/*
 $Id: BrowserController.m,v 1.18 2002/06/24 09:56:52 kindov Exp $
*/

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "BrowserController.h"
#import "ResultsRepository.h"
//#import "Inspector.subproj/InspectorManager.h"
#import <AppKit/AppKit.h>

#import <Foundation/NSString.h>
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSBundle.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSUtilities.h>
#import <Foundation/NSNotification.h>
#ifdef MACOSX
#import <SenBrowserCell.h>
#import <SenBrowserTextCell.h>
#else
#import <SenInterface/SenBrowserCell.h>
#import <SenInterface/SenBrowserTextCell.h>
#endif
#import "NSArray.SenUtilities.h"

#define BROWSER_COLUMN_MIN_WIDTH	@"160"
#define BROWSER_MIN_COL						2

#define FRAME_NAME 	"Browser"
#define FILE_MENUITEM_TAG	(123654)

static int browserColumnWidth;
static int browserColumnCount;
static int browserResizeBehavior;
static NSArray *statusImagesArray=nil;
static NSString *browserColumnWidthKey=@"BrowserColumnWidth";
static NSString *browserColumnCountKey=@"BrowserColumnCount";
static NSString *browserResizeBehaviorKey=@"BrowserResizeBehavior";

//-------------------------------------------------------------------------------------


#if !defined(WIN32) && !defined(MACOSX)
// On MOXS only, NSBrowser does not support key navigation; this Q&D fix corrects this,
// but not when navigating from a branch to its empty content.

@interface NSBrowser_CVLFix : NSBrowser
{
}
@end

@implementation NSBrowser_CVLFix

+ (void) load
{
    [NSBrowser_CVLFix poseAsClass:[NSBrowser class]];
}

- (void) doClick:(id)sender
// Private method in NSBrowser
{
    [super doClick:sender];
    [[self window] makeFirstResponder:self];
}

@end
#endif

// Patch for bug 1000208: Inspector out of sync after a Select All
// Browser does not receive the selectAll: message, it is taken by the matrix
// and browser does not notify that selection changed!
@interface CVLBrowserMatrix:NSMatrix
{
}
@end

@implementation CVLBrowserMatrix

- (void)selectAll:fp12
{
    [super selectAll:fp12];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"CVLBrowserSelectionDidChangeNotification" object:[[self enclosingScrollView] superview]];
}

@end

@interface BrowserController (Private)

- (void) browserResultChanged: (NSNotification *)notification;
- (void) browserResized: (NSNotification *)notification;
- (void) startUpdate:(NSNotification *)notification;
- (void) endUpdate:(NSNotification *)notification;
+ (void)preferencesChanged:(NSNotification *)notification;
- (void)columnSizeChanged:(NSNotification *)aNotification;
- (int) columnForPath: (NSString*) aPath;
- (int) columnForFile: (CVLFile *) aFile;
- (NSBrowserCell *)cellForFile:(CVLFile *)aFile inColumn:(int)column;
- (int)cellRowForFile:(CVLFile *)aFile inColumn:(int)column;
@end

//-------------------------------------------------------------------------------------

@interface BrowserController(BrowserDelegate)
- updateCell:aCell forFile:(CVLFile *)aFile;
@end

//-------------------------------------------------------------------------------------

@implementation BrowserController

+ (void)initialize
{
    if ([self class] == [BrowserController class]) {
        statusImagesArray=[[[[[NSDictionary alloc] initWithContentsOfFile:[[NSBundle bundleForClass:[self class]] pathForResource:@"statusImages" ofType:@"table"]] autorelease] objectForKey:@"defaults"] retain];
//        [[NSUserDefaults standardUserDefaults] registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys:BROWSER_COLUMN_MIN_WIDTH, browserColumnWidthKey, nil]];
// (120) unuseful here, code moved to [CVLDelegate initialize]
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(preferencesChanged:) name:@"PreferencesChanged" object:nil];
        [self preferencesChanged:nil];
    }
}

+ (void)preferencesChanged:(NSNotification *)notification
{
    browserColumnWidth=[[NSUserDefaults standardUserDefaults] integerForKey:browserColumnWidthKey];
    browserColumnCount=[[NSUserDefaults standardUserDefaults] integerForKey:browserColumnCountKey];
    browserResizeBehavior=[[NSUserDefaults standardUserDefaults] integerForKey:browserResizeBehaviorKey];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"ColumnSizeChanged" object:self];
}

- (void)columnSizeChanged:(NSNotification *)aNotification
{
    [delegate viewerFrameSizeChanged:self];
}

+ setImageDict:(NSArray *)dict
{	
  [dict retain];
  [statusImagesArray release];
  statusImagesArray=dict;

  return self;
}


+ (BrowserController*) browserForPath: (NSString*) aPath
{
  return [[[BrowserController alloc] initForPath: aPath] autorelease];
}

- init
{
  return [self initForPath: NSHomeDirectory()];
} // init


- initForPath: (NSString*) pathString;
{
  self= [super init];
    resultsRepository= [[ResultsRepository sharedResultsRepository] retain];
  rootFile = [(id <NSObject>)[CVLFile treeAtPath:pathString] retain];
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(browserResultChanged:)
                                               name:@"ResultsChanged"
                                             object:nil];
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(columnSizeChanged:)
                                               name:@"ColumnSizeChanged"
                                             object:nil];
  return self;
} // initForPath:

- (void) dealloc
{
    int	i;
    
  [[NSNotificationCenter defaultCenter] removeObserver: self];
  [resultsRepository release];
  [rootFile release];
  [filters release];
  [filterProvider release];
  for (i=0;i<=ECInvalidFile;i++) {
    [cellPrototypes[i] release];
      cellPrototypes[i] = nil;
  }
  [browser release];
  //    [blackCellPrototype release];
  //    [grayCellPrototype release];
  [super dealloc];
} // dealloc


- (void)awakeFromNib
{
  SenBrowserTextCell *aCell;
  int i;
  
  cellPrototypes[ECCVSFile]= [[SenBrowserCell cell] retain];

  aCell=[[SenBrowserTextCell alloc] init];
  [cellPrototypes[ECCVSFile] setLeaf:YES];
  [cellPrototypes[ECCVSFile] addTabWithFixedWidth:15];
  [cellPrototypes[ECCVSFile] addTabWithProportionalWidth:100];
  [cellPrototypes[ECCVSFile] setNamePosition:1];
  [cellPrototypes[ECCVSFile] setCell:[[[NSImageCell alloc] init] autorelease] atIndex:0];
  [aCell setDrawsBackground:NO];
  [cellPrototypes[ECCVSFile] setCell:aCell atIndex:1];
  [aCell release];
  [browser setMatrixClass:[CVLBrowserMatrix class]];
  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(select:)
                                               name:@"CVLBrowserSelectionDidChangeNotification"
                                             object:browser];

  for (i=1;i<=ECInvalidFile;i++) {
    cellPrototypes[i]=[cellPrototypes[ECCVSFile] copy];
  }

  aCell=[[SenBrowserTextCell alloc] init];
  [cellPrototypes[ECLocalFile] setCell:aCell atIndex:1];
//  [aCell setTextColor:[NSColor blueColor]];
  [aCell setDrawsBackground:NO];
  [aCell release];

  aCell=[[SenBrowserTextCell alloc] init];
  [cellPrototypes[ECAbsentFile] setCell:aCell atIndex:1];
  [aCell setTextColor:[NSColor darkGrayColor]];
  [aCell release];

  aCell=[[SenBrowserTextCell alloc] init];
  [cellPrototypes[ECInactiveFile] setCell:aCell atIndex:1];
  [aCell setTextColor:[NSColor darkGrayColor]];
  [aCell release];

  aCell=[[SenBrowserTextCell alloc] init];
  [cellPrototypes[ECInvalidFile] setCell:aCell atIndex:1];
  {
      static NSColor	*redColor = nil;

      if(!redColor)
          redColor = [[NSColor colorWithCalibratedRed:0.74 green:0.0 blue:0.0 alpha:1.0] retain];
      [aCell setTextColor:redColor];
  }
  [aCell release];

  filters=[[NSMutableArray alloc] init];
  [browser retain]; // to avoid releasing it by someone else
  [[browser window] release];
  [browser setCellPrototype: cellPrototypes[ECCVSFile]];
  [browser setDelegate: self];
  //    [browser setPath: @""];
//  [browser loadColumnZero];
  [browser setDoubleAction: @selector(doubleSelect:)];
  [browser setAcceptsArrowKeys:YES];
  [browser setSendsActionOnArrowKeys:YES];
  [browser sendAction];		// perform a select: action

  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(browserResized:)
                                               name:NSViewFrameDidChangeNotification
                                               object:browser];
} // awakeFromNib

- (void)setDelegate:(id)aDelegate
{
    delegate=aDelegate;
}

- (void)setFilterProvider:(id)value
{
    [value retain];
    [filterProvider release];
    filterProvider=value;
    [browser loadColumnZero];
}

- (void) reloadData
{
    [browser loadColumnZero];
}

- (void)select: sender
{
  if ([[browser window] isKeyWindow]){
      [[NSNotificationCenter defaultCenter] postNotificationName:@"PathSelected" object: [self selectedPaths]];
  }
} // select:

- (void)doubleSelect: sender
{
  [[NSNotificationCenter defaultCenter] postNotificationName:@"ViewerDoubleSelect" object: self];
} // doubleSelect:


- view
{
  if (!browser) {
    [NSBundle loadNibNamed:@"EasyCVSBrowser.nib" owner:self];
  }
  return browser;
} //view

- (NSString*) rootPath
{
  return [rootFile path];
}

-(NSArray *)relativeSelectedPaths
{
    NSArray *selectedCells;

    selectedCells=[browser selectedCells];

    if ([selectedCells count]) {
        NSMutableArray *result;
        NSString *path;
        id enumerator;
        NSBrowserCell *cell;

        path=[browser pathToColumn:[browser selectedColumn]];
        if ([path length]>1) {
            path=[path substringFromIndex:1];
        } else {
            path=@"";
        }
        result=[NSMutableArray array];
        enumerator=[selectedCells objectEnumerator];

        while (cell=[enumerator nextObject]) {
            [result addObject:[path stringByAppendingPathComponent:[cell stringValue]]];
        }

        return result;
    } else {
        return nil;
    }
}

-(NSArray *)selectedPaths
{
  NSArray *selectedCells;

  selectedCells=[browser selectedCells];

  if ([selectedCells count]) {
    NSMutableArray *result;
    NSString *path;
    id enumerator;
    NSBrowserCell *cell;

    path=[[self rootPath] stringByAppendingPathComponent:[browser pathToColumn:[browser selectedColumn]]];
    result=[NSMutableArray array];
    enumerator=[selectedCells objectEnumerator];

    while (cell=[enumerator nextObject]) {
      [result addObject:[path stringByAppendingPathComponent:[cell stringValue]]];
    }

    return result;
  } else {
      return nil; //    return [NSArray arrayWithObject:rootPath];
  }
}

- (void)selectFiles:(NSSet *)someFiles
{
    NSMutableArray *filePath;
    int column;
    CVLFile *lastColumnSelection;
    CVLFile *columnSelection;
    CVLFile *aFile;
    id enumerator;

    filePath=[NSMutableArray array];
    lastColumnSelection=[someFiles anyObject];
    while ((lastColumnSelection) && (lastColumnSelection!=rootFile)) {
        [filePath insertObject:lastColumnSelection atIndex:0];
        lastColumnSelection=[lastColumnSelection parent];
    }

    if (!lastColumnSelection) return;

    if ([filePath count]) {
        column=0;
        lastColumnSelection=rootFile;

        [filePath removeLastObject];

        enumerator=[filePath objectEnumerator];
        while (columnSelection=[enumerator nextObject]) {
            [browser selectRow:[self cellRowForFile:columnSelection inColumn:column] inColumn:column];
            lastColumnSelection=columnSelection;
            column++;
        }

        // now select all files in last column
        enumerator=[someFiles objectEnumerator];
        while (aFile=[enumerator nextObject]) {
            if ([aFile parent]==lastColumnSelection) {
                [browser selectRow:[self cellRowForFile:aFile inColumn:column] inColumn:column];
            }
        }
    } else {
        [browser selectRow:-1 inColumn:-1];
        // unselect all
    }
    [self select:self];
}

-(BOOL)selectionIsOneDir
{
  NSArray *cells;

  cells=[browser selectedCells];
  return (([cells count] == 1) && (![[cells objectAtIndex:0] isLeaf]));
}

- (NSSize) viewerWillResizetoSize: (NSSize) frameSize
{
    if(browserResizeBehavior){
        // We accept any size
    }
    else{
        int newColumnCount=(int) (((float)(4+frameSize.width)) / (float)(4+browserColumnWidth)+0.5);	

        frameSize.width= newColumnCount*(4+browserColumnWidth)-4;
    }
    requestedBrowserSize=frameSize;
    
    return frameSize;
} // viewerWillResizetoSize:

- (void) browserResized: (NSNotification *)notification
{
    // Invoked ONLY if browser size changed!
    if(browserResizeBehavior){
        [browser setMaxVisibleColumns:browserColumnCount];
    }
    else{
        int newColumnCount;

        newColumnCount= (4+[browser frame].size.width) / (4+browserColumnWidth);
        //    [browser setMinColumnWidth:browserColumnWidth];
        [browser setMaxVisibleColumns: newColumnCount];
    }
}

- (int)cellRowForFile:(CVLFile *)aFile inColumn:(int)column
{
    NSMatrix *matrix=[browser matrixInColumn:column];
    NSBrowserCell *cell = nil;
    int index=0 ,count=[matrix numberOfRows];

    while (index<count) {
        cell=(NSBrowserCell *)[matrix cellAtRow:index column:0];
        if ([aFile isEqual:[cell representedObject]]) {
            break;
        }
        index++;
    }
    
    if (cell) {
        return index;
    } else {
        return -1;
    }
}

- (NSBrowserCell *)cellForFile:(CVLFile *)aFile inColumn:(int)column
{
    NSMatrix *matrix=[browser matrixInColumn:column];
    NSBrowserCell *cell;
    int index=0 ,count=[matrix numberOfRows];

    while (index<count) {
        cell=(NSBrowserCell *)[matrix cellAtRow:index column:0];
        if ([aFile isEqual:[cell representedObject]]) {
            return cell;
        }
        index++;
    }
    return nil;
}

- (NSBrowserCell *)cellForName:(NSString *)aName inColumn:(int)column
{
  NSMatrix *matrix=[browser matrixInColumn:column];
  NSBrowserCell *cell;
  int index=0 ,count=[matrix numberOfRows];

  while (index<count) {
    cell=(NSBrowserCell *)[matrix cellAtRow:index column:0];
    if ([aName isEqual:[cell stringValue]]) {
      return cell;
    }
    index++;
  }
  return nil;
}

- (int) columnForFile: (CVLFile *) aFile
// Move this into a NSBrowser category
{
    return [self columnForPath:[aFile path]];
}

- (int) columnForPath: (NSString*) aPath
// Move this into a NSBrowser category
{	// return column which is displaying 'aPath' or -1 if not found
  NSArray *pathComponents=[aPath pathComponents];
  NSArray *rootPathComponents=[[self rootPath] pathComponents];
  NSArray *localPathArray;
  NSString *pathComponent;
  NSRange localRange;
  id enumerator;
  int count=0;

  localRange.location=0;
  localRange.length=[rootPathComponents count];

  if ([pathComponents count]<localRange.length) {
    return -1;
  }
  localPathArray=[pathComponents subarrayWithRange:localRange];
  if (![localPathArray isEqual:rootPathComponents]) {
    return -1;
  }

  localRange.location=0+localRange.length;
  localRange.length=[pathComponents count]-localRange.length;

  if ([browser selectedColumn]+1<localRange.length) {
    return -1;
  }

  localPathArray=[pathComponents subarrayWithRange:localRange];
  enumerator=[localPathArray objectEnumerator];

  while(pathComponent=[enumerator nextObject]) {
    if (![[[browser selectedCellInColumn:count] stringValue] isEqual:pathComponent]) {
      return -1;
    } else {
      count++;
    }
  }

  return count;
} //columnForPath:

- (void) filteredContentsChanged:(NSNotification *)notification
{
    int column;

    column=[filters indexOfObject:[notification object]];
    [browser reloadColumn: column];
    [self select:self];
}

- (void) browserResultChanged: (NSNotification *)notification
{
    id fileEnum;
    NSBrowserCell *cell;
    NSString *name;
    int column;
    CVLFile *file,*directory;
    ECFileAttributeGroups changes;

    if (!updateCount) {
        if ((resultsRepository == [notification object]) && ([resultsRepository hasChanged])) {	
            [self startUpdate:nil];
            //[[browser window] disableFlushWindow];

            //reloadedPaths=[NSMutableSet set];
            //columnsToDisplay=[NSMutableSet set];
            fileEnum=[[resultsRepository changedFiles] objectEnumerator];

            while (file= [fileEnum nextObject]) {
                directory=[file parent];
                column=[self columnForFile:directory];

                if (column>=0 || file == rootFile) {
                    changes=[file changes];
                    if (changes.status || changes.quickStatus) {
                        //if (![reloadedPaths containsObject: directory]) {
                        if(file != rootFile){
                            name=[[file path] lastPathComponent];
                            if (cell=[self cellForFile:file inColumn:column]) {
                                [cell setLoaded:NO];
                                [browser setNeedsDisplay:YES];
                                //[self updateCell:cell forName:name inDir:dir];
                                //[columnsToDisplay addNewObject:[NSNumber numberWithInt:column]];
                            }
                        }
                        else{
                            (void)[file status]; // In order to have root directory behave like any other dir, we NEED to ask it for its status!!!
                        }
                        //}
                    }
                }
            }

            /*enumerator=[columnsToDisplay objectEnumerator];
            while (columnNumber=[enumerator nextObject]) {
                [browser displayColumn:[columnNumber intValue]];
            }*/

            //                [[browser window] enableFlushWindow];
            [self endUpdate:nil];
        }
    }
} // browserResultChanged:


- (void) startUpdate:(NSNotification *)notification
{
  if (!updateCount) {
    updateCount++;
    [[NSNotificationCenter defaultCenter] postNotificationName:@"StartUpdate" object:self];
  } else {
    updateCount++;
  }
}


- (void) endUpdate:(NSNotification *)notification
{
  updateCount--;
  if (!updateCount) {
    [[NSNotificationCenter defaultCenter] postNotificationName:@"BrowserChanged" object:self];
  }
}


@end

//-------------------------------------------------------------------------------------

@implementation BrowserController(BrowserDelegate)

- updateCell:aCell forFile:(CVLFile *)aFile
{
  NSString *absolutePath;
  NSString *imageName;
  ECStatus status;
  ECFileFlags fileFlags;

  [self startUpdate:nil];
  [resultsRepository startUpdate];
  absolutePath= [aFile path];

  fileFlags=[aFile flags];

  [aCell setLoaded:YES];
  [aCell setRepresentedObject:aFile];
  [aCell setLeaf:[aFile isLeaf]];
  [aCell setStringValue:[aFile name]];
  [aCell useTemplate:cellPrototypes[fileFlags.type]];

  status=[aFile status];
  if (!(imageName=[statusImagesArray objectAtIndex:status.statusType])) {
      imageName=[statusImagesArray objectAtIndex:0];
  }
  
  [aCell setObjectValue:[NSImage imageNamed:imageName] atIndex:0];
#ifdef MACOSX
  // Default system font can not have italic trait!
  if(![aFile hasStickyTagOrDate])
      [aCell setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
  else
      [aCell setFont:[[NSFontManager sharedFontManager] convertFont:[NSFont fontWithName:@"Helvetica" size:[NSFont smallSystemFontSize]] toHaveTrait:NSItalicFontMask]];
#else
  [aCell setFont:[[NSFontManager sharedFontManager] convertFont:[aCell font] toHaveTrait:([aFile hasStickyTagOrDate] ? NSItalicFontMask:NSUnitalicFontMask)]];
#endif

  [aCell setEnabled:YES];
  [resultsRepository endUpdate];
  [self endUpdate:nil];

  return self;
} // updateCell:forName:


#if 0
//#warning (120) a mettre en commentaire d'abord, pour les lazy et remettre pour les very lazy
- (int) browser: sender getNumRowsInColumn:(int) column
{
    char fullPath[MAXPATHLEN+1];
    NSArray* contentArray;

    fullPath[0]= '\0';
    [sender getPath: fullPath toColumn: column];
    strcat(fullPath, "/");
    contentArray= [resultsRepository completedDirForPath: [NSString stringWithCString: fullPath]];
    return [contentArray count];
} // browser:getNumRowsInColumn:
#endif

- (void)browser:(NSBrowser *)sender willDisplayCell:(id)cell atRow:(int)row column:(int)column
{
  [self startUpdate:nil];
  [resultsRepository startUpdate];

  if (![cell isLoaded]) {
    NSArray* contentArray;

    contentArray= [[filters objectAtIndex:column] filteredContents];
    if (row<[contentArray count]) {
        //printf("### load cell at col %d row %d, path %s\n", column, row, [fullPath cString]);
        [self updateCell:cell forFile:[contentArray objectAtIndex:row]];
    } 
  }

  [resultsRepository endUpdate];
  [self endUpdate:nil];
} //  browser:loadCell:atRow:inColumn:column

- (void)browser:(NSBrowser *)sender createRowsForColumn:(int)column inMatrix:(NSMatrix *)matrix
{
  NSString *fullPath;
  NSArray *contentArray;		
  id enumerator;
  int count= 0;
  CVLFile *file;
  DirectoryContentsFilter *filter;
		
  if (![[sender cellPrototype] isKindOfClass:[SenBrowserCell class]]) {	
    // this happens when the browser display itself and before awakeFromNib is called !
    return;
  }
  
  [self startUpdate:nil];
  [resultsRepository startUpdate];

  fullPath=[[self rootPath] stringByAppendingPathComponent:[browser pathToColumn:column]];

  // set up filter if needed
  if (filterProvider) {
      if ([filters count]<=column) {
          filter=[filterProvider filterForDirectory:fullPath];
          [filters addObject:filter];
          [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(filteredContentsChanged:)
                                                       name:@"FilteredContentsChanged"
                                                     object:filter];
      } else if (![[(filter=[filters objectAtIndex:column]) filteredDirectoryPath] isEqual:fullPath]) {
          [[NSNotificationCenter defaultCenter] removeObserver:self name:nil object:filter];
          filter=[filterProvider filterForDirectory:fullPath];
          [filters replaceObjectAtIndex:column withObject:filter];
          [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(filteredContentsChanged:)
                                                       name:@"FilteredContentsChanged"
                                                     object:filter];
      }
      contentArray= [filter filteredContents];
  } else {
      contentArray=nil;
  }
  
  enumerator=[contentArray objectEnumerator];
  while (file=[enumerator nextObject]) {
    [matrix addRow];
    //printf("### fill matrix col %d row %d, path %s\n", column, count, [fullPath cString]);
      [self updateCell:[matrix cellAtRow:count column:0] forFile:file];
    count++;
  }
  [matrix sizeToCells];

  [resultsRepository endUpdate];
  [self endUpdate:nil];

  // Add contextual menu: use File menu
  if(![matrix menu])
      [matrix setMenu:[[[[[[NSApplication sharedApplication] mainMenu] itemWithTag:FILE_MENUITEM_TAG] submenu] copy] autorelease]];
} // browser:fillMatrix:inColumn:

@end


//--------------------------------------------------------------------------------