/*
 $Id: CVLDelegate.m,v 1.22 2002/07/04 12:19:56 stephane Exp $
 */

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "CVLDelegate.h"
#import "BrowserController.h"
#import "WorkAreaViewer.h"
#import "ProgressPanelController.h"
#import "ResultsRepository.h"
#ifdef MACOSX
#import <CvsRequest.h>
#import <CvsCheckoutRequest.h>
#import <CvsLoginRequest.h>
#else
#import <Requests/CvsRequest.h>
#import <Requests/CvsCheckoutRequest.h>
#import <Requests/CvsLoginRequest.h>
#endif
#import "CVLConsole.subproj/CVLConsoleController.h"
#import "InfoPanel.subproj/InfoController.h"
#import "Inspector.subproj/CVLInspectorManager.h"
#import <SenOyster/NSString.SenOysterRegex.h>
#import "NSFileManager.MkDirs.h"
#import "NSFileManager_CVS.h"
#ifdef MACOSX
#import <CvsPserverRepository.h>
#else
#import <Repositories/CvsPserverRepository.h>
#endif
#import "RepositoryViewer.h"
#import <AppKit/AppKit.h>
#import <Foundation/Foundation.h>
#ifdef MACOSX
#import <SenPanelFactory.h>
#import <SenFormPanelController.h>
#import <CVLSelectingFilesRequest.h>
#else
#import <SenPanels/SenPanelFactory.h>
#import <SenPanels/SenFormPanelController.h>
#import <Requests/CVLSelectingFilesRequest.h>
#endif
#import "Comparator.subproj/Comparator.h"
#ifdef MACOSX
#import <CvsImportRequest.h>
#else
#import <Requests/CvsImportRequest.h>
#endif
#import <SenFoundation/SenFoundation.h>
#import <CVLEditorClient.h>


#define BROWSERS_KEY			@"Browsers"
#define RESOURCES_FILENAME		@"Paths.plist"

//-------------------------------------------------------------------------------------

@interface CVLDelegate(Private)
- (void) viewerDidBecomeKey: (NSNotification *)notification;
- (void) viewerWillClose: (NSNotification *)notification;
- (void) openCurrentBrowsersFrom: (NSArray*) browserArray;
- (NSArray*) restoreBrowsersFrom: (NSArray*) browserArray;
- (NSString *)getCvsRoot;
//- (void)goForModuleCvsCheckoutInRepository: (CvsRepository *)aRepository;
//- (void)repositoryUpdated:(NSNotification *)notification;
- (void)setActiveViewer:(id)aViewer;
- (void)viewerSelectionDidChange:(NSNotification *)notification;
- (WorkAreaViewer *) currentViewerShowingFile:(CVLFile *)aFile;
- (NSArray*) existingDirsFrom: (NSArray*) browserArray;
@end

@interface CVLDelegate (ApplicationDelegate)
- (IBAction) showInspectorPanel:(id)sender;
- (IBAction) showProgressPanel:(id)sender;
- (IBAction) showRepositoryViewer:(id)sender;
@end

//-------------------------------------------------------------------------------------

@implementation CVLDelegate

+ (void)initialize
{
    static BOOL initialized = NO;

    /* Make sure code only gets executed once. */
    if (initialized)
    {
        return;
    }
    else
    {
        initialized = YES;

        [NSApp registerServicesMenuSendTypes: [NSArray arrayWithObjects: NSFilenamesPboardType, NSStringPboardType, nil] returnTypes: nil];

#ifndef WIN32
        [[NSUserDefaults standardUserDefaults] registerDefaults:[NSDictionary dictionaryWithObjectsAndKeys:[NSString pathWithComponents:[NSArray arrayWithObjects:@"/", @"usr", @"bin", @"opendiff", nil]], @"OpendiffPath", nil]];
#ifdef MACOSX
        [[NSUserDefaults standardUserDefaults] registerDefaults:[NSDictionary dictionaryWithObjectsAndKeys:[NSString pathWithComponents:[NSArray arrayWithObjects:@"/", @"usr", @"bin", @"cvs", nil]], @"CVSPath", nil]];
#endif
#else WIN32
        [[NSUserDefaults standardUserDefaults] registerDefaults:[NSDictionary dictionaryWithObjectsAndKeys:[NSString pathWithComponents:[NSArray arrayWithObjects:[NSSearchPathForDirectoriesInDomains(NSDeveloperDirectory, NSSystemDomainMask, YES) lastObject], @"Executables", [@"opendiff" stringByAppendingPathExtension:@"exe"], nil]], @"OpendiffPath", nil]];
#endif

        [[NSUserDefaults standardUserDefaults] registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys: @"160", @"BrowserColumnWidth", nil]];
        [[NSUserDefaults standardUserDefaults] registerDefaults: [NSDictionary dictionaryWithObjectsAndKeys: @"2", @"BrowserColumnCount", nil]];
        {
            // Following code was in +[CvsRequest initialize], which is called only when we create the first request. Too late.
#ifdef __ppc__
            NSString* registeredValue= [NSString stringWithString: @"1"];
#else
            NSString* registeredValue= [NSString stringWithString: @"5"];
#endif
            [[NSUserDefaults standardUserDefaults] registerDefaults:[NSDictionary dictionaryWithObjectsAndKeys:registeredValue, @"MaxParallelRequestsCount", @"3", @"CompressionLevel", @"NO", @"ShowBrowsers", nil]];
        }
        [[NSUserDefaults standardUserDefaults] registerDefaults:[NSDictionary dictionaryWithObjectsAndKeys:@"YES", @"CvsTemplateWarning", nil]];
        [[NSUserDefaults standardUserDefaults] registerDefaults:[NSDictionary dictionaryWithObjectsAndKeys:@"%b %d %Y", @"CVLDateFormat", nil]];
    }
    return;
}


+ (NSString*) oldResourceDirectory
{
#ifndef MACOSX
    return [NSHomeDirectory() stringByAppendingPathComponent:[@".AppInfo" stringByAppendingPathComponent:[[NSProcessInfo processInfo] processName]]];
#else
    return [[NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) lastObject] stringByAppendingPathComponent:[[NSProcessInfo processInfo] processName]];
#endif
}

+ (NSString*) resourceDirectory
// Stephane: on devrait faire une categorie pour NSApplication
{
#if 0
    return [[NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) lastObject] stringByAppendingPathComponent:[[NSProcessInfo processInfo] processName]];
#else
    // We no longer want to store pref in ~/.AppInfo/CVL
    // We want to be good citizen and store this in ~/Library/CVL
    // To keeps things backward compatible, we copy content of ~/.AppInfo/CVL in ~/Library/CVL
    // if it has not yet been done, so the user will not loose its preferences.
    // If we cannot write to ~/Library/CVL, we try to write in $TEMP directory

#ifndef MACOSX
    static NSString	*resourceDirectory = nil;

    if(!resourceDirectory){
        NSFileManager	*fileManager = [NSFileManager defaultManager];
        BOOL			workWithTemporaryDirectory = NO;
        
        resourceDirectory = [[NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) lastObject] stringByAppendingPathComponent:[[NSProcessInfo processInfo] processName]];

        NSAssert(resourceDirectory != nil, @"Unable to build resource directory path!");

        if(!([fileManager cvlDirectoryExistsAtPath:resourceDirectory])){
            if(![fileManager cvlDirectoryExistsAtPath:[self oldResourceDirectory]]){
                // Didn't even exist in .AppInfo so create it in ~/Library
                if(![fileManager createAllDirectoriesAtPath:resourceDirectory attributes: nil])
                    workWithTemporaryDirectory = YES;
            }
            else{
                // Exists in .AppInfo (but not in ~/Library) so let's copy it to the new location
                if(![fileManager copyPath:[self oldResourceDirectory] toPath:resourceDirectory handler:nil]){
                    // Error during copy, so let's forget about old preferences...
                    NSLog(@"### Unable to restore preferences by copying %@ to %@. Let's forget about them...", [self oldResourceDirectory], resourceDirectory);
                    if(![fileManager createAllDirectoriesAtPath:resourceDirectory attributes:nil])
                        workWithTemporaryDirectory = YES;
                }
            }
        }

        if(!workWithTemporaryDirectory)
            if(![fileManager isWritableFileAtPath:resourceDirectory])
                workWithTemporaryDirectory = YES;
            else
                [resourceDirectory retain];
        
        if(workWithTemporaryDirectory){
            NSString	*temporaryDir = [[NSTemporaryDirectory() stringByAppendingPathComponent:[[NSProcessInfo processInfo] processName]] stringByAppendingPathComponent:NSUserName()];

            if(NSRunAlertPanel([[NSProcessInfo processInfo] processName], @"Unable to write in directory %@. CVL will temporarily use %@, but you should correct this problem.", @"Quit", @"Continue anyway", nil, resourceDirectory, temporaryDir) == NSAlertDefaultReturn)
                [NSApp terminate:nil];

            NSLog(@"### Unable to create directory %@. CVL needs this directory. Correct this. CVL will temporarily work with %@", resourceDirectory, temporaryDir);
            if(![fileManager cvlDirectoryExistsAtPath:temporaryDir])
                if(![fileManager createAllDirectoriesAtPath:temporaryDir attributes:nil]){
                    (void)NSRunCriticalAlertPanel([[NSProcessInfo processInfo] processName], @"Unable to create directory %@. Correct this problem first.", @"Quit", nil, nil, temporaryDir);
                    [NSApp terminate:nil];
                }

            if(![fileManager isWritableFileAtPath:temporaryDir]){
                (void)NSRunCriticalAlertPanel([[NSProcessInfo processInfo] processName], @"Unable to write in directory %@. Correct this problem first.", @"Quit", nil, nil, temporaryDir);
                [NSApp terminate:nil];
            }
            else
                resourceDirectory = [temporaryDir retain];                
        }
    }
#else
#warning CHECK THIS
    static NSString	*resourceDirectory = nil;

    if(!resourceDirectory){
        NSFileManager	*fileManager = [NSFileManager defaultManager];
        BOOL			workWithTemporaryDirectory = NO;

        // Is there a public API to get the ~/Library/Application Support/CVL directory??
        resourceDirectory = [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) lastObject];
        resourceDirectory = [resourceDirectory stringByAppendingPathComponent:@"Application Support"];
        resourceDirectory = [resourceDirectory stringByAppendingPathComponent:[[NSProcessInfo processInfo] processName]];

        NSAssert(resourceDirectory != nil, @"Unable to build resource directory path!");

        if(![fileManager cvlDirectoryExistsAtPath:resourceDirectory]){
            if(![fileManager cvlDirectoryExistsAtPath:[self oldResourceDirectory]]){
                // Didn't even exist in ~/Library so create it in ~/Library/Application Support
                if(![fileManager createAllDirectoriesAtPath:resourceDirectory attributes: nil])
                    workWithTemporaryDirectory = YES;
            }
            else{
                // Exists in .AppInfo (but not in ~/Library) so let's copy it to the new location
                if(![fileManager copyPath:[self oldResourceDirectory] toPath:resourceDirectory handler:nil]){
                    // Error during copy, so let's forget about old preferences...
                    NSLog(@"### Unable to restore preferences by copying %@ to %@. Let's forget about them...", [self oldResourceDirectory], resourceDirectory);
                    if(![fileManager createAllDirectoriesAtPath:resourceDirectory attributes:nil])
                        workWithTemporaryDirectory = YES;
                }
            }
        }

        if(!workWithTemporaryDirectory)
            if(![fileManager isWritableFileAtPath:resourceDirectory])
                workWithTemporaryDirectory = YES;
            else
                [resourceDirectory retain];

        if(workWithTemporaryDirectory){
            NSString	*temporaryDir = [[NSTemporaryDirectory() stringByAppendingPathComponent:[[NSProcessInfo processInfo] processName]] stringByAppendingPathComponent:NSUserName()];

            if(NSRunAlertPanel([[NSProcessInfo processInfo] processName], @"Unable to write in directory %@. CVL will temporarily use %@, but you should correct this problem.", @"Quit", @"Continue anyway", nil, resourceDirectory, temporaryDir) == NSAlertDefaultReturn)
                [NSApp terminate:nil];

            NSLog(@"### Unable to create directory %@. CVL needs this directory. Correct this. CVL will temporarily work with %@", resourceDirectory, temporaryDir);
            if(![fileManager cvlDirectoryExistsAtPath:temporaryDir])
                if(![fileManager createAllDirectoriesAtPath:temporaryDir attributes:nil]){
                    (void)NSRunCriticalAlertPanel([[NSProcessInfo processInfo] processName], @"Unable to create directory %@. Correct this problem first.", @"Quit", nil, nil, temporaryDir);
                    [NSApp terminate:nil];
                }

            if(![fileManager isWritableFileAtPath:temporaryDir]){
                (void)NSRunCriticalAlertPanel([[NSProcessInfo processInfo] processName], @"Unable to write in directory %@. Correct this problem first.", @"Quit", nil, nil, temporaryDir);
                [NSApp terminate:nil];
            }
            else
                resourceDirectory = [temporaryDir retain];
        }
    }
#endif
    return resourceDirectory;
#endif
} // resourceDirectory

- (NSString*) resourceDirectory
{
    return [[self class] resourceDirectory];
}

+ (NSString*) resourceFilename
{
  return [[self resourceDirectory] stringByAppendingPathComponent:RESOURCES_FILENAME];
} // resourceFilename


- init
{
    NSString* resourceFilename;
    NSDictionary* dict;

    self= [super init];
    activeViewer= nil;
    currentBrowsers= [[NSMutableDictionary dictionaryWithCapacity: 0] retain];
    resultsRepository=[[ResultsRepository sharedResultsRepository] retain];
    [[NSNotificationCenter defaultCenter]
                addObserver:self
                   selector:@selector(browserDidBecomeKey:)
                       name:@"BrowserDidBecomeKey"
                     object:nil];
    [[NSNotificationCenter defaultCenter]
                addObserver:self
                   selector:@selector(viewerDidBecomeKey:)
                       name:@"ViewerDidBecomeKey"
                     object:nil];
    [[NSNotificationCenter defaultCenter]
                addObserver:self
                   selector:@selector(viewerWillClose:)
                       name:@"ViewerWillClose"
                     object:nil];
    [[NSNotificationCenter defaultCenter]
                addObserver:self
                   selector:@selector(requestStateChanged:)
                       name:@"RequestStateChanged"
                     object:nil];
    [[NSNotificationCenter defaultCenter]
                addObserver:self
                   selector:@selector(requestReceivedData:)
                       name:@"RequestReceivedData"
                     object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(updateRepositoryImportServiceEntries:) name:@"RegisteredRepositoriesChanged" object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(pathSelected:) name:@"PathSelected" object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(missingCvsWrappersFile:) name:CvsPserverHasNoCvsWrappersNotification object:nil];

    resourceFilename= [[self class] resourceFilename];
    dict= [[[NSDictionary alloc] initWithContentsOfFile: resourceFilename] autorelease];
    restoredBrowsers= [[self existingDirsFrom:[dict objectForKey: BROWSERS_KEY]] retain];
    
    globalSelection=[[SenSelection alloc] init];
    //  [[CvsRepository new] setLocalCopyPath: [[self class] resourceDirectory]];

    cvlEditorClientConnection = [NSConnection defaultConnection];
    [cvlEditorClientConnection setRootObject:self];
    if(![cvlEditorClientConnection registerName:CVLEditorClientConnectionName]) {
        NSLog(@"Couldn't register %@ connection!", CVLEditorClientConnectionName);
        (void)NSRunAlertPanel([[NSProcessInfo processInfo] processName], @"An error occurred when initializing use of cvs templates; template use has been disabled.", nil, nil, nil);
        cvlEditorClientConnection = nil;
        [[NSUserDefaults standardUserDefaults] setBool:NO forKey:@"UseCvsTemplates"];
    }
    else
        [cvlEditorClientConnection retain];
    
    return self;
} // init


-(void)dealloc
{
  [resultsRepository release];
  [restoredBrowsers release];
  [currentBrowsers release];
  [globalSelection release];
  
  [[NSNotificationCenter defaultCenter] removeObserver:self];
  [cvlEditorClientConnection release];
  [super dealloc];
} // dealloc

- (IBAction) openModule:(id)sender
{
    NSOpenPanel	*panel= [NSOpenPanel openPanel];
    int			value = 0;

    [panel setCanChooseDirectories:YES];
    [panel setTitle:@"Choose"];
    [panel setAllowsMultipleSelection:NO];
    value = [panel runModalForTypes:nil];		// all types

    if(value == NSOKButton)
        [self openViewerForPath:[[panel filenames] objectAtIndex:0]];
} // openModule:

- (void) openViewerForPath:(NSString *)aPath
{
    [[self viewerForPath:aPath] show:self];
}

- (id) viewerForPath:(NSString *)aPath
{
    WorkAreaViewer	*viewer = [currentBrowsers objectForKey:aPath];
    
    if(!viewer){
        viewer = [WorkAreaViewer viewerForPath:aPath];
        [currentBrowsers setObject:viewer forKey:aPath];
    }
    return viewer;
}

- doRepositoryLogin:sender
{
    [[CvsLoginRequest cvsLoginRequest] schedule];
    return self;
}

- (WorkAreaViewer *)viewerWithRootFile:(CVLFile *)aFile
{
    id viewerEnumerator;
    WorkAreaViewer *viewer;

    viewerEnumerator=[currentBrowsers objectEnumerator];

    while (viewer=[viewerEnumerator nextObject]) {
        if (aFile==[viewer rootFile]) {
            return viewer;
        }
    }

    return nil;
}

- (WorkAreaViewer *) currentViewerShowingFile:(CVLFile *)aFile
{
    id viewerEnumerator;
    WorkAreaViewer *viewer;

    viewerEnumerator=[currentBrowsers objectEnumerator];

    while (viewer=[viewerEnumerator nextObject]) {
        if ([aFile isDescendantOf:[viewer rootFile]]) {
            return viewer;
        }
    }

    return nil;
}

- (WorkAreaViewer *)viewerShowingFile:(CVLFile *)aFile
{
    WorkAreaViewer *viewer = [self currentViewerShowingFile:aFile];
	
    if(!viewer)
        viewer = [self newViewerShowingFile:aFile];

    return viewer;
}

- (WorkAreaViewer *)newViewerShowingFile:(CVLFile *)aFile
{
    CVLFile *openedFile;

#if 0
    if (![aFile isLeaf]) {
        openedFile=aFile;
    } else {
        openedFile=[aFile parent];
    }
#else
    if (![aFile isLeaf]) {
        openedFile=aFile;
    } else {
        // We can't always trust isLeaf, because returned status is wrong if file
        // has never been loaded earlier (no up-to-date repository, thus no loadedChildren,
        // thus considered as leaf!)
        // This test should work in most (if not all) cases
        if([[NSFileManager defaultManager] cvlDirectoryExistsAtPath:[aFile path]] && ![[[aFile parent] repository] isWrapper:[aFile path]])
            openedFile=aFile;
        else
            openedFile=[aFile parent];
    }
#endif

    return [self viewerForPath:[openedFile path]];
}

- (void) delayedUpdateDynamicServices:(id)dummy
{
    NSUpdateDynamicServices();
}

- (void) updateRepositoryImportServiceEntries:(id)dummy
{
	NSString		*aPath = [[NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) lastObject] stringByAppendingPathComponent:@"Services"];
    NSMutableArray	*serviceEntries = [NSMutableArray array];
    CvsRepository	*aRepository;
    NSEnumerator	*repositoryEnum = [[[CvsRepository registeredRepositories] sortedArrayUsingSelector:@selector(compareRoot:)] objectEnumerator];
    NSFileManager	*fileManager = [NSFileManager defaultManager];

    // Create ~/Library/Services/CVL.service/Resources if necessary
    aPath = [[aPath stringByAppendingPathComponent:@"CVL.service"] stringByAppendingPathComponent:@"Resources"];
    if(![fileManager fileExistsAtPath:aPath])
        (void)[fileManager createAllDirectoriesAtPath:aPath attributes:nil];

    aPath = [aPath stringByAppendingPathComponent:@"Info.plist"];

    while(aRepository = [repositoryEnum nextObject]){
        NSDictionary	*aService = [NSDictionary dictionaryWithObjectsAndKeys:[NSDictionary dictionaryWithObject:[@"CVL/Import into " stringByAppendingString:[aRepository root]] forKey:@"default"], @"NSMenuItem", @"importIntoRepository", @"NSMessage", @"CVL", @"NSPortName", [NSArray arrayWithObject:NSFilenamesPboardType], @"NSSendTypes", [aRepository root], @"NSUserData", nil];
        
        [serviceEntries addObject:aService];
    }
    (void)[[NSDictionary dictionaryWithObject:serviceEntries forKey:@"NSServices"] writeToFile:aPath atomically:YES];

    // Notif is sent many times on startup, thus we would call NSUpdateDynamicServices() many times at once
    // To avoid message "make_services: someone else is already making services." we play with delayed actions
    [NSRunLoop cancelPreviousPerformRequestsWithTarget:self selector:@selector(delayedUpdateDynamicServices:) object:nil];
    [self performSelector:@selector(delayedUpdateDynamicServices:) withObject:nil afterDelay:0];
}

- (SenSelection *)globalSelection
{
    return globalSelection;
}

- (void) missingCvsWrappersFile:(NSNotification *)aNotif
{
    (void)NSRunInformationalAlertPanel(@"CVL", @"To use wrappers with a cvs pserver (%@), you need a .cvswrappers file in your home directory (%@). This file should be a copy of the server's $CVSROOT/CVSROOT/cvswrappers file.", nil, nil, nil, [[aNotif object] root], [[aNotif object] homeDirectory]);
}

@end

//-------------------------------------------------------------------------------------

@implementation CVLDelegate(RequestObserver)


- (void) requestStateChanged: (NSNotification *)notification
{
#ifdef JA_PATCH
    Request *request=[notification object];

    if ([[request currentState] valueForKey:@"startsCvs"]) {
        if (!(CvsRequest *)[request isQuiet])
          {
            [[CVLConsoleController sharedConsoleController] output: [(CvsRequest *)request shortDescription] bold: YES];
          }
    }
#else
    if ([[notification object] state]==STATE_RUNNING) {
        if ([[notification object] isKindOfClass: [CvsRequest class]])
        {
            CvsRequest *request=[notification object];

            if (![request isQuiet])
            {
                [[CVLConsoleController sharedConsoleController] output: [request shortDescription] bold: YES];
            }
        }
    }
#endif
}


- (void) requestReceivedData:(NSNotification *)notification
{
    if ([[notification object] isKindOfClass: [CvsRequest class]])
    {
        CvsRequest *request=[notification object];
        NSString *newBufPart=[[notification userInfo] objectForKey:@"RequestNewData"];

        if ([[[notification userInfo] objectForKey:@"ErrorStream"] isEqual:@"YES"]) {
            [[CVLConsoleController sharedConsoleController] outputError: newBufPart];
        } else if (![(CvsRequest *)request isQuiet]) {
            [[CVLConsoleController sharedConsoleController] output: newBufPart];
        }
    }
    return;
}

- (void) requestCompleted:(NSNotification *)notification
{ // We assume that this is a checkout request because we only observe these at the moment
    Request *request=[notification object];

    if ([request succeeded])
        [self openViewerForPath:[(CvsCheckoutRequest *)request destinationPath]];
    
    [[NSNotificationCenter defaultCenter]
                      removeObserver:self
                             name:nil
                           object:[notification object]];
}
@end


//-------------------------------------------------------------------------------------

@implementation CVLDelegate(ApplicationDelegate)

#if 0
// maybe useful later for app delegate actions
- (BOOL)validateMenuItem:(id <NSMenuItem>)menuItem
{
  SEL action= [menuItem action];
  NSString* actionString= NSStringFromSelector(action);
  NSRange filesRange= [actionString rangeOfString: @"Files"];
  NSRange workareaRange= [actionString rangeOfString: @"WorkArea"];

  //  NSLog(@"validate action %@", actionString);

  if (filesRange.length)
  {
    // action if for files, check selection > 0
    return ([[activeViewer selectedPaths] count] > 0);
  }
  else if (workareaRange.length)
  {
    // action if for files, check selection > 0
    return (BOOL) (activeViewer != nil);
  }
  else
  {
    return YES;  // validate other actions by default
  }
}
#endif

- (NSArray*) existingDirsFrom: (NSArray*) browserArray
{
  NSMutableArray* result= [NSMutableArray arrayWithCapacity: 0];
  NSEnumerator* bEnum= [browserArray objectEnumerator];	
  id browserInfoDict;
    NSFileManager *fileManager = [NSFileManager defaultManager];

  while ((browserInfoDict= [bEnum nextObject]))
  {
    if ([fileManager cvlDirectoryExistsAtPath: [browserInfoDict objectForKey: @"path"]])
    { // real directory, keep it
      [result addObject: browserInfoDict];
    }
  }
  return result;
} // existingDirsFrom:


- (void) registerToAvatarNotifications
{
    [[NSDistributedNotificationCenter defaultCenter] addObserver:self
                                                        selector:@selector (fileDidSave:)
                                                            name:@"FileDidSave"
                                                          object:@"CVLAvatar"
                                              suspensionBehavior:NSNotificationSuspensionBehaviorDeliverImmediately];

}

- (void) fileDidSave:(NSNotification *) notification
{
    NSString *path = [[notification userInfo] objectForKey:@"Path"];
    [(CVLFile *)[CVLFile treeAtPath:path] invalidateAll];
}

- (void) delayedOpenCurrentBrowsersFrom:(NSArray *)anArray
{
    [self openCurrentBrowsersFrom:anArray];
}

#ifdef WIN32
- (IBAction) showHelp:(id)sender
{
    if(![helpView readRTFDFromFile:[[NSBundle mainBundle] pathForResource:@"CVL" ofType:@"rtfd"]])
        NSBeep();
    else
        [[helpView window] makeKeyAndOrderFront:sender];
}
#endif

- (void) applicationDidFinishLaunching:(NSNotification *)notUsed
{
    NSUserDefaults *defaults=[NSUserDefaults standardUserDefaults];

    NS_DURING
        // If CVL is launched for the first time, let's open the help file!
#ifndef MACOSX
        if(![[defaults persistentDomainForName:[[NSProcessInfo processInfo] processName]] count])
#ifdef WIN32
            [self showHelp:nil];
#else
            [[NSApplication sharedApplication] showHelp:nil];
#endif
#else
        if(![[defaults persistentDomainForName:@"ch.sente.CVL"] count])
            [[NSApplication sharedApplication] showHelp:nil];
#endif

        // services stuff
        [NSApp setServicesProvider: self]; //: [NSArray arrayWithObjects: NSFilenamesPboardType, NSStringPboardType, nil] returnTypes: nil];

        [[NSApp mainMenu] setAutoenablesItems: YES];

        if ([defaults boolForKey:@"ShowInspector"]) {
            [self showInspectorPanel: self];
        }
        if ([defaults boolForKey:@"ShowProgressPanel"]) {
            [self showProgressPanel: self];
        }
        if ([[CVLConsoleController sharedConsoleController] showConsoleAtStartup]) {
            [[CVLConsoleController sharedConsoleController] showWindow: self];
        }

#ifndef WIN32

        if ([defaults boolForKey:@"ShowRepositoryViewer"]) {
            [[RepositoryViewer sharedRepositoryViewer] showWindow:self];
        }

#else

        [[RepositoryViewer sharedRepositoryViewer] showWindow:self]; // always show this window under NT
#endif

//        [self openCurrentBrowsersFrom: restoredBrowsers];
        if ([defaults boolForKey:@"ShowBrowsers"])
            // Thank you to Tom Hageman <trh@xs4all.nl> for this feature
            [self performSelector:@selector(delayedOpenCurrentBrowsersFrom:) withObject:restoredBrowsers afterDelay:0];
        [ProgressPanelController sharedProgressPanelController];	// to force it getting notifs
        [CVLInspectorManager sharedInspector];

        [self registerToAvatarNotifications];

    NS_HANDLER
        NSRunCriticalAlertPanel([localException name], [localException reason], @"Quit", nil, nil);
        [NSApp terminate: nil];
    NS_ENDHANDLER
}


- (void) applicationWillTerminate:(NSNotification *)notUsed
{
  NSString* resourceFilename= [[self class] resourceFilename];
  NSArray* objects= [NSArray arrayWithObjects: [self restoreBrowsersFrom: [currentBrowsers allValues]],nil];
  NSArray* keys= [NSArray arrayWithObjects:BROWSERS_KEY,nil];
  NSDictionary* dict= [NSDictionary dictionaryWithObjects: objects forKeys: keys];
  BOOL ok= NO;

  ok= [dict writeToFile: resourceFilename atomically: NO];
  if (!ok)
  {	// try to create nonexistent path
    [[NSFileManager defaultManager] createAllDirectoriesAtPath: [resourceFilename stringByDeletingLastPathComponent]
                                                    attributes: nil];
    ok= [dict writeToFile: resourceFilename atomically: NO];
  }
    [cvlEditorClientConnection invalidate];
}

#ifdef WIN32
- (BOOL) applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)sender
{
    if(NSRunAlertPanel(@"CVL", @"Do you really want to quit CVL?", @"Yes", @"No", nil) == NSAlertAlternateReturn){
        (void)[self showRepositoryViewer:nil];
        return NO;
    }
    else
        return YES;
}
#endif

- (IBAction) showConsole:(id)sender
{
    [[CVLConsoleController sharedConsoleController] showWindow:sender];
}

- (IBAction) clearConsole:(id)sender
{
    [[CVLConsoleController sharedConsoleController] clearText:sender];
}

- (IBAction) showPreferences:(id)sender
{
    [[[SenPanelFactory sharedPanelFactory] handleOfPanelWithName:@"Preferences"] show:sender];
}

- (IBAction) showInspectorPanel:(id)sender
{
    [[CVLInspectorManager sharedInspector] showWindow:sender];
}

- (IBAction) showRepositoryViewer:(id)sender
{
    [[RepositoryViewer sharedRepositoryViewer] showWindow:sender];
}

- (IBAction) showProgressPanel:(id)sender
{
    [[ProgressPanelController sharedProgressPanelController] showWindow:sender];
}

- (IBAction) showInfoPanel:(id)sender
{
  [[InfoController new] showPanel];
}

- (IBAction) showCvsInfoPanel:(id)sender
{
    [[InfoController new] showCvsPanel];
}

- (IBAction) showComparator:(id)sender
{
    [[Comparator sharedComparator] showWindow:sender];
}

- (NSString *)getCvsRoot
{
  NSString* userCVSRoot= [[NSUserDefaults standardUserDefaults] stringForKey:@"CVSRoot"];
  
  if (!userCVSRoot)
  {
    userCVSRoot= [[[NSProcessInfo processInfo] environment] objectForKey: @"CVSROOT"];
    if (userCVSRoot)
    {
      [[NSUserDefaults standardUserDefaults] setObject: userCVSRoot forKey:@"CVSRoot"];
    }
  }
  return userCVSRoot;
}

- (void)pathSelected:(NSNotification *)notification
{
    NSArray *selectedPaths;
    
    selectedPaths=[notification object];
    [globalSelection setSelectedObjects:[notification object]]; // to refactor to hlod files rather than paths
}

/*
- (void)viewerSelectionDidChange:(NSNotification *)notification
{
    if ([notification object]==[activeViewer selection]) {
//        [globalSelection setSelectedObjects:[[notification object] selectedObjects]];
    }
} */

- (void)setActiveViewer:(id)aViewer;
{
    if (aViewer!=activeViewer) {
//        [[NSNotificationCenter defaultCenter] removeObserver:self name:nil object:[activeViewer selection]];
        
        [activeViewer release];
        activeViewer=aViewer;
        [aViewer retain];
        
//        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(viewerSelectionDidChange:) name:@"SelecionDidChange" object:[activeViewer selection]];
//        [globalSelection setObjects:[[activeViewer selection] objects]];
    }
}

- (void) viewerDidBecomeKey: (NSNotification *) notification
{
    [self setActiveViewer:[notification object]];
}


- (void) viewerWillClose: (NSNotification *)notification
{
    NSArray	*keys = [currentBrowsers allKeysForObject:[notification object]];

    [currentBrowsers removeObjectsForKeys:keys];
    [self setActiveViewer:nil];
}

- (void) openCurrentBrowsersFrom: (NSArray*) browserArray
{
  NSEnumerator* dictEnumerator= [browserArray objectEnumerator];
  NSDictionary* browserDict;

  while((browserDict = [dictEnumerator nextObject])){
      id pool = [[NSAutoreleasePool alloc] init]; // [TRH] attempt to keep temporaries in check.

      [[self viewerForPath:[browserDict objectForKey:@"path"]] showWindowWithDictionary:browserDict];
      [pool release];
  }
}


- (NSArray*) restoreBrowsersFrom: (NSArray*) browserArray
{
  NSMutableArray* restoreArray= [NSMutableArray arrayWithCapacity: 0];
  NSEnumerator* browserEnumerator= [browserArray objectEnumerator];
  id viewer;

  while((viewer = [browserEnumerator nextObject]))
  {
      [restoreArray addObject: [viewer stateDictionary]];	
  }
  return restoreArray;
}

- (BOOL) application:(NSApplication *)sender openFile:(NSString *)filename
{
    // Duplicates code of selectInCVL:userData:error: !!!
    NSSet	*requests;
    BOOL	isDirectory;

    // Let's check directory existence (if we use Recent.bundle, method can be called with yet unexistant filename)
    if(![[NSFileManager defaultManager] cvlFileExistsAtPath:filename isDirectory:&isDirectory])
        return NO;

    if(isDirectory){
#if 0
        [[self viewerShowingFile:[CVLFile treeAtPath:filename]] show:self];
#else
        CVLFile			*aFile = [CVLFile treeAtPath:filename];
        WorkAreaViewer	*viewer = [self currentViewerShowingFile:aFile];

        if(!viewer)
            [[self newViewerShowingFile:aFile] show:self];
        else
            [viewer selectFiles:[NSSet setWithObject:aFile]];
#endif
    }
    else{
        requests = [CVLSelectingFilesRequest requestsForSelectionOfPaths:[NSArray arrayWithObject:filename]];
        [requests makeObjectsPerformSelector:@selector(schedule)];
    }

    return YES; // We don't need to check if directory contains a CVS directory
}

@end

//-------------------------------------------------------------------------------------

@implementation CVLDelegate(ServicesDelegate)

- (void)selectInCVL:(NSPasteboard *)pboard
           userData:(NSString *)data
              error:(NSString **)error
{
    NSArray *pboardFilenames;
    NSArray *types= [pboard types];
    NSSet *requests;
    NSMutableArray	*filenames;
    NSEnumerator	*anEnum;
    NSString		*aPath;
    
    if (![types containsObject:NSFilenamesPboardType]) {
        *error = @"Error: couldn't select filename, pboard couldn't give filename.";
        return;
    }

    pboardFilenames = [pboard propertyListForType:NSFilenamesPboardType];
    if (!pboardFilenames) {
        *error = @"Error: couldn't select filename, pboard couldn't give filename.";
        return;
    }

    // Let's remove files/directories which do not exist in fileSystem
    filenames = [pboardFilenames mutableCopy];
    anEnum = [filenames objectEnumerator];
    while(aPath = [anEnum nextObject])
        if(![[NSFileManager defaultManager] fileExistsAtPath:aPath])
            [filenames removeObject:aPath];
    if(![filenames count]){
        *error = @"Error: files/directories do not exist.";
        [filenames release];
        return;
    }

    anEnum = [filenames objectEnumerator];
    while(aPath = [anEnum nextObject]){
        // See comment in -newViewerShowingFile:
        if([[NSFileManager defaultManager] cvlDirectoryExistsAtPath:aPath] && ![[[[CVLFile treeAtPath:aPath] parent] repository] isWrapper:aPath]){
//        if([[NSFileManager defaultManager] cvlFileExistsAtPath:aPath isDirectory:&isDirectory] && isDirectory){
            WorkAreaViewer	*viewer = [self currentViewerShowingFile:[CVLFile treeAtPath:aPath]];

            if(!viewer)
                [[self newViewerShowingFile:[CVLFile treeAtPath:aPath]] show:self];
            else
                [viewer selectFiles:[NSSet setWithObject:[CVLFile treeAtPath:aPath]]];
            [filenames removeObject:aPath];
        }
    }

    if([filenames count]){
        requests=[CVLSelectingFilesRequest requestsForSelectionOfPaths:filenames];
        [requests makeObjectsPerformSelector:@selector(schedule)];
    }
    [filenames release];
}

- (void) delayedImportOfDirectoriesIntoRepository:(NSDictionary *)aDictionary
{
    NSEnumerator	*dirEnum = [[aDictionary objectForKey:@"directories"] objectEnumerator];
    NSString		*aPath;
    CvsRepository	*aRepository = [aDictionary objectForKey:@"repository"];

    while(aPath = [dirEnum nextObject]){
        // Perhaps we should also display the CVSROOT...
        [[RepositoryViewer sharedRepositoryViewer] importIntoRepository:aRepository importInfo:[NSDictionary dictionaryWithObject:aPath forKey:@"sourcePath"]];
    }
}

- (void) importIntoRepository:(NSPasteboard *)pboard userData:(NSString *)data error:(NSString **)error
{
    NSArray			*pboardFilenames;
    NSArray			*types = [pboard types];
    NSFileManager	*fileManager = [NSFileManager defaultManager];
    NSEnumerator	*fileEnum;
    NSString		*aPath;
    CvsRepository	*aRepository;

    if(![types containsObject:NSFilenamesPboardType]){
        *error = @"Error: couldn't import directory, pboard couldn't give filename.";
        return;
    }

    pboardFilenames = [pboard propertyListForType:NSFilenamesPboardType];
    if (!pboardFilenames) {
        *error = @"Error: couldn't import directory, pboard couldn't give filename. Invalid.";
        return;
    }

    fileEnum = [pboardFilenames objectEnumerator];
    while(aPath = [fileEnum nextObject]){        
    	// Let's check that filename is an existing directory
        if(![fileManager cvlDirectoryExistsAtPath:aPath]){
            *error = [NSString stringWithFormat:@"Directory %@ does not exist. CVL can only import directories.", aPath];
            return;
        }
    }

    aRepository = [CvsRepository repositoryWithRoot:[data stringByStandardizingPath]];
    if(!aRepository){
        *error = [NSString stringWithFormat:@"Unable to get repository for CVSROOT %@", data];
        return;
    }
    
    // We need to delay import, because services have a timeout and we will display a modal panel
    [self performSelector:@selector(delayedImportOfDirectoriesIntoRepository:) withObject:[NSDictionary dictionaryWithObjectsAndKeys:pboardFilenames, @"directories", aRepository, @"repository", nil] afterDelay:0];
}

@end

@interface CVLDelegate(CVLEditorClient) <CVLEditorClient>
@end

@implementation CVLDelegate(CVLEditorClient)

- (NSString *) filteredCommitTemplate:(NSString *)template
{
	// First line is always empty (added automatically by cvs) => remove it
	// CVS adds a block beginning with "CVS: ----------------------------------------------------------------------"
	// up to the end; there's a line beginning with "CVS: Modified Files:": following lines are like this:
	// "CVS:    files...".
    if([[NSUserDefaults standardUserDefaults] boolForKey:@"FilterCvsTemplate"]){
        int				aLength = [template length];
        NSMutableArray	*lines = [NSMutableArray array];
        BOOL			firstLine = YES;
        int				startIndex, endIndex = 0;

        do{
            [template getLineStart:&startIndex end:&endIndex contentsEnd:NULL forRange:NSMakeRange(endIndex, 1)];
            if(firstLine)
                firstLine = NO;
            else{
                NSString	*aLine = [template substringWithRange:NSMakeRange(startIndex, endIndex - startIndex)];

                if(![aLine hasPrefix:@"CVS:"])
                    [lines addObject:aLine];
            }
        }while(endIndex < aLength - 1);

        template = [lines componentsJoinedByString:@"\n"];
    }
    
    return template;
}

- (BOOL) commitWithTemplateInFile:(NSString *)filename
{
    NSString	*templateContent = [NSString stringWithContentsOfFile:filename];
    BOOL		committed = NO;
    
    if(templateContent){
        SenFormPanelController	*commitPanel = [[SenPanelFactory sharedPanelFactory] handleOfPanelWithName:@"CvsCommitPanel"];
    
        [commitPanel setObjectValue:[self filteredCommitTemplate:templateContent] forKey:@"message"];
        [commitPanel setObjectValue:@"Commit File(s)" forKey:@"PanelTitle"];

        if([commitPanel showAndRunModal] == NSOKButton) {
            NSDictionary	*tagInfos = [commitPanel dictionaryValue];
            NSString		*aString = [tagInfos objectForKey:@"message"];
            BOOL			doCommit = YES;
            int				aLength = 0;

            if(aString)
                aLength = [aString length];
            
            // Check that user didn't type lines beginning with "CVS:",
            // because cvs WILL remove them from commit message.
            if([[NSUserDefaults standardUserDefaults] boolForKey:@"FilterCvsTemplate"] && [[NSUserDefaults standardUserDefaults] boolForKey:@"CvsTemplateWarning"] && aLength > 0){
                int	startIndex, endIndex = 0;

                do{
                    [aString getLineStart:&startIndex end:&endIndex contentsEnd:NULL forRange:NSMakeRange(endIndex, 1)];
                    if([[aString substringWithRange:NSMakeRange(startIndex, endIndex - startIndex)] hasPrefix:@"CVS:"]){
                        if(NSRunAlertPanel(@"Commit", @"cvs will remove lines beginning with CVS:. Commit anyway?", @"Cancel", @"Commit", nil) == NSAlertDefaultReturn)
                            doCommit = NO;
                        break;
                    }
                }while(endIndex < aLength - 1);
            }

            if(doCommit){
                if(!aString || [(NSString *)aString length] == 0)
                    (void)NSRunAlertPanel(@"Commit", @"You must give a commit message.", nil, nil, nil);
                else{
                    [aString writeToFile:filename atomically:NO];
                    committed = YES;
                }
            }
        }
    }

	return committed;
}

@end

//-------------------------------------------------------------------------------
