/* CVLFile.m created by ja on Thu 12-Mar-1998 */
/*$Id: CVLFile.m,v 1.29 2002/07/01 09:59:36 kindov Exp $*/

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "CVLFile.h"
#import "ResultsRepository.h"
#import "NSFileManager_CVS.h"
#ifdef MACOSX
#import <CvsRepository.h>
#import <Request.h>
#import <NSString+Lines.h>
#else
#import <Repositories/CvsRepository.h>
#import <Requests/Request.h>
#import <SenFoundation/NSString+Lines.h>
#endif
#import <SenFoundation/SenUtilities.h>
#import "NSArray.SenCategorize.h"
#import "NSArray.SenUtilities.h"
#import "NSString.GentleCompare.h"
#import "NSString+CVL.h"
#import <SenOyster/NSString.SenOysterRegex.h>

static ECFileFlags defaultFileFlags={NO,NO,NO,NO,NO,YES,ECInvalidFile };
static NSString		*CVS_Entries_Directory = nil;

static BOOL	DO_NOT_USE_QUICKSTATUS = NO;


@interface SenFileSystemTree(CVLFile)
- initWithPath:(NSString *) value;
@end

@interface CVLFile (Private)
- (void)_setChildren:(NSArray *)childArray;
- (void)_setFlags:(ECFileFlags)newFlags;
- (void)_setStatus:(ECStatus)newStatus;
- (void)_setModificationDate:(NSDate *)newDate;
- (void)setParent:(id)newParent;

- (void)getFlags;
- (void)getChildren;
- (void)getDifferences;
- (void)getLog;
- (void)getStatus;
- (void)getQuickStatus;
- (void)getTags;
- (void)getRepository;

- (void)getCumulatedStatuses;
- (void)summarizeDirectoryStatus;

- (void)touch;

- (NSComparisonResult)compareOnFileName:(CVLFile *)otherFile;

- (void) setRevisionInWorkArea:(NSString *)value;
- (void) setDateOfLastCheckout:(NSDate *)value;
- (void) setStickyOptions:(NSString *)value;
- (void) setStickyTag:(NSString *)value;
- (void) setStickyDate:(NSDate *)value;
@end

static NSArray *statusTokens=nil;
ECStatus tokenizedStatus[] = {
  {
      ECNoType, ECNoStatus
  }, {
      ECUpToDateType,ECUpToDate
  }, {
      ECLocallyModifiedType, ECLocallyModified
  }, {
      ECLocallyModifiedType, ECLocallyAdded
  }, {
      ECLocallyModifiedType, ECLocallyRemoved
  }, {
      ECNeedsUpdateType, ECNeedsPatch
  }, {
      ECNeedsUpdateType, ECNeedsCheckout
  }, {
      ECNeedsMergeType, ECNeedsMerge
  }, {
      ECNeedsMergeType, ECMixedStatus
  }, {
      ECConflictType ,ECConflict
  }, {
      ECConflictType ,ECConflict2
  }, {
      ECNotCVSType, ECNotCVS
  }, {
      ECNotCVSType, ECContainsNonCVSFiles
  }, {
      ECIgnoredType, ECIgnored
  }, {
      ECUnknownType, ECUnknown
  }, {
      ECUnknownType, ECInvalidEntry
  }
};


@implementation CVLFile
+ (void)initialize
{
    if (!statusTokens) {
        statusTokens=[[NSArray arrayWithObjects:
            @"none",
            @"Up-to-date",
            @"Locally Modified",
            @"Locally Added",
            @"Locally Removed",
            @"Needs Patch",
            @"Needs Checkout",
            @"Needs Merge",
            @"Mixed Status",
            @"Unresolved Conflict",
            @"File had conflicts on merge",
            @"not under CVS control",
            @"contains files not under CVS control",
            @"ignored",
            @"Unknown",
            @"Entry Invalid", nil] retain];
    };
    if(!CVS_Entries_Directory)
        CVS_Entries_Directory = [[@"CVS" stringByAppendingPathComponent:@"Entries"] retain]; // Let's cache the string
    DO_NOT_USE_QUICKSTATUS = [[NSUserDefaults standardUserDefaults] boolForKey:@"NoQuickStatus"];
}

- initWithPath:(NSString *) value
{
    if(self = [super initWithPath:value]){
//        status = tokenizedStatus[ECUpToDate];//tokenizedStatus[ECUnknown];
        flags = defaultFileFlags;
        wasInvalidated = YES;
//        statusWasInvalidated = YES; // Do NOT set it to YES!
    }

    return self;
}

// ------------------ Flags
- (ECFileFlags)flags
{
    if (!have.flags) {
        [self getFlags];
    }
    return flags;
}

- (void)_setFlags:(ECFileFlags)newFlags
{
    if (1) { //if (newFlags!=flags) {
        flags=newFlags;
    }
    loading.flags=NO;
    have.flags=YES;
}

- (void)invalidateFlags
{
    if (loading.flags) {
        changes.flags=YES;
        [self touch];
    } else  if (have.flags) {
        have.flags=NO;       // We keep old value
        changes.flags=YES;
        [self touch];
    }
}

- (NSDate *)modificationDate
{
    if (!have.flags) {
        [self getFlags];
    }
    return modificationDate;
}

- (void)_setModificationDate:(NSDate *)newDate
{
    if (!((newDate==modificationDate) || [newDate isEqual:modificationDate])) {
        [newDate retain];
        [modificationDate release];
        modificationDate=newDate;
    }
    loading.flags=NO;
    have.flags=YES;
}

- (void)getFlags
{
    if (!loading.flags) {
        loading.flags=YES;
//        [[self parent] getChildren];
        // Calling [[self parent] getChildren] will force rescanning fileSystem, which is sometimes unnecessary
        // and VERY time-consuming. Sometimes we'd like just to update some parts of flags.
        if(wasInvalidated){
            if(parent) // At startup, parent is nil; do not un-invalidate file here
                wasInvalidated = NO;
            [[self parent] getChildren];
        }
        else
        (void)[[self parent] loadedChildren];
		// Simply calling [[self parent] loadedChildren] will check if children need loading,
		// but sometimes (after an Update/Commit/...)
        // parent directory would need re-computing statuses (do NOT use summarizeDirectoryStatus or getCumulatedStatuses,
		// because gets recursive and scans whole fileSystem!), as it stays "Being Computed"...
    }
}

// ------------------ Children
- (NSArray *)children
{
    return children;
}

- parent
{
    if (!parent) {
        [self setParent:[super parent]];
    }
    return parent;
}

- (void)setParent:(id)newParent
{
    parent=newParent; // Not retained to avoid cycles
}

- (NSArray *)loadedChildren
{
    if (!have.children) {
        [self getChildren];
    }
    return children;
}

- (void)_setChildren:(NSArray *)childArray
{
    if (!((childArray==children) || [childArray isEqual:children])) {
        [childArray retain];
        [children release];
        children=childArray;
    }
    have.children=YES;
    loading.children=NO;
}

- (void)invalidateChildren
{
    if (loading.children) {
        changes.children=YES;
        [self touch];
    } else if (have.children) {
        [self _setChildren:nil]; // We don't keep old value here
        have.children=NO;
        changes.children=YES;
        [self touch];
    }
    [[self repository] invalidateDir:[self path]];
}

//#define CVL_PROFILING
#ifdef CVL_PROFILING
#define INIT_PROFILE(c)		NSDate	*profileDate[c]; static NSTimeInterval	profileTotal[c]; static NSTimeInterval	profileSum = 0
#define START_PROFILE(i)	profileDate[i] = [[NSDate date] retain];
#define PROFILE(i)	{ NSTimeInterval	interval = -[profileDate[i] timeIntervalSinceNow]; [profileDate[i] release]; profileTotal[i] += interval; profileSum += interval; NSLog(@"PROFILING %@ [%d] = %.2lf (%.2lf / %.2lf)", NSStringFromSelector(_cmd), i, interval, profileTotal[i], profileSum);}
#define PROFILE2(i, p)	{ NSTimeInterval	interval = -[profileDate[i] timeIntervalSinceNow]; [profileDate[i] release]; profileTotal[i] += interval; profileSum += interval; NSLog(@"PROFILING %@ [%d] = %.2lf (%.2lf / %.2lf) %@", NSStringFromSelector(_cmd), i, interval, profileTotal[i], profileSum, p);}
#else
#define INIT_PROFILE(c)
#define START_PROFILE(i)
#define PROFILE(i)
#define PROFILE2(i, p)
#endif

- (void)getChildren
// Profiling: when opening CVL workarea, we spend 20s (total) in this method!!!
{
    INIT_PROFILE(10);
    /*    if(!have.quickStatus && !loading.quickStatus){
        BOOL	isDir;
        
        if(![[NSFileManager defaultManager] cvlFileExistsAtPath:[self path] isDirectory:&isDir])
            isDir = [self flags].isDir;

        if(isDir){
            CvsRepository	*dirRepository;

            [[ResultsRepository sharedResultsRepository] startUpdate];
            dirRepository = [self repository];
            if(dirRepository && [dirRepository root]){
                [[ResultsRepository sharedResultsRepository] getQuickStatusForFile:self];
                [[ResultsRepository sharedResultsRepository] endUpdate];
                return;
            }
            else
                [[ResultsRepository sharedResultsRepository] endUpdate];
        }
    }*/
    if(!loading.children){
        NSFileManager	*fileManager = [NSFileManager defaultManager];
        BOOL			isDir = NO;

        START_PROFILE(0);
        loading.children = YES;
        if(![fileManager cvlFileExistsAtPath:[self path] isDirectory:&isDir])
            isDir = [self flags].isDir;

        if(isDir){
            // It's faster without an autoreleasePool... (10%)
            // [TRH] ...but this makes for a swapping hog on WinNT, so...
            // (and it would just delay the time spent waiting for autorelease to the end of the current runloop iteration, so it is probably better to spend it here and keep the working set manageable.)
            NSAutoreleasePool	*localAP = [[NSAutoreleasePool alloc] init];
            CvsRepository		*dirRepository = [self repository];

            // have.repository is initialized by [self repository] but can be set even if dirRespository is nil
            START_PROFILE(1); // 75%
            if (have.repository) {
                ResultsRepository		*resultsRepository = [ResultsRepository sharedResultsRepository];
                NSMutableArray			*newChildren = [[NSMutableArray alloc] init];
                NSDirectoryEnumerator	*dirContentsEnumerator;
                CVLFile					*file;
                NSString				*fileName, *fullFilePath;
                ECFileFlags				fileFlags;

                [resultsRepository startUpdate];

                // (1) get files from local dir
                START_PROFILE(2); // 60%
                dirContentsEnumerator = [fileManager enumeratorAtPath:[self path]];
                while(fileName = [dirContentsEnumerator nextObject]){
                    [dirContentsEnumerator skipDescendents]; // Perhaps later we can do it deeply, so it's faster!!
//                    START_PROFILE(9); // < 1%
                    fullFilePath = [[self path] stringByAppendingPathComponent:fileName];
                    file = [[self class] treeAtPath:fullFilePath];
                    [newChildren addObject:file];
//                    PROFILE(9);
                    if(dirRepository){
                        // Slow: takes 28%
                        NSDate	*fileModificationDate = nil;

                        if(file->have.flags)
                            fileFlags = [file flags];
                        else
                            fileFlags = defaultFileFlags;

                        START_PROFILE(8); // 10%
                        if([fileManager cvlFileExistsAtPath:fullFilePath isDirectory:&isDir]){
                            fileModificationDate = [[dirContentsEnumerator fileAttributes] objectForKey:NSFileModificationDate];
                            fileFlags.isInWorkArea = YES;
                            fileFlags.isInRepository = NO;
                            fileFlags.isDir = isDir;
                            fileFlags.isWrapper = isDir && ([dirRepository isWrapper:fullFilePath]);
                        }
                        PROFILE(8);
                        fileFlags.isIgnored = fileFlags.isHidden = [dirRepository isIgnored:fullFilePath];
                        [file _setFlags:fileFlags];
                        [file _setModificationDate:fileModificationDate];
                    }
                }
                PROFILE(2);

                START_PROFILE(3); // 5%
                if(dirRepository){
#ifdef DEBUG
                    NSLog(@"++ Flags in dir:%@", [self path]);
#endif

                    // (2) get files from CVS/Entries
                    // 3%
                    {
                        // ./CVS/Entries: List and status of files in the working directory
                        // Each line represents an file or directory known by cvs (for the current directory)
                        // A line begins with / and has six items separated by /
                        // [D]/NAME/REVISION/CHECKOUT_DATE/STICKY_OPTIONS/STICKY_TAG_OR_DATE
                        // Directories have only D/NAME////
                        NSString	*cvsEntries;

                        // Block execution is fast: it takes less than 1% of method execution time
                        cvsEntries = [[NSString alloc] initWithContentsOfFile:[[self path] stringByAppendingPathComponent:CVS_Entries_Directory]];

                        if(cvsEntries){
                            NSArray	*lines = [cvsEntries lines];
                            int		lineIndex = [lines count] - 1;

                            for(; lineIndex >= 0; lineIndex--){
                                NSArray	*cvsEntryInfos = [[lines objectAtIndex:lineIndex] componentsSeparatedByString:@"/"];

                                if([cvsEntryInfos count] >= 6){
                                    NSString	*stickyDateOrTag = [cvsEntryInfos objectAtIndex:5];
                                    
                                    fileName = [cvsEntryInfos objectAtIndex:1]; // NAME
                                    fullFilePath = [[self path] stringByAppendingPathComponent:fileName];
                                    file = [[self class] treeAtPath:fullFilePath];
                                    [newChildren addNewObject:file];

                                    if(file->have.flags)
                                        fileFlags = [file flags];
                                    else
                                        fileFlags = defaultFileFlags;

                                    if(!fileFlags.isInWorkArea){
                                        fileFlags.isDir = [[cvsEntryInfos objectAtIndex:0] isEqual:@"D"]; // Is directory?
                                        fileFlags.isWrapper = ([dirRepository isWrapper:fileName]);
                                        fileFlags.isIgnored = fileFlags.isHidden = [dirRepository isIgnored:fullFilePath];
                                    }
                                    fileFlags.isInCVSEntries = YES;
                                    // We don't know for now because we don't look in the repository from here
                                    fileFlags.isInRepository = fileFlags.isInCVSEntries;
                                    [file _setFlags:fileFlags];

                                    [file setRevisionInWorkArea:[cvsEntryInfos objectAtIndex:2]];
                                    [file setDateOfLastCheckout:[cvsEntryInfos objectAtIndex:3]];

                                    [file setStickyOptions:[cvsEntryInfos objectAtIndex:4]];
                                    have.stickyDateOrTag = YES;
                                    if([stickyDateOrTag length]){
                                        if([stickyDateOrTag hasPrefix:@"T"])
                                            // Valid for tag and revisions!
                                            [file setStickyTag:[stickyDateOrTag substringFromIndex:1]];
                                        else
#warning Check timeZone!
                                            [file setStickyDate:[NSCalendarDate dateWithString:[stickyDateOrTag substringFromIndex:1] calendarFormat:@"%Y.%d.%m.%H.%M.%S"]];
                                    }
                                    else{
                                        [file setStickyTag:@""];
                                        [file setStickyDate:nil];
                                    }
                                }
                            }
                            [cvsEntries release];
                        }
                    }

                    // (3) get files from the repository
                    // This code works only for local repositories
                    // We need something like this for remote repositories too!
                    // Perhaps we should get the info from the repository and add missing files to our list.
                    // But we need a request (cvs -nq update ?) to do this => async!
                    /*                if (showRepositoryFiles) {
                        NSString *repositoryPath=[self repositoryPathForPath:dirPath];
                    NSString *fileNameInRepository;
                    NSString *fileInRepositoryPath;
                    BOOL isDir=NO;

                    if (dirContents=[fileManager directoryContentsAtPath: repositoryPath]) {
                        dirContentsEnumerator=[dirContents objectEnumerator];
                        while (fileNameInRepository=[dirContentsEnumerator nextObject]) {
                            if (![fileNameInRepository isEqual:@"Attic"]) {
                                fileName=nil;
                                fileInRepositoryPath=[repositoryPath stringByAppendingPathComponent:fileNameInRepository];
                                [fileManager cvlFileExistsAtPath:fileInRepositoryPath isDirectory: &isDir];

                                if (!isDir) {
                                    if ([fileNameInRepository hasSuffix:@",v"]) {
                                        fileName=[fileNameInRepository substringToIndex:[fileNameInRepository length]-2];
                                    }
                                } else {
                                    fileName=fileNameInRepository;
                                }

                                if (fileName) {
                                    fileName=[cvsEntryInfos objectAtIndex:1];
                                    fullFilePath=[dirPath stringByAppendingPathComponent:fileName];
                                    file=[[self class] treeAtPath:fullFilePath];
                                    [newChildren addNewObject:file];
                                    fileFlags=[file flags];
                                    fileFlags.isInRepository=YES;
                                    if (!(fileFlags.isInWorkArea || fileFlags.isInCVSEntries)) {
                                        fileFlags.isDir=isDir;
                                        fileFlags.isWrapper=([dirRepository isWrapper: fileName]);
                                        fileFlags.isIgnored=fileFlags.isHidden=[dirRepository shouldHideFile:fileName];
                                    }
                                    [file _setFlags:fileFlags];
                                }
                            }
                        }
                    }

                    // 4 get files from the repository in Attic
                    repositoryPath=[repositoryPath stringByAppendingPathComponent:@"Attic"];
                    if (dirContents=[[NSFileManager defaultManager] directoryContentsAtPath: repositoryPath]) {
                        dirContentsEnumerator=[dirContents objectEnumerator];
                        while (fileNameInRepository=[dirContentsEnumerator nextObject]) {
                            fileName=nil;
                            fileInRepositoryPath=[repositoryPath stringByAppendingPathComponent:fileNameInRepository];

                            if ([fileNameInRepository hasSuffix:@",v"]) {
                                fileName=[fileNameInRepository substringToIndex:[fileNameInRepository length]-2];
                            }

                            if (fileName) {
                                fileName=[cvsEntryInfos objectAtIndex:1];
                                fullFilePath=[dirPath stringByAppendingPathComponent:fileName];
                                file=[[self class] treeAtPath:fullFilePath];
                                [newChildren addNewObject:file];
                                fileFlags=[file flags];
                                fileFlags.isInRepository=YES;
                                if (!(fileFlags.isInWorkArea || fileFlags.isInCVSEntries)) {
                                    fileFlags.isDir=NO;
                                    fileFlags.isWrapper=([dirRepository isWrapper: fileName]);
                                    fileFlags.isIgnored=fileFlags.isHidden=[dirRepository shouldHideFile:fileName];
                                }
                                [file _setFlags:fileFlags];
                            }
                        }
                    }
                    }
                    */
                    // (5) put the whole damn thing in the database
                    // < 1%
                    {
                        // Block execution is fast: it takes less than 0.01% of method execution time
                        int	childIndex = [newChildren count] - 1;

                        for(; childIndex >= 0; childIndex--){
                            file = [newChildren objectAtIndex:childIndex];
                            fileFlags = [file flags];
                            if(!fileFlags.isInWorkArea)
                                fileFlags.type = fileFlags.isInCVSEntries ? (fileFlags.isInRepository ? ECAbsentFile : ECInvalidFile) : ECInactiveFile;
                            else
                                fileFlags.type = fileFlags.isInCVSEntries ? ECCVSFile : (fileFlags.isInRepository ? ECInvalidFile : ECLocalFile);

                            [file _setFlags:fileFlags];
                            if(fileFlags.isIgnored)
                                [file _setStatus:tokenizedStatus[ECIgnored]];
                        }
                    }
                }
                PROFILE(3);

                // < 2%
                if(children){
                    // We need to check if some children, retrieved with quickStatus, are already in the array
                    NSMutableSet	*mySet = [[NSMutableSet alloc] initWithArray:newChildren];

                    //                NSLog(@"Already contains %@", children);
                    [mySet addObjectsFromArray:children];
                    [newChildren setArray:[mySet allObjects]];
                    [mySet release];
                }
                [self _setChildren:[newChildren sortedArrayUsingSelector:@selector(compareOnFileName:)]];
                [resultsRepository endUpdate];
                [newChildren release];

            }
            PROFILE(1);
            [localAP release]; // Use of NSAutoreleasePool does not slow down at all
        } else { // If a leaf CVLFile
            [self _setChildren:nil];
        }
        PROFILE2(0, [self path]);
    }
}

- (NSString *)name
{
    return [[self path] lastPathComponent];
}

- (NSString *)pathInRepository
{
    return nil; //
}

// ------------------ Status

- (ECStatus)status
{
    // Asking for a quickStatus on a file will perform the quickStatus on its parent directory
    // To avoid asking for a quickStatus when we don't need it, e.g. after committing a file,
    // we now check that parent directory does not need a quickStatus: it has not been invalidated,
    // and it is not refreshing.
    if (!have.status && !have.quickStatus) {
        if(DO_NOT_USE_QUICKSTATUS || (flags.isDir && ![self isRealWrapper]) || (statusWasInvalidated && (parent && (!((CVLFile *)parent)->statusWasInvalidated)))){
            [self getStatus];
        }
        else{
            [self getQuickStatus];
        }
    }
    return status;
}

- (void)_setStatus:(ECStatus)newStatus
{
    if (newStatus.tokenizedStatus!=status.tokenizedStatus) {
        status=newStatus;
    }
    loading.quickStatus=NO;
    have.quickStatus=YES;
    if(loading.status){
        have.status=YES;
        loading.status=NO;
    }
    statusWasInvalidated = NO;
}

- (void)_setStatusString:(NSString *)statusString
{
    int tokenIndex;
    
    tokenIndex=[statusTokens indexOfObject:statusString];
    if (tokenIndex==NSNotFound) {
        NSLog (@"AssertionFailed no token for status : %@",statusString);
        tokenIndex=0;
    }
    [self _setStatus:tokenizedStatus[tokenIndex]];
}

- (NSString *)statusString
{
    int statusToken=[self status].tokenizedStatus;
    if (have.quickStatus || have.status) {
        return [statusTokens objectAtIndex:statusToken];
    } else {
        return nil;
    }
}

- (void) setQuickStatusWithString:(NSString *)quickStatus
{
    unichar				statusChar = [quickStatus characterAtIndex:0];
    ResultsRepository	*resultsRepository = [ResultsRepository sharedResultsRepository];
    int					tokenIndex = -1;
    BOOL				isNewFile = NO;
    BOOL				isNewDir = NO;

    // If file is not among parent's children, put it!!!
    // This is the case for files that are new in the repository, thus not registered in the workarea
    if(![[[self parent] children] containsObject:self]){
        if(![[self parent] children])
            [[self parent] _setChildren:[NSArray arrayWithObject:self]];
        else{
            NSMutableArray	*newChildren = [NSMutableArray arrayWithArray:[[self parent] children]];

            [newChildren addObject:self];
            [[self parent] _setChildren:[newChildren sortedArrayUsingSelector:@selector(compareOnFileName:)]];
        }
    }
    switch(statusChar){
        case 'U':
#if 0
            if(![self flags].isInWorkArea)
                isNewFile = YES;
			tokenIndex = 6; // Needs checkout
            if(!have.status){
                [self getStatus]; // We cannot determine if status is NeedsCheckout or NeedsPatch
                return; // Flags will be set correctly
            }
#else
                if(![self flags].isInWorkArea)
                    isNewFile = YES;
                else if(!have.status){
                    [self getStatus]; // We cannot determine if status is NeedsCheckout or NeedsPatch
                    return; // Flags will be set correctly
                }
                tokenIndex = 6; // Needs checkout
#endif
            break;
        case 'A': // Locally added
            tokenIndex = 3;
            break;
        case 'R': // Locally removed
            tokenIndex = 4;
            break;
        case 'M': // Locally modified
            tokenIndex = 2;
            break;
        case 'C': // Conflict OR needs merge, depending on the current status: if currently needsMerge, don't change
            // If file had not yet fetched it status,
            // we have no way (?) to know if file status is currently conflict or if it will be (after update)!
            // Thus we need to launch a full status request...
            if(!have.status){
                [self getStatus];
                return;
            }
            if(status.tokenizedStatus == ECNeedsMerge)
                tokenIndex = 7; // Needs merge
            else
                tokenIndex = 9; // Conflict
            break;
        case '?': // Unknown by cvs or not yet under cvs control
            tokenIndex = [self isLeaf] ? 11:12;
            break;
        case 'u': // Custom flag to handle new directories in repository which are not yet registered in workarea
                  // In fact we cannot know if directory contains valid files,
                  // or if its contents is old. cvs output is not very explicit...
                  // BTW, it does not prune empty dirs when doing quickStatus.
                  // So, as long as we cannot know if dirs are new or old,
                  // let's mark them as unknown, so it will not display
                  // messy names.
            if(flags.isInCVSEntries) // No need to check for have.status flag; flags.isInCVSEntries is up-to-date before entering this method
                tokenIndex = 6; // Needs checkout
            else
                tokenIndex = 14; // Unknown status for cvs (even if it IS known by it...)
            isNewDir = YES;
            break;
        case '=': // Custom flag to tell file is up-to-date
            tokenIndex = 1;
            break;
//        case 'r': // Custom flag to tell file is being removed on next update
//            tokenIndex = 6; // NeedsUpdate
//            break;
        case '!': // Custom flag to tell status output couldn't be parsed, so file needs full status retrieval
            if(!have.status){
                [self getStatus];
                return;
            }
            else{
                [resultsRepository startUpdate];
                
                loading.quickStatus = NO;
                have.quickStatus = YES;
                statusWasInvalidated = NO;
                changes.status = YES;
                changes.quickStatus = YES;
                [resultsRepository fileDidChange:self];

                [resultsRepository endUpdate];
                return;
            }
            break;
        default:
            NSLog(@"Unknown quick status char: %c", (char)statusChar);
            return;
    }

    NSAssert(tokenIndex != -1, @"Did not set tokenIndex!");
    
    [resultsRepository startUpdate];

    [self _setStatus:tokenizedStatus[tokenIndex]];
    loading.quickStatus = NO;
    have.quickStatus = YES;
    statusWasInvalidated = NO;
    changes.status = YES;
    changes.quickStatus = YES;
    if(isNewDir || isNewFile){
        // The case where local file AND repository file both exist is not handled correctly.
		// Some flags are overridden in -getChildren. Worse if we do a refresh...
        // It's a bit to messy to do it correctly now. We need to review many code lines...
        flags.isInRepository = YES;
        if(flags.isInWorkArea){
            flags.type = ECInvalidFile;
        }
        else{
            flags.isInRepository = YES;
            flags.isIgnored = flags.isHidden = [[self repository] isIgnored:[self path]];
            flags.type = ECInactiveFile;
        }
    }
/*    if(isNewDir){
        flags.isDir = YES; // If we set isDir to YES, entry will not be displayed in browser!!
        flags.isWrapper = [[self repository] isWrapper:[self path]];
    }*/
    [resultsRepository fileDidChange:self];

    [resultsRepository endUpdate];
}

- (void)setStatusFromDictionary:(NSDictionary *)results
{
    // Sets status, revisionInWorkArea, revisionInRepository, dateOfLastCheckout, stickyTag, stickyDate and stickyOptions
  id result;
  ResultsRepository *resultsRepository=[ResultsRepository sharedResultsRepository];
  [resultsRepository startUpdate];

  // WARNING: if status could not be done correctly, the assertion will NOT be respected!
//  NSAssert1([results objectForKey:@"status"], @"No status retrieved for %@", self);
  if (result=[results objectForKey:@"status"]) {
      [self _setStatusString:result];
  }
  else
      [self _setStatusString:@"N/A"];
//  NSAssert1([results objectForKey:@"Version"], @"No version retrieved for %@", self);
  // It may happen that no version is retrieved if file has an incorrect version number(?)
  if (result=[results objectForKey:@"Version"]) {
      RETAIN(revisionInWorkArea,result);
  }
  else
      RETAIN(revisionInWorkArea, @"N/A");

  result = [results objectForKey:@"Repository Version"];
  RETAIN(revisionInRepository,result);

  result=[results objectForKey:@"Last Checkout"];
  RETAIN(dateOfLastCheckout,result); // Do NOT check if returned last checkout date! If we do, we can never remove it
  // Notice that in client/server mode, this info is NOT returned by the server

  result=[results objectForKey:@"Sticky Tag"];
  if(!result)
      result = @"";
  RETAIN(stickyTag,result); // Do NOT check if returned sticky tag! If we do, we can never remove it (Bug 1000103)

  result=[results objectForKey:@"Sticky Date"];
  RETAIN(stickyDate,result); // Do NOT check if returned sticky tag! If we do, we can never remove it (Bug 1000103)
  have.stickyDateOrTag = YES;

  result=[results objectForKey:@"Sticky Options"];
  if(!result)
      result = @"";
  RETAIN(stickyOptions,result); // Do NOT check if returned sticky tag! If we do, we can never remove it (Bug 1000103)

  loading.status=NO;
  loading.quickStatus=NO; // ????????????????
  have.status=YES;
  have.quickStatus=YES;
  statusWasInvalidated = NO;
  changes.status=YES;
  changes.quickStatus=YES;
  [resultsRepository fileDidChange:self];

  [resultsRepository endUpdate];
}

- (void)invalidateStatus
{
    statusWasInvalidated = YES;
    if (loading.status || loading.quickStatus) {
        changes.status=YES;
        changes.quickStatus=YES;
        [self touch];
    } else if (have.status || have.quickStatus) {
        status=tokenizedStatus[ECNoStatus];
        have.status=NO;       // We don't keep old value
        changes.status=YES;
        have.quickStatus=NO;       // We don't keep old value
        changes.quickStatus=YES;
        [self touch];
        [stickyOptions release]; stickyOptions = nil;
        [stickyTag release]; stickyTag = nil;
        [stickyDate release]; stickyDate = nil;
        have.stickyDateOrTag = NO;
    }
}

- (BOOL)isLeaf
{
    if (have.flags) {
        return (!flags.isDir || flags.isWrapper);
    } else {
        CvsRepository *dirRepository;
        if (dirRepository=[[self parent] repository]) {
            return (([self loadedChildren]==nil) || ([dirRepository isWrapper:[self path]]));
        } else {
            return ([self loadedChildren]==nil);
        }
    }
}

- (void)getStatus
{
    if (!loading.status) {
        ECFileFlags fileFlags;

        loading.status=YES;
        loading.quickStatus=YES;
        fileFlags=[self flags];
        if (have.flags) {
            if (fileFlags.isIgnored) {
                [self _setStatus:tokenizedStatus[ECIgnored]];
            } else {
                if ([self isLeaf]) {
#if 1
                    if (fileFlags.type==ECLocalFile) {
                        [self _setStatus:tokenizedStatus[ECNotCVS]];
                    } else {
                        [[ResultsRepository sharedResultsRepository] getStatusForFile:self];
                    }
#else
#warning MODIFIED
                    if (!have.repository || [repository root] == nil){
                        [self _setStatus:tokenizedStatus[ECNotCVS]];
                    } else {
                        [[ResultsRepository sharedResultsRepository] getStatusForFile:self];
                    }
#endif
                } else {
                    [self summarizeDirectoryStatus];
                }
            }
        }
    }
}

- (void)getQuickStatus
{
    if (!loading.status && !loading.quickStatus) {
        ECFileFlags fileFlags;

        loading.quickStatus=YES;
        fileFlags=[self flags];
        if (have.flags) {
            if (fileFlags.isIgnored) {
                [self _setStatus:tokenizedStatus[ECIgnored]];
            } else {
                if ([self isLeaf]) {
#if 1
                    if (fileFlags.type==ECLocalFile) {
                        [self _setStatus:tokenizedStatus[ECNotCVS]];
                    } else {
                        [[ResultsRepository sharedResultsRepository] getQuickStatusForFile:self];
                    }
#else
#warning MODIFIED
                    if (!have.repository || [repository root] == nil){
                        [self _setStatus:tokenizedStatus[ECNotCVS]];
                    } else {
                        [[ResultsRepository sharedResultsRepository] getQuickStatusForFile:self];
                    }
#endif
                } else {
                        [self summarizeDirectoryStatus];
                }
            }
        }
    }
}

- (void)summarizeDirectoryStatus
{
    if ([self cumulatedStatuses]) {
        if (!(*cumulatedStatuses)[ECNoStatus]) {
            BOOL modified=NO,needsUpdate=NO,needsMerge=NO,conflict=NO,notCVS=NO,mixed=NO,ignored=NO;
            int statusToken;
            ECStatus childStatus;

            for (statusToken=0;statusToken<=ECUnknown;statusToken++) {
                if ((*cumulatedStatuses)[statusToken]>0) {
                    childStatus=tokenizedStatus[statusToken];
                    if (childStatus.statusType==ECLocallyModifiedType) modified=YES;
                    else if (childStatus.statusType==ECNeedsUpdateType) needsUpdate=YES;
                    else if (childStatus.statusType==ECConflictType) conflict=YES;
                    else if (childStatus.statusType==ECNeedsMergeType) needsMerge=YES;
                    else if (childStatus.statusType==ECNotCVSType) notCVS=YES;
                    else if (childStatus.statusType==ECIgnoredType) ignored=YES;
                }
            }

            statusToken=ECUpToDate; /////////////// Shouldn't it be UNKNOWN?
            if (notCVS) statusToken=ECContainsNonCVSFiles;
            if(needsUpdate)
                if(flags.isInRepository && !flags.isInWorkArea) // In case where dir is in repository but not in work area
                    statusToken = ECNeedsCheckout;
                else
                    statusToken = ECNeedsPatch; // => a folder containing a file <NeedsCheckout> will display <Needs Patch>
            
//            if (needsUpdate) statusToken=ECNeedsPatch; // => a folder containing a file <NeedsCheckout> will display <Needs Patch>
            if (modified) statusToken=ECLocallyModified;
            if (needsMerge) statusToken=ECNeedsMerge;
            if (mixed || (modified && (needsUpdate || needsMerge)) || (needsUpdate && needsMerge)) statusToken=ECMixedStatus;
            if (conflict) statusToken=ECConflict;

            [self _setStatus:tokenizedStatus[statusToken]];
        }
    }
}

// ---------- cumulated status
- (ECCumulatedStatuses *)cumulatedStatuses
{
    if ([self isLeaf]) { // a leaf don't have such a thing
        return NULL;
    }
    if (!have.cumulatedStatuses)
            [self getCumulatedStatuses];

    return cumulatedStatuses;
}

- (void)invalidateCumulatedStatuses
{
    if (loading.cumulatedStatuses) {
        changes.cumulatedStatuses=YES;
        [self touch];
    } else if (have.cumulatedStatuses) {
        have.cumulatedStatuses=NO;       // We keep old value
        changes.cumulatedStatuses=YES;
//        statusWasInvalidated = YES; // Needed?
        [self touch];
    }
}

- (void)getCumulatedStatuses
{
    if (!loading.cumulatedStatuses) {
        int statusToken;
        id childrenEnumerator;
        CVLFile *child;
        ECCumulatedStatuses *childCumulatedStatuses;

        ResultsRepository *resultsRepository=[ResultsRepository sharedResultsRepository];

        [resultsRepository startUpdate];
        loading.cumulatedStatuses=YES;

        [self flags];
        if (have.flags) {
            if (flags.isIgnored) { // no cumulated statuses for ignored files
                if (cumulatedStatuses) {
                    NSZoneFree(NSDefaultMallocZone(), cumulatedStatuses);
                    cumulatedStatuses=NULL;
                }
                loading.cumulatedStatuses=NO;
                have.cumulatedStatuses=YES;
//                statusWasInvalidated = NO; // Needed?
            } else {
                if (!cumulatedStatuses) {
                    cumulatedStatuses=(ECCumulatedStatuses *)NSZoneMalloc(NSDefaultMallocZone(), sizeof(ECCumulatedStatuses));
                }
                for (statusToken=0;statusToken<=ECUnknown;statusToken++) {
                    (*cumulatedStatuses)[statusToken]=0;
                }
                childrenEnumerator=[[self loadedChildren] objectEnumerator];
                while (child=[childrenEnumerator nextObject]) {
                    if ([child isLeaf]) {
                        statusToken=[child status].tokenizedStatus;
                        (*cumulatedStatuses)[statusToken]++;
                    } else {
                        childCumulatedStatuses=[child cumulatedStatuses];
                        if (childCumulatedStatuses) {
                            for (statusToken=0;statusToken<=ECUnknown;statusToken++) {
                                (*cumulatedStatuses)[statusToken]+=(*childCumulatedStatuses)[statusToken];
                            }
                        }
                    }
                }
                // If directory contains only a CVS subdirectory, resulting cumulated statuses will be 0 
				// because CVS is ignored and ignored files return no cumulated statuses
                if([self loadedChildren]){
                    switch([[self loadedChildren] count]){
                        case 0:
                            if(flags.isInCVSEntries) // We MUST check this first: if dir is in CVS entries, dir perhaps needs update
                                (*cumulatedStatuses)[ECNeedsCheckout]++;
                            else
                                // New empty directory not under cvs
                                (*cumulatedStatuses)[ECNotCVS]++;
                            break;
                        case 1: // Not only for 1, but for 1 and more (can contain other ignored files...)
                            if([[[[[self loadedChildren] objectAtIndex:0] path] lastPathComponent] isEqualToString:@"CVS"]){
                    (*cumulatedStatuses)[ECUpToDate]++; // And if it misses some files? => ECNeedsCheckout. It depends on CVS/Entries. BUT missing files recorded in CVS/Entries have already returned status. => check only for missing files that are not yet in workarea
                            }
                            break;
                    }
                }

                loading.cumulatedStatuses=NO; // Shouldn't it be NO as long as children have not yet loaded their cumulated statuses?
                have.cumulatedStatuses=YES;
//                statusWasInvalidated = NO; // Needed?
            }
        }
        [resultsRepository endUpdate];
    }
}

- (NSString *)revisionInWorkArea
{
#if 0
    if (!have.status) {
        [self getStatus];
    }
#else
    if(!revisionInWorkArea && !have.status)
        [self getStatus];
#endif
    return revisionInWorkArea;
}

- (void) setRevisionInWorkArea:(NSString *)value
{
    RETAIN(revisionInWorkArea, value);
}

- (NSString *)revisionInRepository
{
    if (!have.status) {
        [self getStatus];
    }
    return revisionInRepository;
}

- (void) setDateOfLastCheckout:(NSDate *)value
{
    RETAIN(dateOfLastCheckout, value);
}

- (NSDate *)dateOfLastCheckout
{
#if 0
    if (!have.status) {
        [self getStatus];
    }
#else
    if(!dateOfLastCheckout && !have.status)
        [self getStatus];
#endif
    return dateOfLastCheckout;
}

- (NSString *)stickyTag
{
#if 0
    if (!have.status) {
        [self getStatus];
    }
#else
    if(!stickyTag)
        [self getStatus];
#endif
    return stickyTag;
}

- (void) setStickyTag:(NSString *)value
{
    RETAIN(stickyTag, value);
}

- (NSString *)stickyOptions
{
#if 0
    if (!have.status) {
        [self getStatus];
    }
#else
    if(!stickyOptions)
        [self getStatus];
#endif
    return stickyOptions;
}

- (void) setStickyOptions:(NSString *)value
{
    RETAIN(stickyOptions, value);
}

- (NSDate *)stickyDate
{
#if 0
    if (!have.status) {
        [self getStatus];
    }
#else
    if(!have.stickyDateOrTag)
        [self getStatus];
#endif
    return stickyDate;
}

- (void) setStickyDate:(NSDate *)value
{
    RETAIN(stickyDate, value);
}

// ------------- Log
- (NSArray *)log
{
    if (!have.log) {
        [self getLog];
    }
    return log;
}

- (void)setLog:(NSArray *)newLog
{
    if (!((newLog==log) || [newLog isEqual:log])) {
        [newLog retain];
        [log release];
        log=newLog;
        changes.log=YES;
        [self touch];
    }
    have.log=YES;
    loading.log=NO;
}

- (void)_setLog:(NSArray *)newLog
{
    if (!((newLog==log) || [newLog isEqual:log])) {
        [newLog retain];
        [log release];
        log=newLog;
    }
    have.log=YES;
    loading.log=NO;
}

- (void)invalidateLog
{
    if (loading.log) {
        changes.log=YES;
        [self touch];
    } else if (have.log) {
        [self _setLog:nil];
        have.log=NO;
        changes.log=YES;
        [self touch];
    }
}

- (void)getLog
{
    if (!loading.log) {
        ECFileFlags fileFlags;

        fileFlags=[self flags];
        if (have.flags) {
            if ((fileFlags.type==ECLocalFile) || fileFlags.isIgnored || (fileFlags.isDir && !fileFlags.isWrapper)) {
                [self _setLog:nil];
                return ;
            }
        }
        loading.log=YES;
        [[ResultsRepository sharedResultsRepository] getLogForFile:self];
    }
}

// ------------------ Differences

- (NSString *)differencesWithContext:(unsigned)contextLineNumber outputFormat:(CVLDiffOutputFormat)outputFormat
{
    if (!differences || differencesContext != contextLineNumber || diffOutputFormat != outputFormat) {
        differencesContext = contextLineNumber;
        diffOutputFormat = outputFormat;
        [self getDifferences];
    }
    return differences;
}

- (void)setDifferences:(NSString *)newDifferences
{
    if (!((newDifferences==differences) || [newDifferences isEqual:differences])) {
        [newDifferences retain];
        [differences release];
        differences=newDifferences;
        changes.differences=YES;
        [self touch];
    }
    loading.differences=NO;
    have.differences=YES;
}

- (void)_setDifferences:(NSString *)newDifferences
{
    if (!((newDifferences==differences) || [newDifferences isEqual:differences])) {
        [newDifferences retain];
        [differences release];
        differences=newDifferences;
    }
    loading.differences=NO;
    have.differences=YES;
}

- (void)invalidateDifferences
{
    if (loading.differences) {
        changes.differences=YES;
        [self touch];
    } else if (have.differences) {
        [self _setDifferences:nil];
        changes.differences=YES;
        have.differences=NO;
        [self touch];
    }
}

- (void)getDifferences
{
    if (!loading.differences) {
        ECFileFlags fileFlags;

        fileFlags=[self flags];
        if (have.flags) {
            if ((fileFlags.type==ECLocalFile) || fileFlags.isIgnored || fileFlags.isDir /* wrappers included */) {
                [self _setDifferences:nil];
                return;
            }
        }
        loading.differences=YES;
        [[ResultsRepository sharedResultsRepository] getDifferencesForFile:self context:differencesContext outputFormat:diffOutputFormat];
    }
}

// ---------- tags

- (NSArray*) tags
{
    if (!have.tags) {
        [self getTags];
    }
    return tags;
}

- (NSArray *) revisionTags
{
    NSArray	*allTags = [self tags];

    if(allTags){
        NSMutableArray	*revisionTags = [NSMutableArray arrayWithCapacity:[allTags count]];
        NSEnumerator	*anEnum = [allTags objectEnumerator];
        NSDictionary	*aTag;

        while(aTag = [anEnum nextObject])
            if([[aTag objectForKey:@"kind"] isEqualToString:@"revision"])
                [revisionTags addObject:[aTag objectForKey:@"tag"]];

        return revisionTags;
    }

    return nil;
}

- (void) setTags: (NSArray*) newTags
{
    if (!((newTags==tags) || [newTags isEqual:tags])) {
        [newTags retain];
        [tags release];
        tags=newTags;
        changes.tags=YES;
        [self touch];
    }
    have.tags=YES;
    loading.tags=NO;
}

- (void)_setTags:(NSArray *)newTags
{
    if (!((newTags==tags) || [newTags isEqual:tags])) {
        [newTags retain];
        [tags release];
        tags=newTags;
    }
    have.tags=YES;
    loading.tags=NO;
}

- (void)invalidateTags
{
    if (loading.tags) {
        changes.tags=YES;
        [self touch];
    } else if (have.tags) {
        [self _setTags:nil];
        changes.tags=YES;
        have.tags=NO;
        [self touch];
    }
}

- (void)getTags
{
    if (!loading.tags) {
        ECFileFlags fileFlags;

        fileFlags=[self flags];
        if (have.flags) {
            if ((fileFlags.type==ECLocalFile) || fileFlags.isIgnored || (fileFlags.isDir && !fileFlags.isWrapper)) {
                [self _setTags:nil];
                return ;
            }
        }
        loading.tags=YES;
        [[ResultsRepository sharedResultsRepository] getTagsForFile:self];
    }
}


// ---------- repository
- (CvsRepository *)repository
{
    if (!have.repository) {
        [self getRepository];
    }
    return repository;
}

- (void)invalidateRepository
{
    if (loading.repository) {
        changes.repository=YES;
        [self touch];
    } else if (have.repository) {
        changes.repository=YES;
        have.repository=NO;
        [self touch];
    }
}

- (void)getRepository
{
    if (!loading.repository) {
        loading.repository=YES;
        repository=[CvsRepository repositoryForPath:[self path]];
        if (!repository || [repository isUpToDate]) {
            [repository retain];
            loading.repository=NO;
            have.repository=YES;
        } else {
            repository=nil;
        }
    }
}

// ----------------- other methods
- (void)touch
{
    [[ResultsRepository sharedResultsRepository] fileDidChange:self];
}

- (void)invalidateAll
{
#warning BUG?
    // Some bug reports make me think that we are modifying the tree
    // during its traversal...
    ResultsRepository *resultsRepository=[ResultsRepository sharedResultsRepository];

    [resultsRepository startUpdate];
    [self invalidateFlags];
    [self invalidateChildren];
    [self invalidateStatus];
    [self invalidateLog];
    [self invalidateTags];
    [self invalidateDifferences];
    [self invalidateCumulatedStatuses];
    [self invalidateRepository];
    wasInvalidated = YES;
    [resultsRepository endUpdate];
}

- (void)propagateChanges
{
    if (oldChanges.flags || oldChanges.children) {
        [self invalidateStatus];
        [self invalidateCumulatedStatuses];
    }
    if (oldChanges.cumulatedStatuses) {
        BOOL	oldFlag = statusWasInvalidated;

        [self invalidateStatus];
        statusWasInvalidated = oldFlag; // Needed?
        [[self parent] invalidateCumulatedStatuses];
    }
    if (oldChanges.status || oldChanges.quickStatus) {
        [[self parent] invalidateCumulatedStatuses];
    }
    if (oldChanges.repository) {
        if (have.children) {
           [children makeObjectsPerformSelector:@selector(invalidateFlags)];
        }
        [self invalidateFlags];
        [self invalidateChildren];
    }
}

- (void)cumulateChanges
{
    loading.flags&=!changes.flags;
    loading.children&=!changes.children;
    loading.status&=!changes.status;
    loading.quickStatus&=!changes.quickStatus;
    loading.log&=!changes.log;
    loading.differences&=!changes.differences;
    loading.cumulatedStatuses&=!changes.cumulatedStatuses;
    loading.repository&=!changes.repository;

    cumulatedChanges.flags|=changes.flags;
    cumulatedChanges.children|=changes.children;
    cumulatedChanges.status|=changes.status;
    cumulatedChanges.quickStatus|=changes.quickStatus;
    cumulatedChanges.log|=changes.log;
    cumulatedChanges.differences|=changes.differences;
    cumulatedChanges.cumulatedStatuses|=changes.cumulatedStatuses;
    cumulatedChanges.repository|=changes.repository;

    oldChanges=changes;
    
    changes.flags=NO;
    changes.children=NO;
    changes.status=NO;
    changes.quickStatus=NO;
    changes.log=NO;
    changes.differences=NO;
    changes.cumulatedStatuses=NO;
    changes.repository=NO;
}

- (void)clearChanges
{
    
    changes.flags=NO;
    changes.children=NO;
    changes.status=NO;
    changes.quickStatus=NO;
    changes.log=NO;
    changes.differences=NO;
    changes.cumulatedStatuses=NO;
    changes.repository=NO;

    oldChanges=changes;
    cumulatedChanges=changes;
}

- (ECFileAttributeGroups)changes
{
    return cumulatedChanges;
}

- (ECFileAttributeGroups)have
{
    return have;
}

- (NSComparisonResult)compareOnFileName:(CVLFile *)otherFile
{
    return [[[self path] lastPathComponent] gentleCompare:[[otherFile path] lastPathComponent]];
}

- (NSString *) filenameForRevision:(NSString *)aVersion
{
    return [[[self path] lastPathComponent] cvlFilenameForRevision:aVersion];
}

- (NSString *) filenameForDate:(NSString *)aDate
{
    return [[[self path] lastPathComponent] cvlFilenameForDate:aDate];
}

- (NSString *) fileAncestorForRevision:(NSString *)aVersion
{
    NSString	*aFilename = [self name];
    
    // This pattern MUST match the one defined in -precedingVersion
    return [[[self path] stringByDeletingLastPathComponent] stringByAppendingPathComponent:[NSString stringWithFormat:@".#%@.%@", [[[aFilename stringByDeletingPathExtension] stringByAppendingFormat:@"-%@", aVersion] stringByAppendingPathExtension:[aFilename pathExtension]]]];
}

- (NSString *) precedingVersion
// Retrieves preceding version, assuming it is the earliest file matching pattern: .#filename.version
{
    NSString		*currentFile = [self name];
    NSString		*currentDir = [path stringByDeletingLastPathComponent];
    NSArray			*dirContent = [[NSFileManager defaultManager] directoryContentsAtPath:currentDir];
    NSString		*latestVersion = nil;
    NSDate			*latestDate = [NSDate distantPast];
    NSEnumerator	*dirEnum = [dirContent objectEnumerator];
    NSString		*dirEntry;

    while(dirEntry = [dirEnum nextObject]){
        NSArray	*possibleVersions = [dirEntry componentsMatchedByOperator:[NSString stringWithFormat:@"/\\.#%@\\.(.*)/", currentFile]]; // A verifier si le pattern est ok !

        if([possibleVersions count]){
            NSDictionary	*attributes = [[NSFileManager defaultManager] fileAttributesAtPath:[currentDir stringByAppendingPathComponent:dirEntry] traverseLink:NO];
            NSDate			*modDate = [attributes objectForKey:NSFileModificationDate];

            if([modDate compare:latestDate] == NSOrderedDescending){
                latestDate = modDate;
                latestVersion = [possibleVersions objectAtIndex:0];
            }
        }
    }
    return latestVersion;
}

- (NSString *) description
{
    return [[super description] stringByAppendingFormat:@"(%@)", [self path]];
}

- (BOOL) hasBeenRegisteredByRepository
// We cannot rely on isInRepository flag, because it is not up-to-date!
{
    ECTokenizedStatus	tokenizedStatus = [self status].tokenizedStatus;

    // Files which have been locally added then locally removed disappear: they are no longer in CVS/Entries
#warning (Stephane) Does it work for directories?
    return (tokenizedStatus != ECNotCVS && tokenizedStatus != ECNotCVS && tokenizedStatus != ECUnknown && tokenizedStatus != ECIgnored && tokenizedStatus != ECLocallyAdded && tokenizedStatus != ECNoStatus);
}

- (BOOL) isBinary
// Returns YES if sticky options contain -kb
{
    // Shouldn't we assert that stickyOptions must have been already loaded?
    return ([[self stickyOptions] rangeOfString:@"-kb"].length > 0);
}

- (BOOL) isRealWrapper
// Returns YES if file is a directory which is considered as a single file by cvs
{
    // Shouldn't we assert that flags must have been already loaded?
    (void)[self flags];
    return (flags.isDir && flags.isWrapper);
}

- (BOOL) hasStickyTagOrDate
{
    return [self stickyDate] != nil || ([[self stickyTag] length] > 0 && [stickyTag rangeOfString:@"branch:"].length == 0); // stickyTag could also be a branch tag; in this case, we don't care
}

@end
