/* CVLWaitController.m created by stephane on Thu 21-Oct-1999 */
/*$Id: CVLWaitController.m,v 1.2 2000/05/08 14:04:10 stephane Exp $*/

// Copyright (c) 1997-2000, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "CVLWaitController.h"
#import <AppKit/AppKit.h>


NSString	*CVLWaitConditionMetNotification = @"CVLWaitConditionMetNotification";


@interface CVLWaitController(Private)
- (void) setTarget:(id)aTarget selector:(SEL)aSelector userInfo:(id)aUserInfo;
- (void) setCancellable:(BOOL)canBeCancelled;
- (void) wait;
- (void) waiting;
- (void) endWait;
@end

@implementation CVLWaitController

static NSTimeInterval	_defaultGranularity = 0.1;
static NSTimeInterval	_displayThresholdDelay = 0.5;

+ (void) setGranularity:(NSTimeInterval)aGranularity
{
    NSParameterAssert(aGranularity > 0.0);
    _defaultGranularity = aGranularity;
}

+ (NSTimeInterval) granularity
{
    return _defaultGranularity;
}

+ (void) setDisplayThresholdDelay:(NSTimeInterval)aDelay
{
    NSParameterAssert(aDelay >= 0);
    _displayThresholdDelay = aDelay;
}

+ (NSTimeInterval) displayThresholdDelay
{
    return _displayThresholdDelay;
}

+ (CVLWaitController *) waitForConditionTarget:(id)aTarget selector:(SEL)aSelector cancellable:(BOOL)canBeCancelled userInfo:(id)aUserInfo
{
    CVLWaitController	*aController = [[self alloc] init];

    [aController setTarget:aTarget selector:aSelector userInfo:aUserInfo];
    [aController setCancellable:canBeCancelled];
    [aController wait];

    return [aController autorelease];
}

- (id) init
{
	if(self = [super init]){
        message = @"";
        granularity = [[self class] granularity];
        displayThresholdDelay = [[self class] displayThresholdDelay];
	}

	return self;
}

- (void) dealloc
{
    [message release];
    [target release];
    [userInfo release];
    [waitPanel release];
    [timer invalidate];
    [timer release];

    [super dealloc];
}

- (void) setTarget:(id)aTarget selector:(SEL)aSelector userInfo:(id)aUserInfo
{
    NSParameterAssert(aTarget != nil);
    NSParameterAssert(aSelector != NULL);
    NSParameterAssert([aTarget respondsToSelector:aSelector]);
    
    target = [aTarget retain];
    selector = aSelector;
    userInfo = [aUserInfo retain];
}

- (void) setCancellable:(BOOL)canBeCancelled
{
    cancellable = canBeCancelled;
    if(!cancellable){
        [cancelButton removeFromSuperview];
        cancelButton = nil;
    }
}

- (void) displayPanelIfNeeded
{
    if(displayThresholdDelay < 0.0){
        NSAssert([NSBundle loadNibNamed:@"CVLWait" owner:self] != NO, @"Unable to load nib named CVLWait");
        [messageTextField setStringValue:message];
        [[messageTextField cell] setWraps:YES];
        [waitPanel center];
        [waitPanel makeKeyAndOrderFront:nil];
    }
}

- (void) wait
{
    NSAssert(timer == nil, @"-[CVLWaitController wait] has already been called once.");

    displayThresholdDelay -= granularity;
    [self displayPanelIfNeeded];
    timer = [[NSTimer scheduledTimerWithTimeInterval:granularity target:self selector:@selector(waiting) userInfo:nil repeats:YES] retain];
}

- (void) waiting
{
    [progressIndicator animate:self];
    if([target performSelector:selector] != nil)
        [self endWait];
    else{
        displayThresholdDelay -= granularity;
        [self displayPanelIfNeeded];
    }
}

- (void) endWait
{
    [waitPanel orderOut:self];
    [self retain]; // It might happen that the only retainer on self is the timer, so let's retain ourself before invalidating timer...
    [timer invalidate];
    [[NSNotificationCenter defaultCenter] postNotificationName:CVLWaitConditionMetNotification object:self userInfo:(userInfo ? [NSDictionary dictionaryWithObjectsAndKeys:userInfo, @"UserInfo", nil]:nil)];
    [timer release];
    timer = nil;
    [self release];
}

- (void) setWaitMessage:(NSString *)aMessage
{
    if(aMessage){
        [message autorelease];
        message = [aMessage retain];
    }
    else{
        [message autorelease];
        message = @"";
    }
    [messageTextField setStringValue:message];
}

- (IBAction) cancelWaiting:(id)sender
{
    aborted = YES;
    [self endWait];
}

- (BOOL) waitCancelled
{
    return aborted;
}

@end
