/* Comparator.m created by stephane on Wed 29-Sep-1999 */
/*$Id: Comparator.m,v 1.11 2002/06/24 09:58:03 kindov Exp $*/

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "Comparator.h"
#import "../CVLFile.h"
#import "../CVLDelegate.h"
#import "../SenFoundationCVLCandidates.subproj/NSArray_RevisionComparison.h"
#import "../ResultsRepository.h"
#ifdef MACOSX
#import <CVLOpendiffRequest.h>
#else
#import <Requests/CVLOpendiffRequest.h>
#endif
#import <SenFoundation/SenSelection.h>
#import <AppKit/AppKit.h>


#define CURRENT_REVISION_STRING	@"Current"


@interface Comparator(Private)
- (void) updateGUIAndShowHideAncestor:(BOOL)flag;
@end

@implementation Comparator

static Comparator	*sharedInstance = nil;

+ (Comparator *) sharedComparator
{
    if(!sharedInstance)
        sharedInstance = [[self alloc] init];

    return sharedInstance;
}

- (id) init
{
    if(self = [self initWithWindowNibName:@"Comparator"]){
        // Like CVLInspectorManager: suffers the same bugs
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(selectionChanged:) name:SenSelectionDidChangeNotification object:[(CVLDelegate *)[[NSApplication sharedApplication] delegate] globalSelection]];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(resultsChanged:) name:@"ResultsChanged" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(viewerWillClose:) name:@"ViewerWillClose" object:nil];
#ifdef MACOSX
        [self setWindowFrameAutosaveName:@"Comparisons"];
#endif
        dateFormatter = [[NSDateFormatter alloc] initWithDateFormat:[[NSUserDefaults standardUserDefaults] stringForKey:@"CVLDateFormat"] allowNaturalLanguage:YES];
    }

    return self;
}

- (void) dealloc
{
    [comparisonParameterDictionary release];
    [file release];
    [dateFormatter release];
    
    [super dealloc];
}

- (void) windowDidLoad
{
    [super windowDidLoad];

    // Normally we shouldn't need to implement this; we could simply call [self setWindowFrameAutosaveName:@"Comparisons"] in init
    // BUT... if we do it, superclass' implementation does reposition window if it is not fully visible on screen,
    // which is VERY annoyable.
#ifndef MACOSX
    if(![[self window] setFrameAutosaveName:@"Comparisons"])
        [[self window] setFrameUsingName:@"Comparisons"];
#endif

    [ancestorSwitch performClick:nil]; // Hide ancestor part on startup
    [self updateGUIAndShowHideAncestor:NO];
}

- (void) comparisonForFile:(CVLFile *)aFile parameters:(NSDictionary *)parameterDictionary orderFront:(BOOL)orderFront
{
    if(![aFile isLeaf]){
        // We do not support comparison of directories (limited by CvsUpdateRequest)
        aFile = nil;
        parameterDictionary = nil;
	}
    [comparisonParameterDictionary autorelease];
    comparisonParameterDictionary = [parameterDictionary copyWithZone:[self zone]];
    [file autorelease];
    file = [aFile retain];

    if(orderFront || ([self isWindowLoaded] && [[self window] isVisible])){
        [self updateGUIAndShowHideAncestor:orderFront];
        if(orderFront)
            [self showWindow:self];
    }
}

- (void) comparisonForFile:(CVLFile *)aFile parameters:(NSDictionary *)parameterDictionary
{
    [self comparisonForFile:aFile parameters:parameterDictionary orderFront:YES];
}

- (id) objectValueFromDateAsString:(NSString *)string
{
    if([freeFormSwitch state])
        return string;
    else
        return [NSCalendarDate dateWithString:string calendarFormat:@"%Y-%m-%dT%H:%M:%S%z"];
}

- (void) updateGUIAndShowHideAncestor:(BOOL)showHideAncestor
{
    NSDictionary	*aDict;
    NSString		*aString;
    NSArray			*versionDictionaries = [file log];
    NSMutableArray	*values = [NSMutableArray arrayWithCapacity:[versionDictionaries count]];
    NSEnumerator	*anEnum;
    BOOL			enabled = (file != nil);
    BOOL			displayAncestor = NO;

    (void)[self window]; // Force loading of nib if necessary

    [leftRevisionPopup removeAllItems];
    [rightRevisionPopup removeAllItems];
    [ancestorRevisionPopup removeAllItems];
    [leftTagPopup removeAllItems];
    [rightTagPopup removeAllItems];
    [ancestorTagPopup removeAllItems];

    [leftMatrix setEnabled:enabled];
    [leftRevisionPopup setEnabled:enabled];
    [leftTagPopup setEnabled:enabled];
    [leftDateTextField setEnabled:enabled];
    [rightMatrix setEnabled:enabled];
    [rightRevisionPopup setEnabled:enabled];
    [rightTagPopup setEnabled:enabled];
    [rightDateTextField setEnabled:enabled];
    [ancestorMatrix setEnabled:enabled];
    [ancestorRevisionPopup setEnabled:enabled];
    [ancestorTagPopup setEnabled:enabled];
    [ancestorDateTextField setEnabled:enabled];
    [mergeTextField setEnabled:enabled];
    [mergeChooserButton setEnabled:enabled];
    [comparisonButton setEnabled:enabled];

    if(!enabled){
        [pathTextField setStringValue:@""];
        [leftDateTextField setObjectValue:nil];
        [rightDateTextField setObjectValue:nil];
        [ancestorDateTextField setObjectValue:nil];
        [mergeTextField setStringValue:@""];
        return;
    }

    [pathTextField setStringValue:[[file path] stringByAbbreviatingWithTildeInPath]];

    anEnum = [[versionDictionaries sortedRevisionArrayWithKey:@"revision"] objectEnumerator];
    while(aDict = [anEnum nextObject])
        [values addObject:[aDict objectForKey:@"revision"]];

	[leftRevisionPopup addItemWithTitle:CURRENT_REVISION_STRING];
    [leftRevisionPopup addItemsWithTitles:values];
    [rightRevisionPopup addItemWithTitle:CURRENT_REVISION_STRING];
    [rightRevisionPopup addItemsWithTitles:values];
    [ancestorRevisionPopup addItemWithTitle:CURRENT_REVISION_STRING];
    [ancestorRevisionPopup addItemsWithTitles:values];

    [values removeAllObjects];
    versionDictionaries = [file tags];
    anEnum = [[versionDictionaries sortedRevisionArrayWithKey:@"tag"] objectEnumerator];
    while(aDict = [anEnum nextObject])
        [values addObject:[aDict objectForKey:@"tag"]];

    [leftTagPopup addItemsWithTitles:values];
    [rightTagPopup addItemsWithTitles:values];
    [ancestorTagPopup addItemsWithTitles:values];

    [leftMatrix selectCellAtRow:0 column:0];
    aString = [comparisonParameterDictionary objectForKey:@"LeftRevision"];
    if(aString)
        [leftRevisionPopup selectItemWithTitle:aString];
    else{
        [leftRevisionPopup selectItemAtIndex:0];
        aString = [comparisonParameterDictionary objectForKey:@"LeftTag"];
        if(aString){
            [leftTagPopup selectItemWithTitle:aString];
            [leftMatrix selectCellAtRow:1 column:0];
        }
        else{
            if([leftTagPopup numberOfItems] > 0)
                [leftTagPopup selectItemAtIndex:0];
            else{
                [[leftMatrix cellAtRow:1 column:0] setEnabled:NO];
                [leftTagPopup setEnabled:NO];
            }
            aString = [comparisonParameterDictionary objectForKey:@"LeftDate"];
            if(aString){
                [leftDateTextField setObjectValue:[self objectValueFromDateAsString:aString]];
                [leftMatrix selectCellAtRow:2 column:0];
            }
            else
                [leftDateTextField setObjectValue:nil];
        }
    }

    [rightMatrix selectCellAtRow:0 column:0];
    aString = [comparisonParameterDictionary objectForKey:@"RightRevision"];
    if(aString)
        [rightRevisionPopup selectItemWithTitle:aString];
    else{
        [rightRevisionPopup selectItemAtIndex:0];
        aString = [comparisonParameterDictionary objectForKey:@"RightTag"];
        if(aString){
            [rightTagPopup selectItemWithTitle:aString];
            [rightMatrix selectCellAtRow:1 column:0];
        }
        else{
            if([rightTagPopup numberOfItems] > 0)
                [rightTagPopup selectItemAtIndex:0];
            else{
                [[rightMatrix cellAtRow:1 column:0] setEnabled:NO];
                [rightTagPopup setEnabled:NO];
            }
            aString = [comparisonParameterDictionary objectForKey:@"RightDate"];
            if(aString){
                [rightDateTextField setObjectValue:[self objectValueFromDateAsString:aString]];
                [rightMatrix selectCellAtRow:2 column:0];
            }
            else
                [rightDateTextField setObjectValue:nil];
        }
    }

    [ancestorMatrix selectCellAtRow:0 column:0];
    aString = [comparisonParameterDictionary objectForKey:@"AncestorRevision"];
    if(aString){
        [ancestorRevisionPopup selectItemWithTitle:aString];
        displayAncestor = YES;
    }
    else{
        [ancestorRevisionPopup selectItemAtIndex:0];
        aString = [comparisonParameterDictionary objectForKey:@"AncestorTag"];
        if(aString){
            [ancestorTagPopup selectItemWithTitle:aString];
            [ancestorMatrix selectCellAtRow:1 column:0];
            displayAncestor = YES;
        }
        else{
            if([ancestorTagPopup numberOfItems] > 0)
                [ancestorTagPopup selectItemAtIndex:0];
            else{
                [[ancestorMatrix cellAtRow:1 column:0] setEnabled:NO];
                [ancestorTagPopup setEnabled:NO];
            }
            aString = [comparisonParameterDictionary objectForKey:@"AncestorDate"];
            if(aString){
                [ancestorDateTextField setObjectValue:[self objectValueFromDateAsString:aString]];
                [ancestorMatrix selectCellAtRow:2 column:0];
                displayAncestor = YES;
            }
            else
                [ancestorDateTextField setObjectValue:nil];
        }
    }

    if(showHideAncestor && [ancestorSwitch state] != displayAncestor)
        [ancestorSwitch performClick:nil];

    aString = [comparisonParameterDictionary objectForKey:@"MergeFile"];
    if(aString)
        [mergeTextField setStringValue:[aString stringByAbbreviatingWithTildeInPath]];
    else
        [mergeTextField setStringValue:[[file path] stringByAbbreviatingWithTildeInPath]];
}

- (NSString *) dateAsStringForTextField:(NSTextField *)textField
{
    if([textField objectValue]){
        if([freeFormSwitch state])
            return [NSString stringWithString:[textField stringValue]];
        else
            // http://www.w3.org/TR/NOTE-datetime
            // ISO8601
            // e.g. YYYY-MM-DDThh:mm:ss.sTZD (eg 1997-07-16T19:20:30.45+01:00)
            return [(NSCalendarDate *)[textField objectValue] descriptionWithCalendarFormat:@"%Y-%m-%dT%H:%M:%S%z"];
    }
    else
        return nil;
}

- (IBAction) compareVersions:(id)sender
{
    // Do comparison by launching opendiffRequest
    NSMutableDictionary	*currentParams = [NSMutableDictionary dictionary];
    CVLOpendiffRequest	*aRequest;
    NSString			*aString;

    switch([leftMatrix selectedRow]){
        case 0:
            if([leftRevisionPopup indexOfSelectedItem] != 0){
                aString = [leftRevisionPopup titleOfSelectedItem];
                [currentParams setObject:aString forKey:@"LeftRevision"];
            }
			break;
        case 1:
            aString = [leftTagPopup titleOfSelectedItem];
            if(aString)
                [currentParams setObject:aString forKey:@"LeftTag"];
			break;
        case 2:
            aString = [self dateAsStringForTextField:leftDateTextField];
            if([aString length] != 0)
                [currentParams setObject:aString forKey:@"LeftDate"];
			break;
    }

    switch([rightMatrix selectedRow]){
        case 0:
            if([rightRevisionPopup indexOfSelectedItem] != 0){
                aString = [rightRevisionPopup titleOfSelectedItem];
                [currentParams setObject:aString forKey:@"RightRevision"];
            }
            break;
        case 1:
            aString = [rightTagPopup titleOfSelectedItem];
            if(aString)
                [currentParams setObject:aString forKey:@"RightTag"];
            break;
        case 2:
            aString = [self dateAsStringForTextField:rightDateTextField];
            if([aString length] != 0)
                [currentParams setObject:aString forKey:@"RightDate"];
            break;
    }

    if([ancestorSwitch state])
        switch([ancestorMatrix selectedRow]){
            case 0:
                if([ancestorRevisionPopup indexOfSelectedItem] != 0){
                    aString = [ancestorRevisionPopup titleOfSelectedItem];
                    [currentParams setObject:aString forKey:@"AncestorRevision"];
                }
                break;
            case 1:
                aString = [ancestorTagPopup titleOfSelectedItem];
                if(aString && [aString length])
                    [currentParams setObject:aString forKey:@"AncestorTag"];
                    break;
            case 2:
                aString = [self dateAsStringForTextField:ancestorDateTextField];
                if([aString length] != 0)
                    [currentParams setObject:aString forKey:@"AncestorDate"];
                    break;
        }

	aString = [mergeTextField stringValue];
    if([aString length] != 0)
        [currentParams setObject:aString forKey:@"MergeFile"];
    
    aRequest = [[CVLOpendiffRequest alloc] initWithFile:file parameters:currentParams];
    [aRequest schedule];
    [aRequest release];
}

- (IBAction) useAncestor:(id)sender
{
    NSRect	aRect = [[self window] frame];

    if([ancestorSwitch state]){
        // Display ancestor part
        NSView	*nextKeyView = [ancestorSwitch nextKeyView];

        aRect.size.width += NSWidth([ancestorBox frame]);
        [[self window] setFrame:aRect display:YES];
        [ancestorDateTextField setNextKeyView:nextKeyView];
        [ancestorSwitch setNextKeyView:ancestorMatrix];
    }
    else{
        // Hide ancestor part
        NSView		*nextKeyView = [ancestorDateTextField nextKeyView];
        NSResponder	*currentFirstResponder = [[self window] firstResponder];

        aRect.size.width -= NSWidth([ancestorBox frame]);
        [[self window] setFrame:aRect display:YES];
        [ancestorSwitch setNextKeyView:nextKeyView];
        [ancestorDateTextField setNextKeyView:nil];

        if(currentFirstResponder == ancestorMatrix || currentFirstResponder == ancestorRevisionPopup || currentFirstResponder == ancestorTagPopup || currentFirstResponder == ancestorDateTextField)
            [[self window] makeFirstResponder:ancestorSwitch];
    }
}

- (IBAction) choiceChanged:(id)sender
{
    int	tag = [sender tag];

    if(tag < 10)
        [leftMatrix selectCellAtRow:[sender tag] column:0];
    else if(tag < 20)
        [rightMatrix selectCellAtRow:(tag - 10) column:0];
    else
        [ancestorMatrix selectCellAtRow:(tag - 20) column:0];
}

- (void) comparisonForPath:(NSString *)aPath
{
    if(aPath)
        [self comparisonForFile:[CVLFile treeAtPath:aPath] parameters:nil orderFront:NO];
    else
        [self comparisonForFile:nil parameters:nil orderFront:NO];
}

- (void) selectionChanged:(NSNotification *)aNotif
{
    if([[aNotif object] count] == 1)
        [self comparisonForPath:(NSString *)[[aNotif object] selectedObject]];
    else
        [self comparisonForPath:nil]; // We don't support multiselections
}

- (void) resultsChanged:(NSNotification *)notification
{
    if(![[self window] isVisible])
        return;
    
    if(file && [[notification object] hasChanged])
        if([[[notification object] changedFiles] containsObject:file])
            [self updateGUIAndShowHideAncestor:NO];
}

- (void) viewerWillClose:(NSNotification *)notification
{
    [self selectionChanged:nil];
}

- (IBAction) showWindow:(id)sender
{
    NSArray	*selectedFiles = [[(CVLDelegate *)[[NSApplication sharedApplication] delegate] globalSelection] selectedObjects];
    
    [super showWindow:sender];
    [self toggleFreeFormDate:nil];
    if([selectedFiles count] == 1)
        [self comparisonForPath:[selectedFiles lastObject]];
    else
        [self comparisonForPath:nil]; // We don't support multiselections
}

- (IBAction) toggleFreeFormDate:(id)sender
{
    NSEnumerator	*anEnum = [[NSArray arrayWithObjects:leftDateTextField, rightDateTextField, ancestorDateTextField, nil] objectEnumerator];
    NSTextField		*aTextField;
    BOOL			freeForm = [freeFormSwitch state];

    while(aTextField = [anEnum nextObject]){
        if(freeForm){
            [[aTextField cell] setFormatter:nil];
            [aTextField setToolTip:@"Enter date using any format cvs can understand"];
        }
        else{
            [[aTextField cell] setFormatter:dateFormatter];
            [aTextField setToolTip:[NSString stringWithFormat:@"Enter date like '%@'", [[NSCalendarDate calendarDate] descriptionWithCalendarFormat:[dateFormatter dateFormat]]]];
        }
    }
}

@end
