/* $Id: VLLogInspector.m,v 1.14 2002/06/21 10:08:50 stephane Exp $ */

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "VLLogInspector.h"
#import <ResultsRepository.h>
#ifdef MACOSX
#import <NSArray_RevisionComparison.h>
#else
#import <SenFoundation/NSArray_RevisionComparison.h>
#endif
#ifdef MACOSX
#import <CVLOpendiffRequest.h>
#else
#import <Requests/CVLOpendiffRequest.h>
#endif

#import <AppKit/AppKit.h>
#import <Foundation/Foundation.h>
#import "CVLFile.h"


@interface VLLogInspector (NSTableDataSource)

- (NSArray*) cachedSortedLogArray;
- (int)numberOfRowsInTableView:(NSTableView *)tableView;
- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row;

@end

//-------------------------------------------------------------------------------------
@implementation VLLogInspector

- (void) awakeFromNib
{
    [table setDoubleAction:@selector(showDifference:)];
    [table setTarget:self];
    lastSelectedRows = [[NSMutableArray alloc] initWithCapacity:2];
}

- (void) compareLeftKey:(NSString *)lkey value:(NSString *)lvalue withRightKey:(NSString *)rkey value:(NSString *)rvalue
{
    // Do comparison by launching opendiffRequest
    NSMutableDictionary	*currentParams = [NSMutableDictionary dictionary];
    CVLOpendiffRequest	*aRequest;

    [currentParams setObject:lvalue forKey:lkey];
    if(rvalue)
        [currentParams setObject:rvalue forKey:rkey];

    aRequest = [[CVLOpendiffRequest alloc] initWithFile:[CVLFile treeAtPath:[[self inspected] objectAtIndex:0]] parameters:currentParams];
    [aRequest schedule];
    [aRequest release];
}

- (IBAction) showDifference:(id)sender
{
    NSEnumerator	*anEnum = [table selectedRowEnumerator];
    NSNumber		*aRow = [anEnum nextObject];

    if(aRow){
        NSString	*lvalue;
        NSString	*rvalue = nil;

        lvalue = [[[self cachedSortedLogArray] objectAtIndex:[aRow intValue]] objectForKey:@"revision"];
        aRow = [anEnum nextObject];
        if(aRow)
            rvalue = [[[self cachedSortedLogArray] objectAtIndex:[aRow intValue]] objectForKey:@"revision"];
        [self compareLeftKey:@"LeftRevision" value:lvalue withRightKey:(aRow ? @"RightRevision":nil) value:rvalue];
    }
    else
        NSBeep();
}

- (IBAction) showLog:sender
{
}


- (IBAction )select:sender
{
  NSString* logMsg= nil;

  if (([[self cachedSortedLogArray] count] > 0) && ([table selectedRow] >= 0))
  {
    NSDictionary* logDict= [[self cachedSortedLogArray] objectAtIndex: [table selectedRow]];

    logMsg= [logDict objectForKey: @"msg"];
   // [date setTimeFromNSDate: (NSCalendarDate*) [logDict objectForKey: @"date"]];
   // [date setStringValue: [[logDict objectForKey: @"date"] description]];
  }
  else
  {
    logMsg= @"";
    //[date setTimeFromNSDate: [[NSDate distantPast] dateWithCalendarFormat: NULL
    //                                                           timeZone: NULL]];
   // [date setStringValue: [[NSDate distantPast] description]];
  }

  [logField setString:[[logMsg copy] autorelease]];
 // [date display];
} // select:


- (void) selectVersionOfElement: (id) anElement
{  // find row of current element revision
    if (logArray) {
        NSString* elementRevision= (NSString*)[(CVLFile *)[CVLFile treeAtPath:anElement] revisionInWorkArea];
        BOOL found= NO;
        int i=0;
        int count= [[self cachedSortedLogArray] count];

        while ((!found) && (i < count))
          {
            NSDictionary* dict= [[self cachedSortedLogArray] objectAtIndex: i];

            found= [[dict objectForKey: @"revision"] isEqualToString: elementRevision];
            i++;
          }
        [table deselectAll:nil]; // If selected index does not change, no notif will be sent => deselect then reselect every time
        if (found)
          {  // select this row in table
            [table selectRow: i-1 byExtendingSelection: NO];
            [table scrollRowToVisible: i-1];
          }
    }
    [self select: nil];
}


- (void) update
{
    ResultsRepository* resultRep = [ResultsRepository sharedResultsRepository];
    NSString* element= [[self inspected] objectAtIndex: 0];
    id newLogArray;

    [resultRep startUpdate];
    newLogArray=[(CVLFile *)[CVLFile treeAtPath:element] log];
    if (logArray != newLogArray) {
        [logArray release];

        if ([newLogArray isKindOfClass: [NSArray class]]) {
            logArray = [newLogArray retain];
        }
        else {
            logArray = nil;
        }
        needsSorting= YES;
        [table reloadData];
        [self selectVersionOfElement: element];
    }
    [resultRep endUpdate];
}
@end

//-------------------------------------------------------------------------------------

@implementation VLLogInspector (NSTableViewDelegate)

- (void) tableView:(NSTableView *)tableView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
    if([cell isKindOfClass:[NSTextFieldCell class]]){
        NSColor	*textColor = [NSColor controlTextColor];

        if([[[[self cachedSortedLogArray] objectAtIndex:row] objectForKey:@"state"] isEqualToString:@"dead"]){
            // State <dead> means that file has been removed from cvs and committed
            static NSColor	*redColor = nil;

            if(!redColor)
                redColor = [[NSColor colorWithCalibratedRed:0.74 green:0.0 blue:0.0 alpha:1.0] retain];
            textColor = redColor;
        }
        [(NSTextFieldCell *)cell setTextColor:textColor];
    }
}

- (void) tableViewSelectionIsChanging:(NSNotification *)aNotification
{
    NSEnumerator	*anEnum = [table selectedRowEnumerator];
    NSNumber		*aRow;

    isChangingSelection = YES;
    switch([lastSelectedRows count]){
        case 0:
            aRow = [anEnum nextObject];
            if(aRow){
                [lastSelectedRows addObject:aRow];
                aRow = [anEnum nextObject];
                if(aRow){
                    [lastSelectedRows addObject:aRow];
                    while(aRow = [anEnum nextObject])
                        [table deselectRow:[aRow intValue]];
                }
            }
			break;
        case 1:{
            BOOL		added = NO;
            NSNumber	*lastSelectedRow = nil;
            
            while(aRow = [anEnum nextObject]){
                if(!lastSelectedRow)
                    lastSelectedRow = [lastSelectedRows lastObject];
                if(![lastSelectedRow isEqual:aRow]){
                    if(added)
                        [table deselectRow:[aRow intValue]];
                    else{
                        [lastSelectedRows addObject:aRow];
                        added = YES;
                    }
                }
            }
            if(!lastSelectedRow)
                [lastSelectedRows removeAllObjects];
            break;
        }
        case 2:{
            NSArray	*oldSelectedRows = [NSArray arrayWithArray:lastSelectedRows];

            while(aRow = [anEnum nextObject]){
                if(![oldSelectedRows containsObject:aRow]){
                    [table deselectRow:[[lastSelectedRows objectAtIndex:0] intValue]];
                    [lastSelectedRows removeObjectAtIndex:0];
                    [lastSelectedRows addObject:aRow];
                }
            }
            oldSelectedRows = [[table selectedRowEnumerator] allObjects];
            if(![oldSelectedRows containsObject:[lastSelectedRows objectAtIndex:1]])
                [lastSelectedRows removeObjectAtIndex:1];
            if(![oldSelectedRows containsObject:[lastSelectedRows objectAtIndex:0]])
                [lastSelectedRows removeObjectAtIndex:0];
        }
    }
    isChangingSelection = NO;
}

- (void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
    if(isChangingSelection)
        return;
    [self tableViewSelectionIsChanging:nil]; // In case of selectAll:, there is no isChanging notification
    [self select: nil];
}

- (void) tableViewColumnDidMove: (NSNotification*) aNotification
{
    if (([aNotification object] == table) && (([[[aNotification userInfo] objectForKey: @"NSOldColumn"] intValue] == 0) ||
                                              ([[[aNotification userInfo] objectForKey: @"NSNewColumn"] intValue] == 0)))
    {
        needsSorting= YES;
        [table reloadData]; // to force redisplaying all columns (and not only the moved ones)
    }
}

@end

//-------------------------------------------------------------------------------------


@implementation VLLogInspector (NSTableDataSource)


- (NSArray*) cachedSortedLogArray
{
  static id cachedLogArray= nil;

  if (needsSorting)
  {
      NSString* currentKey= [[[table tableColumns] objectAtIndex: 0] identifier];

      [cachedLogArray release];
      cachedLogArray= [[logArray sortedRevisionArrayWithKey: currentKey] retain];
      needsSorting= NO;
  }
  return cachedLogArray;
}


- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
  if (tableView == table)
  {
    return [logArray count];    
  }
  return 0;
} // numberOfRowsInTableView:


- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
  if (tableView == table)
  {
    id theValue = [[[self cachedSortedLogArray] objectAtIndex: row] objectForKey: [tableColumn identifier]];

    if (!theValue)
    {
      return @""; 
    }
    return theValue;
  }
  return nil;
}

//- (void)tableView:(NSTableView *)tableView setObjectValue:(id)object forTableColumn:(NSTableColumn *)tableColumn row:(int)row;

@end

//-------------------------------------------------------------------------------------
