/* VLTagInspector.m created by vincent on Wed 20-May-1998 */
/*$Id: VLTagInspector.m,v 1.8 2002/06/21 10:08:51 stephane Exp $*/

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "VLTagInspector.h"
#import <ResultsRepository.h>
#ifdef MACOSX
#import <NSArray_RevisionComparison.h>
#else
#import <SenFoundation/NSArray_RevisionComparison.h>
#endif
#ifdef MACOSX
#import <CVLOpendiffRequest.h>
#else
#import <Requests/CVLOpendiffRequest.h>
#endif

#import <AppKit/AppKit.h>
#import "CVLFile.h"

// most of the code below comes from VLLogInspector 

@interface VLTagInspector (NSTableDataSource)

- (NSArray*) cachedSortedTagArray;
- (int)numberOfRowsInTableView:(NSTableView *)tableView;
- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row;

@end

@implementation VLTagInspector

- (void) awakeFromNib
{
    [table setDoubleAction:@selector(showDifference:)];
    [table setTarget:self];
    lastSelectedRows = [[NSMutableArray alloc] initWithCapacity:2];
}

- (void) compareLeftKey:(NSString *)lkey value:(NSString *)lvalue withRightKey:(NSString *)rkey value:(NSString *)rvalue
{
    // Do comparison by launching opendiffRequest
    NSMutableDictionary	*currentParams = [NSMutableDictionary dictionary];
    CVLOpendiffRequest	*aRequest;

    [currentParams setObject:lvalue forKey:lkey];
    if(rvalue)
        [currentParams setObject:rvalue forKey:rkey];

    aRequest = [[CVLOpendiffRequest alloc] initWithFile:[CVLFile treeAtPath:[[self inspected] objectAtIndex:0]] parameters:currentParams];
    [aRequest schedule];
    [aRequest release];
}

- (IBAction) showDifference:(id)sender
{
    NSEnumerator	*anEnum = [table selectedRowEnumerator];
    NSNumber		*aRow = [anEnum nextObject];

    if(aRow){
        NSString	*lvalue;
        NSString	*rvalue = nil;

        lvalue = [[[self cachedSortedTagArray] objectAtIndex:[aRow intValue]] objectForKey:@"tag"];
        aRow = [anEnum nextObject];
        if(aRow)
            rvalue = [[[self cachedSortedTagArray] objectAtIndex:[aRow intValue]] objectForKey:@"tag"];
        [self compareLeftKey:@"LeftTag" value:lvalue withRightKey:(aRow ? @"RightTag":nil) value:rvalue];
    }
    else
        NSBeep();
}

- (IBAction )select:sender
{
}

- (void) update
{
    ResultsRepository* resultRep = [ResultsRepository sharedResultsRepository];
    NSString* element= [[self inspected] objectAtIndex: 0];
    id newTagArray;

    [resultRep startUpdate];
    newTagArray=[(CVLFile *)[CVLFile treeAtPath:element] tags];
    if (tagArray != newTagArray) {
        [tagArray release];

        if ([newTagArray isKindOfClass: [NSArray class]]) {
            tagArray = [newTagArray retain];
        }
        else {
            tagArray = nil;
        }
        needsSorting= YES;
        [table reloadData];
       // [self selectVersionOfElement: element];
    }
    [resultRep endUpdate];
}

@end

@implementation VLTagInspector (NSTableViewDelegate)

- (void) tableViewSelectionIsChanging:(NSNotification *)aNotification
{
    NSEnumerator	*anEnum = [table selectedRowEnumerator];
    NSNumber		*aRow;

    isChangingSelection = YES;
    switch([lastSelectedRows count]){
        case 0:
            aRow = [anEnum nextObject];
            if(aRow){
                [lastSelectedRows addObject:aRow];
                aRow = [anEnum nextObject];
                if(aRow){
                    [lastSelectedRows addObject:aRow];
                    while(aRow = [anEnum nextObject])
                        [table deselectRow:[aRow intValue]];
                }
            }
            break;
        case 1:{
            BOOL		added = NO;
            NSNumber	*lastSelectedRow = nil;

            while(aRow = [anEnum nextObject]){
                if(!lastSelectedRow)
                    lastSelectedRow = [lastSelectedRows lastObject];
                if(![lastSelectedRow isEqual:aRow]){
                    if(added)
                        [table deselectRow:[aRow intValue]];
                    else{
                        [lastSelectedRows addObject:aRow];
                        added = YES;
                    }
                }
            }
            if(!lastSelectedRow)
                [lastSelectedRows removeAllObjects];
            break;
        }
        case 2:{
            NSArray	*oldSelectedRows = [NSArray arrayWithArray:lastSelectedRows];

            while(aRow = [anEnum nextObject]){
                if(![oldSelectedRows containsObject:aRow]){
                    [table deselectRow:[[lastSelectedRows objectAtIndex:0] intValue]];
                    [lastSelectedRows removeObjectAtIndex:0];
                    [lastSelectedRows addObject:aRow];
                }
            }
            oldSelectedRows = [[table selectedRowEnumerator] allObjects];
            if(![oldSelectedRows containsObject:[lastSelectedRows objectAtIndex:1]])
                [lastSelectedRows removeObjectAtIndex:1];
            if(![oldSelectedRows containsObject:[lastSelectedRows objectAtIndex:0]])
                [lastSelectedRows removeObjectAtIndex:0];
        }
    }
    isChangingSelection = NO;
}

- (void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
    if ([aNotification object] == table)
    {
        if(isChangingSelection)
            return;
        [self tableViewSelectionIsChanging:nil]; // In case of selectAll:, there is no isChanging notification
        [self select:nil];
    }
}

- (void) tableViewColumnDidMove: (NSNotification*) aNotification
{
    if (([aNotification object] == table) && (([[[aNotification userInfo] objectForKey: @"NSOldColumn"] intValue] == 0) ||
                                              ([[[aNotification userInfo] objectForKey: @"NSNewColumn"] intValue] == 0)))
    {
        needsSorting= YES;
        [table reloadData]; // to force redisplaying all columns (and not only the moved ones)
    }
}

@end


//-------------------------------------------------------------------------------------

@implementation VLTagInspector (NSTableDataSource)


- (NSArray*) cachedSortedTagArray
{
  static id cachedTagArray= nil;

  if (needsSorting)
  {
      NSString* currentKey= [[[table tableColumns] objectAtIndex: 0] identifier];

      [cachedTagArray release];
      cachedTagArray= [[tagArray sortedRevisionArrayWithKey: currentKey] retain];
      needsSorting= NO;
  }
  return cachedTagArray;
}


- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
  if (tableView == table)
  {
    return [tagArray count];
  }
  return 0;
} // numberOfRowsInTableView:


- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
  if (tableView == table)
  {
    id theValue = [[[self cachedSortedTagArray] objectAtIndex: row] objectForKey: [tableColumn identifier]];

    if (!theValue)
    {
      return @"";
    }
    return theValue;
  }
  return nil;
}

@end

