/*
   $Id: ProgressPanelController.m,v 1.11 2002/07/04 12:21:40 stephane Exp $
   */

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "ProgressPanelController.h"
#ifdef MACOSX
#import <Request.h>
#else
#import <Requests/Request.h>
#endif

#import <AppKit/AppKit.h>


static ProgressPanelController *theController = nil;
//-------------------------------------------------------------------------------------
@interface ProgressPanelController(Private)
- (void) requestStateChanged:(NSNotification *)notification;
- (void) queueRequest: (Request *)theRequest;
- (void) delayedRemoveRequest:(Request *)theRequest;
- (void) removeRequest:(Request *)theRequest;
@end

//-------------------------------------------------------------------------------------

@implementation ProgressPanelController

+ (void) initialize
{
	[[NSUserDefaults standardUserDefaults] registerDefaults:[NSDictionary dictionaryWithObjectsAndKeys:@"NO", @"ShowProgressPanel", nil]];
}

+ (id) sharedProgressPanelController
{
    if(!theController)
      theController = [[ProgressPanelController alloc] init];

    return theController;
}

- (id) init
{
    if(self = [self initWithWindowNibName:@"ProgressPanel"]){
        requests = [[NSMutableArray allocWithZone:[self zone]] init];
#ifdef JA_PATCH
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(requestStateChanged:)
                                                     name:@"RequestStateDidChange"
                                                   object:nil];
#else
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(requestStateChanged:)
                                                     name:@"RequestStateChanged"
                                                   object:nil];
#endif
#ifdef MACOSX
        [self setWindowFrameAutosaveName:@"ProgressPanel"];
#endif
    }

    return self;
}

- (void) dealloc
{
    [requests release];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:@"RequestStateChanged" object:nil];
    
    [super dealloc];
}

- (void) windowDidLoad
{
    [super windowDidLoad];

#ifndef MACOSX
    // Normally we shouldn't need to implement this; we could simply call [self setWindowFrameAutosaveName:@"ProgressPanel"] in init
    // BUT... if we do it, superclass' implementation does reposition window if it is not fully visible on screen,
    // which is VERY annoyable.
    if(![[self window] setFrameAutosaveName:@"ProgressPanel"])
        [[self window] setFrameUsingName:@"ProgressPanel"];
#endif

    [tableView setHeaderView:nil]; // Remove column headers
    [(NSTextFieldCell *)[[tableView tableColumnWithIdentifier:@"operation"] dataCell] setTextColor:[NSColor darkGrayColor]];
    [(NSTextFieldCell *)[[tableView tableColumnWithIdentifier:@"filename"] dataCell] setTextColor:[NSColor blackColor]];
    [(NSTextFieldCell *)[[tableView tableColumnWithIdentifier:@"state"] dataCell] setTextColor:[NSColor darkGrayColor]];
    [tableView sizeLastColumnToFit];

    [tableView reloadData];
    [self tableViewSelectionDidChange:nil];
    [[tableView enclosingScrollView] setNeedsDisplay:YES]; // If we redisplay only the tableView, there is a small glitch due to remove of headerView
}

- (IBAction) showWindow:(id)sender;
{
    [super showWindow:sender];
    [[NSUserDefaults standardUserDefaults] setBool:YES forKey:@"ShowProgressPanel"];
}

- (void) updateRequest:(Request *)aRequest
{
    [tableView reloadData];
    [self tableViewSelectionDidChange:nil];
    [tableView setNeedsDisplay:YES];
}

- (void) requestStateChanged:(NSNotification *)notification
{
    Request	*request = [notification object];

    if(![requests containsObject:request])
        [requests addObject:request];

#ifdef JA_PATCH
    if([[request currentState] isTerminal])
#else
    if([request state] == STATE_ENDED)
#endif
        [self delayedRemoveRequest:request];

    [self updateRequest:request];
}

- (void) queueRequest:(Request *)theRequest
{
    [requests insertObject:theRequest atIndex:0];
    [tableView reloadData];
    [self tableViewSelectionDidChange:nil];
    [tableView setNeedsDisplay:YES];
}

- (void) delayedRemoveRequest:(Request *)theRequest
{
    [self performSelector:@selector(removeRequest:) withObject:theRequest afterDelay:1.0 inModes:[NSArray arrayWithObjects:NSDefaultRunLoopMode, NSModalPanelRunLoopMode, nil]];
}

- (void) removeRequest:(Request *)theRequest
{
    [theRequest retain];
    [requests removeObject:theRequest];
    [tableView reloadData];
    [self tableViewSelectionDidChange:nil];
    [tableView setNeedsDisplay:YES];
    [theRequest release];
}

- (int) numberOfRowsInTableView:(NSTableView *)tableView
{
    return [requests count];
}

- (id) tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
    if([[tableColumn identifier] isEqualToString:@"operation"])
        return [[requests objectAtIndex:row] cmdTitle];
    else if([[tableColumn identifier] isEqualToString:@"filename"])
        return [[requests objectAtIndex:row] summary];
    else if([[tableColumn identifier] isEqualToString:@"state"])
        return [[requests objectAtIndex:row] stateString];
    else
        return nil;
}

- (IBAction) interruptSelectedRequests:(id)sender
{
    NSEnumerator	*anEnum = [tableView selectedRowEnumerator];
    id				anObject;
    NSMutableArray	*selectedRequests = [NSMutableArray array];

    while(anObject = [anEnum nextObject])
        [selectedRequests addObject:[requests objectAtIndex:[anObject intValue]]];

    anEnum = [selectedRequests objectEnumerator];
    while(anObject = [anEnum nextObject])
        [anObject cancel];
}

- (BOOL) tableView:(NSTableView *)aTableView shouldSelectRow:(int)rowIndex
{
    return [[requests objectAtIndex:rowIndex] canBeCancelled];
}

- (void) tableViewSelectionDidChange:(NSNotification *)aNotification
{
    NSEnumerator	*anEnum = [tableView selectedRowEnumerator];
    NSNumber		*aRow;
    BOOL			canBeCancelled = NO;

    while(aRow = [anEnum nextObject]){
        if([[requests objectAtIndex:[aRow intValue]] canBeCancelled]){
            canBeCancelled = YES;
            break;
        }
    }

    [killButton setEnabled:canBeCancelled];
}

@end

//-------------------------------------------------------------------------------------

@implementation ProgressPanelController (WindowDelegate)

- (void) windowWillClose:(NSNotification *)notification
{
    if([[NSApplication sharedApplication] isRunning]) // On NT, method also is called when app is terminating!
        [[NSUserDefaults standardUserDefaults] setBool:NO forKey:@"ShowProgressPanel"];
}

@end

//-------------------------------------------------------------------------------------

