/* CvsPserverRepository.m created by ja on Mon 16-Feb-1998 */
/*$Id: CvsPserverRepository.m,v 1.8 2002/07/01 16:17:00 kindov Exp $*/

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "CvsPserverRepository.h"
#ifdef MACOSX
#import <CvsLoginRequest.h>
#import <CVLNTLoginRequest.h>
#else
#import <Requests/CvsLoginRequest.h>
#import <Requests/CVLNTLoginRequest.h>
#endif
#import <SenOyster/SenOyster.h>
#import <SenOyster/NSString.SenOysterRegex.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSEnumerator.h>
#import <Foundation/NSNotification.h>
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSFileManager.h>


NSString	*CvsPserverHasNoCvsWrappersNotification = @"CvsPserverHasNoCvsWrappersNotification";


@interface CvsPserverRepository (Private)
- (BOOL) hasPermanentPassword;
- (void)loginEnded:(NSNotification *)notification;
- (void) checkLocalCvsWrappersFile;
@end

@implementation CvsPserverRepository
- initWithMethod:(NSString *)theMethod root:(NSString *)repositoryRoot
{
    if (self=[super initWithMethod:theMethod root:repositoryRoot]) {
        if ([self hasPermanentPassword]) {
            isLoggedIn=YES;
            [self checkLocalCvsWrappersFile];
        }
    }

    return self;
}

- initWithProperties:(NSDictionary *)properties
{
    NSString*thePassword;

    if (self=[super initWithProperties:properties]) {
        thePassword=[properties objectForKey:@"password"];
        if (thePassword && ![thePassword isEqual:@""]) {
            password=[thePassword retain];
        } else {
            if ([self hasPermanentPassword]) {
                isLoggedIn=YES;
                [self checkLocalCvsWrappersFile];
            }
        }
    }
    return self;
}

- (void)dealloc
{
    [password release];
    [loginRequest release];

    [super dealloc];
}

- (id) init
{
    self= [super init];
    isLoggedIn= NO;

    return self;
}

- (NSDictionary *)properties
{
    return [NSDictionary dictionaryWithObjectsAndKeys:method,@"method",user,@"user",host,@"host",path,@"path",password,@"password",nil];
}

- (BOOL)needsLogin
{
    return YES;
}

- (void)setIsLoggedIn:(BOOL)flag
{
    if (isLoggedIn!=flag) {
        isLoggedIn=flag;
        if (!isLoggedIn) {
            [password release];
            password=nil;
        }
        else
            [self checkLocalCvsWrappersFile];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"RepositoryChanged" object:self];
    }
}

- (BOOL)isReadyForRequests
{
    return isLoggedIn;
}

- (BOOL)isLoggedIn
{
    return isLoggedIn;
}

- (Request *)gettingReadyRequest
{
    return [self loginRequest];
}

- (Request *)loginRequest
{
    if (isLoggedIn) {
        return nil;
    } else {
        if (!loginRequest) {

#ifndef WIN32
            loginRequest=[CvsLoginRequest cvsLoginRequestForRepository:self];
            if (password) {
                [(CvsLoginRequest *)loginRequest setPassword:password];
            }
#else
            loginRequest= [CVLNTLoginRequest CVLNTLoginRequestForRepository: self];
#endif

            [[NSNotificationCenter defaultCenter]
                          addObserver:self
                             selector:@selector(loginEnded:)
                                 name:@"RequestCompleted"
                               object:loginRequest];
            [loginRequest schedule];
            [loginRequest retain];
        }
        return loginRequest;
    }
}

- (void)loginEnded:(NSNotification *)notification
{
    [[NSNotificationCenter defaultCenter]
                  removeObserver:self
                         name:@"RequestCompleted"
                       object:loginRequest];
    [loginRequest release];
    loginRequest=nil;
}

- (BOOL) hasPermanentPassword
{
    NSString* mContent= [[NSString alloc] initWithContentsOfFile:[[self homeDirectory] stringByAppendingPathComponent:@".cvspass"]];

    if (mContent) {
        NSMutableString* passContent;
        NSEnumerator* lineEnum;
        NSArray* linesArray;
        NSString* lineString;
        int num= 0;

        passContent= [mContent mutableCopy];
        // remove empty lines
#ifdef WIN32
        num= [passContent applyReplacementOperator: [NSString stringWithCString: "s/\\s+\\r\\n/\n/g"]];
#endif
        num= [passContent applyReplacementOperator: [NSString stringWithCString: "s/\\s+\\n/\n/g"]];
        linesArray= [passContent componentsSeparatedByOperator: @"/\\n/"];
        lineEnum= [linesArray objectEnumerator];
        while ((lineString= [lineEnum nextObject]))
        {
            NSArray* wordsFromLine= [lineString componentsSeparatedBySpace];
            if ([wordsFromLine count] >= 2)
            {
                NSString * fileRepositoryRoot;

                fileRepositoryRoot=[wordsFromLine objectAtIndex:0];
                if ([fileRepositoryRoot isEqual:[self root]]){
                    [passContent release];
                    [mContent release];
                    return YES;
                }
            }
        }
        [passContent release];
        [mContent release];
    }
    return NO;
}

- (void) checkLocalCvsWrappersFile
{
    if(!checkedLocalCVSWrappers){
        checkedLocalCVSWrappers = YES;
        if(![[NSFileManager defaultManager] fileExistsAtPath:[[self homeDirectory] stringByAppendingPathComponent:@".cvswrappers"]])
            [[NSNotificationCenter defaultCenter] postNotificationName:CvsPserverHasNoCvsWrappersNotification object:self];
    }
}

@end
