/* CvsRemoteRepository.m created by ja on Wed 11-Mar-1998 */
/*$Id: CvsRemoteRepository.m,v 1.10 2002/07/01 09:54:38 kindov Exp $*/

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "CvsRemoteRepository.h"
#import <SenOyster/SenOyster.h>
#import <SenOyster/NSString.SenOysterRegex.h>
#import <AppKit/NSPanel.h>
#ifdef MACOSX
#import <Foundation/NSArray.h>
#import <Foundation/NSDictionary.h>
#endif

@implementation CvsRemoteRepository
+ (NSString *)rootForProperties:(NSDictionary *)properties
{
    NSString *theMethod,*theUser,*theHost,*theRepositoryPath;

    theMethod=[properties objectForKey:@"method"];
    theUser=[properties objectForKey:@"user"];
    theHost=[properties objectForKey:@"host"];
    theRepositoryPath=[properties objectForKey:@"path"];
    if(theRepositoryPath && ![theRepositoryPath hasPrefix:@"/"]){
        // Let's check that the path is an ABSOLUTE one!
        // If we don't and it is not, repository will create an absolutely ;-)
        // awful checkoutDir!!!
        // WARNING: we cannot use method -[NSString isAbsolute], because on NT
        // it checks for backslashes!!! 
		// WARNING: on NT, path can contain drive letter!!
        if(!([theRepositoryPath length] > 2 && [theRepositoryPath characterAtIndex:1] == ':' && [theRepositoryPath characterAtIndex:2] == '/'))
           theRepositoryPath = [@"/" stringByAppendingString:theRepositoryPath];
	}

    if (theMethod && theUser && theHost && theRepositoryPath) {
        if (![theMethod isEqual:@"remote"]) {
            if (!theUser || [theUser isEqual:@""]) {
                return [NSString stringWithFormat:@":%@:%@:%@",theMethod,theHost,theRepositoryPath];
            } else {
                return [NSString stringWithFormat:@":%@:%@@%@:%@",theMethod,theUser,theHost,theRepositoryPath];
            }
        } else {
            if (!theUser || [theUser isEqual:@""]) {
                return [NSString stringWithFormat:@"%@:%@",theHost,theRepositoryPath];
            } else {
                return [NSString stringWithFormat:@"%@@%@:%@",theUser,theHost,theRepositoryPath];
            }
        }
    } else {
        return nil;
    }
}

- initWithMethod:(NSString *)theMethod root:(NSString *)repositoryRoot
{
    BOOL validRootString=NO;
    NSArray *matchingResult;
    user=host=path=nil;
    
    if (self=[super initWithMethod:theMethod root:repositoryRoot]) {
        if ([theMethod isEqual:@"remote"]) {
            matchingResult=[repositoryRoot componentsMatchedByOperator:@"/^(\\w+)@([0-9a-zA-Z_\\-\\.]+):(.+)$/"];
            if ([matchingResult count]==3) {
                user=[[matchingResult objectAtIndex:0] retain];
                host=[[matchingResult objectAtIndex:1] retain];
                path=[[matchingResult objectAtIndex:2] retain];
                validRootString=YES;
            } else { // try without user
                matchingResult=[repositoryRoot componentsMatchedByOperator:@"/^([0-9a-zA-Z_\\-\\.]+):(.+)$/"];
                if ([matchingResult count]==2) {
                    user=@"";
                    host=[[matchingResult objectAtIndex:0] retain];
                    path=[[matchingResult objectAtIndex:1] retain];
                    validRootString=YES;
                }
            }
        } else {
            matchingResult=[repositoryRoot componentsMatchedByOperator:@"/^:(\\w*):(\\w+)@([0-9a-zA-Z_\\-\\.]+):(.+)$/"];
            if ([matchingResult count]==4) { // try without user
                user=[[matchingResult objectAtIndex:1] retain];
                host=[[matchingResult objectAtIndex:2] retain];
                path=[[matchingResult objectAtIndex:3] retain];
                validRootString=YES;
            } else {
                matchingResult=[repositoryRoot componentsMatchedByOperator:@"/^:(\\w*):([0-9a-zA-Z_\\-\\.]+):(.+)$/"];
                if ([matchingResult count]==3) {
                    user=@"";
                    host=[[matchingResult objectAtIndex:1] retain];
                    path=[[matchingResult objectAtIndex:2] retain];
                    validRootString=YES;
                }
            }
        }
        if (!validRootString) {
            NSRunAlertPanel(@"Cvs problem", @"The repository root string you have given is invalid.", nil, nil, nil);
            [self release];
            return nil;
        }
    }

    return self;
}

- initWithProperties:(NSDictionary *)properties
{
    NSString *theMethod,*theUser,*theHost,*theRepositoryPath;

    theMethod=[properties objectForKey:@"method"];
    theUser=[properties objectForKey:@"user"];
    theHost=[properties objectForKey:@"host"];
    theRepositoryPath=[properties objectForKey:@"path"];

    if (self=[super initWithMethod:theMethod root:[[self class] rootForProperties:properties]]) {
        user=[theUser retain];
        host=[theHost retain];
        path=[theRepositoryPath retain];
    }
    return self;
}

- (NSDictionary *)properties
{
    return [NSDictionary dictionaryWithObjectsAndKeys:method,@"method",user,@"user",host,@"host",path,@"path",nil];
}

- (void)dealloc
{
    [user release];
    [host release];
    [path release];

    [super dealloc];
}

@end
