/* CvsRepository.h created by vincent on Thu 13-Nov-1997 */

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import <Foundation/NSObject.h>

@class NSArray, NSMutableArray, NSMutableDictionary, NSMutableString, NSMutableSet, NSDictionary, NSString, Request, CvsLoginRequest, CvsCheckoutRequest, CvsModule;

@interface CvsRepository : NSObject
{
    BOOL upToDate;
    BOOL isUpdating;
    NSString *root;
    NSString *workAreaPath;
    NSMutableArray	*modules;
    NSArray* baseIgnoreWildCardsArray;
    NSArray* baseWrapperWildCardsArray;
    NSMutableDictionary *ignoredPatternDict;
    NSMutableDictionary *wrapperPatternDict;
    NSMutableSet *dirsControlled; //cached for efficiency
    CvsCheckoutRequest *checkoutRequest;
    NSString *method;
    BOOL	cvsRootCheckoutFailed;
    NSMutableDictionary	*environment;
}
// do not subclass the following methods
+ (NSArray *) registeredRepositories;
+ (CvsRepository *) nullRepository;
+ (CvsRepository *) defaultRepository;
+ (CvsRepository *) repositoryWithRoot:(NSString *)aPath;
+ (CvsRepository *) repositoryWithProperties:(NSDictionary *)properties;
+ (Class)repositoryClassForMethod:(NSString *)methodName;
+ (void) registerRepository:(CvsRepository *)repository;
+ (NSString*) cvsRootWorkAreasPath;
+ (CvsRepository *)repositoryForPath:(NSString *)aPath;
+ (BOOL)disposeRepository:(CvsRepository *)repository;
+ (BOOL) isRepositoryToBeDisposed:(CvsRepository *)repository;

// do not call, subclass only
+ (NSString *)rootForProperties:(NSDictionary *)properties;
- initWithMethod:(NSString *)theMethod root:(NSString *)root;
- initWithProperties:(NSDictionary *)dictionary;

- (NSString *) homeDirectory;

// no restrictions
- (NSDictionary *)properties;
- (BOOL)isUpToDate;
- (BOOL)isUsed;
- (CvsCheckoutRequest *)checkoutRequest;
- (NSString *)workAreaPath;
- (NSArray *)modulesSymbolicNames;
- (NSString *)root;
- (BOOL)isWrapper: (NSString*) aPath;
- (BOOL)isIgnored: (NSString*) aPath;
- (BOOL)isLocal;  // repository answering yes must implement the 'Local' informal protocol
- (BOOL)needsLogin; // repository answering yes must implement the 'Login' informal protocol
- (BOOL)isReadyForRequests;
- (Request *)gettingReadyRequest;
- (NSArray *) modules;
- (CvsModule *) moduleWithSymbolicName:(NSString *)aName;
- (BOOL) cvsRootCheckoutFailed;

- (CvsCheckoutRequest *) checkoutAgain; // Forces a new checkout
- (BOOL) isUpdating;
- (BOOL) isUpToDate_WithoutRefresh; // Does NOT force to be up-to-date; simply returns current status

- (NSDictionary *) environment;
- (void) setEnvironment:(NSDictionary *)aDict;
- (BOOL) isInheritedEnvironmentKey:(NSString *)aKey value:(NSString *)aValue; // Local overriding of environment, or standard environment?

- (void) invalidateDir:(NSString *)dirPath; // Invalidates ignored patterns for that dir

@end

@interface CvsRepository (Login)
- (void)setIsLoggedIn:(BOOL)flag;
- (BOOL)isLoggedIn;
- (Request *)loginRequest;
@end

@interface CvsRepository (Local)
- (NSString *)path;
@end