/* CvsRepository.m created by vincent on Thu 13-Nov-1997 */
/*$Id: CvsRepository.m,v 1.22 2002/07/01 16:01:33 stephane Exp $*/

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "CvsRepository.h"
#ifdef MACOSX
#import <CvsCheckoutRequest.h>
#else
#import <Requests/CvsCheckoutRequest.h>
#endif
#import "../NSFileManager.MkDirs.h"
#import <SenOyster/SenOyster.h>
#import <SenOyster/NSString.SenOysterRegex.h>
#import "../ResultsRepository.h"
#import "../CVLFile.h"
#import <AppKit/NSApplication.h>
#ifdef MACOSX
#import <CvsModule.h>
#else
#import <Modules/CvsModule.h>
#endif
#import "../CVLDelegate.h"
#import <AppKit/NSPanel.h>
#ifdef MACOSX
#import <NSString+Lines.h>
#else
#import <SenFoundation/NSString+Lines.h>
#endif
#import "NSString+CVL.h"

static NSMutableDictionary* repositories= nil;
static CvsRepository * nullRepository=nil;
static NSMutableSet * repositoriesToRemove=nil;
static NSDictionary* repositoryClasses=nil;


@interface CvsRepository (Private)
+ (NSString*) cvsRootWorkAreasPath;
+ (NSString*) rootAtPath: (NSString*)aPath;
+ (CvsRepository *)repositoryWithWorkAreaAtPath:(NSString *)aPath;
+ (NSDictionary *)repositories;
- (CvsRepository *)initNullRepository;
- (void)setWorkAreaPath:(NSString *)thePath;
- (NSArray *) arrayByBuildingBaseIgnoreWildCards;
- (NSArray *) arrayByBuildingBaseWrapperWildCards;
- (void) checkoutEnded:(NSNotification *)notification;
- (void) registerDir:(NSString *)dirPath;
- (void) invalidateControlledDirs;
- (void) invalidateCaches;
@end

/*
 We could make a category on NSNotificationCenter to add -addObserver:selector:name:repository:
 to be able to observe all notifications sent by a specific repository, for specific actions.
 */

@implementation CvsRepository
+ (void)initialize
{
    [[NSNotificationCenter defaultCenter]
                  addObserver:self
                     selector:@selector(applicationWillTerminate:)
                         name:NSApplicationWillTerminateNotification
                       object:NSApp];

    repositoryClasses=[[NSDictionary alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"RepositoryClasses" ofType:@"plist"]];
    if (!repositoryClasses) {
        NSLog(@"Warning: cannot open repository classes property list");
    }
}

+ (NSArray *) registeredRepositories
{
    return [[self repositories] allValues];
}

+ (CvsRepository *) repositoryWithRoot:(NSString *)repositoryRoot;
{
    CvsRepository *result=nil;
    NSString *repositoryMethod;
    
    if (!repositoryRoot) {
        return [self nullRepository];
    }
    // Stephane: could we allow [NSString defaultCStringEncoding] or NSUTF8StringEncoding ?
    if (![repositoryRoot canBeConvertedToEncoding:NSASCIIStringEncoding]) {
        NSLog(@"Assertion failed: repository root not ascii");
        return nil;
    }
    result=[[self repositories] objectForKey:repositoryRoot];
    if (!result) {
        if ([repositoryRoot rangeOfString:@":"].length==1) { // contains a :
            NSArray *matchingResult=[repositoryRoot componentsMatchedByOperator:@"/^:(\\w*):(.+)$/"];
            if ([matchingResult count]==2) {
                repositoryMethod=[matchingResult objectAtIndex:0];
            } else {
                repositoryMethod=@"remote";
            }
        } else {
            repositoryMethod=@"local";
        }
        result=[[[self repositoryClassForMethod:repositoryMethod] alloc] initWithMethod:repositoryMethod root:repositoryRoot]; 

        [result autorelease];
    }

    return result;
}

+ (CvsRepository *)nullRepository
{
    if (!nullRepository) {
        nullRepository=[[self alloc] initNullRepository];
    }

    return nullRepository;
}

+ (CvsRepository *) defaultRepository;
{
    return [self repositoryWithRoot:[[NSUserDefaults standardUserDefaults] stringForKey:@"CVSRoot"]];
}


+ (NSString *)rootForProperties:(NSDictionary *)properties;
{
    NSString *repositoryRoot;

    repositoryRoot=[properties objectForKey:@"root"];
    if (repositoryRoot) {
        return repositoryRoot;
    } else {
        return nil;
    }
}

+ (CvsRepository *) repositoryWithProperties:(NSDictionary *)properties
{
    NSString *repositoryMethod;
    NSString *repositoryRoot=nil;
    CvsRepository *result=nil;
    
    repositoryMethod=[properties objectForKey:@"method"];
    if (!repositoryMethod) {
        repositoryRoot=[properties objectForKey:@"root"];
        if (repositoryRoot) {
            return [self repositoryWithRoot:repositoryRoot];
        }
        NSLog(@"Assertion failed: no method in properties for repository");
        return nil;
    } else {
        repositoryRoot=[[self repositoryClassForMethod:repositoryMethod] rootForProperties:properties];
    }
    
    if (repositoryRoot) {
        // Stephane: could we allow [NSString defaultCStringEncoding] ?
        if (![repositoryRoot canBeConvertedToEncoding:NSASCIIStringEncoding]) {
            NSLog(@"Assertion failed: repository root not ascii");
            return nil;
        }
        result=[[self repositories] objectForKey:repositoryRoot];
        if (!result) {
            result=[[[[self repositoryClassForMethod:repositoryMethod] alloc] initWithProperties:properties] autorelease];
        }
    }

    return result;
}

+ (Class)repositoryClassForMethod:(NSString *)methodName
{
    NSString *className;
    Class repositoryClass;

    className=[repositoryClasses objectForKey:methodName];

    if (className) {
        if (repositoryClass=NSClassFromString(className)) {  // Look for a class serving our needs
            return repositoryClass;
        }
    }
    return [CvsRepository class];
}

+ (NSString*) cvsRootWorkAreasPath
{
    return [[[NSApp delegate] resourceDirectory] stringByAppendingPathComponent:@"Repositories"];
} // resourceDirectory

+ (NSDictionary *)repositories
{
    if (!repositories) {
        NSArray *repositoriesDirContents;
        NSFileManager *fileManager=[NSFileManager defaultManager];
        id enumerator;
        NSString *aWorkAreaPath;
        NSString *cvsRootWorkAreasPath;
        CvsRepository *loadedRepository;
        
        repositories=[[NSMutableDictionary alloc] init];
        cvsRootWorkAreasPath=[self cvsRootWorkAreasPath];

        repositoriesDirContents=[fileManager directoryContentsAtPath:cvsRootWorkAreasPath];
        enumerator=[repositoriesDirContents objectEnumerator];
        while (aWorkAreaPath=[enumerator nextObject]) {
            aWorkAreaPath=[[cvsRootWorkAreasPath stringByAppendingPathComponent:aWorkAreaPath] stringByAppendingPathComponent:@"CVSROOT"];

            loadedRepository=[self repositoryWithWorkAreaAtPath:aWorkAreaPath];
            if (loadedRepository) {
                [self registerRepository:loadedRepository];
            } else {
                [fileManager removeFileAtPath:[aWorkAreaPath stringByDeletingLastPathComponent] handler:nil];
            }
        }
//        [self parseCvsPasswordFile];
    }
    return repositories;
}

+ (void) registerRepository:(CvsRepository *)repository
{
    if (repositoriesToRemove && [repositoriesToRemove containsObject:repository])
        [repositoriesToRemove removeObject:repository];

    if (repository!=nullRepository) {
        // (Stephane) Shouldn't we update the repository ASAP?
//        (void)[repository isUpToDate]; // Forces update of repository
        [repositories setObject:repository forKey:[repository root]];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"RegisteredRepositoriesChanged" object:self];
    }
}

+ (CvsRepository *)repositoryForPath:(NSString *)path
{
    NSString *filePath;
    NSString *cvsRoot;
    CvsRepository *result;

    path=[path stringByStandardizingPath];
    
    filePath=[[path stringByAppendingPathComponent:@"CVS"] stringByAppendingPathComponent:@"Root"];
    cvsRoot= [self rootAtPath: filePath];

    result=[CvsRepository repositoryWithRoot:cvsRoot];
    [result registerDir:path];

    return result;
}

- (id) init
{
    if(self = [super init]){
        ignoredPatternDict = [[NSMutableDictionary allocWithZone:[self zone]] initWithCapacity:50];
        wrapperPatternDict = [[NSMutableDictionary allocWithZone:[self zone]] initWithCapacity:20];
        dirsControlled = [[NSMutableSet allocWithZone:[self zone]] initWithCapacity:50];
    }
    
    return self;
}

- (void) dealloc
{
    [baseWrapperWildCardsArray release];
    [baseIgnoreWildCardsArray release];
    [ignoredPatternDict release];
    [wrapperPatternDict release];
    [checkoutRequest release];
    [workAreaPath release];
    [root release];
    [dirsControlled release];
    [modules release];
    [environment release];
    [method release];

    [super dealloc];
}

- (CvsRepository *)initNullRepository
{
    if (self=[self init]) {
        upToDate=YES;
        root=nil;
        workAreaPath=nil;
    }
    return self;
}

- (NSString *) environmentKey
{
    return [@"Environment-" stringByAppendingString:[self root]];
}

- initWithMethod:(NSString *)theMethod root:(NSString *)repositoryRoot
{
    if (self=[self init]) {
        NSDictionary	*aDict;
        NSMutableDictionary *mutableDict = [[[NSProcessInfo processInfo] environment] mutableCopy];
        
        method=[theMethod retain];
        upToDate=NO;
        root=[repositoryRoot retain];
        environment = [[NSMutableDictionary allocWithZone:[self zone]] init];
        aDict = [[NSUserDefaults standardUserDefaults] dictionaryForKey:[self environmentKey]];
        if(aDict)
            [mutableDict addEntriesFromDictionary:aDict];
        [self setEnvironment:mutableDict];
        [mutableDict release];
    }
    return self;
}


+ (NSString*) rootAtPath: (NSString*) aPath
{
    NSString*  currentRoot= [NSString stringWithContentsOfFile:aPath];

#ifdef WIN32
    if (currentRoot && [currentRoot length]>1) {
        currentRoot=[currentRoot substringToIndex:[currentRoot length]-2];
    }
#else
    if (currentRoot && [currentRoot length]>0) {
        currentRoot=[currentRoot substringToIndex:[currentRoot length]-1];
    }
#endif

    return currentRoot;
}


- initWithProperties:(NSDictionary *)dictionary
{
    NSString *repositoryMethod;
    NSString *repositoryRoot;

    repositoryMethod=[dictionary objectForKey:@"method"];
    repositoryRoot=[dictionary objectForKey:@"root"];

    return [self initWithMethod:repositoryMethod root:repositoryRoot];
}

- (NSDictionary *)properties
{
    return [NSDictionary dictionaryWithObjectsAndKeys:method,@"method",root,@"root",nil];
}

+ (CvsRepository *)repositoryWithWorkAreaAtPath:(NSString *)aPath
{
    CvsRepository *result;
    NSFileManager *fileManager=[NSFileManager defaultManager];
    NSString *pathOfRootFile;
    NSString *repositoryRoot= nil;

    pathOfRootFile=[[aPath stringByAppendingPathComponent:@"CVS"] stringByAppendingPathComponent:@"Root"];

    if ([fileManager fileExistsAtPath:pathOfRootFile]) {
        repositoryRoot= [[self class] rootAtPath: pathOfRootFile];
    }

    result=[self repositoryWithRoot:repositoryRoot];
    if (result) {
        [result setWorkAreaPath:aPath];
    }

    return result;
}

- (BOOL)isLocal
{
    return NO;
}

- (BOOL)needsLogin
{
    return NO;
}

- (void) invalidateDir:(NSString *)dirPath
{
    [ignoredPatternDict removeObjectForKey:dirPath];
    [wrapperPatternDict removeObjectForKey:dirPath];
}

- (void) registerDir:(NSString *)dirPath
{
    if (dirPath)
        [dirsControlled addObject:dirPath];
}

- (void)setWorkAreaPath:(NSString *)thePath
{
    [thePath retain];
    [workAreaPath release];
    workAreaPath=thePath;
}

- (NSString *)workAreaPath
{
    if (!workAreaPath) {
        NSString *checkoutDir;

        checkoutDir=[[[self class] cvsRootWorkAreasPath] stringByAppendingPathComponent:[[root lastPathComponent] stringByAppendingString:[[NSProcessInfo processInfo] globallyUniqueString]]];
        workAreaPath=[[checkoutDir stringByAppendingPathComponent:@"CVSROOT"] retain];
    }

    return workAreaPath;
}

- (NSArray *)modulesSymbolicNames
{
    if ([self isUpToDate]) {
        NSEnumerator	*anEnum = [[modules sortedArrayUsingSelector:@selector(compareSymbolicName:)] objectEnumerator];
        CvsModule		*aModule;
        NSMutableArray	*names = [NSMutableArray arrayWithCapacity:[modules count]];

        while(aModule = [anEnum nextObject])
            [names addObject:[aModule symbolicName]];
        return names;
    } else {
        return nil;
    }
}

- (NSArray *) modules
{
    if([self isUpToDate])
        return modules;
    else
        return nil;
}

- (CvsModule *) moduleWithSymbolicName:(NSString *)aName
{
    NSEnumerator	*anEnum = [[self modules] objectEnumerator];
    CvsModule		*aModule;

    while(aModule = [anEnum nextObject])
        if([[aModule symbolicName] isEqualToString:aName])
            return aModule;
    return nil;
}

- (void) appendShellPatternsFromString:(NSString *)string toArray:(NSMutableArray *)patterns
{
    NSArray		*lines = [string lines];
    int			i = 0, max = [lines count];
    NSString	*aLine;

    for(; i < max; i++){
        NSArray		*words;
        int			j = 0, wordMax;
        NSString	*aWord;

        aLine = [lines objectAtIndex:i];
        words = [aLine componentsSeparatedByString:@" "];
        wordMax = [words count];
        for(; j < wordMax; j++){
            aWord = [words objectAtIndex:j];
            if(![aWord isEqualToString:@""])
                if([aWord isEqualToString:@"!"])
                    [patterns removeAllObjects];
                else
                    [patterns addObject:aWord];
        }
    }
}

- (NSArray *) baseIgnoreWildCardsArray
{
    if(baseIgnoreWildCardsArray == nil){
        // These wildcards come from src/ignore.c, variable ign_default
        NSString	*repositoryIgnoreString = [[NSString alloc] initWithContentsOfFile:[[self workAreaPath] stringByAppendingPathComponent:@"cvsignore"]];
        NSString	*homeIgnoreString = [[NSString alloc] initWithContentsOfFile:[[self homeDirectory] stringByAppendingPathComponent:@".cvsignore"]];
        NSString	*environmentIgnoreString = [[self environment] objectForKey:@"CVSIGNORE"];

        baseIgnoreWildCardsArray = [[NSMutableArray allocWithZone:[self zone]] initWithObjects:@"RCS", @"SCCS", @"CVS", @"CVS.adm", @"RCSLOG", @"cvslog.*", @"tags", @"TAGS", @".make.state", @".nse_depinfo", @"*~", @"#*", @".#*", @",*", @"_$*", @"*$", @"*.old", @"*.bak", @"*.BAK", @"*.orig", @"*.rej", @".del-*", @"*.a", @"*.olb", @"*.o", @"*.obj", @"*.so", @"*.exe", @"*.Z", @"*.elc", @"*.ln", @"core", nil];
        [self appendShellPatternsFromString:repositoryIgnoreString toArray:(NSMutableArray *)baseIgnoreWildCardsArray];
        [self appendShellPatternsFromString:homeIgnoreString toArray:(NSMutableArray *)baseIgnoreWildCardsArray];
        [self appendShellPatternsFromString:environmentIgnoreString toArray:(NSMutableArray *)baseIgnoreWildCardsArray];

        [repositoryIgnoreString release];
        [homeIgnoreString release];
    }

    return baseIgnoreWildCardsArray;
}

- (NSArray *) baseWrapperWildCardsArray
{
    if(baseWrapperWildCardsArray == nil){
        NSString	*repositoryWrapperString = [[NSString alloc] initWithContentsOfFile:[[self workAreaPath] stringByAppendingPathComponent:@"cvswrappers"]];
        NSString	*homeWrapperString = [[NSString alloc] initWithContentsOfFile:[[self homeDirectory] stringByAppendingPathComponent:@".cvswrappers"]];
        NSString	*clientEnvironmentWrapperString = [[self environment] objectForKey:@"CVS_CLIENT_WRAPPER_FILE"]; // Dunno whether it's necessary... See src/wrapper.c
        NSString	*environmentWrapperString = [[self environment] objectForKey:@"CVSWRAPPERS"];

        baseWrapperWildCardsArray = [[NSMutableArray allocWithZone:[self zone]] init];
        [self appendShellPatternsFromString:repositoryWrapperString toArray:(NSMutableArray *)baseWrapperWildCardsArray];
        [self appendShellPatternsFromString:homeWrapperString toArray:(NSMutableArray *)baseWrapperWildCardsArray];
        [self appendShellPatternsFromString:clientEnvironmentWrapperString toArray:(NSMutableArray *)baseWrapperWildCardsArray];
        [self appendShellPatternsFromString:environmentWrapperString toArray:(NSMutableArray *)baseWrapperWildCardsArray];

        [repositoryWrapperString release];
        [homeWrapperString release];
    }
    
    return baseWrapperWildCardsArray;
}

- (void) setModulesFromFile:(NSString *)aPath
{
    NSMutableArray	*newModule = [[CvsModule modulesWithContentsOfFile:aPath forRepository:self] retain];
    
    [modules release];
    modules = newModule;
}

- (BOOL) isWrapper:(NSString *)aPath
{
    if(!root)
        return NO;
    else{
        NSString	*currentDirPath = [aPath stringByDeletingLastPathComponent];
        NSArray		*patterns = [[wrapperPatternDict objectForKey:currentDirPath] retain];
        BOOL		result;
        
        if(!patterns){
            // Not build yet, build it now!
            NSString	*currentDirWrapperString = [[NSString alloc] initWithContentsOfFile:[currentDirPath stringByAppendingPathComponent:@".cvswrappers"]];

            patterns = [[NSMutableArray alloc] initWithArray:[self baseWrapperWildCardsArray]];
            [self appendShellPatternsFromString:currentDirWrapperString toArray:(NSMutableArray *)patterns];
            [wrapperPatternDict setObject:patterns forKey:currentDirPath];
            [currentDirWrapperString release];
        }
        result = [[aPath lastPathComponent] cvlFilenameMatchesShellPatterns:patterns];
        [patterns release];

        return result;
    }
} // isWrapper:


- (BOOL) isIgnored:(NSString*) aPath
{
    if(!root)
        return NO;
    else{
        NSString	*currentDirPath = [aPath stringByDeletingLastPathComponent];
        NSArray		*patterns = [[ignoredPatternDict objectForKey:currentDirPath] retain];
        BOOL		result;

        if(!patterns){
            // Not build yet, build it now!
            NSString	*currentDirIgnoreString = [[NSString alloc] initWithContentsOfFile:[currentDirPath stringByAppendingPathComponent:@".cvsignore"]];

            patterns = [[NSMutableArray alloc] initWithArray:[self baseIgnoreWildCardsArray]];
            [self appendShellPatternsFromString:currentDirIgnoreString toArray:(NSMutableArray *)patterns];
            [ignoredPatternDict setObject:patterns forKey:currentDirPath];
            [currentDirIgnoreString release];
        }
        result = [[aPath lastPathComponent] cvlFilenameMatchesShellPatterns:patterns];
        [patterns release];

        return result;
    }
}

- (NSString *)root
{
    return root;
}

- (BOOL) isUpdating
{
    return isUpdating;
}

- (BOOL) isUpToDate_WithoutRefresh
// Does NOT force to be up-to-date; simply returns current status
{
    return upToDate;
}

- (void) doCheckout
{
    if(isUpdating)
        return;
    else{
        NSString		*checkoutDir;
        NSFileManager	*fileManager = [NSFileManager defaultManager];

        checkoutDir = [[self workAreaPath] stringByDeletingLastPathComponent];
        if([fileManager createAllDirectoriesAtPath:checkoutDir attributes:nil])
            (void)[self checkoutRequest];
        else
            (void)NSRunAlertPanel(@"CVL", @"CVL needs to create directory %@ and is unable to do it. No repository can be opened!", nil, nil, nil, checkoutDir);
    }
}

- (BOOL)isUpToDate
{
    if(isUpdating)
        return NO;
    if(upToDate)
        return YES;
    [self doCheckout];
    return NO;
}

- (CvsCheckoutRequest *)checkoutRequest
{
    if (upToDate) {
        return nil;
    } else {
        if (!checkoutRequest) {
            isUpdating = YES;
            [[self class] registerRepository:self];
            checkoutRequest=[CvsCheckoutRequest cvsCheckoutRequestForModule:@"CVSROOT" inRepository:self toPath:[[self workAreaPath] stringByDeletingLastPathComponent]];
            [[NSNotificationCenter defaultCenter]
                          addObserver:self
                             selector:@selector(checkoutEnded:)
                                 name:@"RequestCompleted"
                               object:checkoutRequest];
            [checkoutRequest setIsQuiet: YES];
            [checkoutRequest schedule];
            [checkoutRequest retain];
        }
        return checkoutRequest;
    }
}

- (void) invalidateCaches
{
    [baseIgnoreWildCardsArray release];
    baseIgnoreWildCardsArray = nil;
    [ignoredPatternDict removeAllObjects];
    [baseWrapperWildCardsArray release];
    baseWrapperWildCardsArray = nil;
    [wrapperPatternDict removeAllObjects];
}

- (void) checkoutEnded:(NSNotification *)notification
{
    if ([notification object]==checkoutRequest) {
        isUpdating = NO;
        [[NSNotificationCenter defaultCenter]
                      removeObserver:self
                             name:@"RequestCompleted"
                           object:checkoutRequest];
        [checkoutRequest release];
        checkoutRequest=nil;
    }
    if ([[notification object] succeeded]) {
        cvsRootCheckoutFailed = NO;
        [self setModulesFromFile:[[self workAreaPath] stringByAppendingPathComponent:@"modules"]];
        [self invalidateCaches];
        upToDate=YES;
        [self invalidateControlledDirs];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"RepositoryChanged" object:self];
    }
    else{
        // In case of login on a pserver/remote repository, CVSROOT may be impossible to get.
        // In this case, we can fake to have a CVSROOT with nearly nothing in it.
        // This is a temporary patch.
        NSFileManager	*fileManager = [NSFileManager defaultManager];
        NSString		*aPath = [[self workAreaPath] stringByAppendingPathComponent:@"cvswrappers"];

        cvsRootCheckoutFailed = YES;
        if(![fileManager fileExistsAtPath:aPath])
#ifdef WIN32
            [@"# This is a dummy cvswrappers file, as CVL couldn't get one from the repository\r\n" writeToFile:aPath atomically:NO];
#else
            [@"# This is a dummy cvswrappers file, as CVL couldn't get one from the repository\n" writeToFile:aPath atomically:NO];
#endif
        aPath = [[self workAreaPath] stringByAppendingPathComponent:@"modules"];
        if(![fileManager fileExistsAtPath:aPath])
#ifdef WIN32
            [@"# This is a dummy modules file, as CVL couldn't get one from the repository\r\n" writeToFile:aPath atomically:NO];
#else
            [@"# This is a dummy modules file, as CVL couldn't get one from the repository\n" writeToFile:aPath atomically:NO];
#endif
        aPath = [[self workAreaPath] stringByAppendingPathComponent:@"cvsignore"];
        if(![fileManager fileExistsAtPath:aPath])
            // Do NOT write comments in this file, because it is NOT allowed by cvs!!!
#ifdef WIN32
            [@"\r\n" writeToFile:aPath atomically:NO];
#else
            [@"\n" writeToFile:aPath atomically:NO];
#endif

        [self setModulesFromFile:[[self workAreaPath] stringByAppendingPathComponent:@"modules"]];
        [self invalidateCaches];
        upToDate=YES;
        [self invalidateControlledDirs];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"RepositoryChanged" object:self];
    }
}

- (BOOL) cvsRootCheckoutFailed
{
    return cvsRootCheckoutFailed;
}

- (void)invalidateControlledDirs
{
    id enumerator;
    NSString *aPath;
    CVLFile *file;
    ResultsRepository *resultsReopository=[ResultsRepository sharedResultsRepository];
    
    enumerator=[dirsControlled objectEnumerator];
    [resultsReopository startUpdate];

    while (aPath=[enumerator nextObject]) {
        file=[CVLFile treeAtPath:aPath];
        [file invalidateRepository];
    }

    [resultsReopository endUpdate];
}

- (BOOL)isUsed
{
    return ([dirsControlled count]>0);
}

- (void)cleanUp
{
    NSFileManager *fileManager=[NSFileManager defaultManager];

    [fileManager removeFileAtPath:[[self workAreaPath] stringByDeletingLastPathComponent] handler:nil];
    [[NSUserDefaults standardUserDefaults] removeObjectForKey:[self environmentKey]];
}

- (BOOL)isReadyForRequests
{
    return YES;
}

- (Request *)gettingReadyRequest
{
    return nil;
}

+ (void)applicationWillTerminate:(NSNotification *)notification
{
    id enumerator;
    CvsRepository *repository;

    enumerator=[repositoriesToRemove objectEnumerator];
    while (repository=[enumerator nextObject]) {
        [repository cleanUp];
        [repositories removeObjectForKey:[repository root]];
    }
}

+ (BOOL)disposeRepository:(CvsRepository *)repository
{
    if ([repository isUsed]) {
        if (!repositoriesToRemove) {
            repositoriesToRemove=[[NSMutableSet alloc] init];
        }
        [repositoriesToRemove addObject:repository];
        return NO;
    } else {
        [repository cleanUp]; 
        [repositories removeObjectForKey:[repository root]];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"RegisteredRepositoriesChanged" object:self];
        return YES;
    }
}

+ (BOOL) isRepositoryToBeDisposed:(CvsRepository *)repository
{
    if(repositoriesToRemove)
        return [repositoriesToRemove containsObject:repository];
    return NO;
}

- (CvsCheckoutRequest *) checkoutAgain
{
	if(isUpdating)
        return checkoutRequest;
    if(!upToDate)
        (void)[self checkoutRequest];
    else{
        if(!checkoutRequest){
            isUpdating = YES;
            checkoutRequest = [CvsCheckoutRequest cvsCheckoutRequestForModule:@"CVSROOT" inRepository:self toPath:[[self workAreaPath] stringByDeletingLastPathComponent]];
            [[NSNotificationCenter defaultCenter]
                              addObserver:self
                                 selector:@selector(checkoutEnded:)
                                     name:@"RequestCompleted"
                                   object:checkoutRequest];
            [checkoutRequest setIsQuiet:YES];
            [checkoutRequest schedule];
            [checkoutRequest retain];
        }
    }
    return checkoutRequest;
}

- (NSDictionary *) environment
{
    return environment;
}

- (void) setEnvironment:(NSDictionary *)aDict
{
    NSMutableDictionary	*customEnv = [NSMutableDictionary dictionary];
    NSEnumerator		*anEnum = [aDict keyEnumerator];
    NSString			*aKey;

    while(aKey = [anEnum nextObject])
        if(![self isInheritedEnvironmentKey:aKey value:[aDict objectForKey:aKey]])
            [customEnv setObject:[aDict objectForKey:aKey] forKey:aKey];
    
    [environment setDictionary:aDict];
    if(![environment objectForKey:@"HOME"])
        // On Windows we absolutely need that variable!
        [environment setObject:NSHomeDirectory() forKey:@"HOME"];
    if([customEnv count])
        [[NSUserDefaults standardUserDefaults] setObject:customEnv forKey:[self environmentKey]];
    else
        [[NSUserDefaults standardUserDefaults] removeObjectForKey:[self environmentKey]];
}

- (BOOL) isInheritedEnvironmentKey:(NSString *)aKey value:(NSString *)aValue
{
    return [[[[NSProcessInfo processInfo] environment] objectForKey:aKey] isEqualToString:aValue];
}

- (NSString *) homeDirectory
{
    return [[self environment] objectForKey:@"HOME"];
}
@end
