// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "RepositoryViewer.h"
#import "RepositoryDataSource.h"
#ifdef MACOSX
#import <CvsImportRequest.h>
#import <CvsCheckoutRequest.h>
#import <CvsCommitRequest.h>
#import <SenPanelFactory.h>
#import <SenFormPanelController.h>
#import <NSView.SenReplacing.h>
#import <CvsInitRequest.h>
#import <CvsModule.h>
#import <SelectorRequest.h>
#else
#import <Requests/CvsImportRequest.h>
#import <Requests/CvsCheckoutRequest.h>
#import <Requests/CvsCommitRequest.h>
#import <SenPanels/SenPanelFactory.h>
#import <SenPanels/SenFormPanelController.h>
#import <SenInterface/NSView.SenReplacing.h>
#import <Requests/CvsInitRequest.h>
#import <Modules/CvsModule.h>
#import <Requests/SelectorRequest.h>
#endif
#import "../CVLConsole.subproj/CVLConsoleController.h"
#import <AppKit/AppKit.h>

static RepositoryViewer * sharedRepositoryViewer = nil;

// The FRAME_NAME has been modified, because the min size of the new window is greater the the min size of the old one
// So, to avoid displaying mess, we rename the FRAME_NAME.
#define FRAME_NAME	@"RepositoryViewerNEW"

@interface NSObject(CVLAppDelegate)
- (void) openViewerForPath:(NSString *)aPath; // Target is CVLDelegate
@end

@interface RepositoryViewer(Private)
- (void) selectionChanged:(NSNotification *)notification;
- (void) initializeTabView;
- (void) reloadTabViewItemView;
- (void) selectRepositoryAtIndex:(int)anIndex;
- (void) selectRepository:(CvsRepository *)aRepository;
@end


@implementation RepositoryViewer

+ (RepositoryViewer *) sharedRepositoryViewer
{
    if(!sharedRepositoryViewer)
        sharedRepositoryViewer = [[self alloc] init];

    return sharedRepositoryViewer;
}

- (id) init
{
    if(self = [self initWithWindowNibName:@"RepositoryViewer"]){
#ifdef MACOSX
        [self setWindowFrameAutosaveName:FRAME_NAME];
#endif
        [[NSNotificationCenter defaultCenter]
                        addObserver:self
                           selector:@selector(reloadTable:)
                               name:@"RegisteredRepositoriesChanged"
                             object:nil];
        [[NSNotificationCenter defaultCenter]
                        addObserver:self
                           selector:@selector(reloadTable:)
                               name:@"RepositoryChanged"
                             object:nil];
        dateFormatter = [[NSDateFormatter alloc] initWithDateFormat:[[NSUserDefaults standardUserDefaults] stringForKey:@"CVLDateFormat"] allowNaturalLanguage:YES];
    }
    return self;
}

- (void) windowDidLoad
{
    NSString	*cvsRoot;
    int			cvsRootRow = -1;

    [super windowDidLoad];

#ifndef MACOSX
    // Normally we shouldn't need to implement this; we could simply call [self setWindowFrameAutosaveName:@"ConsolePanel"] in init
    // BUT... if we do it, superclass' implementation does reposition window if it is not fully visible on screen,
    // which is VERY annoyable.
    if(![[self window] setFrameAutosaveName:FRAME_NAME])
        [[self window] setFrameUsingName:FRAME_NAME];
#endif
    
    [tabView retain];
    [noRepositoryView retain];
    [tabViewItemView retain];
    [tabViewEnvItemView retain];
    [self initializeTabView];

    [[NSNotificationCenter defaultCenter]
                    addObserver:self
                       selector:@selector(selectionChanged:)
                           name:NSTableViewSelectionDidChangeNotification
                         object:tableView];
    [[NSNotificationCenter defaultCenter]
                    addObserver:self
                       selector:@selector(envVariableSelectionChanged:)
                           name:NSTableViewSelectionDidChangeNotification
                         object:envTableView];
    if(cvsRoot = [[NSUserDefaults standardUserDefaults] stringForKey:@"CVSRoot"])
        cvsRootRow = [[tableView dataSource] rowOfObject:[CvsRepository repositoryWithRoot:cvsRoot]];
	if(cvsRootRow != -1)
        [self selectRepositoryAtIndex:cvsRootRow];
    else
        [self selectionChanged:nil]; // We want to update the tabView if no default cvsRoot thus no repository

    [[self window] setMiniwindowImage:[NSImage imageNamed:@"appicon"]];
}

- (void) dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [tabView release];
    [tabViewItemView release];
    [tabViewEnvItemView release];
    [noRepositoryView release];
    [[self window] setDelegate:nil];
    [dateFormatter release];

    [super dealloc];
}

- (void) reloadTable:(NSNotification *)notification
{
    [tableView reloadData];
}

- (CvsRepository *) selectedRepository
{
    if([tableView selectedRow] != -1)
        return (CvsRepository *)[[tableView dataSource] objectAtRow:[tableView selectedRow]];
    else
        return nil;
}

- (void) selectionChanged:(NSNotification *)notification
{
    CvsRepository	*repository = [self selectedRepository];
    BOOL			hasValidSelectedRepository = (repository != nil) && [repository isUpToDate_WithoutRefresh];

    if(hasValidSelectedRepository){
        [[NSUserDefaults standardUserDefaults] setObject:[repository root] forKey:@"CVSRoot"];
        if([tabView window] != [self window])
            [tabView senReplaceView:noRepositoryView];
        [self reloadTabViewItemView];
    }
    else if([repository isUpdating]){
        if([tabView window] != [self window])
            [tabView senReplaceView:noRepositoryView];
        [self reloadTabViewItemView];
    }
    else if([noRepositoryView window] != [self window])
            [noRepositoryView senReplaceView:tabView];

    [removeButton setEnabled:hasValidSelectedRepository];
    [importButton setEnabled:hasValidSelectedRepository];
    [checkoutButton setEnabled:hasValidSelectedRepository];
    [openWorkareaButton setEnabled:hasValidSelectedRepository && ![repository cvsRootCheckoutFailed]];
}

- (void) envVariableSelectionChanged:(NSNotification *)notification
{
    [removeEnvVariableButton setEnabled:([envTableView selectedRow] >= 0)];
}

- (void) selectRepository:(CvsRepository *)aRepository
{
    int	anIndex = [[tableView dataSource] rowOfObject:aRepository];

    [self selectRepositoryAtIndex:anIndex];
}

- (void) selectRepositoryAtIndex:(int)anIndex
{
    // Force selection of newly added repository
    anIndex = MIN([tableView numberOfRows] - 1, anIndex); // Do not go out of bounds; select last one if anIndex is to big
    if(anIndex >= 0){
        // Notif IS sent, but lazy loading cannot work, because delegate is not asked if it allows selection => ask explicitly
        if([[tableView delegate] tableView:tableView shouldSelectRow:anIndex]){
            // Bug 1000062: If selected index does not change, no notif will be sent => deselect then reselect every time
            [tableView deselectAll:nil];
            [tableView selectRow:anIndex byExtendingSelection:NO];
            [tableView scrollRowToVisible:anIndex];
        }
    }
    else
        [tableView deselectAll:nil];
}

- (IBAction) showWindow:(id)sender
{
    [super showWindow:sender];
    [[NSUserDefaults standardUserDefaults] setBool:YES forKey:@"ShowRepositoryViewer"];
}

- (IBAction) addRepository:(id)sender
{
    SenFormPanelController	*panel;
    CvsRepository			*repository;
    NSDictionary			*repositporyProperties;

    panel = [[SenPanelFactory sharedPanelFactory] handleOfPanelWithName:@"RepositoryChoice"];

    if([panel showAndRunModal] == NSOKButton){
        repositporyProperties = [panel dictionaryValue];
        repository = [CvsRepository repositoryWithProperties:repositporyProperties];
        if(repository){
            [CvsRepository registerRepository:repository];
            if([repository needsLogin]){
                Request		*loginRequest;
                NSString	*password;

                password = [repositporyProperties objectForKey:@"password"];
                if(password && ![password isEqual:@""])
                    loginRequest = [repository loginRequest];
            }
            [self selectRepository:repository];
        }
    }
}

- (IBAction) removeRepository:(id)sender
{
    CvsRepository	*repository;

    if(repository = [self selectedRepository]){
        int	aRow = [[tableView dataSource] rowOfObject:repository];
        
        if(![CvsRepository disposeRepository:repository])
#ifdef MACOSX
            NSBeginInformationalAlertSheet(@"Removing a repository", nil, nil, nil, [self window], nil, NULL, NULL, NULL, @"This repository is currently in use, it will be removed from your list of repositories when the application terminates.");
#else
            NSRunInformationalAlertPanel(@"Removing a repository", @"This repository is currently in use, it will be removed from your list of repositories when the application terminates.", nil, nil, nil);
#endif
        else
            [self selectRepositoryAtIndex:aRow];
    }
}

- (IBAction) repositoryImport:(id)sender
{
    [self importIntoRepository:[CvsRepository defaultRepository] importInfo:nil];
}

- (BOOL) runModalImport
{
    SenFormPanelController	*importController = [[SenPanelFactory sharedPanelFactory] handleOfPanelWithName:@"CvsImportPanel"];
    
    if([importController showAndRunModal] == NSOKButton){
        // We need to check that all fields are filled (and valid) before sending request,
        // else empty fields have side-effects!!!

        if(![importController objectValueForKey:@"module"] || [(NSString *)[importController objectValueForKey:@"module"] length] == 0 || ![importController objectValueForKey:@"sourcePath"] || [(NSString *)[importController objectValueForKey:@"sourcePath"] length] == 0 || ![importController objectValueForKey:@"releaseTag"] || [(NSString *)[importController objectValueForKey:@"releaseTag"] length] == 0 || ![importController objectValueForKey:@"vendorTag"] || [(NSString *)[importController objectValueForKey:@"vendorTag"] length] == 0 || ![importController objectValueForKey:@"message"] || [(NSString *)[importController objectValueForKey:@"message"] length] == 0){
#ifdef MACOSX
            NSBeginAlertSheet(@"Import", nil, nil, nil, [self window], self, NULL, @selector(importSheetDidDismiss:returnCode:contextInfo:), NULL, @"You need to give a valid path for the new repository, of the form computer:/path/to/repository or /path/to/repository.");
            return NO;
#else
            (void)NSRunAlertPanel(@"Import", @"You must fill all fields.", nil, nil, nil);
            return YES;
#endif
        }
        // release and vendor tags may not be the same
        if([[importController objectValueForKey:@"vendorTag"] isEqualToString:[importController objectValueForKey:@"releaseTag"]]){
#ifdef MACOSX
#else
            (void)NSRunAlertPanel(@"Import", @"Release and vendor tags may not be the same.", nil, nil, nil);
#endif
            return YES;
        }

        [[CvsImportRequest cvsImportRequestForModule:[importController objectValueForKey:@"module"]
                                        inRepository:[importController objectValueForKey:@"repository"]
                                          importPath:[importController objectValueForKey:@"sourcePath"]
                                          releaseTag:[importController objectValueForKey:@"releaseTag"]
                                           vendorTag:[importController objectValueForKey:@"vendorTag"]
                                             message:[importController objectValueForKey:@"message"]] schedule];
    }
    return NO;
}

- (void) importSheetDidDismiss:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    (void)[self runModalImport];
}

- (void) importIntoRepository:(CvsRepository *)aRepository importInfo:(NSDictionary *)importInfo
{
    SenFormPanelController	*importController = [[SenPanelFactory sharedPanelFactory] handleOfPanelWithName:@"CvsImportPanel"];
    NSEnumerator			*keyEnum = [importInfo keyEnumerator];
    id						aKey;

    while(aKey = [keyEnum nextObject])
        [importController setObjectValue:[importInfo objectForKey:aKey] forKey:aKey];
        
    [importController setObjectValue:aRepository forKey:@"repository"];
    do{
    }while([self runModalImport]);        
}

- (void) goForModuleCvsCheckoutInRepository:(CvsRepository *)aRepository
{
    CvsCheckoutRequest		*request;
    SenFormPanelController	*moduleController = [[SenPanelFactory sharedPanelFactory] handleOfPanelWithName:@"CvsCheckoutModulePanel"];

    checkoutDateTextField = [[[moduleController formController] controlForKey:@"date"] retain];
    freeFormSwitch = [[[moduleController formController] controlForKey:@"freeForm"] retain];
    [freeFormSwitch setTarget:self];
    [freeFormSwitch setAction:@selector(toggleFreeFormDate:)];
    [self toggleFreeFormDate:nil];
    [moduleController setObjectValue:aRepository forKey:@"repository"];
    if([moduleController showAndRunModal] == NSOKButton){
        NSString	*dateString;
        NSString	*directoryName;

        if([checkoutDateTextField objectValue]){
            if([freeFormSwitch state])
                dateString = [NSString stringWithString:[moduleController objectValueForKey:@"date"]];
            else
                // http://www.w3.org/TR/NOTE-datetime
                // ISO8601
                // e.g. YYYY-MM-DDThh:mm:ss.sTZD (eg 1997-07-16T19:20:30.45+01:00)
                dateString = [(NSCalendarDate *)[moduleController objectValueForKey:@"date"] descriptionWithCalendarFormat:@"%Y-%m-%dT%H:%M:%S%z"];
        }
        else
            dateString = nil;

        request = [CvsCheckoutRequest cvsCheckoutRequestForModule:[moduleController objectValueForKey:@"module"]
                                                     inRepository:aRepository
                                                           toPath:[moduleController objectValueForKey:@"workAreaPath"]
                                                         revision:[moduleController objectValueForKey:@"revision"]
                                                             date:dateString
                                          removesStickyAttributes:[[moduleController objectValueForKey:@"removesStickyAttributes"] boolValue]];
        directoryName = [moduleController objectValueForKey:@"directoryName"];
        if([directoryName length])
            [request setDestinationPath:directoryName];
        [[NSNotificationCenter defaultCenter]
                    addObserver:self
                       selector:@selector(requestCompleted:)
                           name:@"RequestCompleted"
                         object:request];
        [request schedule];
    }
    [checkoutDateTextField release]; checkoutDateTextField = nil;
    [freeFormSwitch release]; freeFormSwitch = nil;
}

- (IBAction) repositoryCheckout:(id)sender
{
    CvsRepository	*aRepository = [CvsRepository defaultRepository];
    
    if([aRepository isUpToDate])
        [self goForModuleCvsCheckoutInRepository:aRepository];
    else
        [[NSNotificationCenter defaultCenter]
                      addObserver:self
                         selector:@selector(repositoryUpdated:)
                             name:@"RequestCompleted"
                           object:[aRepository checkoutRequest]];
}

- (IBAction) openRepositoryWorkarea:(id)sender
{
    CvsRepository	*repository = [self selectedRepository];

    NSAssert(repository != nil, @"No repository is selected!");
    NSAssert([repository isUpToDate], @"Repository is not up-to-date!");

    [[NSApp delegate] openViewerForPath:[repository workAreaPath]]; // Message sent to CVLDelegate...
}

- (void) repositoryUpdated:(NSNotification *)notification
{
    CvsRequest	*request = [notification object];
    
    NSParameterAssert(notification != nil);
    [[NSNotificationCenter defaultCenter]
                  removeObserver:self
                            name:@"RequestCompleted"
                          object:request];

    if([request succeeded])
        [[NSRunLoop currentRunLoop] performSelector:@selector(goForModuleCvsCheckoutInRepository:)
                                             target:self
                                           argument:[request repository]
                                              order:0
                                              modes:[NSArray arrayWithObjects:NSDefaultRunLoopMode, NSModalPanelRunLoopMode, nil]];
}

- (void) requestCompleted:(NSNotification *)notification
{
    [[NSNotificationCenter defaultCenter] removeObserver:self name:[notification name] object:[notification object]];    
    [[NSApp delegate] requestCompleted:notification];
}

- (IBAction) newRepository:(id)sender
{
    SenFormPanelController	*newRepositoryPanel = [[SenPanelFactory sharedPanelFactory] handleOfPanelWithName:@"CvsNewRepositoryPanel"];

    if([newRepositoryPanel showAndRunModal] == NSOKButton){
        NSDictionary	*tagInfos = [newRepositoryPanel dictionaryValue];
        NSString		*path = [tagInfos objectForKey:@"path"];

        if(path && [path length] > 0){
            CvsInitRequest	*initRequest = [CvsInitRequest cvsInitRequestWithPath:path];

            [initRequest schedule];
            [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(initRequestCompleted:) name:@"RequestCompleted" object:initRequest];
        }
        else
#ifdef MACOSX
            NSBeginAlertSheet(@"Invalid Repository", nil, nil, nil, [self window], nil, NULL, NULL, NULL, @"You need to give a valid path for the new repository, of the form computer:/path/to/repository or /path/to/repository.");
#else
            (void)NSRunAlertPanel(@"New Repository", @"You need to give a valid path for the new repository, of the form computer:/path/to/repository or /path/to/repository.", nil, nil, nil);
#endif
    }
}

- (void) initRequestCompleted:(NSNotification *)aNotif
{
    CvsRepository	*repository;
    NSDictionary	*repositoryProperties;

    NSParameterAssert(aNotif != nil && [[aNotif object] isKindOfClass:[CvsInitRequest class]]);

    [[NSNotificationCenter defaultCenter] removeObserver:self name:[aNotif name] object:[aNotif object]];
    if(![[aNotif object] succeeded]){
#ifdef MACOSX
        NSBeginAlertSheet(@"Unable to create new repository", nil, nil, nil, [self window], nil, NULL, NULL, NULL, @"Unable to create repository at path %@.", [[aNotif object] newRepositoryPath]);
#else
        (void)NSRunAlertPanel(@"New Repository", @"Unable to create repository at path %@", nil, nil, nil, [[aNotif object] newRepositoryPath]);
#endif
        return;
    }
    
    repositoryProperties = [[aNotif object] result];
    repository = [CvsRepository repositoryWithProperties:repositoryProperties];
    if(repository){
        [CvsRepository registerRepository:repository];
        if([repository needsLogin]){
            Request		*loginRequest;
            NSString	*password;

            password = [repositoryProperties objectForKey:@"password"];
            if(password && ![password isEqual:@""])
                loginRequest = [repository loginRequest];
        }
        [self selectRepository:repository];
    }
}

- (BOOL) checkModuleDescription:(NSString *)moduleDescription forRepository:(CvsRepository *)aRepository
{
    NSArray	*errors = [CvsModule parseModuleDescription:moduleDescription forRepository:aRepository generatesModules:NO];

    if(errors && [errors count]){
        [[CVLConsoleController sharedConsoleController] outputError:[[errors componentsJoinedByString:@"\n"] stringByAppendingString:@"\n"]];
        (void)NSRunAlertPanel(@"File error", @"Modules file content is not correct. See Console log output.", nil, nil, nil);
        return NO;
    }
    return YES;
}

- (IBAction) validateRepositoryParams:(id)sender
{
    NSString		*aFileName = [[tabView selectedTabViewItem] identifier];
    CvsRepository	*repository = [self selectedRepository];

    NSAssert(repository != nil, @"No repository is selected!");
    NSAssert(aFileName != nil, @"Selected tab identifier is nil");

    if([aFileName isEqualToString:@"modules"]){
        if([self checkModuleDescription:[textView string] forRepository:repository])
#ifdef MACOSX
            NSBeginInformationalAlertSheet(@"File consistency", nil, nil, nil, [self window], nil, NULL, NULL, NULL, @"File %@ is consistent.", [[tabView selectedTabViewItem] identifier]);
#else
            (void)NSRunInformationalAlertPanel(@"File consistency", @"File %@ is consistent.", nil, nil, nil, [[tabView selectedTabViewItem] identifier]);
#endif
    }
    else
#ifdef MACOSX
        NSBeginInformationalAlertSheet(@"File consistency", nil, nil, nil, [self window], nil, NULL, NULL, NULL, @"Feature not yet implemented for file %@.", [[tabView selectedTabViewItem] identifier]);
#else
        (void)NSRunInformationalAlertPanel(@"File consistency", @"Feature not yet implemented for file %@.", nil, nil, nil, [[tabView selectedTabViewItem] identifier]);
#endif
}

- (IBAction) saveRepositoryParams:(id)sender
{
    NSString				*aName = [[tabView selectedTabViewItem] identifier];
    NSString				*aFileName;
	CvsCommitRequest		*aRequest;
	SelectorRequest			*reloadRequest;
    SenFormPanelController	*commitPanel;
    CvsRepository			*repository = [self selectedRepository];

    NSAssert(repository != nil, @"No repository is selected!");
    NSAssert(aName != nil, @"Selected tab identifier is nil");

    if([aName isEqualToString:@"modules"] && [[NSUserDefaults standardUserDefaults] boolForKey:@"ValidateModulesOnSave"])
        if(![self checkModuleDescription:[textView string] forRepository:repository])
            return;

    aFileName = [[repository workAreaPath] stringByAppendingPathComponent:aName];
    if(![[textView string] writeToFile:aFileName atomically:NO]){
#ifdef MACOSX
        NSBeginAlertSheet(@"Save", nil, nil, nil, [self window], nil, NULL, NULL, NULL, @"Unable to save file %@.", aFileName);
#else
        (void)NSRunAlertPanel(@"Save", @"Unable to save file %@.", nil, nil, nil, aFileName);
#endif
        return;
    }

    if([[NSUserDefaults standardUserDefaults] boolForKey:@"UseCvsTemplates"]){
        aRequest = [CvsCommitRequest cvsCommitRequestForFiles:[NSArray arrayWithObject:aName] inPath:[repository workAreaPath] message:nil];
        reloadRequest = [SelectorRequest requestWithTarget:self selector:@selector(reloadRepository:) argument:repository];
        [reloadRequest addPrecedingRequest:aRequest];
        [reloadRequest schedule];
    }
    else{
        commitPanel = [[SenPanelFactory sharedPanelFactory] handleOfPanelWithName:@"CvsCommitPanel"];
        [commitPanel setObjectValue:[NSString stringWithFormat:@"Commit %@ File", aName] forKey:@"PanelTitle"];

        if([commitPanel showAndRunModal] == NSOKButton){
            NSDictionary		*tagInfos = [commitPanel dictionaryValue];
            CvsCommitRequest	*aRequest;
            SelectorRequest		*reloadRequest;

            if(![tagInfos objectForKey:@"message"] || [(NSString *)[tagInfos objectForKey:@"message"] length] == 0){
#ifdef MACOSX
                NSBeginAlertSheet(@"Unable to commit", nil, nil, nil, [self window], nil, NULL, NULL, NULL, @"You must give a commit message.");
#else
                (void)NSRunAlertPanel(@"Commit", @"You must give a commit message.", nil, nil, nil);
#endif
                return;
            }

            aRequest = [CvsCommitRequest cvsCommitRequestForFiles:[NSArray arrayWithObject:aName] inPath:[repository workAreaPath] message:[tagInfos objectForKey:@"message"]];
            reloadRequest = [SelectorRequest requestWithTarget:self selector:@selector(reloadRepository:) argument:repository];
            [reloadRequest addPrecedingRequest:aRequest];
            [reloadRequest schedule];
        }
    }
}

- (void) reloadRepository:(CvsRepository *)aRepository
{
    [self reloadTabViewItemView];
    [aRepository checkoutAgain];
}

- (IBAction) revertRepositoryParams:(id)sender
{
    // Force reloading/parsing of ALL files; we need to DELETE file first, then do a checkout on it
    NSString		*aFileName = [[tabView selectedTabViewItem] identifier];
    CvsRepository	*repository = [self selectedRepository];
    CvsRequest		*aRequest;

    NSAssert(repository != nil, @"No repository is selected!");
    NSAssert(aFileName != nil, @"Selected tab identifier is nil");

    (void)[[NSFileManager defaultManager] removeFileAtPath:[[repository workAreaPath] stringByAppendingPathComponent:aFileName] handler:nil];

    aRequest = [[self selectedRepository] checkoutAgain];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(fileRequestCompleted:) name:@"RequestCompleted" object:aRequest];
    [self reloadTabViewItemView];
}

- (void) fileRequestCompleted:(NSNotification *)aNotif
{
    [self reloadTabViewItemView];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:[aNotif name] object:[aNotif object]];    
}

- (void) initializeTabView
{
    NSTabViewItem	*anItem;

    initializingTabs = YES;
    
    // Tab for <module> file
    anItem = [[NSTabViewItem alloc] initWithIdentifier:@"modules"];
    [anItem setLabel:@"Modules"];
    [tabView addTabViewItem:anItem];

    // Tab for <cvsignore> file
    anItem = [[NSTabViewItem alloc] initWithIdentifier:@"cvsignore"];
    [anItem setLabel:@"Ignored Files"];
    [tabView addTabViewItem:anItem];

    // Tab for <cvswrappers> file
    anItem = [[NSTabViewItem alloc] initWithIdentifier:@"cvswrappers"];
    [anItem setLabel:@"Wrappers"];
    [tabView addTabViewItem:anItem];

    // Tab for environment variables
    anItem = [[NSTabViewItem alloc] initWithIdentifier:@"environment"];
    [anItem setLabel:@"Environment"];
    [tabView addTabViewItem:anItem];

    initializingTabs = NO;
}

- (void) tabView:(NSTabView *)tabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem
{
    if([[tabViewItem identifier] isEqualToString:@"environment"]){
        [tabViewItem setView:tabViewEnvItemView];
        [tabViewItem setInitialFirstResponder:envTableView];
    }
    else{
        [tabViewItem setView:tabViewItemView];
        [tabViewItem setInitialFirstResponder:textView];
    }

    if(initializingTabs) // Notif is already sent when initialize tabs. Too early for us.
        return;

    [self reloadTabViewItemView];
}

- (void) reloadTabViewItemView
{
    CvsRepository	*repository = [self selectedRepository];
    NSString		*aName = [[tabView selectedTabViewItem] identifier];
    BOOL			fileAction = YES;

    NSAssert(repository != nil, @"Trying to reload tab view when no repository is selected!");
    NSAssert(aName != nil, @"[[tabView selectedTabViewItem] identifier] is nil!");

    if([aName isEqualToString:@"environment"]){
        [addEnvVariableButton setEnabled:YES];
        [removeEnvVariableButton setEnabled:([envTableView selectedRow] >= 0)];
        [envTableView reloadData];
    }
    else{
        if([repository isUpToDate]){
            NSString	*aFileName = [[repository workAreaPath] stringByAppendingPathComponent:aName];
            NSString	*fileContents = [NSString stringWithContentsOfFile:aFileName];

            if(fileContents)
                [textView setString:fileContents];
            else{
                [textView setString:@"N/A"];
                fileAction = NO;
            }
        }
        else{
            if([repository isUpdating])
                [textView setString:@"Information is being loaded. Please wait..."];
            else
                [textView setString:@"N/A"];
            fileAction = NO;
        }
        fileAction = fileAction && ![repository cvsRootCheckoutFailed];
        [textView setEditable:fileAction];
        [saveRepositoryParamsButton setEnabled:fileAction];
        [validateRepositoryParamsButton setEnabled:fileAction];
        [revertRepositoryParamsButton setEnabled:![repository isUpdating]]; // This button should always be enabled, even if file does not exist. Exception: CVSROOT is being updated.
    }
    [openWorkareaButton setEnabled:![repository cvsRootCheckoutFailed] && ![repository isUpdating]];
}

- (float) splitView:(NSSplitView *)sender constrainMinCoordinate:(float)proposedCoord ofSubviewAt:(int)offset
{
    return MAX(proposedCoord, 85);
}

- (float) splitView:(NSSplitView *)sender constrainMaxCoordinate:(float)proposedCoord ofSubviewAt:(int)offset
{
    return MIN(proposedCoord, NSHeight([sender frame]) - 80);
}

- (BOOL) splitView:(NSSplitView *)sender canCollapseSubview:(NSView *)subview
{
    if([[sender subviews] indexOfObject:subview] == 1)
        return YES; // Only subview containing details can be collapsed
    return NO;
}

- (BOOL) tableView:(NSTableView *)aTableView shouldSelectRow:(int)row
{
    if(aTableView == envTableView)
        return YES;
    else{
        // Force refresh (checkout) of repository, in a lazy way
        // This call forces the update (checkout) of the repository; as the result is obtained asynchronously,
        // we first receive NO, and later we are notified of the results
        CvsRepository	*repository = [[tableView dataSource] objectAtRow:row];

        if([CvsRepository isRepositoryToBeDisposed:repository])
            return YES;
        if(![repository isUpToDate])
            [[NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(repositoryIsUpdating:) userInfo:nil repeats:YES] retain];
    }
    return YES;
}

- (void) repositoryIsUpdating:(NSTimer *)aTimer
{
    CvsRepository	*selectedRepository = [self selectedRepository];
    
    if([CvsRepository isRepositoryToBeDisposed:selectedRepository] || [selectedRepository isUpToDate_WithoutRefresh]){
        [aTimer invalidate];
        [aTimer release];
        [self selectionChanged:nil];
    }
    else if(![selectedRepository isUpdating]){
        // Updating failed; warn user and remove repository from list
        int	aRow = [[tableView dataSource] rowOfObject:selectedRepository];

        [self selectRepositoryAtIndex:aRow];
        if(![CvsRepository disposeRepository:selectedRepository])
#ifdef MACOSX
            NSBeginInformationalAlertSheet(@"Invalid Repository", nil, nil, nil, [self window], nil, NULL, NULL, NULL, @"This repository is currently in use, but is invalid. It will be removed from your list of repositories when the application terminates.");
#else
            (void)NSRunInformationalAlertPanel(@"Invalid Repository", @"This repository is currently in use, but is invalid. It will be removed from your list of repositories when the application terminates.", nil, nil, nil);
#endif
        else
#ifdef MACOSX
            NSBeginInformationalAlertSheet(@"Invalid Repository", nil, nil, nil, [self window], nil, NULL, NULL, NULL, @"This repository is invalid. It is removed from your list of repositories.");
#else
            (void)NSRunInformationalAlertPanel(@"Invalid Repository", @"This repository is invalid. It is removed from your list of repositories.", nil, nil, nil);
#endif
    }
}

- (BOOL) validateMenuItem:(id <NSMenuItem>)menuItem
{
    if([menuItem action] == @selector(newRepository:) || [menuItem action] == @selector(addRepository:))
        return YES;
    if([self selectedRepository])
        return YES;
    else
        return NO;
}

- (IBAction) addEnvironmentVariable:(id)sender
{
    CvsRepository		*selectedRepository = [self selectedRepository];
    NSMutableDictionary	*newDict = [[selectedRepository environment] mutableCopy];

    [newDict setObject:@"" forKey:@""];
    [selectedRepository setEnvironment:newDict];
    [newDict release];
    [envTableView reloadData];
}

- (IBAction) removeEnvironmentVariable:(id)sender
{
    CvsRepository		*selectedRepository = [self selectedRepository];
    NSMutableDictionary	*newDict = [[selectedRepository environment] mutableCopy];
    NSEnumerator		*selectedRowEnumerator = [envTableView selectedRowEnumerator];
    NSNumber			*aRow;
    NSTableColumn		*tableColumn = [envTableView tableColumnWithIdentifier:@"key"];

    while(aRow = [selectedRowEnumerator nextObject])
        [newDict removeObjectForKey:[self tableView:envTableView objectValueForTableColumn:tableColumn row:[aRow intValue]]];        

    [selectedRepository setEnvironment:newDict];
    [newDict release];
    [envTableView reloadData];
}

- (int) numberOfRowsInTableView:(NSTableView *)tableView
{
    return [[[self selectedRepository] environment] count];
}

- (id) tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
    CvsRepository	*selectedRepository = [self selectedRepository];
    NSArray			*orderedKeys = [[[selectedRepository environment] allKeys] sortedArrayUsingSelector:@selector(compare:)];
    
    if([[tableColumn identifier] isEqualToString:@"key"])
        return [orderedKeys objectAtIndex:row];
    else
        return [[selectedRepository environment] objectForKey:[orderedKeys objectAtIndex:row]];
}

- (void) tableView:(NSTableView *)tableView setObjectValue:(id)object forTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
    CvsRepository		*selectedRepository = [self selectedRepository];
    NSArray				*orderedKeys = [[[selectedRepository environment] allKeys] sortedArrayUsingSelector:@selector(compare:)];
    NSMutableDictionary	*newDict = [[selectedRepository environment] mutableCopy];

    if([[tableColumn identifier] isEqualToString:@"key"]){
        if(![orderedKeys containsObject:object]){
            [newDict setObject:[newDict objectForKey:[orderedKeys objectAtIndex:row]] forKey:object];
            [newDict removeObjectForKey:[orderedKeys objectAtIndex:row]];
        }
    }
    else
        [newDict setObject:object forKey:[orderedKeys objectAtIndex:row]];
    [selectedRepository setEnvironment:newDict];
    [newDict release];
    [envTableView reloadData];
}

- (void) tableView:(NSTableView *)aTableView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
    NSString	*aKey, *aValue;
    BOOL		inherited;

    if(aTableView != envTableView)
        return;
    aKey = [[aTableView dataSource] tableView:aTableView objectValueForTableColumn:[envTableView tableColumnWithIdentifier:@"key"] row:row];
    aValue = [[aTableView dataSource] tableView:aTableView objectValueForTableColumn:[envTableView tableColumnWithIdentifier:@"value"] row:row];
    inherited = [[self selectedRepository] isInheritedEnvironmentKey:aKey value:aValue];
#ifdef MACOSX
    if(inherited)
      [cell setFont:[[NSFontManager sharedFontManager] convertFont:[NSFont fontWithName:@"Helvetica" size:[[cell font] pointSize]] toHaveTrait:NSItalicFontMask]];
    else
      [cell setFont:[[NSFontManager sharedFontManager] convertFont:[NSFont systemFontOfSize:[[cell font] pointSize]] toHaveTrait:NSItalicFontMask]];
#else
    [cell setFont:[[NSFontManager sharedFontManager] convertFont:[cell font] toHaveTrait:(inherited ? NSItalicFontMask:NSUnitalicFontMask)]];
#endif
}

- (IBAction) toggleFreeFormDate:(id)sender
{
    if([freeFormSwitch state]){
        [[checkoutDateTextField cell] setFormatter:nil];
        [checkoutDateTextField setToolTip:@"Enter date using any format cvs can understand"];
    }
    else{
        [[checkoutDateTextField cell] setFormatter:dateFormatter];
        [checkoutDateTextField setToolTip:[NSString stringWithFormat:@"Enter date like '%@'", [[NSCalendarDate calendarDate] descriptionWithCalendarFormat:[dateFormatter dateFormat]]]];
    }
}

@end

@implementation RepositoryViewer (WindowDelegate)

- (void) windowDidBecomeKey:(NSNotification *)notification
{
    [[NSUserDefaults standardUserDefaults] setBool:YES forKey:@"ShowRepositoryViewer"];
}

- (void) windowWillClose:(NSNotification *)notification
{
    if([[NSApplication sharedApplication] isRunning]) // On NT, method also is called when app is terminating!
        [[NSUserDefaults standardUserDefaults] setBool:NO forKey:@"ShowRepositoryViewer"];
}

@end
