/* CVLOpendiffRequest.m created by stephane on Mon 27-Sep-1999 */
/*$Id: CVLOpendiffRequest.m,v 1.7 2002/07/04 12:26:32 stephane Exp $*/

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "CVLOpendiffRequest.h"
#ifdef MACOSX
#import <CvsUpdateRequest.h>
#else
#import <Requests/CvsUpdateRequest.h>
#endif
#import "../CVLFile.h"
#import <Foundation/Foundation.h>


static NSString	*opendiffPath = nil;

// WARNING
// CvsUpdateRequest does not support updates of directories if we try to restore old versions
// So CVLOpendiffRequest must never be invoked for directories (or multiselection)

// (Stephane) Instead of using opendiff, we could perhaps use FileMerge Services:
// FileMerge/Compare Files      => two files as parameters?
// FileMerge/Compare To Master  => three files as parameters?
// How do we set the merge file?

@interface CVLOpendiffRequest(Private)
+ (void) preferencesChanged:(NSNotification *)notification;
- (id) initWithFile:(NSString *)fileFullPath andFile:(NSString *)fileFullPath2; // Generic initializer for <opendiff fileFullPath fileFullPath2>
- (id) initWithCVLFile:(CVLFile *)file; // Calls -initWithFile:andFile: and adds arguments -merge previousRevision
- (id) initWithCVLFileAndAncestor:(CVLFile *)file; // Calls -initWithFile:andFile: and adds arguments -ancestor previousRevision -merge file
#ifdef JA_PATCH
- (CvsUpdateRequest *)getAndObserveUpdateRequestForFile:(CVLFile *)aFile revision:(NSString *)revision date:(NSString *)dateString toFile:(NSString *)destinationPath;
#endif
@end

@implementation CVLOpendiffRequest

+ (void) initialize
{
    static BOOL	initialized = NO;
    
    [super initialize];
    if(!initialized){
        initialized = YES;
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(preferencesChanged:) name:@"PreferencesChanged" object:nil];
        [self preferencesChanged:nil];
    }
}

+ (void) preferencesChanged:(NSNotification *)notification
{
    [opendiffPath release];
    opendiffPath = [[[NSUserDefaults standardUserDefaults] objectForKey:@"OpendiffPath"] retain];
/*  
	// This test should be done in preferences panel...  
	if(opendiffPath){
        NSString	*aString = opendiffPath;

        [opendiffPath autorelease];
        opendiffPath = nil;
        NSAssert1([aString length] && [[NSFileManager defaultManager] isExecutableFileAtPath:aString], @"No executable for opendiff path %@", aString);
    	[aString retain];
    	opendiffPath = aString;
    }*/
}

+ (BOOL) opendiffIsValid
{
    return opendiffPath != nil;
}

+ (CVLOpendiffRequest *) opendiffRequestForFile:(NSString *)fileFullPath
{
    CVLFile		*file;
    ECStatus	currentECStatus;

    if(!opendiffPath)
        return nil;

    file = [CVLFile treeAtPath:fileFullPath];
    currentECStatus = [file status]; // Stephane: shouldn't we check that status is up-to-date?
    
    if([file isLeaf]){
        if(currentECStatus.statusType == ECConflictType)
            return [[[self alloc] initWithCVLFileAndAncestor:file] autorelease];
        else
            return [[[self alloc] initWithCVLFile:file] autorelease];
    }
    else{
        // <file> is a directory: currently, this doesn't work for directories...
        // Don't do anything...
        return nil;
    }
}

#ifdef JA_PATCH
- (id) initWithCVLFile:(CVLFile *)file //prv
{
    NSParameterAssert([file isLeaf]);
    if (self = [self initWithTitle:@"opendiff"]) {
        NSString	*repoVersion = [file revisionInRepository]; // prv we are not sure the version is available !!!!
        leftPath = [[NSTemporaryDirectory() stringByAppendingPathComponent:[file filenameForRevision:repoVersion]] retain];
        leftRequest=[[self getAndObserveUpdateRequestForFile:file revision:repoVersion date:nil toFile:leftPath] retain];
        rightPath=[[file path] retain];
        mergePath = [[file path] retain];
    }

    return self;
}

- (id) initWithCVLFileAndAncestor:(CVLFile *)file
{
    NSParameterAssert([file isLeaf]);
    if(self = [self initWithCVLFile:file]){
        NSString	*previousVersion = [file precedingVersion];// prv we are not sure the version is available !!!!
        ancestorPath = [[NSTemporaryDirectory() stringByAppendingPathComponent:[file filenameForRevision:previousVersion]] retain];
        leftPath = [[file fileAncestorForRevision:previousVersion] retain];
        ancestorRequest=[[self getAndObserveUpdateRequestForFile:file revision:previousVersion date:nil toFile:ancestorPath] retain];
        rightPath=[[file path] retain];
        mergePath = [[file path] retain];
    }

    return self;
}

- (CvsUpdateRequest *)getAndObserveUpdateRequestForFile:(CVLFile *)aFile revision:(NSString *)revision date:(NSString *)dateString toFile:(NSString *)destinationPath
{
    CvsUpdateRequest *theRequest;

theRequest=[CvsUpdateRequest cvsUpdateRequestForFile:[aFile name] inPath:[[aFile path] stringByDeletingLastPathComponent] revision:revision date:dateString toFile:destinationPath];
    if (theRequest) {
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(getVersionRequestCompleted:) name:@"RequestCompleted" object:theRequest];
    }
    return theRequest;
}

- (void)getVersionRequestCompleted:(NSNotification *)notification
{
    Request *aRequest=[notification object];

    [[NSNotificationCenter defaultCenter] removeObserver:self name:@"RequestCompleted" object:aRequest];
    if (aRequest==leftRequest) { [leftRequest release];leftRequest=nil; }
    if (aRequest==rightRequest) { [rightRequest release];rightRequest=nil; }
    if (aRequest==ancestorRequest) { [ancestorRequest release];ancestorRequest=nil; }
    if (![aRequest succeeded]) {
        allGetVersionRequestsSucceeded=NO; // should cancel the others
    }
    [self updateState];
}

- (void)start
{
    [leftRequest schedule];
    [rightRequest schedule];
    [ancestorRequest schedule];
    allGetVersionRequestsSucceeded=YES;
}

- (BOOL)allGetVersionRequestsEnded
{
    return (!leftRequest && !rightRequest && !ancestorRequest);
}

- (BOOL)readyForTask
{
    return [self allGetVersionRequestsEnded] && allGetVersionRequestsSucceeded;
}

- (BOOL)failedToGetReady
{
    return [self allGetVersionRequestsEnded] && !allGetVersionRequestsSucceeded;
}

- (id) initWithFile:(CVLFile *)file parameters:(NSDictionary *)parameterDictionary
{
    NSString		*leftVersion, *rightVersion, *ancestorVersion;
    NSString		*leftDatedVersion = nil, *rightDatedVersion = nil, *ancestorDatedVersion = nil;

    NSParameterAssert(file != nil);
    NSParameterAssert([file isLeaf]);

    leftVersion = [parameterDictionary objectForKey:@"LeftRevision"];
    if(!leftVersion){
        leftVersion = [parameterDictionary objectForKey:@"LeftTag"];
        if(!leftVersion){
            leftDatedVersion = [parameterDictionary objectForKey:@"LeftDate"];
            if(leftDatedVersion){
                leftPath = [[NSTemporaryDirectory() stringByAppendingPathComponent:[file filenameForDate:leftDatedVersion]] retain];
                leftRequest=[[self getAndObserveUpdateRequestForFile:file revision:nil date:leftDatedVersion toFile:leftPath] retain];
            }
        }
    }
    if(!leftPath)
        if(!leftVersion)
            leftPath = [file path];
        else{
            leftPath = [NSTemporaryDirectory() stringByAppendingPathComponent:[file filenameForRevision:leftVersion]];
            leftRequest=[[self getAndObserveUpdateRequestForFile:file revision:leftVersion date:nil toFile:leftPath] retain];
        }

    rightVersion = [parameterDictionary objectForKey:@"RightRevision"];
    if(!rightVersion){
        rightVersion = [parameterDictionary objectForKey:@"RightTag"];
        if(!rightVersion){
            rightDatedVersion = [parameterDictionary objectForKey:@"RightDate"];
            if(rightDatedVersion){
                rightPath = [NSTemporaryDirectory() stringByAppendingPathComponent:[file filenameForDate:rightDatedVersion]];
                rightRequest=[[self getAndObserveUpdateRequestForFile:file revision:nil date:rightDatedVersion toFile:rightPath] retain];
            }
        }
    }
    if(!rightPath)
        if(!rightVersion)
            rightPath = [file path];
        else{
            rightPath = [NSTemporaryDirectory() stringByAppendingPathComponent:[file filenameForRevision:rightVersion]];
            rightRequest=[[self getAndObserveUpdateRequestForFile:file revision:rightVersion date:nil toFile:rightPath] retain];
        }

    ancestorVersion = [parameterDictionary objectForKey:@"AncestorRevision"];
    if(!ancestorVersion){
        ancestorVersion = [parameterDictionary objectForKey:@"AncestorTag"];
        if(!ancestorVersion){
            ancestorDatedVersion = [parameterDictionary objectForKey:@"AncestorDate"];
            if(ancestorDatedVersion){
                ancestorPath = [NSTemporaryDirectory() stringByAppendingPathComponent:[file filenameForDate:ancestorDatedVersion]];
                ancestorRequest=[[self getAndObserveUpdateRequestForFile:file revision:nil date:ancestorDatedVersion toFile:ancestorPath] retain];
            }
        }
    }
    if(!ancestorPath && ancestorVersion){
        ancestorPath = [NSTemporaryDirectory() stringByAppendingPathComponent:[file filenameForRevision:ancestorVersion]];
        ancestorRequest=[[self getAndObserveUpdateRequestForFile:file revision:ancestorVersion date:nil toFile:ancestorPath] retain];
    }

    mergePath = [parameterDictionary objectForKey:@"MergeFile"];
    if(!mergePath)
        mergePath = [[file path] retain];

    return self;
}

- (BOOL)setUpTask
{
    if ([super setUpTask]) {
        [task setLaunchPath:opendiffPath];
        if (ancestorPath) {
            [task setArguments:[NSArray arrayWithObjects:leftPath, rightPath, @"-ancestor", ancestorPath, @"-merge", mergePath, nil]];
        } else {
            [task setArguments:[NSArray arrayWithObjects:leftPath, rightPath, @"-merge", mergePath, nil]];
        }
        return YES;
    }
    return NO;
}
#else
- (id) initWithTitle:(NSString *)cmdString
{
    if(!opendiffPath){
        [self dealloc];
        return nil;
    }

    if(self = [super initWithTitle:cmdString]){
        NSTask	*opendiffTask = [[NSTask alloc] init];

        [opendiffTask setLaunchPath:opendiffPath];
        [self setTask:opendiffTask];
        [opendiffTask release];
    }

    return self;
}

- (id) initWithFile:(NSString *)fileFullPath andFile:(NSString *)fileFullPath2
{
    if(self = [self initWithTitle:@"opendiff"])
        [task setArguments:[NSArray arrayWithObjects:fileFullPath, fileFullPath2, nil]];

    return self;
}

- (id) initWithCVLFile:(CVLFile *)file
{
#warning BUG: if file has no full status yet, revisionInRepository is nil => name will contain ending dot
    // We must be sure that we have a version number => we perhaps need to delay the request
    NSString	*repoVersion = [file revisionInRepository];
    NSString	*repoVersionFullPath = [NSTemporaryDirectory() stringByAppendingPathComponent:[file filenameForRevision:repoVersion]];

    NSParameterAssert([file isLeaf]);
    if(self = [self initWithFile:repoVersionFullPath andFile:[file path]]){
        CvsUpdateRequest	*repoVersionRequest = [CvsUpdateRequest cvsUpdateRequestForFile:[file name] inPath:[[file path] stringByDeletingLastPathComponent] revision:repoVersion date:nil toFile:repoVersionFullPath];

        [task setArguments:[[task arguments] arrayByAddingObjectsFromArray:[NSArray arrayWithObjects:@"-merge", [file path], nil]]];
        [self addPrecedingRequest:repoVersionRequest];
    }

    return self;
}

- (id) initWithCVLFileAndAncestor:(CVLFile *)file
{
    NSParameterAssert([file isLeaf]);
    if(self = [self initWithCVLFile:file]){
        NSString	*previousVersion = [file precedingVersion];
        NSString	*ancestorFullPath = [NSTemporaryDirectory() stringByAppendingPathComponent:[file filenameForRevision:previousVersion]];
        NSString	*unMergedFullPath = [file fileAncestorForRevision:previousVersion];
        NSArray		*args = [task arguments];
        CvsUpdateRequest	*ancestorRequest = [CvsUpdateRequest cvsUpdateRequestForFile:[file name] inPath:[[file path] stringByDeletingLastPathComponent] revision:previousVersion date:nil toFile:ancestorFullPath];

        [self addPrecedingRequest:ancestorRequest];
        // Make opendiff /tmp/file.repo .#file.previous -ancestor /tmp/file.previous -merge file
        [task setArguments:[NSArray arrayWithObjects:[args objectAtIndex:0], unMergedFullPath, @"-ancestor", ancestorFullPath, [args objectAtIndex:2], [args objectAtIndex:3], nil]];
    }

    return self;
}

- (id) initWithFile:(CVLFile *)file parameters:(NSDictionary *)parameterDictionary
{
    NSString		*leftVersion, *rightVersion, *ancestorVersion;
    NSString		*leftDatedVersion = nil, *rightDatedVersion = nil, *ancestorDatedVersion = nil;
    NSString		*leftVersionFullPath = nil, *rightVersionFullPath = nil, *ancestorVersionFullPath = nil;
    NSString		*mergePath;
    NSMutableArray	*requests = [NSMutableArray array];

    NSParameterAssert(file != nil);
    NSParameterAssert([file isLeaf]);

    leftVersion = [parameterDictionary objectForKey:@"LeftRevision"];
    if(!leftVersion){
        leftVersion = [parameterDictionary objectForKey:@"LeftTag"];
        if(!leftVersion){
            leftDatedVersion = [parameterDictionary objectForKey:@"LeftDate"];
            if(leftDatedVersion){
                leftVersionFullPath = [NSTemporaryDirectory() stringByAppendingPathComponent:[file filenameForDate:leftDatedVersion]];
                [requests addObject:[CvsUpdateRequest cvsUpdateRequestForFile:[file name] inPath:[[file path] stringByDeletingLastPathComponent] revision:nil date:leftDatedVersion toFile:leftVersionFullPath]];
            }
        }
    }
    if(!leftVersionFullPath)
        if(!leftVersion)
            leftVersionFullPath = [file path];
        else{
            leftVersionFullPath = [NSTemporaryDirectory() stringByAppendingPathComponent:[file filenameForRevision:leftVersion]];
            [requests addObject:[CvsUpdateRequest cvsUpdateRequestForFile:[file name] inPath:[[file path] stringByDeletingLastPathComponent] revision:leftVersion date:nil toFile:leftVersionFullPath]];
        }

    rightVersion = [parameterDictionary objectForKey:@"RightRevision"];
    if(!rightVersion){
        rightVersion = [parameterDictionary objectForKey:@"RightTag"];
        if(!rightVersion){
            rightDatedVersion = [parameterDictionary objectForKey:@"RightDate"];
            if(rightDatedVersion){
                rightVersionFullPath = [NSTemporaryDirectory() stringByAppendingPathComponent:[file filenameForDate:rightDatedVersion]];
                [requests addObject:[CvsUpdateRequest cvsUpdateRequestForFile:[file name] inPath:[[file path] stringByDeletingLastPathComponent] revision:nil date:rightDatedVersion toFile:rightVersionFullPath]];
            }
        }
    }
    if(!rightVersionFullPath)
        if(!rightVersion)
            rightVersionFullPath = [file path];
        else{
            rightVersionFullPath = [NSTemporaryDirectory() stringByAppendingPathComponent:[file filenameForRevision:rightVersion]];
            [requests addObject:[CvsUpdateRequest cvsUpdateRequestForFile:[file name] inPath:[[file path] stringByDeletingLastPathComponent] revision:rightVersion date:nil toFile:rightVersionFullPath]];
        }

    ancestorVersion = [parameterDictionary objectForKey:@"AncestorRevision"];
    if(!ancestorVersion){
        ancestorVersion = [parameterDictionary objectForKey:@"AncestorTag"];
        if(!ancestorVersion){
            ancestorDatedVersion = [parameterDictionary objectForKey:@"AncestorDate"];
            if(ancestorDatedVersion){
                ancestorVersionFullPath = [NSTemporaryDirectory() stringByAppendingPathComponent:[file filenameForDate:ancestorDatedVersion]];
                [requests addObject:[CvsUpdateRequest cvsUpdateRequestForFile:[file name] inPath:[[file path] stringByDeletingLastPathComponent] revision:nil date:ancestorDatedVersion toFile:ancestorVersionFullPath]];
            }
        }
    }
    if(!ancestorVersionFullPath && ancestorVersion){
        ancestorVersionFullPath = [NSTemporaryDirectory() stringByAppendingPathComponent:[file filenameForRevision:ancestorVersion]];
        [requests addObject:[CvsUpdateRequest cvsUpdateRequestForFile:[file name] inPath:[[file path] stringByDeletingLastPathComponent] revision:ancestorVersion date:nil toFile:ancestorVersionFullPath]];
    }

    mergePath = [parameterDictionary objectForKey:@"MergeFile"];
    if(!mergePath)
        mergePath = [file path];

    if(self = [self initWithFile:leftVersionFullPath andFile:rightVersionFullPath]){
        CvsUpdateRequest	*aRequest;
        NSEnumerator		*anEnum = [requests objectEnumerator];

        if(ancestorVersionFullPath)
            [task setArguments:[NSArray arrayWithObjects:leftVersionFullPath, rightVersionFullPath, @"-ancestor", ancestorVersionFullPath, @"-merge", mergePath, nil]];
        else
            [task setArguments:[NSArray arrayWithObjects:leftVersionFullPath, rightVersionFullPath, @"-merge", mergePath, nil]];

        while(aRequest = [anEnum nextObject])
            [self addPrecedingRequest:aRequest];
    }

    return self;
}
#endif

@end
