/* CVLUnwrapRequest.m created by stephane on Fri 25-Feb-2000 */
/*$Id: CVLUnwrapRequest.m,v 1.1 2002/06/14 13:28:09 stephane Exp $*/
/* Copyright (c) 1997, Sen:te Ltd.  All rights reserved. */

#import "CVLUnwrapRequest.h"
#import "../CVLFile.h"


static NSString	*unwrapPath = nil;
#ifdef WIN32
static NSString	*shPath = nil;
#endif


@interface CVLUnwrapRequest(Private)
+ (void) preferencesChanged:(NSNotification *)notification;
- (id) initWithCVLFile:(CVLFile *)aFile;
@end

@implementation CVLUnwrapRequest

+ (void) initialize
{
    static BOOL	initialized = NO;

    [super initialize];
    if(!initialized){
        initialized = YES;
        [[NSUserDefaults standardUserDefaults] registerDefaults:[NSDictionary dictionaryWithObject:[[NSBundle mainBundle] pathForResource:@"unwrap" ofType:@"sh"] forKey:@"unwrap"]];
#ifdef WIN32
        [[NSUserDefaults standardUserDefaults] registerDefaults:[NSDictionary dictionaryWithObject:[NSString pathWithComponents:[NSArray arrayWithObjects:[NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSSystemDomainMask, YES) lastObject], @"Executables", [@"sh" stringByAppendingPathExtension:@"exe"], nil]] forKey:@"sh.exe"]];
#endif
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(preferencesChanged:) name:@"PreferencesChanged" object:nil];
        [self preferencesChanged:nil];
    }
}

+ (void) preferencesChanged:(NSNotification *)notification
{
    [unwrapPath release];
    unwrapPath = [[[NSUserDefaults standardUserDefaults] objectForKey:@"unwrap"] retain];
#ifdef WIN32
    [shPath release];
    shPath = [[[NSUserDefaults standardUserDefaults] objectForKey:@"sh.exe"] retain];
#endif
}

+ (BOOL) unwrapIsValid
{
#ifdef WIN32
    return shPath != nil && unwrapPath != nil;
#else
    return unwrapPath != nil;
#endif
}

+ (id) unwrapRequestForWrapper:(NSString *)fileFullPath
{
    return [[[self alloc] initWithCVLFile:[CVLFile treeAtPath:fileFullPath]] autorelease];
}

- (id) initWithCVLFile:(CVLFile *)aFile
{
    if(![[self class] unwrapIsValid]){
        [self dealloc];
        return nil;
    }

    if(self = [super initWithTitle:@"unwrap"]){
        NSTask	*unwrapTask = [[NSTask alloc] init];

#ifdef WIN32
        [unwrapTask setLaunchPath:shPath];
        [self setTask:unwrapTask];
        [unwrapTask release];
        [task setArguments:[NSArray arrayWithObjects:unwrapPath, [[aFile path] lastPathComponent], nil]];
#else
        [unwrapTask setLaunchPath:unwrapPath];
        [self setTask:unwrapTask];
        [unwrapTask release];
        [task setArguments:[NSArray arrayWithObject:[[aFile path] lastPathComponent]]];
#endif
        [task setCurrentDirectoryPath:[[aFile path] stringByDeletingLastPathComponent]];
    }
    
    return self;
}

@end
