/* CvsCheckoutRequest.m created by vincent on Wed 26-Nov-1997 */
/*$Id: CvsCheckoutRequest.m,v 1.8 2002/06/14 13:19:33 stephane Exp $*/

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "CvsCheckoutRequest.h"
#ifdef MACOSX
#import <CvsRepository.h>
#else
#import <Repositories/CvsRepository.h>
#endif
#import <Foundation/NSArray.h>
#import <Foundation/NSPathUtilities.h>
#ifdef MACOSX
#import <CvsModule.h>
#else
#import <Modules/CvsModule.h>
#endif


@implementation CvsCheckoutRequest

+ (CvsCheckoutRequest *)cvsCheckoutRequestForModule:(NSString *)module inRepository:(CvsRepository *)aRepository toPath:(NSString *)aPath
{
    return [self cvsCheckoutRequestForModule:module inRepository:aRepository toPath:aPath revision:nil date:nil];
}

+ (CvsCheckoutRequest *)cvsCheckoutRequestForModule:(NSString *)module inRepository:(CvsRepository *)aRepository toPath:(NSString *)aPath revision:(NSString *)aRevision date:(NSString *)aDate
{
    return [self cvsCheckoutRequestForModule:module inRepository:aRepository toPath:aPath revision:aRevision date:aDate removesStickyAttributes:NO];
}

+ (CvsCheckoutRequest *)cvsCheckoutRequestForModule:(NSString *)module inRepository:(CvsRepository *)aRepository toPath:(NSString *)aPath revision:(NSString *)aRevision date:(NSString *)aDate removesStickyAttributes:(BOOL)removesStickyAttributesFlag
{
    CvsCheckoutRequest *request;

    if (request=[self requestWithCmd: 10 title:@"checkout" path:aPath files:[NSArray arrayWithObject: module]]) {
        [request setRepository:aRepository];
        [request setRevision:aRevision];
        [request setDate:aDate];
        [request setRemovesStickyAttributes:removesStickyAttributesFlag];
    }

    return request;
}
/*
+ (id) cvsCheckoutRequestForFile: (NSString*) aFile inPath: (NSString*) aPath
{
    CvsCheckoutRequest *request;

    if (request=[self requestWithCmd: 10 title:@"checkout" path:aPath files: [NSArray arrayWithObject: aFile]])
            [request setRepository:[CvsRepository defaultRepository]];

        return request;
}

+ (id) cvsCheckoutRequestForFile: (NSString*) aFile inDestinationPath: (NSString*) aDestinationPath;
// allow checkout with another dir name
// Stephane: this method is currently NOT used, is it?
// Perhaps we should add a textField in checkout panel to allow new name...
// Or we could check if given path exists: if it doesn't, it means that user wants
// to rename the destination path.
{
    CvsCheckoutRequest* newRequest;

    newRequest=[self requestWithCmd: 10
                              title:@"checkout"
                               path: [aDestinationPath stringByDeletingLastPathComponent]
                              files: [NSArray arrayWithObject: aFile]];
    if (newRequest) {
        [newRequest setDestinationPath: aDestinationPath];
        [newRequest setRepository:[CvsRepository defaultRepository]];
    }
    return newRequest;
}
*/
- (NSString *)module
{
    return [files objectAtIndex:0];
}

- (void) dealloc
{
    [destinationPath release];
    [revision release];
    [date release];
    [super dealloc];    
}

- (void) setDestinationPath: (NSString*) aString
{
    if (destinationPath != aString)
    {
        [destinationPath release];
        destinationPath= [aString retain];
    }
}

- (NSString*) destinationPath
{
    if (destinationPath)
    {
        return [[self path] stringByAppendingPathComponent:destinationPath];
    }
    else{
        NSString	*moduleName = [files objectAtIndex:0];
        CvsModule	*aModule = [repository moduleWithSymbolicName:moduleName];

        if(aModule)
            return [[self path] stringByAppendingPathComponent:[aModule outputDirectoryName]];
        else
            // It may happen that modules are not defined in <modules> file. It is not an error.
            return [[self path] stringByAppendingPathComponent:moduleName];
    }
}


- (NSArray *) cvsOptions
{
    NSArray		*previousOptions = [super cvsOptions];
    NSArray		*newOptions = nil;
    unsigned	compressionLevel = [self compressionLevel];

    if(compressionLevel)
        newOptions = [NSArray arrayWithObject:[NSString stringWithFormat:@"-z%d", compressionLevel]];

    if(newOptions)
        return [previousOptions arrayByAddingObjectsFromArray:newOptions];
    else
        return previousOptions;
}


- (NSArray *)cvsCommandOptions
{
    NSMutableArray *options;

    options=[NSMutableArray array];

    [options addObject:@"-P"]; // Prune (remove) empty directories
    
    if (revision && ![revision isEqual:@""]) {
        [options addObject:@"-r"];
        [options addObject:revision];
    }
    if (date && ![date isEqual:@""]) {
        [options addObject:@"-D"];
        [options addObject:date];
    }
    if (destinationPath)
    {
        [options addObject:@"-d"];
        [options addObject:[destinationPath lastPathComponent]];
    }
    if(removesStickyAttributes)
        [options addObject:@"-A"];

    return options;
}

- (void)setRevision:(NSString *)aRevision
{
    [aRevision retain];
    [revision release];
    revision=aRevision;
}

- (void)setDate:(NSString *)aDate
{
    [aDate retain];
    [date release];
    date=aDate;
}

- (void) setRemovesStickyAttributes:(BOOL)flag
{
    removesStickyAttributes = flag;
}

- (NSArray *)cvsCommandArguments
{
    return [self files];
}

- (NSString *)cvsWorkingDirectory
{
    return [self path];
}


@end
