/* CvsLogRequest.m created by ja on Thu 04-Sep-1997 */
/*$Id: CvsLogRequest.m,v 1.7 2002/07/04 12:30:03 stephane Exp $*/

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "CvsLogRequest.h"
#import <SenOyster/NSString.SenOysterRegex.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSAutoreleasePool.h>
#import <Foundation/NSCalendarDate.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSTimeZone.h>

#ifndef WIN32

static NSString* CvsLogWorkingFilePattern = @"/Working file:\\s+(.*)\\n/";

#else

static NSString* CvsLogWorkingFilePattern = @"/Working file:\\s+(.*)\\r\\n/";

#endif

@interface CvsLogRequest (Private)
- (void)initResult;
@end

@implementation CvsLogRequest
+ (CvsLogRequest *)cvsLogRequestForFiles:(NSArray *)someFiles inPath:(NSString *)aPath
{
    return [self requestWithCmd:1 title:@"log" path:aPath files:someFiles];
}

- (void)dealloc
{
    [parsingBuffer release];
    [result release];
    [logPath release];
    
    [super dealloc];
}

- (NSArray *)cvsCommandOptions
{
    return nil;
}

- (NSArray *)cvsCommandArguments
{
    return [self files];
}

- (NSString *)cvsWorkingDirectory
{
    return [self path];
}

-(NSDictionary *)result
{
  return result;
}

- (void)initResult
{ 
    result=[[NSMutableDictionary alloc] init];
}

- (BOOL)setUpTask
{
    if ([super setUpTask]) {

        parsingBuffer=[[NSMutableString alloc] init];

        if (!result) {
            [self initResult];
        }
        return YES;
    } else {
        return NO;
    }
}

- (void) readLogFrom: (NSString*) aString
  // scan 'aString' and update log info, if any
{
    NSAutoreleasePool *subpool = [[NSAutoreleasePool alloc] init];
    NSArray* matchArray= [aString componentsMatchedByOperator: CvsLogWorkingFilePattern];
    NSArray* linesArray;
    int linesCount= 0;
    int i= 0;
    NSString* revision= nil;
    NSString* author= nil;
    NSString* message= nil;
    NSCalendarDate* date= nil;
    NSString	*lineModif = nil;
    NSString	*stateString = nil;

    if ([matchArray count] > 0) {
        [logPath release];
        logPath= [[[self path] stringByAppendingPathComponent: [matchArray objectAtIndex: 0]] retain];
    }
    
    linesArray= [aString componentsSeparatedByOperator: CvsRequestNewLinePattern];
    linesCount= [linesArray count];

    while (!(revision) && (i < linesCount)) {
        NSString* line= [linesArray objectAtIndex: i];

        if (([line length] > 10) && ([[line substringToIndex: 9] isEqualToString: @"revision "])) {
            revision= [line substringFromIndex: 9];
        }
        i++;
    }
    if ((revision) && (i < linesCount)) {
        NSString* nextLine= [linesArray objectAtIndex: i];

        // Warning: <lines: xxx> is optional!
        matchArray= [nextLine componentsMatchedByOperator:@"/date:\\s+(.*);\\s+author:\\s+(.*);\\s+state:\\s+(.*);(\\s+lines:\\s+(.*))?/"];
        if ([matchArray count] > 4) { // all infos are present
            author= [matchArray objectAtIndex: 1];
            // log is the only cvs subcommand which returns dates in UTC timezone
            date= [NSCalendarDate dateWithString: [[matchArray objectAtIndex: 0] stringByAppendingString: @" +0000"]
                                  calendarFormat: @"%Y/%m/%d %H:%M:%S %z"];
            [date setTimeZone:[NSTimeZone timeZoneWithName:@"UTC"]];
                lineModif = [matchArray objectAtIndex:4];
            stateString = [matchArray objectAtIndex:2];
        }
    }
    if ((date) && (i+1 < linesCount)) {	// infos found, get complete log and fill dictionary
        NSRange theRange= {i+1, linesCount - i - 1};		
        NSArray* logArray= [linesArray subarrayWithRange: theRange];

        message= [logArray componentsJoinedByString: RequestNewLineString];
    }
    if (message) {	// all infos are filled, set them into dictionary
        NSMutableDictionary* resultDict= [result objectForKey: logPath];
        NSMutableArray* newArray= [NSMutableArray arrayWithCapacity: 0];
        NSMutableDictionary *logDict= [NSMutableDictionary dictionaryWithCapacity: 0];

        if (!resultDict)
        {
          resultDict=[NSMutableDictionary dictionary];
          [result setObject:resultDict forKey: logPath];
        }
        [logDict setObject: revision forKey: @"revision"];
        [logDict setObject: date forKey: @"date"];
        [logDict setObject: author forKey: @"author"];
        [logDict setObject: message forKey: @"msg"];
        [logDict setObject: lineModif forKey: @"modifs"];
        [logDict setObject: stateString forKey: @"state"];
        //NSLog(@"log dict %@\n log array %@\n", [logDict description], [[resultDict objectForKey: @"log"] description]);

        [newArray addObjectsFromArray: [resultDict objectForKey: @"log"]];
        [newArray addObject: logDict];
        [resultDict setObject: newArray forKey: @"log"];
    }
    [subpool release];
} // readLogFrom:

- (void)parseOutput:(NSString *)data
{
  NSRange sentinelRange;
  NSArray* results= nil;
  int rCount= 0;
  int j= 0;

  [parsingBuffer appendString: data];
  // sentinel to avoid deleting last split pettern
  sentinelRange.length= 2;
  [parsingBuffer appendString: @"xx"];

  // split sur ==== ou sur ---
  results= [parsingBuffer componentsSeparatedByOperator: CvsRequestLineOfEqualsOrMinusPattern];
  rCount= [results count];
  j= 0;
  for (j= 0; j< rCount-1; j++)
  {	// give all results to the reader
    NSString* aResult= [results objectAtIndex: j];
    if ([aResult length] > .0)
    {
      [self readLogFrom: aResult];
    }
  }
  [parsingBuffer setString: [results objectAtIndex: rCount -1]];
  sentinelRange.location= [parsingBuffer length] -  sentinelRange.length;
  [parsingBuffer deleteCharactersInRange: sentinelRange];
} //

#ifdef JA_PATCH
- (void)flushBuffers
{
    [self readLogFrom: parsingBuffer];
    [parsingBuffer release];
    parsingBuffer=nil;
}

- (void)endWithSuccess;
{
    [self flushBuffers];
}

- (void)endWithFailure;
{
    [self flushBuffers];
}
#else
- (void)end
{
    [self readLogFrom: parsingBuffer];
    [parsingBuffer release];
    parsingBuffer=nil;

    if (!result) {
        [self initResult];
    }
    [super endWithoutInvalidation];
}
#endif
@end
