/* CvsLoginRequest.m created by ja on Thu 04-Dec-1997 */
/*$Id: CvsLoginRequest.m,v 1.6 2002/07/04 12:30:03 stephane Exp $*/

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "CvsLoginRequest.h"
#ifdef MACOSX
#import <CvsRepository.h>
#import <SenPanelFactory.h>
#import <SenFormPanelController.h>
#else
#import <Repositories/CvsRepository.h>
#import <SenPanels/SenPanelFactory.h>
#import <SenPanels/SenFormPanelController.h>
#endif

#ifndef JA_PATCH
@interface CvsLoginRequest (Private)
- (NSString *)askPassword;
@end
#endif

@implementation CvsLoginRequest
+ (CvsLoginRequest *)cvsLoginRequest
{
    return [self cvsLoginRequestForRepository:[CvsRepository defaultRepository]];
}

+ (CvsLoginRequest *)cvsLoginRequestForRepository:(CvsRepository *)aRepository
{
    CvsLoginRequest *request;

    if ([aRepository needsLogin]) {
        if (request=[self requestWithCmd: 12 title:@"login" path:nil files:nil])
            [request setRepository:aRepository];
    } else {
        return nil;
    }

    return request;
}

+ (CvsLoginRequest *)cvsLoginRequestForRepository:(CvsRepository *)aRepository withPassword:(NSString *)passwordString
{
    CvsLoginRequest *request;

    if (request=[self requestWithCmd: 12 title:@"login" path:nil files:nil]) {
        [request setRepository:aRepository];
        [request setPassword:[passwordString retain]];
    }

    return request;
}

#ifdef JA_PATCH
+(State *)initialState
{
    static BOOL triedInitialState=NO;
    static State *initialState=nil;

    if (!triedInitialState) {
        initialState=[[State initialStateForStateFile:[[NSBundle bundleForClass:[self class]] pathForResource:@"CvsLogin" ofType:@"fsm"]] retain];
        triedInitialState=YES;
    }
    return initialState;
}
#endif

- (void)setPassword:(NSString *)passwordString
{
    [passwordString retain];
    [password release];
    password=passwordString;
}

- (NSString *)cvsCommand
{
    return @"login";
}

#ifdef JA_PATCH
- (void)askPassword
{
    SenFormPanelController *panel;

    panel=[[SenPanelFactory sharedPanelFactory] handleOfPanelWithName:@"CvsPserverLogin"];
    [panel setDictionaryValue:[repository properties]];
    if ([panel showAndRunModal]==NSOKButton) {
        [self setPassword:[panel objectValueForKey:@"password"]];
    } else {
        [self setPassword:nil];
    }
}
#else
- (NSString *)askPassword
{
    SenFormPanelController *panel;

    panel=[[SenPanelFactory sharedPanelFactory] handleOfPanelWithName:@"CvsPserverLogin"];
    [panel setDictionaryValue:[repository properties]];
    if ([panel showAndRunModal]==NSOKButton) {
        return [panel objectValueForKey:@"password"];
    } else {
        return nil;
    }
}

- (void)resumeNow
{
    switch (internalState) {
        case STATE_LOGIN_FIRST_PANEL:
            if (password=[self askPassword]) {
                [password retain];
                internalState=STATE_NOT_STARTED;
                [self setState:STATE_READY];
            } else {
                success=NO;
                [self end];
            }
            break;

        case STATE_LOGIN_PANEL:
            if (password=[self askPassword]) {
                [password retain];
                internalState=STATE_CVS_RETRY_AFTER_LOGIN;
                [self setState:STATE_READY];
            } else {
                success=NO;
                [self end];
            }
            break;

        default:
            [super resumeNow];
            break;
    }
}
#endif

- (BOOL) setUpTask {
    if ([super setUpTask]) {
        NSPipe *aPipe=[NSPipe pipe];

#ifdef DEBUG
        NSLog([[task environment] description]);
#endif
        cvsInput=[[aPipe fileHandleForWriting] retain];

        [task setStandardInput:aPipe];

        // Stephane: could we allow [NSString defaultCStringEncoding] ?
        [cvsInput writeData:[[password stringByAppendingString: RequestNewLineString] dataUsingEncoding:NSASCIIStringEncoding
                                                                                   allowLossyConversion:YES]];
        //[cvsInput closeFile];

        parseByLine=NO;
        
        return YES;
    }
    return NO;
}



- (void)parseError:(NSString *)data
{
    if (!summedOutput) summedOutput=[[NSMutableString alloc] init];
    [summedOutput appendString:data];

    if ([summedOutput hasSuffix:@"CVS password: "]) {
 //       if (password) {
 //           [cvsInput writeData: [[password stringByAppendingString: RequestNewLineString] dataUsingEncoding:NSASCIIStringEncoding allowLossyConversion:YES]];
 //       }
 //       [summedOutput setString:@""];
    }
}

#ifdef JA_PATCH
- (id)taskTerminated:(NSNotification *)notification
{
    [cvsInput closeFile];
    [cvsInput release];
    cvsInput=nil;

    return [super taskTerminated:notification];
}

-(NSString *)summary
{
    return @"";
}

- (BOOL) canRunAgainstRequest:(Request *)runningRequest
{
    return !([runningRequest isKindOfClass:[CvsLoginRequest class]]);
}

- (NSString *)shortDescription
{
    return [NSString stringWithFormat: @"%@ in %@\n", [self cmdTitle], [repository root]];
}

- (void) endWithSuccess
{
    [repository setIsLoggedIn:YES];
}

- (void)endWithFailure
{
    [repository setIsLoggedIn:NO];
}

- (BOOL)havePassword
{
    return password!=nil;
}

- (BOOL)haveNoPassword
{
    return password==nil;
}
#else
- (void)taskEnded
{
    if (success) {
        [self endConditionReached];
    } else {
        [password release];
        password=nil;
        internalState=STATE_LOGIN_PANEL;
        [self setState:STATE_READY];
    }
}

- (void)end
{
    [cvsInput closeFile];
    [cvsInput release];
    cvsInput=nil;

    if (internalState!=STATE_NOT_STARTED) {
        [repository setIsLoggedIn:success];
    }
    [super endWithoutInvalidation];
}

-(NSString *)summary
{
    return @"";
}

- (BOOL) canContinue
{
    return [self precedingRequestsEnded];
}

- (void)schedule
{
    if (!password) {
        internalState=STATE_LOGIN_FIRST_PANEL;
    }
    [super schedule];
}

- (BOOL) canRunAgainstRequest:(Request *)runningRequest
{
    return !([runningRequest isKindOfClass:[CvsLoginRequest class]]);
}

- (NSString *)shortDescription
{
    return [NSString stringWithFormat: @"%@ in %@\n", [self cmdTitle], [repository root]];
}
#endif

@end
