/* CvsQuickStatusRequest.m created by stephane on Tue 05-Oct-1999 */
/*$Id: CvsQuickStatusRequest.m,v 1.16 2002/07/04 12:28:01 stephane Exp $*/

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "CvsQuickStatusRequest.h"
#import <SenOyster/NSString.SenOysterRegex.h>
#import <Foundation/Foundation.h>
#import "../CVLFile.h"
#ifdef MACOSX
#import <CvsRepository.h>
#import <NSString+Lines.h>
#else
#import <Repositories/CvsRepository.h>
#import <SenFoundation/NSString+Lines.h>
#endif


@interface CvsQuickStatusRequest(Private)
- (void) readStatusesFromString:(NSString *)aBuffer;
- (void) parseString:(NSString *)data usingBuffer:(NSMutableString *)aBuffer;
@end

@implementation CvsQuickStatusRequest

+ (CvsQuickStatusRequest *) cvsQuickStatusRequestFromPath:(NSString *)aPath
{
    CvsQuickStatusRequest	*request = [self requestWithCmd:CVS_QUICK_STATUS_CMD_TAG title:@"quick status" path:aPath files:nil];

    [request setIsQuiet:YES];
#if 0
NSAssert1([[(CVLFile *)[CVLFile treeAtPath:aPath] repository] root] != nil, @"No repository root for %@", aPath);
#else
	if(![[(CVLFile *)[CVLFile treeAtPath:aPath] repository] root])
        return nil;
#endif

    return request;
}

- (id) init
{
    if(self = [super init]){
        parsingBuffer = [[NSMutableString allocWithZone:[self zone]] init];
        errorParsingBuffer = [[NSMutableString allocWithZone:[self zone]] init];
    }
    // If we get truncated lines, we will output bad results!!

    return self;
}

- (void) dealloc
{
    [fileStatuses release];
    [parsingBuffer release];
    [errorParsingBuffer release];

    [super dealloc];
}

- (NSArray *) cvsOptions
{
    NSArray		*previousOptions = [super cvsOptions];
    NSArray		*newOptions = [NSArray arrayWithObjects: @"-q", @"-n", nil];
    unsigned	compressionLevel = [self compressionLevel];

    if(compressionLevel)
        newOptions = [newOptions arrayByAddingObject:[NSString stringWithFormat:@"-z%d", compressionLevel]];

    return [previousOptions arrayByAddingObjectsFromArray:newOptions];
}

- (NSArray *) cvsCommandArguments
{
    return [NSArray array];
}

- (NSArray *) cvsCommandOptions
{
    // -P option has absolutely NO effect on the output!!!
    // Using -d option should be optional, as it also logs directories which
    // have definitely been removed!!!
    // Currently we will retrieve them, but mark them as UNKNOWN
//    if([[NSUserDefaults standardUserDefaults] boolForKey:@"QuickStatusShowsMissingDirectories"])
        return [NSArray arrayWithObjects:@"-d", nil]; // (pseudo) create missing directories
//    return [NSArray array];
}

- (NSString *) cvsWorkingDirectory
{
    return [self path];
}

- (void) parseError:(NSString *)data
{
    [self parseString:data usingBuffer:errorParsingBuffer];
    [super parseError:data];
}

- (void) parseOutput:(NSString *)data
{
    [self parseString:data usingBuffer:parsingBuffer];
}

- (void) parseString:(NSString *)data usingBuffer:(NSMutableString *)aBuffer
{
    NSRange	sentinelRange;
    NSArray	*results = nil;
    int		rCount = 0;
    int		j = 0;

    [aBuffer appendString:data];
    // sentinel to avoid deleting last split pattern
    sentinelRange.length = 2;
    [aBuffer appendString:@"xx"];

    results = [aBuffer lines];
    rCount = [results count];

    // Read all lines except the last one which may be uncomplete!
#ifdef JA_PATCH
    for(j = rCount - 2; j >= 0; j--){ // Give all results to the reader
        NSString	*aResult = [results objectAtIndex:j];

        if([aResult length] > .0)
            [self readStatusesFromString:aResult];
    }
#else
    for(j = rCount - 2; j >= 0; j--){ // Give all results to the reader
        NSString	*aResult = [results objectAtIndex:j];

        if([aResult length] > .0)
            [self readStatusesFromString:aResult];
    }
#endif

    [aBuffer setString:[results objectAtIndex:rCount - 1]];
    sentinelRange.location = [aBuffer length] - sentinelRange.length;
    [aBuffer deleteCharactersInRange:sentinelRange];
}

- (void) readStatusesFromString:(NSString *)aBuffer
{
    // Up-to-date files are not displayed
    // Locally added files are prefixed with A
    // Locally added directories are NOT listed: only their content is listed
    // Locally removed files are prefixed with R
    // Locally removed directories are NOT listed: only their content is listed

    NSArray			*lines = [aBuffer lines];
    NSEnumerator	*statusLineEnum = [lines objectEnumerator];
    NSString		*aLine;

    if(!fileStatuses)
        fileStatuses = [[NSMutableDictionary allocWithZone:[self zone]] initWithCapacity:[lines count]];
    
    while(aLine = [statusLineEnum nextObject]){
        unichar	status;

        if([aLine length] < 3)
            continue;

        status = [aLine characterAtIndex:0];
        switch(status){
            case 'U': // Needs checkout or needs patch if already exists locally or needs merge if ???
            case 'A': // Locally added; locally added dirs are not listed: only their content is listed (=> problem if dir empty)
            case 'R': // Locally removed
            case 'M': // Locally modified
                if([aLine characterAtIndex:1] != ' ')
                    // Merging differences between 1.11 and 1.16 into GPGKey.m
                    break;
            case 'C': // Conflict
            case '?': // Unknown by cvs, or contains non cvs files
                [fileStatuses setObject:[NSString stringWithCharacters:&status length:1] forKey:[[self path] stringByAppendingPathComponent:[aLine substringFromIndex:2]]];
                break;
            case 'c': // In fact comes as an error (stderr), termination status != 0 (NEEDS TO BE CHECKED!!!!!!)
#warning New outputs
// cvs [update aborted]: end of file from server (consult above messages if any)
                
    			// WARNING: in pserver mode, messages are a bit different: they begin with
       			// <cvs server> instead of <cvs update>
				{
                    NSString	*filename;

                    if([aLine hasPrefix:@"cvs"]){
                        // cvs: inflate: unknown compression method
                        // cvs: dying gasps from XXX unexpected
                        // cvs: warning: unrecognized response XXX from cvs server
                        if([aLine hasSuffix:@": inflate: unknown compression method"] || ([aLine rangeOfString:@": dying gasps from "].length > 0 && [aLine hasSuffix:@" unexpected"]) || [aLine rangeOfString:@": warning: unrecognized response "].length > 0 && [aLine hasSuffix:@" from cvs server"])
                            // Trace option (-t) is ON. Ignore output.
                            continue;
                        // cvs update aborted: stat failed for XXX
                        // cvs update aborted: reading from server: Input/output error
                        if([aLine rangeOfString:@"update aborted: stat failed for "].length > 0 || [aLine rangeOfString:@"server aborted: stat failed for "].length > 0 || [aLine hasSuffix:@"update aborted: reading from server: Input/output error"] || [aLine hasSuffix:@"server aborted: reading from server: Input/output error"]){
                            updateAborted = YES;
                            continue;
                        }
                    }

                    // cvs XXX: New directory `XXX' -- ignored
                    filename = [[aLine componentsMatchedByOperator:@"/^cvs(.*): New directory .(.*). -- ignored$/"] lastObject];
                    if(filename && [filename length])
                        [fileStatuses setObject:@"u" forKey:[[self path] stringByAppendingPathComponent:filename]];
                    else{
                        // cvs update: move away XXX; it is in the way
                        filename = [[aLine componentsMatchedByOperator:@"/^cvs(.*): move away (.*); it is in the way$/"] lastObject];
                        // We can ignore this case
                        if(!filename || ![filename length]){
                            // cvs update: XXX is not (any longer) pertinent
                            // cvs server: warning: XXX is not (any longer) pertinent
                            filename = [[aLine componentsMatchedByOperator:@"/^cvs(.*): (.*) is no longer in the repository$/"] lastObject];
                            if(!filename || ![filename length])
                                // cvs server: conflict: XXX is modified but no longer in the repository
                                filename = [[aLine componentsMatchedByOperator:@"/^cvs(.*): conflict: (.*) is modified but no longer in the repository$/"] lastObject];
                            if(!filename || ![filename length]){
                                // cvs update aborted: could not find desired version XXX in XXX,v
                                // cvs server aborted: could not find desired version XXX in XXX,v
                                filename = [[aLine componentsMatchedByOperator:@"/^cvs(.*) \\[((update)|(server)) aborted\\]: could not find desired version (.*) in (.*),v$/"] lastObject];
                                if(!filename || ![filename length]){
                                    // cvs update: XXX is not (any longer) pertinent
                                    // cvs server: warning: XXX is not (any longer) pertinent
                                    filename = [[aLine componentsMatchedByOperator:@"/^cvs(.*): (warning: )*(.*) is not \\(any longer\\) pertinent$/"] lastObject];
                                    if(!filename || ![filename length]){
                                        // cvs XXX: warning: XXX was lost
                                        filename = [[aLine componentsMatchedByOperator:@"/^cvs(.*): warning: (.*) was lost$/"] lastObject];
                                        if(!filename || ![filename length]){
                                            // cvs XXX: conflicts found in XXX
                                            filename = [[aLine componentsMatchedByOperator:@"/^cvs(.*): conflicts found in (.*)$/"] lastObject];
                                            if(!filename || ![filename length]){
                                                // cvs update: Sorry, you don't have read/write access to the history file
                                                if([aLine hasSuffix:@"Sorry, you don't have read/write access to the history file"] || [aLine rangeOfString:@"Permission denied"].length > 0)
                                                    updateAborted = YES;
                                                else{
                                                    // In this case, we will perform a full status request
                                                    NSLog(@"Unknown <cvs update> output(1): %@", aLine);
                                                    if([aLine hasPrefix:@"cvs"] && ([aLine rangeOfString:@"update aborted: "].length > 0 || [aLine rangeOfString:@"server aborted: "].length > 0))
                                                        updateAborted = YES;
                                                }
                                            }
                                            else{
                                                [fileStatuses setObject:@"C" forKey:[[self path] stringByAppendingPathComponent:filename]];
                                            }
                                        }
                                        else{
                                            // File should become NeedsUpdate
                                            [fileStatuses setObject:@"U" forKey:[[self path] stringByAppendingPathComponent:filename]];
                                        }
                                    }
                                    else{
                                        // File should become NeedsUpdate
                                        [fileStatuses setObject:@"U" forKey:[[self path] stringByAppendingPathComponent:filename]];
                                    }
                                }
                                else{
                                    // In this case, we should warn user, because local CVS info is inconsistent!!!
                                    [fileStatuses setObject:@"!" forKey:[[self path] stringByAppendingPathComponent:filename]]; // Perform full status
                                    updateAborted = YES;
                                }
                            }
                            else
                                [fileStatuses setObject:@"!" forKey:[[self path] stringByAppendingPathComponent:filename]]; // Perform full status
                        }
                    }
                }
                break;
            case 'S':
                if(![aLine hasPrefix:@"S->"])
                    // In this case, let's perform a full status request
                    NSLog(@"Unknown <cvs update> output: %@", aLine);
                // Else trace option (-t) is ON. Ignore output.
                break;
            default:
                if(![aLine isMatchedByOperator:@"/^(\\s*)->(.*)/"] && ![aLine hasPrefix:@"retrieving revision "] && ![aLine isEqualToString:@"rcsmerge: warning: conflicts during merge"] && ![aLine hasPrefix:@"Unknown host "] && ![aLine isEqualToString:@"No such file or directory"] && ![aLine hasPrefix:@"cannot change mode for "])
                    // In this case, let's perform a full status request
                    NSLog(@"Unknown <cvs update> output: %@", aLine);
                // Else, trace option (-t) is ON. Ignore output.
        }
    }
}

- (NSDictionary *) result
{
    return fileStatuses;
}

#ifdef JA_PATCH
- (void) closeReadHandle
{
    [self readStatusesFromString:parsingBuffer]; // Parse the last line, finally
    [parsingBuffer release];
    parsingBuffer = nil;

    [super closeReadHandle];
}

- (void) closeErrorReadHandle
{
    [self readStatusesFromString:errorParsingBuffer]; // Parse the last line, finally
    [errorParsingBuffer release];
    errorParsingBuffer = nil;

    [super closeErrorReadHandle];
}

- (void)endWithSuccess;
{
}

- (void)endWithFailure;
{
}
#else
- (void) end
{
    [self readStatusesFromString:parsingBuffer]; // Parse the last line, finally
    [parsingBuffer release];
    parsingBuffer = nil;

    [self readStatusesFromString:errorParsingBuffer]; // Parse the last line, finally
    [errorParsingBuffer release];
    errorParsingBuffer = nil;

    [super endWithoutInvalidation];
}
#endif

- (int) priority
{
    return -9; // Higher priority than CvsStatusRequest
}

- (BOOL) statusAborted
{
    return updateAborted;
}

@end
