/*
 $Id: CvsRequest.m,v 1.15 2002/07/04 12:30:03 stephane Exp $
 */

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "CvsRequest.h"
#ifdef MACOSX
#import <CvsRepository.h>
#else
#import <Repositories/CvsRepository.h>
#endif
#import "../NSArray.SenCategorize.h"
#import "../ResultsRepository.h"
#import "../CVLFile.h"
#import <AppKit/NSApplication.h>
#import <AppKit/NSPanel.h>
#import "../CVLDelegate.h"
#import <SenFoundation/NSObject_SenTree.h>
#import "../NSFileManager_CVS.h"

static NSString *cvsPath= nil;
static NSString* cvsCmds[CVS_CMD_COUNT];
static int maxRequestsCount= 0;
static int defaultCompressionLevel = 3;


#ifndef WIN32

NSString* CvsRequestNewLinePattern= @"/\\n/";
NSString* CvsRequestLineOfEqualsOrMinusPattern= @"/=+\\n|-+\\n/";

#else WIN32

NSString* CvsRequestNewLinePattern= @"/\\r\\n/";
NSString* CvsRequestLineOfEqualsOrMinusPattern= @"/=+\\r\\n|-+\\r\\n/";

#endif


//----------------------------------------------------------------------------------------------------------------------

@interface CvsRequest (Private)
+ (void)preferencesChanged:(NSNotification *)notification;
- (void)repositoryChanged:(NSNotification *)notification;
- (void)restart;
@end

//----------------------------------------------------------------------------------------------------------------------

@implementation CvsRequest
+ (void)initialize
{
    cvsCmds[0]=@"status";
    cvsCmds[1]=@"log";
    cvsCmds[2]=@"update";
    cvsCmds[CVS_REMOVE_CMD_TAG]=@"remove";
    cvsCmds[CVS_DIFF_CMD_TAG]=@"diff";
    cvsCmds[CVS_OBJCOMMENT_CMD_TAG]=@"admin";
    cvsCmds[CVS_NOKEYWDEXP_CMD_TAG]=@"admin";
    cvsCmds[7]=@"commit";
    cvsCmds[8]=@"tag";
    cvsCmds[CVS_IMPORT_CMD_TAG]=@"import";
    cvsCmds[10]=@"checkout";
    cvsCmds[11]=@"add";
    cvsCmds[12]=@"UNIX";
    cvsCmds[CVS_VERSION_TAG]=@"version";
    cvsCmds[CVS_INIT_CMD_TAG]=@"init";
    cvsCmds[CVS_QUICK_STATUS_CMD_TAG]=@"update";
    cvsCmds[CVS_RELEASE_CMD_TAG]=@"release";

    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(preferencesChanged:) name:@"PreferencesChanged" object:nil];
    [self preferencesChanged:nil];
}

+ (NSDictionary*) canonicalizePath: (NSString*) aPath andFiles: (NSArray*) someFiles
    // remove any subpath from files and return common dir as key and files as value (only one common path is allowed)
{
    NSString* commonPath= aPath;

    if ([someFiles count])
    {
        NSMutableDictionary* subpathsDict= [[someFiles categorizeUsingMethod: @selector(stringByDeletingLastPathComponent)] mutableCopy];

        [subpathsDict removeObjectForKey: @""]; // remove files without any subpath

        NSAssert([[subpathsDict allKeys] count] <= 1, @"Cannot handle more than one common subpath!");

        if ([[subpathsDict allKeys] count] == 1)
        {
            NSEnumerator* fileEnum= [someFiles objectEnumerator];
            NSString* currentPath= nil;
            NSMutableArray* filesWithoutPath= [NSMutableArray arrayWithCapacity: [someFiles count]];

            while (currentPath= [fileEnum nextObject])
            {
                [filesWithoutPath addObject: [currentPath lastPathComponent]];
            }
            commonPath= [aPath stringByAppendingPathComponent: [[subpathsDict allKeys] objectAtIndex: 0]];

#ifdef DEBUG
            NSLog(@"multi path in files %@, commonPath: %@, new files %@", someFiles, commonPath, filesWithoutPath); // Stephane: this is not an error...
#endif

            [subpathsDict release];
            return [NSDictionary dictionaryWithObject: filesWithoutPath forKey: commonPath];
        }
        else
        {
            [subpathsDict release];
            return [NSDictionary dictionaryWithObject: someFiles forKey: commonPath];
        }
    }
    else
    {
        return [NSDictionary dictionaryWithObject: [NSArray array] forKey: commonPath];
    }
}


+ requestWithCmd:(int)aCmd title:(NSString *)cmdString path:(NSString *)aPath files:(NSArray *)someFiles
{
    return [[[self alloc] initWithCmd:aCmd title:cmdString path:aPath files:someFiles] autorelease];
}

+ (void)preferencesChanged:(NSNotification *)notification
{
    [cvsPath release];
    cvsPath=[[[NSUserDefaults standardUserDefaults] objectForKey:@"CVSPath"] retain];
    maxRequestsCount= MAX (1, [[[NSUserDefaults standardUserDefaults] objectForKey: @"MaxParallelRequestsCount"] intValue]);
    defaultCompressionLevel = [[NSUserDefaults standardUserDefaults] integerForKey:@"CompressionLevel"];
}

- initWithCmd:(int)aCmd title:(NSString *)cmdString path:(NSString *)aPath files:(NSArray *)someFiles
{
    cmd=aCmd;
    self=[self initWithTitle: cmdString];
    [self setPath:aPath];
    [self setFiles:someFiles];
    [self setRepository:[CvsRepository repositoryForPath:aPath]];

    return self;
}

-(void)dealloc
{
    [repository release];
    [path release];
    [files release];

    [super dealloc];
}

- (CvsRepository *)repository
{
    return repository;
}

- (void)setRepository:(CvsRepository *)aRepository
{
    if (repository) {
        [repository release];
        [[NSNotificationCenter defaultCenter] removeObserver:self                                                                    name:@"RepositoryChanged" object:repository];
    }
    [aRepository retain];
    repository=aRepository;
    if (repository) {
        [[NSNotificationCenter defaultCenter] addObserver:self                                                                    selector:@selector(repositoryChanged:) name:@"RepositoryChanged" object:repository];
    }
}

- (void)repositoryChanged:(NSNotification *)notification
{
#ifdef JA_PATCH
    [self updateState];
#else
    if (state!=STATE_RUNNING) {
        if (![repository isReadyForRequests]) {
            if (![self canContinue]) {
                [self setState:STATE_WAITING];
            } else {
                [self setState:STATE_READY];
            }
        }
    }
#endif
}

-(int)cmd
{
    return cmd;
}

- (NSArray *)cvsOptions
{
    return [NSArray arrayWithObjects:@"-d",[repository root],nil];
}

- (NSString *)cvsCommand
{
    return cvsCmds[cmd];
}

#ifndef JA_PATCH
- (void) resumeNow
{
    switch (internalState) {
        case STATE_CVS_RETRY_AFTER_LOGIN:
            [self taskCleanUp];
            if ([self startTask]) {
                [self setState:STATE_RUNNING];
            } else {
                success=NO;
                [self endConditionReached];
            }
                break;

        default:
            [super resumeNow];
            break;
    }
}
#endif

- (NSArray *)cvsCommandOptions
{
  // this switch will disappear when all requests will be real classes
    switch (cmd) {

        case CVS_REMOVE_CMD_TAG:
            return [NSArray arrayWithObject:@"-fR"];
            break;

        case CVS_OBJCOMMENT_CMD_TAG:
            return [NSArray arrayWithObject:@"-c"];
            break;

        case CVS_NOKEYWDEXP_CMD_TAG:
            return [NSArray arrayWithObject:@"-ko"];
            break;

        default:
            return nil;
            break;
    }
}

- (NSString *)cvsWorkingDirectory
{
    if (cmd == CVS_REMOVE_CMD_TAG) {
        return [self path];
    }

    return nil;
}

- (NSArray *)cvsCommandArguments
{
    if ((cmd == CVS_OBJCOMMENT_CMD_TAG) || (cmd == CVS_NOKEYWDEXP_CMD_TAG))
    {	// cmds without arguments
        return [NSArray arrayWithObject:[self path]];
    } else if (cmd == CVS_TAG_CMD_TAG)
    {
        NSMutableArray *cmdArgs=[NSMutableArray arrayWithArray:[self files]];
        [cmdArgs addObject:[self path]];

        return cmdArgs;
    }
    return [self files];
}

- (void)setPath:(NSString *)value
{
    [value retain];
    [path release];
    path=value;
}

-(NSString *)path
{
    return path;
}


-(NSString *)summary
{
    if ([[self files] count] == 1)
    {
        return [[self files] objectAtIndex: 0];
    }
    else
    {
        return [[self path] lastPathComponent];
    }
}


- (void)setFiles:(NSArray *)value
{
    [value retain];
    [files release];
    files=value;
}

-(NSArray *)files
{
    return files;
}

-(BOOL)setUpTask
{ // (120) some code is duplicated in super class, some cleaning is required on both..
    NSMutableArray *taskArgs= [NSMutableArray array];

    if ([super setUpTask])
    {
        NSDictionary	*environment = [[self repository] environment];
        
        [taskArgs addObjectsFromArray:[self cvsOptions]];
        if([self cvsCommand]){
            // We need to check this for the special case where we ask --version! There is no cvs command...
            [taskArgs addObject:[self cvsCommand]];
            [taskArgs addObjectsFromArray:[self cvsCommandOptions]];
            [taskArgs addObjectsFromArray:[self cvsCommandArguments]];
        }
        if(environment)
            [task setEnvironment:environment];
        if ((cvsPath) && ([cvsPath cStringLength])) {
            [task setLaunchPath:cvsPath];
        } else {
            [NSApp sendAction: @selector(showPreferences:) to: nil from: self];
            (void)NSRunAlertPanel(@"Cvs problem", @"No path set for the cvs binary, please set it in the preferences panel.", nil, nil, nil);
            return NO;
        }

        {
            NSString *workingDirectory;

            if (workingDirectory=[self cvsWorkingDirectory]) {
                NSFileManager *manager = [NSFileManager defaultManager];

                if([manager cvlDirectoryExistsAtPath:workingDirectory]){
                    [task setCurrentDirectoryPath:workingDirectory];
                } else {
                    return NO;
                }
            }
        }

        if (cmd == CVS_ADD_CMD_TAG)
        {
            NSPipe *aPipe=[NSPipe pipe];
            NSFileHandle *input=[aPipe fileHandleForWriting];

            [task setStandardInput:aPipe];
            
      // [input writeData:[NSData dataWithBytesNoCopy:"y\n" length:2]]; //gives malloc-error 5 !
      //      [input writeData:[NSData dataWithBytes:"y\n" length:2]];
            [input writeData: [[@"y" stringByAppendingString: RequestNewLineString] dataUsingEncoding:NSASCIIStringEncoding
                                                                                 allowLossyConversion:YES]];
            [input closeFile];
        }

        if (cmd == CVS_COMMIT_CMD_TAG){
            if([[NSUserDefaults standardUserDefaults] boolForKey:@"UseCvsTemplates"])
                {
                // In case of user cancelling commit, cvs waits for an answer from its stdin:
                // let's reply we want to _a_bort the operation.
                NSPipe			*aPipe = [NSPipe pipe];
                NSFileHandle	*input = [aPipe fileHandleForWriting];

                [task setStandardInput:aPipe];

                [input writeData:[[@"a" stringByAppendingString: RequestNewLineString] dataUsingEncoding:NSASCIIStringEncoding
                                                                                    allowLossyConversion:YES]];
                [input closeFile];
                }
        }

        [task setArguments:taskArgs];
        return YES;
    }
    else
    {
        return NO;
    }
}

#ifndef JA_PATCH
- (void)taskEnded
{
    if (internalState==STATE_CVS_TRY_LOGIN) {
        internalState=STATE_CVS_RETRY_AFTER_LOGIN;
        if (![self canContinue]) {
            [self setState:STATE_WAITING];
        } else {
            [self setState:STATE_READY];
        }
    } else {
        [self endConditionReached];
    }
}

- (BOOL) canContinue
{
    if ([repository isReadyForRequests]) {
        return [super canContinue];
    } else {
        [self addPrecedingRequest:(Request *)[repository gettingReadyRequest]];
    }
    return NO;
}
#endif

- (void)parseError:(NSString *)data
{
    if ([repository needsLogin]) {
        if ([data rangeOfString:@"to log in first"].length) {
#ifdef JA_PATCH
            noLogin=YES;
#else
            internalState=STATE_CVS_TRY_LOGIN;
#endif
            [repository setIsLoggedIn:NO];
        } else if ([data rangeOfString:@"authorization failed:"].length) {
#ifdef JA_PATCH
            noLogin=YES;
#else
            internalState=STATE_CVS_TRY_LOGIN;
#endif
            [repository setIsLoggedIn:NO];
        }
    }

    return;
}

-(void)updateFileStatuses
{
    ResultsRepository *resultsRepository=[ResultsRepository sharedResultsRepository];
    CVLFile *file;

    [resultsRepository startUpdate];

    if (![self files]) {
        [(CVLFile *)[CVLFile treeAtPath:[self path]] traversePostorder:@selector(invalidateAll)];
    } else {
        id enumerator=[[self files] objectEnumerator];
        NSString *aPath;

        while (aPath=[enumerator nextObject]) {
            aPath=[[self path] stringByAppendingPathComponent:aPath];
            file=[CVLFile treeAtPath:aPath];
            [file traversePostorder:@selector(invalidateAll)];
            /*
             fileFlags=[file flags];
             if (!fileFlags.isDir || fileFlags.isWrapper) {
                 [file invalidateAll];
             } else {
                 [[CVLFile treeAtPath:[self path]] traversePostorder:@"invalidateAll"];
                 [resultsRepository invalidateResultsForDirWithPath:aPath recursively:YES];
             }
             */
        }
    }
    [resultsRepository endUpdate];
}

#ifdef JA_PATCH
-(void)endWithSuccess
{
    [self updateFileStatuses];
}

-(void)endWithFailure
{
    [self updateFileStatuses];
}
#else
-(void)end
{
    [self updateFileStatuses];

    [super end];
}

- (void) endWithoutInvalidation
{
    [super end];
}

- (NSString *)description
{
    NSMutableDictionary* dict= [NSMutableDictionary dictionaryWithCapacity: 4];

    [dict setObject: (cmdTitle ? cmdTitle : @"") forKey: @"cmdTitle"];
    [dict setObject: (path ? path : @"") forKey: @"path"];
    [dict setObject: ([files count] ? [files description] : @"") forKey: @"args"];
    [dict setObject: [self stateString] forKey: @"state"];

    return [dict description];
}
#endif

- (BOOL) canRunAgainstRequests:(NSSet *)runningRequests
{
    id enumerator;
    CvsRequest *aRequest;
    int requestCount=0;

    enumerator=[runningRequests objectEnumerator];
    while (aRequest=[enumerator nextObject]) {
        if ([aRequest isKindOfClass:[CvsRequest class]]) {
            requestCount++;
            if (requestCount>=maxRequestsCount) return NO;
            if (![self canRunAgainstRequest:aRequest]) return NO;
        }

    }

    return YES;
}

- (BOOL) canRunAgainstRequest:(Request *)runningRequest
{
    //  ResultsRepository* resultsRepository= [ResultsRepository sharedResultsRepository];

    if ([(CvsRequest *)runningRequest path]) { //(NO) was here, but locks reappeared !
        return !([path isEqualToString:[(CvsRequest *)runningRequest path]]);
        //                ([[resultsRepository repositoryPathForPath: path] isEqualToString: [resultsRepository repositoryPathForPath: [(CvsRequest *)runningRequest path]]])) ;
    } else {
        return YES;
    }
}

- (unsigned) compressionLevel
{
    return defaultCompressionLevel;
}

#ifdef JA_PATCH
+(State *)initialState
{
    static BOOL triedInitialState=NO;
    static State *initialState=nil;

    if (!triedInitialState) {
        initialState=[[State initialStateForStateFile:[[NSBundle bundleForClass:[self class]] pathForResource:@"CvsTask" ofType:@"fsm"]] retain];
        triedInitialState=YES;
    }
    return initialState;
}

- (BOOL)readyForCvsTask
{
    return !noLogin;
}

- (BOOL)repositoryIsReady
{
    return [[self repository] isReadyForRequests];
}

- (BOOL)repositoryIsNotReady
{
    return ![[self repository] isReadyForRequests];
}

- (BOOL)repositoryFailed
{
    return (!repositoryRequest && ![[self repository] isReadyForRequests]);
}

- (BOOL)taskFailedBecauseOfAccess
{
    return noLogin;
}

- (BOOL)startTask
{
    noLogin=NO;
    return [super startTask];
}

- (void)waitForRepository
{
    repositoryRequest=[[[self repository] gettingReadyRequest] retain];
    if (repositoryRequest) {
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(repositoryDone:) name:@"RequestCompleted" object:repositoryRequest];
    } else {
        [self updateState];
    }
}

- (void)repositoryDone:(NSNotification *)notification
{
    [[NSNotificationCenter defaultCenter] removeObserver:self name:@"RequestCompleted" object:repositoryRequest];
    [repositoryRequest release];
    repositoryRequest=nil;
    [self updateState];
}
#endif

@end

//----------------------------------------------------------------------------------------------------------------------

@implementation CvsRequest (TemporaryHereWaitingForProperClasses)

- (NSString *)shortDescription
{
    return [NSString stringWithFormat: @"%@ in %@\n", [self cmdTitle], [self path]];
}

@end
