/* CvsStatusRequest.m created by ja on Thu 04-Sep-1997 */

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "CvsStatusRequest.h"
#import "../ResultsRepository.h"
#import <SenOyster/NSString.SenOysterRegex.h>
#import "../NSArray.SenCategorize.h"
#import "../CVLFile.h"
#import <SenFoundation/SenAssertion.h>

#define CVS_STATUS_STATUS_KEYWORD	@"status"
#define CVS_RESULT_VERSION_KEYWORD	@"Version"
#define CVS_REPOSITORY_VERSION_KEYWORD	@"Repository Version"
#define CVS_RESULT_CODATE_KEYWORD	@"Last Checkout"
#define CVS_RESULT_STAG_KEYWORD		@"Sticky Tag"
#define CVS_RESULT_SDATE_KEYWORD	@"Sticky Date"
#define CVS_RESULT_SOPTIONS_KEYWORD	@"Sticky Options"

#ifndef WIN32

NSString* CvsStatusExaminingPattern= @"/cvs status: Examining\\s+(.*)\\n/";
NSString* CvsStatusStatusPattern= @"/Status:\\s+(.*)\\n/";
NSString* CvsStatusRevisionPattern= @"/Working revision:\\s+(.*)\\n/";
NSString* CvsStatusStickyTagPattern= @"/Sticky Tag:\\s+(.*)\\n/";
NSString* CvsStatusStickyOptionsPattern= @"/Sticky Options:\\s+(.*)\\n/";
NSString* CvsStatusStickyDatePattern= @"/Sticky Date:\\s+(.*)\\n/";
NSString* CvsStatusLineOfEqualsPattern= @"/=+\\n/";

#else WIN32

NSString* CvsStatusExaminingPattern= @"/cvs status: Examining\\s+(.*)\\r\\n/";
NSString* CvsStatusStatusPattern= @"/Status:\\s+(.*)\\r\\n/";
NSString* CvsStatusRevisionPattern= @"/Working revision:\\s+(.*)\\r\\n/";
NSString* CvsStatusStickyTagPattern= @"/Sticky Tag:\\s+(.*)\\r\\n/";
NSString* CvsStatusStickyOptionsPattern= @"/Sticky Options:\\s+(.*)\\r\\n/";
NSString* CvsStatusStickyDatePattern= @"/Sticky Date:\\s+(.*)\\r\\n/";
NSString* CvsStatusLineOfEqualsPattern= @"/=+\\r\\n/";

#endif

/*
 Example of a full status output (without tag info):
===================================================================
File: fileName         	Status: aStatus

   Working revision:	aWorkAreaVersion	aWorkAreaRevisionDate
   Repository revision:	aRepositoryVersion
   Sticky Tag:		aStickyTag
   Sticky Date:		aStickyDate
   Sticky Options:	aStickyOption

 Note that aWorkAreaRevisionDate does NOT appear in client/server mode!
 Lines up to Repository revision ALWAYS appear.
 fileName can be "no file %s"
 aWorkAreaVersion can be "No entry for %s", or "New file!"
 aRepositoryVersion can be "No revision control file"
 aStickyDate, aStickyTag, aStickyOption can be "(none)".
 aStickyTag can be "aBranch (branch)" or "aRevision (revision)", or " - MISSING from RCS file!"
*/

@interface CvsStatusRequest (Private)
- (void)initResult;
@end

@implementation CvsStatusRequest
+ (CvsStatusRequest *)cvsStatusRequestForFiles:(NSArray *)someFiles inPath:(NSString *)aPath
{
    NSArray *checkedFiles=nil;
    NSDictionary* pathDict;
    NSString* commonPath;

    if (someFiles) {
        pathDict=[[self class] canonicalizePath: aPath andFiles: someFiles];
        commonPath= [[pathDict allKeys] objectAtIndex: 0];
        checkedFiles= [pathDict objectForKey: commonPath];
    } else {
        commonPath=aPath;
    }

    return [self requestWithCmd:0 title:@"status" path: commonPath files: checkedFiles];
}

- (void)dealloc
{
    [parsingBuffer release];
    [result release];
    [statusCmdDir release];
    
    [super dealloc];
}

- (NSArray *)cvsOptions
{
    NSArray		*previousOptions = [super cvsOptions];
    NSArray		*newOptions = nil;
    unsigned	compressionLevel = [self compressionLevel];

    if(compressionLevel)
        newOptions = [NSArray arrayWithObject:[NSString stringWithFormat:@"-z%d", compressionLevel]];

    if(newOptions)
        return [previousOptions arrayByAddingObjectsFromArray:newOptions];
    else
        return previousOptions;
}

- (NSArray *)cvsCommandOptions
{
    return [NSArray arrayWithObject:@"-l"];
}

- (NSArray *)cvsCommandArguments
{
    return [self files];
}

- (NSString *)cvsWorkingDirectory
{
    return [self path];
}

-(NSDictionary *)result
{
  return result;
}

- (void)initResult
{
    NSMutableDictionary *resultDict;
    NSString *filePath;
    CVLFile *file;
    id argsEnumerator;
    ResultsRepository *resultsRepository=[ResultsRepository sharedResultsRepository];

    result=[[NSMutableDictionary alloc] init];
    if ([self files]) {
        [resultsRepository startUpdate];

        argsEnumerator=[[self files] objectEnumerator];
        while (filePath=[argsEnumerator nextObject]) {
            filePath=[path stringByAppendingPathComponent:filePath];
            file=[CVLFile treeAtPath:filePath];
            if([file isLeaf]){
               // NSCalendarDate* pastDate= [[NSDate distantPast] dateWithCalendarFormat: NULL
               //                                                             timeZone: NULL];
                resultDict=[NSMutableDictionary dictionary];
                [resultDict setObject:@"Unknown" forKey:CVS_STATUS_STATUS_KEYWORD];
                //[resultDict setObject:@"none" forKey:CVS_RESULT_VERSION_KEYWORD];
                // [resultDict setObject: pastDate forKey:CVS_RESULT_CODATE_KEYWORD];
                //[resultDict setObject:@"none" forKey: CVS_REPOSITORY_VERSION_KEYWORD];
                [result setObject:resultDict forKey:filePath];
            }
        }
        [resultsRepository endUpdate];
    }
}

- (BOOL)setUpTask
{
    BOOL up;

    up=[super setUpTask];

#ifdef JA_PATCH
    if (!result) {
        [self initResult];
    }
    if (up) {
        parsingBuffer=[[NSMutableString alloc] init];
    }
#else
    if (up) {
        if (!result) {
            [self initResult];
        }
        parsingBuffer=[[NSMutableString alloc] init];
    }
#endif

    return up;
}

- (void) readStatusFrom: (NSString*) aString
    // scan 'aString' and update status, if any
{
  NSAutoreleasePool *subpool = [[NSAutoreleasePool alloc] init];
  NSMutableDictionary *statusDict;
  NSString* aPath= NULL;
  NSString* aFilename= NULL;
//  NSArray* matchArray= [aString componentsMatchedByOperator: CvsStatusExaminingPattern]; obsolete (120)
  NSArray* matchArray= nil;

  [statusCmdDir release];
#if 0
  // obsolete (120)
  if ([matchArray count] > 0) {
      statusCmdDir= [[matchArray objectAtIndex: 0] retain];
      SEN_DEBUG(@"Examining is still alive!");
  } else
#endif
  {	//	get path info from cmd queue
    ECFileFlags fileFlags;

    statusCmdDir=[self path];
    fileFlags=[(CVLFile *)[CVLFile treeAtPath:statusCmdDir] flags];
    [statusCmdDir retain];
  }
  matchArray= [aString componentsMatchedByOperator: @"/File:\\s+(no file)?\\s*(.*)\\t/"];
  if ([matchArray count] > 1)
  {		// assign after removing trailing spaces
    aFilename= [[matchArray objectAtIndex: 1] stringByApplyingReplacementOperator: @"s/\\s+$//g"];
    aPath= [statusCmdDir stringByAppendingPathComponent: aFilename];
    statusDict=[result objectForKey:aPath];
    if (!statusDict) {
      statusDict=[NSMutableDictionary dictionary];
      [result setObject:statusDict forKey:aPath];
    }
    matchArray= [aString componentsMatchedByOperator: CvsStatusStatusPattern]; //(120)
    if ([matchArray count] > 0) {
      NSString* statusString= [matchArray objectAtIndex: 0];
      [statusDict setObject:statusString forKey:CVS_STATUS_STATUS_KEYWORD];
    }
    matchArray= [aString componentsMatchedByOperator: CvsStatusRevisionPattern];
    if ([matchArray count] > 0)
    {
      NSArray* splitArray= NULL;
      NSString* revisionString= [matchArray objectAtIndex: 0];
      splitArray= [revisionString componentsSeparatedByOperator: @"/\\t/"];
      if ([splitArray count] > 1) {
          NSString* dateString= [splitArray objectAtIndex: 1];
          // cvs outputs time and dates in LOCAL timezones! Except for the cvs log command which returns UTC timezone
          // (Stephane) It seems that this is wrong: it outputs an UTC date here...
          NSCalendarDate* aDate= [NSCalendarDate dateWithString: [dateString stringByAppendingString: @" +0000"]
                                                 calendarFormat: @"%a %b %d %H:%M:%S %Y %z"];

          [statusDict setObject: [splitArray objectAtIndex: 0] forKey: CVS_RESULT_VERSION_KEYWORD];
          // BUG 1000063: it may happen that second element is NOT a date, but a description like "Result of merge"!
          if(aDate)
              [statusDict setObject:  aDate forKey: CVS_RESULT_CODATE_KEYWORD];
      }
      else
      {
        [statusDict setObject: revisionString forKey: CVS_RESULT_VERSION_KEYWORD];
      }
      matchArray= [aString componentsMatchedByOperator: @"/Repository revision:\\s+(.*)\\t/"];
      // this pattern could be improved to get repository file path (same as above: match without \t and then split on \t's)
      if ([matchArray count] > 0)
      {
        NSString* repoRevisionString= [matchArray objectAtIndex: 0];
        [statusDict setObject: repoRevisionString forKey: CVS_REPOSITORY_VERSION_KEYWORD];
      }
      matchArray= [aString componentsMatchedByOperator: CvsStatusStickyTagPattern];
      if ([matchArray count] > 0)
      {
        NSString* stickyTagString= [matchArray objectAtIndex: 0];
        if (![stickyTagString isEqualToString: @"(none)"])
        {
          [statusDict setObject: stickyTagString forKey: CVS_RESULT_STAG_KEYWORD];
        }
      }
      matchArray= [aString componentsMatchedByOperator: CvsStatusStickyOptionsPattern];
      if ([matchArray count] > 0)
      {
        NSString* stickyOptionString= [matchArray objectAtIndex: 0];
        if (![stickyOptionString isEqualToString: @"(none)"])
        {
          [statusDict setObject: stickyOptionString forKey: CVS_RESULT_SOPTIONS_KEYWORD];
        }
      }
      matchArray= [aString componentsMatchedByOperator: CvsStatusStickyDatePattern];
      if ([matchArray count] > 0)
      {
        NSString* stickyDateString= [matchArray objectAtIndex: 0];
        if (![stickyDateString isEqualToString: @"(none)"])
        {
            // cvs outputs time and dates in LOCAL timezones! Except for the cvs log command which returns UTC timezone
            // (Stephane) This is wrong: it outputs an UTC date here...
            NSCalendarDate* aDate= [NSCalendarDate dateWithString: [stickyDateString stringByAppendingString: @" +0000"]
                                                 calendarFormat: @"%Y.%m.%d.%H.%M.%S %z"];

          if (aDate)
            [statusDict setObject: aDate forKey: CVS_RESULT_SDATE_KEYWORD];
          else
              NSLog(@"-[CvsStatusRequest readStatusFrom:]: could not retrieve date from %@", stickyDateString);
        }
      }
    }
  }
  [subpool release];
} // readStatusFrom:


- (void)parseOutput:(NSString *)data
{
  NSRange sentinelRange;
  NSArray* results= nil;
  int rCount= 0;
  int j= 0;

  [parsingBuffer appendString: data];
  // sentinel to avoid deleting last split pettern
  sentinelRange.length= 2;
  [parsingBuffer appendString: @"xx"];

  // split sur ====
  results= [parsingBuffer componentsSeparatedByOperator: CvsStatusLineOfEqualsPattern];
  rCount= [results count];
  for (j= 0; j< rCount-1; j++)
  {	// give all results to the reader
    NSString* aResult= [results objectAtIndex: j];
    if ([aResult length] > .0)
    {
      [self readStatusFrom: aResult];
    }
  }
  [parsingBuffer setString: [results objectAtIndex: rCount -1]];
  sentinelRange.location= [parsingBuffer length] -  sentinelRange.length;
  [parsingBuffer deleteCharactersInRange: sentinelRange];
}


- (void)parseError:(NSString *)data
{
    [self parseOutput: data];
    [super parseError:data];
}

#ifdef JA_PATCH
- (void)flushBuffers
{
    [self readStatusFrom: parsingBuffer];
    [parsingBuffer release];
    parsingBuffer=nil;
}

- (void)endWithSuccess;
{
    [self flushBuffers];
}

- (void)endWithFailure;
{
    [self flushBuffers];
}
#else
- (void)end
{
    [self readStatusFrom: parsingBuffer];
    [parsingBuffer release];
    parsingBuffer=nil;

    if (!result) {
        [self initResult];
    }
    [super endWithoutInvalidation];
}
#endif

- (int)priority
{
    return -10;
}

@end
