/* CvsUpdateRequest.m created by ja on Fri 29-Aug-1997 */

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "CvsUpdateRequest.h"
#import <Foundation/NSArray.h>
#import <Foundation/NSFileHandle.h>
#import <Foundation/NSFileManager.h>
#import "CVLUnwrapRequest.h"
#import "../CVLFile.h"


@implementation CvsUpdateRequest

#ifdef WIN32
+ (void) initialize
{
    static BOOL	initialized = NO;

    [super initialize];
    if(!initialized){
        initialized = YES;
        [[NSUserDefaults standardUserDefaults] registerDefaults:[NSDictionary dictionaryWithObject:@"YES" forKey:@"DisableUnwrap"]];
    }
}
#endif

+ (CvsUpdateRequest *)cvsUpdateRequestForFiles:(NSArray *)someFiles inPath:(NSString *)aPath
{
    return [self cvsUpdateRequestForFiles: someFiles inPath: aPath revision: nil date: nil];
}

+ (CvsUpdateRequest *)cvsUpdateRequestForFiles:(NSArray *)someFiles inPath:(NSString *)aPath removesStickyAttributes:(BOOL)removesStickyAttributesFlag
{
    return [self cvsUpdateRequestForFiles: someFiles inPath: aPath revision: nil date: nil removesStickyAttributes:removesStickyAttributesFlag];
}

+ (CvsUpdateRequest *)cvsUpdateRequestForFiles:(NSArray *)someFiles inPath:(NSString *)aPath revision:(NSString *)aRevision date:(NSString *)aDate
{
    return [self cvsUpdateRequestForFiles:someFiles inPath:aPath revision:aRevision date:aDate removesStickyAttributes:NO];
}

+ (CvsUpdateRequest *)cvsUpdateRequestForFiles:(NSArray *)someFiles inPath:(NSString *)aPath revision:(NSString *)aRevision date:(NSString *)aDate removesStickyAttributes:(BOOL)removesStickyAttributesFlag
{
    NSArray *checkedFiles=nil;
    NSDictionary* pathDict;
    NSString* commonPath;
    CvsUpdateRequest *newRequest;

    if (someFiles) {
        pathDict=[[self class] canonicalizePath: aPath andFiles: someFiles];
        commonPath= [[pathDict allKeys] objectAtIndex: 0];
        checkedFiles= [pathDict objectForKey: commonPath];
    } else {
        commonPath=aPath;
    }

    newRequest=[self requestWithCmd:2 title:@"update" path: commonPath files: checkedFiles];
    
    if (aRevision)
    {
        [newRequest setRevision:aRevision];
    }
    if (aDate)
    {
        [newRequest setDate:aDate];
    }
    [newRequest setRemovesStickyAttributes:removesStickyAttributesFlag];
    return newRequest;
}

+ (CvsUpdateRequest *)cvsUpdateRequestForFile:(NSString *)file inPath:(NSString *)aPath revision:(NSString *)aRevision date:(NSString *)aDate toFile:(NSString *)fullPath
{
    return [self cvsUpdateRequestForFile:file inPath:aPath revision:aRevision date:aDate removesStickyAttributes:NO toFile:fullPath];
}

+ (CvsUpdateRequest *)cvsUpdateRequestForFile:(NSString *)file inPath:(NSString *)aPath revision:(NSString *)aRevision date:(NSString *)aDate removesStickyAttributes:(BOOL)removesStickyAttributesFlag toFile:(NSString *)fullPath
{
    NSDictionary* pathDict= [[self class] canonicalizePath: aPath andFiles: [NSArray arrayWithObject:file]];
    NSString* commonPath= [[pathDict allKeys] objectAtIndex: 0];
    CvsUpdateRequest *newRequest;

    newRequest=[self requestWithCmd:2 title:@"update" path: commonPath files: [pathDict objectForKey: commonPath]];
    [newRequest setIsQuiet: NO];
    if (aRevision)
    {
        [newRequest setRevision:aRevision];
    }
    if (aDate)
    {
        [newRequest setDate:aDate];
    }
    [newRequest setDestinationFilePath:fullPath];
    [newRequest setRemovesStickyAttributes:removesStickyAttributesFlag];
    return newRequest;
}

- (void)dealloc
{
    [destinationFile release];
    [revision release];
    [date release];

    [super dealloc];
}

- (void)setRevision:(NSString *)aRevision
{
    [aRevision retain];
    [revision release];
    revision=aRevision;
}

- (void)setDate:(NSString *)aDate
{
    [aDate retain];
    [date release];
    date=aDate;
}

- (void) setRemovesStickyAttributes:(BOOL)flag
{
    removesStickyAttributes = flag;
}

- (void)setDestinationFilePath:(NSString *)fullPath
{
    [fullPath retain];
    [destinationFile release];
    destinationFile=fullPath;
}

- (NSString *) destinationFilePath
{
    return destinationFile;
}

- (id)outputFile
{
    if (destinationFile) {
      NSFileHandle *file;
      NSFileManager* fileManager= [NSFileManager defaultManager];

      // Stephane: there is currently a bug with cvs and wrappers: wrappers are not restored correctly
      // when doing an update with date or revision or tag: they are still wrapped (gnutar-gzipped)!!
      // What we could do is trying to gunzip and gnutar xf them...
      if (![fileManager fileExistsAtPath: destinationFile])
      {
        [fileManager createFileAtPath: destinationFile
                             contents: nil
                           attributes: [NSDictionary dictionaryWithObject: [NSNumber numberWithInt: 420]
                                                                   forKey: NSFilePosixPermissions]]; // = o644
                                                                                                     // Stephane: could it be here that we have problems with NFS?
      }
      file=[NSFileHandle fileHandleForWritingAtPath:destinationFile];
      if (file) {
        return file;
      } else {
        return nil;
      }
    } else {
      return [super outputFile];
    }
}


- (NSArray *)cvsOptions
{
    NSArray		*previousOptions = [super cvsOptions];
    NSArray		*newOptions = nil;
    unsigned	compressionLevel = [self compressionLevel];

    if(compressionLevel)
        newOptions = [NSArray arrayWithObjects:@"-q", [NSString stringWithFormat:@"-z%d", compressionLevel], nil];

    if(newOptions)
        return [previousOptions arrayByAddingObjectsFromArray:newOptions];
    else
        return [previousOptions arrayByAddingObject:@"-q"];
}


- (NSArray *)cvsCommandOptions
{
    NSMutableArray *options;

    options=[NSMutableArray arrayWithObjects:@"-d", @"-P", nil];
    if (destinationFile)
    {
      [options insertObject: @"-p" atIndex: 0];
    }
    if (revision) {
        [options addObject:@"-r"];
        [options addObject:revision];
    }
    if (date) {
        [options addObject:@"-D"];
        [options addObject:date];
    }
    if(removesStickyAttributes)
        [options addObject:@"-A"];

    return options;
}

- (NSArray *)cvsCommandArguments
{
    return [self files];
}

- (NSString *)cvsWorkingDirectory
{
    return [self path];
}

- (void) unwrapEnd:(NSNotification *)aNotif
{
    // We need to rename the unwrapped file to the destination file
    (void)[[NSFileManager defaultManager] movePath:[[destinationFile stringByDeletingLastPathComponent] stringByAppendingPathComponent:[files lastObject]] toPath:destinationFile handler:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:[aNotif name] object:aNotif];
    [super endWithoutInvalidation];
}

#ifdef JA_PATCH
- (void)endWithSuccess;
{
    if (!destinationFile) [self updateFileStatuses];
}

- (void)endWithFailure;
{
    if (!destinationFile) [self updateFileStatuses];
}
#else
- (void)end
{
    if (destinationFile)
    {
        // In case of destinationFile, there is only one input file
        if(![[NSUserDefaults standardUserDefaults] boolForKey:@"DisableUnwrap"] && [(CVLFile *)[CVLFile treeAtPath:[[self path] stringByAppendingPathComponent:[[self files] lastObject]]] isRealWrapper]){
            CVLUnwrapRequest	*unwrapRequest = [CVLUnwrapRequest unwrapRequestForWrapper:destinationFile];

            [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(unwrapEnd:) name:@"RequestCompleted" object:unwrapRequest];
            [unwrapRequest schedule];
        }
        else
            [super endWithoutInvalidation];
    }
    else
    {
        [super end];
    }
}
#endif

@end
