/* CvsVerboseStatusRequest.m created by vincent on Fri 22-May-1998 */
/*$Id: CvsVerboseStatusRequest.m,v 1.6 2002/07/04 12:31:32 stephane Exp $*/

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "CvsVerboseStatusRequest.h"
#import <SenOyster/NSString.SenOysterRegex.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSAutoreleasePool.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSPathUtilities.h>

#ifndef WIN32

static NSString* CvsExistingTagsPattern = @"/\\s+Existing Tags:\\n/";

#else

static NSString* CvsExistingTagsPattern = @"/\\s+Existing Tags:\\r\\n/";

#endif


@implementation CvsVerboseStatusRequest
+ (CvsVerboseStatusRequest *)cvsVerboseStatusRequestForFiles:(NSArray *)someFiles inPath:(NSString *)aPath
{
    return [self requestWithCmd:13 title:@"get tags" path:aPath files:someFiles];
}

- (void)dealloc
{
    [parsingBuffer release];
    [result release];
    [tagsPath release];

    [super dealloc];
}

- (NSString*) cvsCommand
{
    return @"status";
}

- (NSArray *)cvsCommandOptions
{
    return [NSArray arrayWithObject: @"-lv"];
}

- (NSArray *)cvsCommandArguments
{
    return [self files];
}

- (NSString *)cvsWorkingDirectory
{
    return [self path];
}

-(NSDictionary *)result
{
  return result;
}

- (void)initResult
{
    result=[[NSMutableDictionary alloc] init];
}

- (BOOL)setUpTask
{
#ifdef JA_PATCH
    if (!result) {
        [self initResult];
    }

    if ([super setUpTask]) {
        parsingBuffer=[[NSMutableString alloc] init];

        return YES;
    } else {
        return NO;
    }
#else
    if ([super setUpTask]) {

        parsingBuffer=[[NSMutableString alloc] init];

        if (!result) {
            [self initResult];
        }
        return YES;
    } else {
        return NO;
    }
#endif
}

- (void) readTagsFrom: (NSString*) aString
  // scan 'aString' and update tags info, if any
{
    NSAutoreleasePool *subpool = [[NSAutoreleasePool alloc] init];
    NSArray* matchArray= [aString componentsSeparatedByOperator: CvsExistingTagsPattern];
    if ([matchArray count] == 2)
    {
        NSString* infoPart= [matchArray objectAtIndex: 0];
        NSString* tagsPart= [matchArray objectAtIndex: 1];
        NSArray* linesArray;
        int i= 0;

        matchArray= [infoPart componentsMatchedByOperator: @"/File:\\s+(no file)?\\s*(.*)\\t/"];
        if ([matchArray count] > 0)
        { 		// assign after removing trailing spaces
            NSString* aFilename= [[matchArray objectAtIndex: 1] stringByApplyingReplacementOperator: @"s/\\s+$//g"];

            [tagsPath release];
            tagsPath= [[[self path] stringByAppendingPathComponent: aFilename] retain];
        }

        linesArray= [tagsPart componentsSeparatedByOperator: CvsRequestNewLinePattern];
        i= [linesArray count];

        while (i--)
        {
            NSString* line= [linesArray objectAtIndex: i];

            matchArray= [line componentsMatchedByOperator:@"/\\t+(.*)\\s+\\((.*):\\s+(.*)\\)/"]; //\s+(.*)\s+\((.*):\s+(.*)\)
            if ([matchArray count] > 0)
            { // all infos are present, set them into dictionary
                NSMutableDictionary* resultDict= [result objectForKey: tagsPath];
                NSMutableArray* newArray= [NSMutableArray arrayWithCapacity: 0];
                NSMutableDictionary *tagsDict= [NSMutableDictionary dictionaryWithCapacity: 0];

                NSString* currentTag= [[matchArray objectAtIndex: 0] stringByApplyingReplacementOperator: @"s/\\s+$//g"];
                NSString* tagKind= [matchArray objectAtIndex: 1];
                NSString* revision= [matchArray objectAtIndex: 2];

                if (!resultDict)
                {
                    resultDict=[NSMutableDictionary dictionary];
                    [result setObject:resultDict forKey: tagsPath];
                }
                [tagsDict setObject: currentTag forKey: @"tag"];
                [tagsDict setObject: tagKind forKey: @"kind"];
                [tagsDict setObject: revision forKey: @"revision"];

                [newArray addObjectsFromArray: [resultDict objectForKey: @"tags"]];
                [newArray addObject: tagsDict];
                [resultDict setObject: newArray forKey: @"tags"];
            }
        }
    }
    [subpool release];
} // readTagsFrom:

- (void)parseOutput:(NSString *)data
{
    NSRange sentinelRange;
    NSArray* results= nil;
    int rCount= 0;
    int j= 0;

    [parsingBuffer appendString: data];
    // sentinel to avoid deleting last split pettern
    sentinelRange.length= 2;
    [parsingBuffer appendString: @"xx"];

    // split sur ==== ou sur ---
    results= [parsingBuffer componentsSeparatedByOperator: CvsRequestLineOfEqualsOrMinusPattern];
    rCount= [results count];
    j= 0;
    for (j= 0; j< rCount-1; j++)
    {	// give all results to the reader
        NSString* aResult= [results objectAtIndex: j];
        if ([aResult length] > .0)
        {
            [self readTagsFrom: aResult];
        }
    }
    [parsingBuffer setString: [results objectAtIndex: rCount -1]];
    sentinelRange.location= [parsingBuffer length] -  sentinelRange.length;
    [parsingBuffer deleteCharactersInRange: sentinelRange];
} //

#ifdef JA_PATCH
- (void)flushBuffers
{
    [self readTagsFrom: parsingBuffer];
    [parsingBuffer release];
    parsingBuffer=nil;
}

- (void)endWithSuccess;
{
    [self flushBuffers];
}

- (void)endWithFailure;
{
    [self flushBuffers];
}
#else
- (void)end
{
    [self readTagsFrom: parsingBuffer];
    [parsingBuffer release];
    parsingBuffer=nil;

    if (!result) {
        [self initResult];
    }
    [super endWithoutInvalidation];
}
#endif

@end

