/* Request.h created by vincent on Fri 14-Nov-1997 */

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import <Foundation/NSObject.h>
#ifdef JA_PATCH
#import "State.h"
#endif


@class NSSet, NSMutableSet;


#define STATE_IDLE 0
#define STATE_READY 4
#define STATE_RUNNING 1
#define STATE_WAITING 2
#define STATE_ENDED 3

#define STATE_NOT_STARTED 0

#ifndef WIN32

extern NSString* RequestNewLineString;

#else WIN32

extern NSString* RequestNewLineString;

#endif


@interface Request : NSObject
{
#ifdef JA_PATCH
    NSString *cmdTitle;
    BOOL success;
    int order;

    State *currentState;
#else
    NSString *cmdTitle;
    int state;
    int internalState;
    int endConditionsCount;
    NSMutableSet *precedingRequests;
    BOOL success;
    int order;
#endif
}

#ifdef JA_PATCH
+ (State *)initialState;
- (State *)currentState;
- (void)updateState;
#endif

+ requestWithTitle:(NSString *)cmdString;
- initWithTitle:(NSString *)cmdString;

- (void)schedule;
#ifndef JA_PATCH
- (int)state;
- (void)setState:(int)aState;
- (void)start;

// methods used by the Scheduler, modify the state and send a notification
- (void)resumeNow; // NB: there is a method -(BOOL)resume declared in NSSound
#endif
- (void)setOrder:(int)aNumber;

- (NSComparisonResult)hasPriorityComparedToRequest:(Request *)anotherRequest;
- (int)priority;
- (int)order;

// methods called by other objects
//- (void)stop;
//- (void)pause;
- (void)schedule;

#ifndef JA_PATCH
// methods called by the request 
- (void)end;
- (void)cancel;
- (BOOL) canBeCancelled;

- (BOOL)addPrecedingRequest:(Request *)aRequest;
- (NSSet *)precedingRequests;
- (BOOL) canContinue; // called by scheduler for transition WAITING->RUNNING
- (BOOL) precedingRequestsEnded;
#endif
- (BOOL) canRunAgainstRequests:(NSSet *)runningRequests;
- (BOOL) canRunAgainstRequest:(Request *)runningRequest;

// query methods
- (NSString *)cmdTitle;
- (NSString *)summary;
- (NSString *)stateString;

#ifndef JA_PATCH
-(void)endConditionReached;
#endif

- (BOOL)succeeded;

- (NSString *)shortDescription;
@end
