/* Request.m created by vincent on Fri 14-Nov-1997 */

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "Request.h"
#import "../CVLScheduler.h"
#import <Foundation/NSEnumerator.h>
#import <Foundation/NSNotification.h>
#import <Foundation/NSSet.h>
#import <Foundation/NSString.h>

#ifndef WIN32

NSString* RequestNewLineString= @"\n";

#else WIN32

NSString* RequestNewLineString= @"\r\n";

#endif


#ifndef JA_PATCH
@interface Request (Private)
- (void)end;
- (void)endConditionReached;
- (void)precedingRequestCompleted:(NSNotification *)notification;
@end
#endif

@implementation Request
+ requestWithTitle:(NSString *)cmdString
{
  return [[[self alloc] initWithTitle: cmdString] autorelease];
}


- initWithTitle:(NSString *)cmdString
{
  self=[self init];
  cmdTitle=[cmdString retain];
#ifndef JA_PATCH
  state=STATE_IDLE;
#endif
  
  return self;
}

-(void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [cmdTitle release];
#ifndef JA_PATCH
    [precedingRequests release];
#endif
    
    [super dealloc];
}

#ifndef JA_PATCH
-(void)start
{
    success=YES;
    [self setState:STATE_RUNNING];
}

- (void)resumeNow
{
    if (internalState==STATE_NOT_STARTED) {
        [self start];
    }
}

-(void)endConditionReached
{
    endConditionsCount--;
    if (!endConditionsCount) {
        [self end];
    }
}

-(void)cancel
{
    if([self canBeCancelled]){
        success=NO;
        [self end];
    }
}

- (BOOL) canBeCancelled
{
    return [self state] != STATE_ENDED;
}

-(void)end
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [self setState:STATE_ENDED];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"RequestCompleted" object:self];
    [self release];
}
#endif

-(NSString *)cmdTitle
{
  return cmdTitle;
}

-(NSString *)summary
{
    return @"";
}

#ifdef JA_PATCH
-(State *)currentState
{
    return currentState;
}

- (NSString *)stateString
{
    return [currentState valueForKey:@"description"];
}
#else
-(int)state
{
  return state;
}

- (NSString *)stateString
{
    switch (state) {
        case STATE_IDLE:
            return @"";
            break;

        case STATE_READY:
            return @"Ready";
            break;
            
        case STATE_WAITING:
            return @"Waiting";
            break;

        case STATE_RUNNING:
            return @"Running";
            break;

        case STATE_ENDED:
            if (success)
                return @"Done";
            else
                return @"Canceled";
            
            break;
    }

    return @"";
}

- (void)setState:(int)aState
{
    state=aState;
    [[NSNotificationCenter defaultCenter] postNotificationName:@"RequestStateChanged" object:self];
}

- (BOOL)addPrecedingRequest:(Request *)aRequest
{
    if (!precedingRequests) {
        precedingRequests=[[NSMutableSet alloc] initWithCapacity:1];
    }
    [precedingRequests addObject:aRequest];
    [[NSNotificationCenter defaultCenter] addObserver:self                    selector:@selector(precedingRequestCompleted:)
                                                 name:@"RequestCompleted"
                                               object:aRequest];

    return YES;
}

- (void)precedingRequestCompleted:(NSNotification *)notification
{
    Request *aRequest=[notification object];
    
    [[NSNotificationCenter defaultCenter] removeObserver:self
                                                    name:@"RequestCompleted"
                                                  object:aRequest];
    if (![aRequest succeeded]) {
        [self cancel];
    } else {
        [precedingRequests removeObject:aRequest];
        if ([self canContinue]) {
            [self setState:STATE_READY];
        }
    }
}

- (NSSet *)precedingRequests
{
    return precedingRequests;
}

- (BOOL) precedingRequestsEnded
{
    return ([[self precedingRequests] count]==0);
}
#endif

- (BOOL) canRunAgainstRequests:(NSSet *)runningRequests
{
    id enumerator;
    Request *aRequest;

    enumerator=[runningRequests objectEnumerator];
    while (aRequest=[enumerator nextObject]) {
        if (![self canRunAgainstRequest:aRequest]) return NO;
    }
    return YES;
}

- (BOOL) canRunAgainstRequest:(Request *)runningRequest
{
    return YES;
}

#ifndef JA_PATCH
- (BOOL) canContinue
{
    return [self precedingRequestsEnded];
}

- (void)schedule
{
    [self retain];
    [[CVLScheduler sharedScheduler] scheduleRequest:self];
    if ([self canContinue]) {
        [self setState:STATE_READY];
    } else {
        [self setState:STATE_WAITING];
    }
}
#endif

- (BOOL)succeeded
{
    return success;
}

- (NSString *)shortDescription
{
    return cmdTitle;
}

- (void)setOrder:(int)aNumber
{
    order=aNumber;
}

- (int)order
{
    return order;
}

- (int)priority
{
    return 0;
}

- (NSComparisonResult)hasPriorityComparedToRequest:(Request *)anotherRequest
{
    int comparison;

    comparison=[self priority]-[anotherRequest priority];

    if (comparison==0) comparison=[anotherRequest order]-[self order];
    
    if (comparison==0) return NSOrderedSame;
    if (comparison<0) return NSOrderedDescending;
    return NSOrderedAscending;
}

#ifdef JA_PATCH
+ (State *)initialState
{
    return nil;
}

- (void)updateState
{
    State *nextState;

    if (!currentState) {
        nextState=[[self class] initialState];
        NSLog (@"Next state",nextState);
    } else {
        nextState=[currentState nextStateForMachine:self];
    }
    if (nextState) {
        SEL selector=[nextState enteringSelector];

        [nextState retain];
        [currentState release];
        currentState=nextState;
        if (selector!=NULL) {
            [self performSelector:selector];
        }
        [[NSNotificationCenter defaultCenter] postNotificationName:@"RequestStateDidChange" object:self];
        if ([currentState isTerminal]) {
            [[NSNotificationCenter defaultCenter] postNotificationName:@"RequestCompleted" object:self];
        } else {
            [self updateState];
        }
    }
}

- (void)schedule
{
    [[CVLScheduler sharedScheduler] scheduleRequest:self];
    [self updateState];
}
#endif

@end
