/* TaskRequest.m created by ja on Mon 24-Nov-1997 */
/*$Id: TaskRequest.m,v 1.16 2002/07/04 12:33:50 stephane Exp $*/

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "TaskRequest.h"
#import <AppKit/NSPanel.h>
#import <AppKit/NSApplication.h> // For NSModalPanelRunLoopMode
#ifdef MACOSX
#import <Foundation/NSCharacterSet.h>
#import <Foundation/NSNotification.h>
#import <Foundation/NSException.h>
#import <Foundation/NSUserDefaults.h>
#import <Foundation/NSFileHandle.h>
#import <Foundation/NSTask.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSProcessInfo.h>
#import <Foundation/NSRunLoop.h>
#endif

#define MIN_DATA_LEN_BEFORE_NOTIF 50

static NSCharacterSet* newLineCharacterSet= nil;
static BOOL				taskTracingEnabled = NO;

#ifndef JA_PATCH
@interface TaskRequest (Private)
- (void)cancel;
- (void)end;
- (void)endConditionReached;
@end
#endif


@implementation TaskRequest

+ (void) initialize
{
    [super initialize];
    taskTracingEnabled = [[NSUserDefaults standardUserDefaults] boolForKey:@"TaskTracingEnabled"];
}

- init
{
    self = [super init];

    lastReceivedString= [[NSMutableString stringWithCapacity: 0] retain];
    lastReceivedErrorString= [[NSMutableString stringWithCapacity: 0] retain];
    if (!newLineCharacterSet)
    {
        newLineCharacterSet= [[NSCharacterSet characterSetWithCharactersInString: RequestNewLineString] retain];
    }
    parseByLine=YES;
    
    return self;
}

-(void) dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [lastReceivedString release];
    [lastReceivedErrorString release];
    [errorPipe release];
    [pipe release];
    [readHandle release];
    [errorReadHandle release];
    [task release];
    [super dealloc];
}

-(void)start
{
#ifndef JA_PATCH
    [super start];

    if (![self startTask]) {
        [self cancel];
        return;
    }
    endConditionsCount++;
    internalState=STATE_TASK_RUN_TASK;
#endif
}

-(BOOL)startTask
{
#ifdef JA_PATCH
    success=YES;
    if ([self setUpTask]) {
        if(taskTracingEnabled)
            NSLog(@"%@> %@ %@ [%@]", [task currentDirectoryPath], [task launchPath], [task arguments], [task environment] );

        NS_DURING
            [[NSNotificationCenter defaultCenter] addObserver:self
                                                     selector:@selector(taskTerminated:)
                                                         name:NSTaskDidTerminateNotification
                                                       object:task];
            [task launch];
        NS_HANDLER
            [[NSNotificationCenter defaultCenter] removeObserver:self
                                                         name:NSTaskDidTerminateNotification
                                                       object:task];
            NSRunAlertPanel(@"Request problem", @"%@", @"OK", nil, nil,
                            localException);
            success=NO;
        NS_ENDHANDLER
    } else {
        success=NO;
    }
    return success;
#else
    BOOL returnValue=YES;
    if ([self setUpTask]) {
        if(taskTracingEnabled)
            NSLog(@"%@> %@ %@ [%@]", [task currentDirectoryPath], [task launchPath], [task arguments], [task environment] );

        NS_DURING
            [[NSNotificationCenter defaultCenter] addObserver:self
                                                     selector:@selector(taskTerminated:)
                                                         name:NSTaskDidTerminateNotification
                                                       object:task];
            [task launch];
            taskEndConditionsCount++;
        NS_HANDLER
            [[NSNotificationCenter defaultCenter] removeObserver:self
                                                         name:NSTaskDidTerminateNotification
                                                       object:task];
            NSRunAlertPanel(@"Request problem", @"%@", @"OK", nil, nil,
                            localException);
            returnValue=NO;
        NS_ENDHANDLER
    } else {
        returnValue=NO;
    }
    return returnValue;
#endif
}

- (void)taskCleanUp
{
    [task release];
    task=nil;
}

#ifdef JA_PATCH
- (BOOL)taskWasSuccessfull
{
    return (!task && !readHandle && !errorReadHandle && success);
}

- (BOOL)taskWasAFailure
{
    return (!task && !readHandle && !errorReadHandle && !success);
}
#else
- (void) cancel
{
    if([self canBeCancelled] && task && [task isRunning])
        [task interrupt]; // Should we call -terminate instead?
    else
        [super cancel];
}

- (void)taskEnded
{
    [self endConditionReached];
}

-(void)taskEndConditionReached
{
    taskEndConditionsCount--;
    if (!taskEndConditionsCount) {
        [self taskEnded];
    }
}
#endif

- (id)outputFile
{
    if (!pipe) {
        pipe = [NSPipe pipe];
        readHandle = [pipe fileHandleForReading];

        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(dataReceived:) name:NSFileHandleReadCompletionNotification object:readHandle];

        [readHandle readInBackgroundAndNotifyForModes:[NSArray arrayWithObjects:NSDefaultRunLoopMode, NSModalPanelRunLoopMode, nil]];
        [pipe retain];
        [readHandle retain];

        taskEndConditionsCount++;
    }
    return pipe;
}

- (id)errorFile
{
    if (!errorPipe) {
        errorPipe = [NSPipe pipe];
        errorReadHandle = [errorPipe fileHandleForReading];

        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(errorReceived:) name:NSFileHandleReadCompletionNotification object:errorReadHandle];

        [errorReadHandle readInBackgroundAndNotifyForModes:[NSArray arrayWithObjects:NSDefaultRunLoopMode, NSModalPanelRunLoopMode, nil]];
        [errorPipe retain];
        [errorReadHandle retain];

        taskEndConditionsCount++;
    }
    return errorPipe;
}

/*
- (void)close
{
    if (task) {
        [task terminate];
        [task release];
        [[NSNotificationCenter defaultCenter] removeObserver:self name:NSTaskDidTerminateNotification object:task];
        task=nil;
    }
    if (readHandle) {
        [readHandle release];
        [pipe release];
        [[NSNotificationCenter defaultCenter] removeObserver:self
                                                        name:@"NSFileHandleReadCompletionNotification"
                                                      object:readHandle];
    }
    if (errorReadHandle) {
        [errorReadHandle release];
        [errorPipe release];
        [[NSNotificationCenter defaultCenter] removeObserver:self
                                                        name:@"NSFileHandleReadCompletionNotification"
                                                      object:errorReadHandle];
    }
}*/

- (BOOL)isQuiet
{
    return isQuiet;
}

- (void)setIsQuiet:(BOOL)value
{
    isQuiet=value;
}


- (void) setTask: (NSTask*) aTask
{
    if (task != aTask)
    {
        [task release];
        task= [aTask retain];
    }
}

-(NSString *)summary
{
    return [[task arguments] description];
}

#ifdef JA_PATCH
- (void) resumeNow
{
    taskAuthorized=YES;
    [self updateState];
}

- (BOOL)authorizedToRun
{
    return taskAuthorized;
}
#endif

-(BOOL)setUpTask
{
    if (!task) {
        static NSDictionary	*env = nil;
        if(!env)env = [[[NSProcessInfo processInfo] environment] retain];
        task=[[NSTask alloc] init];
        [task setEnvironment:env];
        // It seems that -[NSProcessInfo environment] is time-consuming (15 samples @ 50ms = 0.75s)
		// If there is no internal cache, we should create our own cache, as it is often repeated
    }

    [task setStandardOutput:[self outputFile]];
    [task setStandardError:[self errorFile]];
    return YES;
}


- (void) effectiveStringReceived: (NSString*) aString
{
    [[NSNotificationCenter defaultCenter] postNotificationName:@"RequestReceivedData" object:self userInfo:[NSDictionary dictionaryWithObject:aString forKey:@"RequestNewData"]];
    if (parseByLine) [self parseOutput:aString];
}

- (void) effectiveErrorStringReceived: (NSString*) aString
{
    [[NSNotificationCenter defaultCenter] postNotificationName:@"RequestReceivedData" object:self userInfo:[NSDictionary dictionaryWithObjectsAndKeys: aString,@"RequestNewData", @"YES",@"ErrorStream",nil]];
    if (parseByLine) [self parseError:aString];
}


- (void)dataReceived:(NSNotification *)notification
{
    NSDictionary *info=[notification userInfo];
    NSString* receivedString= [[NSString alloc] initWithData: [info objectForKey:NSFileHandleNotificationDataItem]
                                                    encoding: [NSString defaultCStringEncoding]];
    NSRange newLineRange= [receivedString rangeOfCharacterFromSet: newLineCharacterSet];

    if(taskTracingEnabled && [receivedString length])
        NSLog(@"<< %@", receivedString);
    
    [lastReceivedString appendString: receivedString];
    if (!parseByLine) {
        [self parseOutput:receivedString];
    }
    
    if (newLineRange.length)
    {
        [self effectiveStringReceived: lastReceivedString];
        [lastReceivedString setString: @""]; 
    }
    if ([receivedString cStringLength])
    {
    [readHandle readInBackgroundAndNotifyForModes:[NSArray arrayWithObjects:NSDefaultRunLoopMode, NSModalPanelRunLoopMode, nil]];
    }
    else
    { //end of file
        if ([lastReceivedString cStringLength])
        {
            [self effectiveStringReceived: lastReceivedString];
            [lastReceivedString setString: @""]; 
        }
#ifdef JA_PATCH
        [self closeReadHandle];

        [self updateState];
#else
        [[NSNotificationCenter defaultCenter] removeObserver:self
                                                        name:NSFileHandleReadCompletionNotification
                                                      object:readHandle];
        [readHandle closeFile];
        [readHandle release];
        readHandle=nil;
        [pipe release];
        pipe=nil;

        [self taskEndConditionReached];
#endif
    }
    [receivedString release];
}

#ifdef JA_PATCH
- (void)closeReadHandle
{
    [[NSNotificationCenter defaultCenter] removeObserver:self name:NSFileHandleReadCompletionNotification object:readHandle];
    [readHandle closeFile];
    [readHandle release];
    readHandle=nil;
    [pipe release];
    pipe=nil;
}
#endif

- (void)errorReceived:(NSNotification *)notification
{
    NSDictionary *info=[notification userInfo];
    NSString* receivedString= [[NSString alloc] initWithData: [info objectForKey:NSFileHandleNotificationDataItem]
                                                    encoding: [NSString defaultCStringEncoding]];
    NSRange newLineRange= [receivedString rangeOfCharacterFromSet: newLineCharacterSet];

    if(taskTracingEnabled && [receivedString length])
        NSLog(@"#< %@", receivedString);

    [lastReceivedErrorString appendString: receivedString];
    if (!parseByLine) {
        [self parseError:receivedString];
    }

    if (newLineRange.length)
    {
        [self effectiveErrorStringReceived : lastReceivedErrorString];
        [lastReceivedErrorString setString: @""]; // reset
    }
    if ([receivedString cStringLength])
    {
    [errorReadHandle readInBackgroundAndNotifyForModes:[NSArray arrayWithObjects:NSDefaultRunLoopMode, NSModalPanelRunLoopMode, nil]];
    }
    else
    { //end of file
        if ([lastReceivedErrorString cStringLength])
        {
            [self effectiveErrorStringReceived: lastReceivedErrorString];
            [lastReceivedErrorString setString: @""]; // reset
        }

#ifdef JA_PATCH
        [self closeErrorReadHandle];

        [self updateState];
#else
        [[NSNotificationCenter defaultCenter] removeObserver:self
                                                        name:NSFileHandleReadCompletionNotification
                                                      object:errorReadHandle];
        [errorReadHandle closeFile];
        [errorReadHandle release];
        errorReadHandle=nil;
        [errorPipe release];
        errorPipe=nil;
        [self taskEndConditionReached];
#endif
    }
    [receivedString release];
}

#ifdef JA_PATCH
- (void)closeErrorReadHandle
{
    [[NSNotificationCenter defaultCenter] removeObserver:self name:NSFileHandleReadCompletionNotification object:errorReadHandle];
    [errorReadHandle closeFile];
    [errorReadHandle release];
    errorReadHandle=nil;
    [errorPipe release];
    errorPipe=nil;
}
#endif

- (id)taskTerminated:(NSNotification *)notification
{
    int status = [[notification object] terminationStatus];

    if (status) success=NO;
    else success=YES;
    if(taskTracingEnabled)
        if(status)
            NSLog(@"### Task termination status: %d", status);
        else
            NSLog(@"<No task error>");

    [[NSNotificationCenter defaultCenter] removeObserver:self name:NSTaskDidTerminateNotification object:task];
#ifdef JA_PATCH
    [task release];
    task=nil;

    [self updateState];
#else
    [self taskEndConditionReached];
#endif
    return self;
}

- (void)parseOutput:(NSString *)data
{
    return;
}

- (void)parseError:(NSString *)data
{
    return;
}

#ifdef JA_PATCH
+(State *)initialState
{
    static BOOL triedInitialState=NO;
    static State *initialState=nil;

    if (!triedInitialState) {
        initialState=[[State initialStateForStateFile:[[NSBundle bundleForClass:[self class]] pathForResource:@"Request" ofType:@"fsm"]] retain];
        triedInitialState=YES;
    }
    return initialState;
}
#endif

@end


