/*
 $Id: ResultsRepository.m,v 1.16 2002/06/17 16:26:13 kindov Exp $
 */

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "ResultsRepository.h"
#ifdef MACOSX
#import <CvsRequest.h>
#import <CvsRepository.h>
#import <CvsStatusRequest.h>
#import <CvsLogRequest.h>
#import <CvsDiffRequest.h>
#import <CvsVerboseStatusRequest.h>
#import <CvsQuickStatusRequest.h>
#else
#import <Requests/CvsRequest.h>
#import <Repositories/CvsRepository.h>
#import <Requests/CvsStatusRequest.h>
#import <Requests/CvsLogRequest.h>
#import <Requests/CvsDiffRequest.h>
#import <Requests/CvsVerboseStatusRequest.h>
#import <Requests/CvsQuickStatusRequest.h>
#endif
#import "CVLFile.h"
#import "NSArray.SenCategorize.h"
#import "NSArray.SenUtilities.h"
#import <Foundation/NSDictionary.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSNotification.h>
#import <Foundation/NSUtilities.h>
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSDate.h>
#import <AppKit/NSApplication.h> // For NSModalPanelRunLoopMode

static ResultsRepository *resultsRepository=nil;
@interface ResultsRepository (private)
- startUpdate:(NSNotification *)notification;
- endUpdate:(NSNotification *)notification;
- cmdEnded:(NSNotification *)notification;
- (void)propagateRecursiveChanges;
- (void)calculateChanges;
- (void)getShowRepositoryFiles:(NSNotification *)notification;
@end

@implementation ResultsRepository
+ (ResultsRepository *)sharedResultsRepository;
{
  if (!resultsRepository) {
    return resultsRepository=[[[self alloc] init] autorelease];
  } else {
    return resultsRepository;
  }
}

- init
{
  if (self=[super init]) {
    changedFiles=[[NSMutableSet alloc] init];
    statusQueue=[[NSMutableArray alloc] init];
    quickStatusQueue=[[NSMutableArray alloc] init];
    //statusRequests=[[NSMutableArray alloc] init];
    logQueue= [[NSMutableArray alloc] init];
    logRequests=[[NSMutableArray alloc] init];
    tagsQueue= [[NSMutableArray alloc] init];
    tagsRequests= [[NSMutableArray alloc] init];
    quickStatusTargetPaths = [[NSMutableSet alloc] init];
    [[NSNotificationCenter defaultCenter]
                  addObserver:self
                     selector:@selector(cmdEnded:)
                         name:@"RequestCompleted"
                       object:nil];

//    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(getShowRepositoryFiles:) name:@"PreferencesChanged" object:nil];
//    [self getShowRepositoryFiles:nil];
  }
  return self;
}

/*
- (void)getShowRepositoryFiles:(NSNotification *)notification
{
    BOOL oldShowRepositoryFiles=showRepositoryFiles;
    showRepositoryFiles=[[NSUserDefaults standardUserDefaults] boolForKey:@"ShowRepositoryFiles"];

    if (oldShowRepositoryFiles!=showRepositoryFiles) {
        [self invalidateResultsForDirWithPath:@"" andKey:@"dir" recursively:YES];
    }
} */

- (void)dealloc
{
  [[NSNotificationCenter defaultCenter]
                removeObserver:self];
  [changedFiles release];
  [statusQueue release];
  [quickStatusQueue release];
  //[statusRequests release];
  [logQueue release];
  [logRequests release];
  [tagsQueue release];
  [tagsRequests release];
  [quickStatusTargetPaths release];
  [super dealloc];
}

- (void)setStatusesFromDictionary:(NSDictionary *)aDictionary
{
  id enumerator;
  NSString *path;
  CVLFile *file;

  enumerator=[aDictionary keyEnumerator];
  [self startUpdate:nil];
  while (path=[enumerator nextObject]) {
      file=[CVLFile treeAtPath:path];
      [file setStatusFromDictionary:[aDictionary objectForKey:path]];
  }
  [self endUpdate:nil];
}

- (void)setQuickStatusesFromDictionary:(NSDictionary *)aDictionary successful:(BOOL)wasSuccessful parentPath:(NSString *)parentPath
{
    NSEnumerator	*enumerator = [aDictionary keyEnumerator];
    NSString		*path;

    [self startUpdate:nil];
    while(path = [enumerator nextObject]){
        CVLFile	*file = [CVLFile treeAtPath:path];

        [file setQuickStatusWithString:[aDictionary objectForKey:path]];
        [quickStatusTargetPaths removeObject:path];
    }
    // All other files are up-to-date (except ignored ones and files whose parent is unknown by cvs)!!!
    // We need to inform these files, because there status is currently set to loading.status = YES!
    // WARNING: it may happen that quickStatus is not finished and returns prematurately with an error
    // In this case, we need to perform a FULL STATUS on the remaining files!
    // SECOND WARNING: if two or more quickStatus are performed at the same time,
    // there is a concurrency problem: we may not remove files which are not children of the qs directory.
    enumerator = [quickStatusTargetPaths objectEnumerator];
    while(path = [enumerator nextObject]){
        if([path hasPrefix:parentPath]){
            CVLFile	*file = [CVLFile treeAtPath:path];

            if(!wasSuccessful){
                [file setQuickStatusWithString:@"!"];
            }
            else if([(CVLFile *)[file parent] status].tokenizedStatus == ECContainsNonCVSFiles)
                [file setQuickStatusWithString:@"?"];
            else
                [file setQuickStatusWithString:@"="];
            [quickStatusTargetPaths removeObject:path];
        }
    }
    [self endUpdate:nil];
//    [quickStatusTargetPaths removeAllObjects];
}

- (void)setLogsFromDictionary:(NSDictionary *)aDictionary
{
  id enumerator;
  NSString *path;
  CVLFile *file;

  enumerator=[aDictionary keyEnumerator];
  [self startUpdate:nil];
  while (path=[enumerator nextObject]) {
      file=[CVLFile treeAtPath:path];
      [file setLog:[[aDictionary objectForKey:path] objectForKey:@"log"]];
  }
  [self endUpdate:nil];
}

- (void)setDifferencesFromDictionary:(NSDictionary *)aDictionary
{
  id enumerator;
  NSString *path;
  CVLFile *file;

  enumerator=[aDictionary keyEnumerator];
  [self startUpdate:nil];
  while (path=[enumerator nextObject]) {
      file=[CVLFile treeAtPath:path];
      [file setDifferences:[[aDictionary objectForKey:path] objectForKey:@"diff"]];
  }
  [self endUpdate:nil];
}


- (void)setTagsFromDictionary:(NSDictionary *)aDictionary
{
  id enumerator;
  NSString *path;
  CVLFile *file;

  enumerator=[aDictionary keyEnumerator];
  [self startUpdate:nil];
  while (path=[enumerator nextObject]) {
      file=[CVLFile treeAtPath:path];
      [file setTags:[[aDictionary objectForKey:path] objectForKey:@"tags"]];
  }
  [self endUpdate:nil];
}


- (void)fileDidChange:(CVLFile *)file
{
    [self startUpdate:nil];
    [changedFiles addObject:file];
    [self endUpdate:nil];
}

- (void)startUpdate
{
  [self startUpdate:nil];
}

- (void)endUpdate;
{
  [self endUpdate:nil];
}

- startUpdate:(NSNotification *)notification
{
  if (!updateCount) {
    updateCount++;
    [[NSNotificationCenter defaultCenter] postNotificationName:@"StartUpdate"
                                                        object:self];
  } else {
    updateCount++;
  }

  return self;
}

- endUpdate:(NSNotification *)notification
{
    if (updateCount==1) {
        NSMutableSet *allChangedFiles=[[NSMutableSet alloc] init];
        while ([changedFiles count]) {
            [changedFiles makeObjectsPerformSelector:@selector(cumulateChanges)];
            [allChangedFiles unionSet:changedFiles];
            oldChangedFiles=changedFiles;
            changedFiles=[[NSMutableSet alloc] init];
            [oldChangedFiles makeObjectsPerformSelector:@selector(propagateChanges)];
        }
        [oldChangedFiles release];
        oldChangedFiles=allChangedFiles;
        [[NSNotificationCenter defaultCenter]
                                postNotificationName:@"ResultsChanged"
                                              object:self];
        [allChangedFiles makeObjectsPerformSelector:@selector(clearChanges)];
        [allChangedFiles removeAllObjects];
        [allChangedFiles release];
        oldChangedFiles=nil;
    }

  updateCount--;

  return self;
}

/*
- (void)calculateChanges
{
  NSMutableDictionary *finalChanges=[[NSMutableSet alloc] init];

  while ([changes count]) {
      oldChangedFiles=changedFiles;
      changedFiles=[[NSMutableSet alloc] init];
      {
          //merge changes
          id pathEnumerator=[oldChangedFiles keyEnumerator];
          NSMutableArray *changedKeys;
          NSString *path;

          while (path=[pathEnumerator nextObject]) {
              if (changedKeys=[finalChanges objectForKey:path])  {
                  [changedKeys addNewObjectsFromArray:[oldChanges objectForKey:path]];
              } else {
                  [finalChanges setObject:[[[oldChangedFiles objectForKey:path] mutableCopy] autorelease] forKey:path];
              }
          }
      }
      [self propagateRecursiveChanges];
      [oldChangedFiles release];
      oldChangedFiles=nil;
  }
  [changedFiles release];
  changedFiles=finalChanges;
} */
/*
- (void)propagateRecursiveChanges
{
  id enumerator;
  NSArray *changedKeys;
  NSString *path;

  enumerator=[oldChanges keyEnumerator];
  while (path=[enumerator nextObject])
  {
    changedKeys=[oldChanges objectForKey:path];
    if ([changedKeys containsObject:@"status"]) {
      [self invalidateResultForPath:[path stringByDeletingLastPathComponent] andKey:@"status"];
    }
    if ([changedKeys containsObject:@"dir"]) {
      [self invalidateResultForPath:path andKey:@"status"];
    }
  }
}
*/

- (BOOL)hasChanged
{
    return ([oldChangedFiles count]>0);
}

-(void)runStatus:(id)sender
{
  NSDictionary *sortedRequests;
  id dirsEnumerator;
  NSString *dir;
  CvsStatusRequest *request;
  id pathEnumerator;
  NSString *path;
  NSMutableArray *files;

  performStatusPending=NO;
  sortedRequests=[statusQueue categorizeUsingMethod:@selector(stringByDeletingLastPathComponent)];
  [statusQueue removeAllObjects];

  dirsEnumerator=[sortedRequests keyEnumerator];
  [self startUpdate];
  while (dir=[dirsEnumerator nextObject]) {
    //if (![statusRequests containsObject:dir]) {
    //    [statusRequests addObject:dir];
        files=[NSMutableArray array];
        pathEnumerator=[[sortedRequests objectForKey:dir] objectEnumerator];

        while (path=[pathEnumerator nextObject]) {
            [files addObject:[path lastPathComponent]];
        }
        request=[CvsStatusRequest cvsStatusRequestForFiles:files inPath:dir];
        [request setIsQuiet:YES];
        [request schedule];
    //}
  }
  [self endUpdate];
}

- (void) runQuickStatus:(id)sender
{
    NSEnumerator	*dirsEnumerator;
    NSString		*dir;

    performQuickStatusPending = NO;

    [self startUpdate];

    dirsEnumerator = [quickStatusQueue objectEnumerator];
    while(dir = [dirsEnumerator nextObject]){
#ifdef WIN32
        [[CvsQuickStatusRequest cvsQuickStatusRequestFromPath:[dir substringToIndex:[dir length] - 1]] schedule]; // On NT, we need to remove the ending \ character...
#else
        [[CvsQuickStatusRequest cvsQuickStatusRequestFromPath:dir] schedule];
#endif
    }

    [quickStatusQueue removeAllObjects];
    [self endUpdate];
}


-(void)runLog:(id)sender
{
  NSDictionary *sortedRequests;
  id dirsEnumerator;
  NSString *dir;
  CvsLogRequest *request;
  id pathEnumerator;
  NSString *path;
  NSMutableArray *files;

  performLogPending=NO;
  sortedRequests=[logQueue categorizeUsingMethod:@selector(stringByDeletingLastPathComponent)];
  [logQueue removeAllObjects];

  dirsEnumerator=[sortedRequests keyEnumerator];
  [self startUpdate];
  while (dir=[dirsEnumerator nextObject]) {
    if (![logRequests containsObject:dir]) {
        [logRequests addObject:dir];
        files=[NSMutableArray array];
        pathEnumerator=[[sortedRequests objectForKey:dir] objectEnumerator];

        while (path=[pathEnumerator nextObject]) {
            [files addObject:[path lastPathComponent]];
        }
        request=[CvsLogRequest cvsLogRequestForFiles:files inPath:dir];
        [request setIsQuiet:YES];
        [request schedule];
    }
  }
  [self endUpdate];
}


-(void)runTags:(id)sender
{
  NSDictionary *sortedRequests;
  id dirsEnumerator;
  NSString *dir;
  CvsVerboseStatusRequest *request;
  id pathEnumerator;
  NSString *path;
  NSMutableArray *files;

  performTagsPending=NO;
  sortedRequests=[tagsQueue categorizeUsingMethod:@selector(stringByDeletingLastPathComponent)];
  [tagsQueue removeAllObjects];

  dirsEnumerator=[sortedRequests keyEnumerator];
  [self startUpdate];
  while (dir=[dirsEnumerator nextObject]) {
    if (![tagsRequests containsObject:dir]) {
        [tagsRequests addObject:dir];
        files=[NSMutableArray array];
        pathEnumerator=[[sortedRequests objectForKey:dir] objectEnumerator];

        while (path=[pathEnumerator nextObject]) {
            [files addObject:[path lastPathComponent]];
        }
        request=[CvsVerboseStatusRequest cvsVerboseStatusRequestForFiles:files inPath:dir];
        [request setIsQuiet:YES];
        [request schedule];
    }
  }
  [self endUpdate];
}

- cmdEnded:(NSNotification *)notification
{
    if ([[notification object] isKindOfClass: [CvsRequest class]])
    {
        CvsRequest *request=[notification object];
        NSString *path=[request path];

        switch([request cmd]){
            case CVS_STATUS_CMD_TAG:
                //if ([statusRequests containsObject:path]) {
                //    [statusRequests removeObject:path];
                //}
                [self setStatusesFromDictionary:[(CvsStatusRequest *)request result]];
                break;
            case CVS_LOG_CMD_TAG:
                if ([logRequests containsObject:path]) {
                    [logRequests removeObject:path];
                }
                [self setLogsFromDictionary:[(CvsLogRequest *)request result]];
                break;
            case CVS_DIFF_CMD_TAG:
                [self setDifferencesFromDictionary:[(CvsDiffRequest *)request result]];
                break;
            case CVS_GET_TAGS_CMD_TAG:
                if ([tagsRequests containsObject:path]) {
                    [tagsRequests removeObject:path];
                }
                [self setTagsFromDictionary:[(CvsVerboseStatusRequest *)request result]];
                break;
            case CVS_QUICK_STATUS_CMD_TAG:
                [self setQuickStatusesFromDictionary:[(CvsQuickStatusRequest *)request result] successful:![(CvsQuickStatusRequest *)request statusAborted] parentPath:[request path]];
                break;
        }
    }
    return self;
}

- (void)getStatusForFile:(CVLFile *)file
{
    NSString *path=[file path];
    
    if (![statusQueue containsObject:path]) {
        [statusQueue addObject:path];
        if (!performStatusPending) {
            performStatusPending=YES;
            [[NSRunLoop currentRunLoop] performSelector:@selector(runStatus:)
                                                 target:self
                                               argument:nil
                                                  order:0
                                                  modes:[NSArray arrayWithObjects:NSDefaultRunLoopMode, NSModalPanelRunLoopMode, nil]];
        }
    }
}

- (void) getQuickStatusForFile:(CVLFile *)file
{
    NSString	*path = [file path];

    if(![statusQueue containsObject:path]){
        NSEnumerator	*anEnum = [[NSArray arrayWithArray:quickStatusQueue] objectEnumerator];
        NSString		*aDir, *pathDir;

//        if(!performQuickStatusPending)
//            [quickStatusTargetPaths removeAllObjects];
        
        path = [path stringByStandardizingPath];
        pathDir = [path stringByDeletingLastPathComponent];
#ifdef WIN32
        pathDir = [pathDir stringByAppendingString:@"\\"];
#else
		pathDir = [pathDir stringByAppendingString:@"/"];
#endif
        [quickStatusTargetPaths addObject:path];
        while(aDir = [anEnum nextObject])
            // We are now sure that all dirs have a / suffix!
            if([path hasPrefix:aDir])
                return;
            else if([aDir hasPrefix:pathDir])
                [quickStatusQueue removeObject:aDir]; // Isn't it dangerous? If someone's observer of the request... Not dangerous, as we create the requests, and nobody else than self if observer of the request.

        [quickStatusQueue addObject:pathDir];
        if(!performQuickStatusPending){
            performQuickStatusPending = YES;
            [[NSRunLoop currentRunLoop] performSelector:@selector(runQuickStatus:) target:self argument:nil order:0 modes:[NSArray arrayWithObjects:NSDefaultRunLoopMode, NSModalPanelRunLoopMode, nil]];
        }
    }
}

- (void)getLogForFile:(CVLFile *)file
{
    NSString *path=[file path];
    if (![logQueue containsObject:path]) {
        [logQueue addObject:path];
        if (!performLogPending) {
            performLogPending=YES;
            [[NSRunLoop currentRunLoop] performSelector:@selector(runLog:)
                                                 target:self
                                               argument:nil
                                                  order:0
                                                  modes:[NSArray arrayWithObjects:NSDefaultRunLoopMode, NSModalPanelRunLoopMode, nil]];
        }
    }
}

- (void)getDifferencesForFile:(CVLFile *)file context:(CVLDiffOutputFormat)context outputFormat:(CVLDiffOutputFormat)outputFormat
{
    NSString *path=[file path];
    NSString *currentDir= [path stringByDeletingLastPathComponent];
    NSArray *someFiles= [NSArray arrayWithObject: [path lastPathComponent]];
    CvsDiffRequest *request= [CvsDiffRequest cvsDiffRequestAtPath: currentDir files: someFiles context:context outputFormat:outputFormat];

    [request setIsQuiet:YES];
    [request schedule];
}


- (void)getTagsForFile:(CVLFile *)file
{
    NSString *path=[file path];
    if (![tagsQueue containsObject:path]) {
        [tagsQueue addObject:path];
        if (!performTagsPending) {
            performTagsPending=YES;
            [[NSRunLoop currentRunLoop] performSelector:@selector(runTags:)
                                                 target:self
                                               argument:nil
                                                  order:0
                                                  modes:[NSArray arrayWithObjects:NSDefaultRunLoopMode, NSModalPanelRunLoopMode, nil]];
        }
    }
}

/********************************************************************/
/* Lazy methods */

/*
- (NSString *)obtainRepositoryPath:(NSString *)fullPath andKey:(NSString *)key
{
  NSString *path;

  if (!fullPath) {
    [[[NSException alloc] init] raise];
  }
  path=[[[NSString alloc] initWithContentsOfFile:[[fullPath stringByAppendingPathComponent:@"CVS"] stringByAppendingPathComponent:@"Repository"]] autorelease];

  path=[path substringToIndex:[path length]-1];

  if (!path) {
    if ([fullPath length] > 0) {
      if (path=[self repositoryPathForPath:[fullPath stringByDeletingLastPathComponent]]) {
        path=[path stringByAppendingPathComponent:[fullPath lastPathComponent]];
      } else {
        path=@"";
      }
    } else {
      path= @"";
    }
  }

  [self startUpdate:nil];
  [self setResult:path forPath:fullPath andKey:key];
  [self endUpdate:nil];

  return path;
} */

/*
- (NSObject *)obtainMTimeForPath:(NSString *)path andKey:(NSString *)key
{
  NSDictionary* attributes= [[NSFileManager defaultManager] fileAttributesAtPath: path traverseLink: NO];
  NSDate* mDate= [attributes objectForKey: NSFileModificationDate];
  if (mDate)
  {
    [self setResult: [mDate dateWithCalendarFormat: nil timeZone: nil] forPath:path andKey:@"mod Time"];
  }

  return mDate;
} // obtainIsLeafAndMTimeForPath:andKey:
*/

- (NSSet *)changedFiles
{
    return oldChangedFiles;
}
@end

