/*$Id: RetrievePanelController.m,v 1.9 2002/06/14 12:47:06 stephane Exp $*/

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "RetrievePanelController.h"
#import <SenOyster/NSString.SenOysterRegex.h>
#import "NSString+CVL.h"
#import <CVLFile.h>
#import <ResultsRepository.h>

static RetrievePanelController *retrieveControllerInstance;

@implementation RetrievePanelController

+ new
{
    if (!retrieveControllerInstance) {
        retrieveControllerInstance= [[RetrievePanelController alloc] init];
    }
    return retrieveControllerInstance;
} // new


- init
{
    self= [super init];
    dateFormatter = [[NSDateFormatter alloc] initWithDateFormat:[[NSUserDefaults standardUserDefaults] stringForKey:@"CVLDateFormat"] allowNaturalLanguage:YES];
    return self;
} // init


- (void)dealloc
{
    [dateFormatter release];
    [super dealloc];
}

- show: sender
{
    if (!panel)
      {
        [NSBundle loadNibNamed:@"RetrievePanel" owner:self];
        [dateCell setObjectValue:nil];
      }
    [panel orderFront: self];
    return self;
}

- hide: sender
{
    if (panel)
        [panel orderOut: self];
    return self;
} // hide:

/*
- (void) resultsChanged:(NSNotification *)notification
{
    if([[notification object] hasChanged])
        if([[[notification object] changedFiles] containsObject:selectedFile]){
            tags = [selectedFile revisionTags];
            [revisionCombo reloadData];
        }
}
*/
- (int) runModalForFiles: (NSArray*) files inPath:(NSString *)path
{
    // Rhapsody: bug with combobox in modal panel: unable to choose, or move very fast!
    BOOL oneSelected= ([files count] == 1);
    BOOL openButtonEnabled= oneSelected;

    [self show: self];

    [self toggleFreeFormDate:nil];
    [revisionCombo setStringValue:@""];
    
    if (oneSelected)
    {        
        [destPath release];
        destPath= [[[files objectAtIndex: 0] lastPathComponent] retain];
        // Reload tags
        selectedFile = [(CVLFile *)[CVLFile treeAtPath:[path stringByAppendingPathComponent:[files objectAtIndex: 0]]] retain];
        tags = [selectedFile revisionTags];
        if(!tags){
            // Observe notification sent after tags have been retrieved
//            [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(resultsChanged:) name:@"ResultsChanged" object:nil];
// BUG: we cannot get this notification before we quit modal mode, 
// because NSTaskDidTerminateNotification is not sent before modal is over!
            // We wait for this notification in -[TaskRequest taskTerminated:]
            // seems there is no way to force sending of the notification
            // during modal.
        }
        else
            [tags retain];
    }
    [revisionCombo reloadData];
    [openButton setEnabled: openButtonEnabled];
    [saveAsButton setEnabled: oneSelected];
    [NSApp runModalForWindow: panel];
    [tags release]; tags = nil;
    [[NSNotificationCenter defaultCenter] removeObserver:self name:@"ResultsChanged" object:nil];
    [selectedFile release]; selectedFile = nil;
    [self hide: self];
    return result;	
} // runModal


- cancel:sender
{
    result=[sender tag];
    
    [NSApp stopModal];
    return self;
}


- ok:sender
{
    result=[sender tag];

    [NSApp stopModal];
  return self;
}

- (IBAction) toggleFreeFormDate:(id)sender
{
    if([freeFormSwitch state]){
        [[dateCell cell] setFormatter:nil];
        [dateCell setToolTip:@"Enter date using any format cvs can understand"];
    }
    else{
        [[dateCell cell] setFormatter:dateFormatter];
        [dateCell setToolTip:[NSString stringWithFormat:@"Enter date like '%@'", [[NSCalendarDate calendarDate] descriptionWithCalendarFormat:[dateFormatter dateFormat]]]];
    }
}

- (NSString *) dateString
{
    if([dateCell objectValue]){
        if([freeFormSwitch state])
            return [NSString stringWithString:[dateCell stringValue]];
        else
            // http://www.w3.org/TR/NOTE-datetime
            // ISO8601
            // e.g. YYYY-MM-DDThh:mm:ss.sTZD (eg 1997-07-16T19:20:30.45+01:00)
            return [(NSCalendarDate *)[dateCell objectValue] descriptionWithCalendarFormat:@"%Y-%m-%dT%H:%M:%S%z"];
    }
    else
        return nil;
}

- (NSString *) revision
{
    NSString	*aString = [revisionCombo stringValue];

    if([aString length])
        return [NSString stringWithString:aString];
    else
        return nil;
}

- (BOOL) removesStickyAttributes
{
    return [stickySwitch state];
}

- (NSString*) buildDestinationPath
{
    NSString	*resultPath = destPath;
    NSString	*aString = [self dateString];

    if(aString)
        resultPath = [resultPath cvlFilenameForDate:aString];
    else{
        aString = [self revision];
        if(aString)
            resultPath = [resultPath cvlFilenameForRevision:aString];
    }

    return resultPath;
}


- (NSString *) destinationPath
{
    return destPath;
}


- (void) saveAs: sender
{
    NSSavePanel* sPanel= [NSSavePanel savePanel];
    int resultCode= 0;

    resultCode= [sPanel runModalForDirectory: @"" file: [[self buildDestinationPath] lastPathComponent]];
    if (resultCode == NSOKButton)
    {
        [destPath release];
        destPath= [[sPanel filename] retain];
        result= CVL_RETRIEVE_SAVE_AS;
        [NSApp stopModal];
    }
}


/*
- selectPath:sender
{
    NSOpenPanel *aPanel= [NSOpenPanel openPanel];
    int value= 0;
    [aPanel setCanChooseDirectories: YES];
    [aPanel setAllowsMultipleSelection: NO];
    [aPanel setTitle: @"Create Work Area in Folder"];
    value= [aPanel runModalForTypes: nil];		// all types
    if (value == NSOKButton) {
        NSArray* filenames= [aPanel filenames];
        [pathField setStringValue:[filenames objectAtIndex:0]];
    }
    [aPanel setTitle: @"Open"];	// restore original title
    return self;
} // selectPath:


- select: sender
{
    if (sender == browser)
      {
        NSString *selection=[[browser selectedCell] stringValue];
        if (selection)
            [moduleField setStringValue: selection];
      }
    return self;
} // select:


- (NSString*) selectedPath
{
    return selectedPath;
} // selectedPath


- (NSString*) selectedModule
{
    return [moduleField stringValue];
} // selectedModule
*/

- (int)numberOfItemsInComboBox:(NSComboBox *)aComboBox
{
    return [tags count];
}

- (id)comboBox:(NSComboBox *)aComboBox objectValueForItemAtIndex:(int)index
{
    return [tags objectAtIndex:index];
}

@end
