/* SenBrowserCell.m created by ja on Tue 11-Feb-1997 */
/*$Id: SenBrowserCell.m,v 1.7 2002/06/14 12:23:00 stephane Exp $*/

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "SenBrowserCell.h"
#ifdef MACOSX
#import <NSValue_Null.h>
#else
#import <SenFoundation/NSValue_Null.h>
#endif

@interface SenBrowserCell (Private)
- (void)willModifyTabs;
- (void)willModifyCells;
- (NSMutableArray *)tabWidths;
- (NSMutableArray *)cells;
- (NSRect)_availableFrameInFrame:(NSRect)aRect;
@end

// This helper function works around the fact that -[NSArray copyWithZone:]
// only does a shallow copy now.  This was leading to elements of the cells 
// referencing a dealloced NSMatrix (stashed in the private _controlView ivar).
//
static id copyArrayWithZone(NSArray *array, NSZone *zone) {
    if (array) {
        unsigned index, count = [array count];
        NSMutableArray *result = [[NSMutableArray allocWithZone:zone] initWithCapacity:count];
        for(index=0; index<count; index++) {
            id object = [array objectAtIndex:index];
            if ([object respondsToSelector:@selector(copyWithZone:)]) {
                object = [object copyWithZone:zone];
            } else {
                object = [object retain];
            }
            [result addObject:object];
            [object release];
        }
        return result;
    } else {
        return nil;
    }
}

@implementation SenBrowserCell
+ (SenBrowserCell *)cell
{
    return [[[self alloc] init] autorelease];
}

- (id)init
{
    if (self=[super init]) {
        cells=[[NSMutableArray allocWithZone:[self zone]] init];
        tabWidths=[[NSMutableArray allocWithZone:[self zone]] init];
        values=[[NSMutableArray allocWithZone:[self zone]] init];
        [self setObjectValue:@""];

        reusedCells=NO;
        reusedTabs=NO;
    }
    return self;
}

- (void)dealloc
{
    [tabWidths release];
    [cells release];
    [values release];
    
    [super dealloc];
}

- (id)copyWithZone:(NSZone *)zone
{
    SenBrowserCell *copy;
    int index;

    if(copy = [super copyWithZone:zone]) {
        copy->cells=copyArrayWithZone(cells, zone); // or copy  retain (120)
        copy->tabWidths=[tabWidths retain];
        copy->values=[[NSMutableArray allocWithZone:[self zone]] init];
        index=0;
        while (index<[tabWidths count]) {
            [copy->values addObject:[NSValue nullValue]];
            index++;
        }

        copy->reusedCells=YES; //120
        copy->reusedTabs=YES;
        copy->namePosition=namePosition;
    }
    return copy;
}

- (NSRect)_availableFrameInFrame:(NSRect)aRect
{
    aRect.size.width-=[[[self class] branchImage] size].width;

    if (aRect.size.width<0) {
        aRect.size.width=0;
    }
    return aRect;
}

- (NSSize)cellSize
{
    NSSize resultingSize=[super cellSize];
    int count,index;
    float tabWidth;
    
    count=[tabWidths count];
    index=0;
    while (index<count) {
        tabWidth=[[tabWidths objectAtIndex:index] floatValue];
        if (tabWidth<0) {
//            resultingSize.width+=tabWidth;
        }
        resultingSize.height=MAX([[cells objectAtIndex:index] cellSize].height,resultingSize.height);
        index++;
    }

    return resultingSize;
}

- (void)willModifyTabs
{
    if (reusedTabs) {
        NSMutableArray *newTabWidths;

        reusedTabs=NO;

        newTabWidths=[tabWidths mutableCopyWithZone:[self zone]];
        [tabWidths release];
        tabWidths=newTabWidths;
    }
}

- (void)willModifyCells
{
    if (reusedCells) {
        NSMutableArray *newCells;

        reusedCells=NO;

        newCells=[cells mutableCopyWithZone:[self zone]];
        [cells release];
        cells=newCells;
    }
}

- (void)useTemplate:(SenBrowserCell *)anotherCell
{
    [self setTabWidths:[anotherCell tabWidths]];
    [self setCells:[anotherCell cells]];
    [self setNamePosition:[anotherCell namePosition]];
}

- (void) setTabWidths:(NSArray *)someTabs
{
    reusedTabs=YES;

    [someTabs retain];
    [tabWidths release];
    tabWidths=(NSMutableArray *)someTabs;
}

- (NSArray *)tabWidths
{
    return tabWidths;
}

- (void) addTabWithFixedWidth:(float)width
{
    [self willModifyTabs];

    [tabWidths addObject:[NSNumber numberWithFloat:width]];
    [cells addObject:[NSValue nullValue]];
    [values addObject:[NSValue nullValue]];
}

- (void) addTabWithProportionalWidth:(float)width
{
    [self willModifyTabs];

    [tabWidths addObject:[NSNumber numberWithFloat:-width]];
    [cells addObject:[NSValue nullValue]];
    [values addObject:[NSValue nullValue]];
}

- (void) setTabFixedWidth:(float)width atIndex:(int)index
{
    [self willModifyTabs];

    [tabWidths replaceObjectAtIndex:index withObject:[NSNumber numberWithFloat:width]];
}

- (void) setTabProportionalWidth:(float)width atIndex:(int)index
{
    [self willModifyTabs];

    [tabWidths replaceObjectAtIndex:index withObject:[NSNumber numberWithFloat:-width]];
}

- (void) setCells:(NSArray *)someCells
{
    reusedCells=YES;

//    [someCells retain];
    [cells release];
    cells=(NSMutableArray *)copyArrayWithZone(someCells, [self zone]); //reusedCells say that it's not mutable
}

- (NSArray *)cells
{
    return cells;
}

- (void) setCell:(NSCell *)cell atIndex:(int)index
{
    [self willModifyCells];

    [cells replaceObjectAtIndex:index withObject:cell];
}

- (NSCell *) cellAtIndex:(int)index
{
    return [cells objectAtIndex:index];
}

- (void) setObjectValue:(id)value atIndex:(int)index
{
    if (!values) {
        [super setObjectValue:value];
    }
    [values replaceObjectAtIndex:index withObject:value];
}

- (id)objectValueAtIndex:(int)index;
{
    id object;

    if (!values) {
        return [super objectValue];
    }

    object=[values objectAtIndex:index];
    if (object==[NSValue nullValue]) {
        return nil;
    } else {
        return object;
    }
}

- (void)setNamePosition:(int)value
{
    namePosition=value;
}

- (int)namePosition
{
    return namePosition;
}

- (NSString *)stringValue
{
    if (!values) {
        return [super stringValue];
    }
    return [[[self objectValueAtIndex:namePosition] retain] autorelease];
}

- (void)setStringValue:(NSString *)aString
{
    if (!values) {
        [super setStringValue:aString];
    }
    [self setObjectValue:aString atIndex:namePosition];
}

- (void)set
{
	[super set];
}

- (void)setState:(int)flag
{
    id cellEnumerator;
    NSCell *cell;

    cellEnumerator=[cells objectEnumerator];
    while (cell=[cellEnumerator nextObject]) {
        [cell setState:flag];
    }
    [super setState:flag];
}

- (void)setFont:(NSFont *)fontObj
{
    id cellEnumerator;
    NSCell *cell;

    cellEnumerator=[cells objectEnumerator];
    while (cell=[cellEnumerator nextObject]) {
        [cell setFont:fontObj];
    }
    [super setFont:fontObj];
}

- (void)highlight:(BOOL)flag withFrame:(NSRect)cellFrame inView:(NSView *)aView
{
    [self setState:flag];
    [aView setNeedsDisplayInRect:cellFrame];
//    [super highlight:flag withFrame:cellFrame inView:aView];
//    [self drawInteriorWithFrame:cellFrame inView:aView];
}
/*
- (void)highlight:(BOOL)flag withFrame:(NSRect)cellFrame inView:(NSView *)aView
{
    float proportionalTotalWidth;
    float proportionalAvailableWidth;
    float tabWidth=0;
    int index,count;
    float tabStart,tabEnd;
    NSRect subCellFrame;
    NSCell *cell;

    [super highlight:flag withFrame:cellFrame inView:aView];
    cellFrame=[self _availableFrameInFrame:cellFrame];
    subCellFrame=cellFrame;
    proportionalTotalWidth=0;
    proportionalAvailableWidth=NSWidth(cellFrame);
    count=[tabWidths count];
    index=0;
    while (index<count) {
        tabWidth=[[tabWidths objectAtIndex:index] floatValue];
        if (tabWidth<0) {
            proportionalTotalWidth-=tabWidth;
        } else {
            proportionalAvailableWidth-=tabWidth;
        }
        index++;
    }

    index=0;
    tabStart=tabEnd=NSMinX(cellFrame);

    while ((index<count) && (tabEnd<NSMaxX(cellFrame))) {
        tabStart=tabEnd;
        tabWidth=[[tabWidths objectAtIndex:index] floatValue];
        if (tabWidth<0) {
            if (proportionalAvailableWidth>0) {
                tabWidth=(-tabWidth/proportionalTotalWidth)*proportionalAvailableWidth;
            } else {
                tabWidth=0;
            }
        }

        tabEnd=tabStart+tabWidth;

        if (tabEnd>NSMaxX(cellFrame)) {
            tabEnd=NSMaxX(cellFrame);
            tabWidth=tabEnd-tabStart;
        }

        if (tabWidth>0) {
            subCellFrame.origin.x=tabStart;
            subCellFrame.size.width=tabWidth;

            cell=[cells objectAtIndex:index];
            [cell setObjectValue:[values objectAtIndex:index]];
            [cell highlight:flag withFrame:subCellFrame inView:aView];
        }
        index++;
    }
} */

- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView *)aView 
{
    float proportionalTotalWidth;
    float proportionalAvailableWidth;
    float tabWidth=0;
    int index,count;
    float tabStart,tabEnd;
    NSRect subCellFrame,availableCellFrame;
    NSCell *cell;

    availableCellFrame=[self _availableFrameInFrame:cellFrame];
    subCellFrame=availableCellFrame;
    proportionalTotalWidth=0;
    proportionalAvailableWidth=NSWidth(availableCellFrame);
    count=[tabWidths count];
    index=0;
    while (index<count) {
        tabWidth=[[tabWidths objectAtIndex:index] floatValue];
        if (tabWidth<0) {
            proportionalTotalWidth-=tabWidth;
        } else {
            proportionalAvailableWidth-=tabWidth;
        }
        index++;
    }

    [super drawInteriorWithFrame:cellFrame inView:aView];

    index=0;
    tabStart=tabEnd=NSMinX(availableCellFrame);

    while ((index<count) && (tabEnd<NSMaxX(availableCellFrame))) {
        tabStart=tabEnd;
        tabWidth=[[tabWidths objectAtIndex:index] floatValue];
        if (tabWidth<0) {
            if (proportionalAvailableWidth>0) {
                tabWidth=(-tabWidth/proportionalTotalWidth)*proportionalAvailableWidth;
            } else {
                tabWidth=0;
            }
        }

        tabEnd=tabStart+tabWidth;

        if (tabEnd>NSMaxX(availableCellFrame)) {
            tabEnd=NSMaxX(availableCellFrame);
            tabWidth=tabEnd-tabStart;
        }

        if (tabWidth>0) {
            subCellFrame.origin.x=tabStart;
            subCellFrame.size.width=tabWidth;

            cell=[cells objectAtIndex:index];
            [cell setObjectValue:[values objectAtIndex:index]];
            if ([cell state]!=[self state]) {
                [cell setState:[self state]];
            }
            [cell drawWithFrame:subCellFrame inView:aView];
        }
        index++;
    } 

}

@end
