/* SenFormController.m created by ja on Tue 24-Feb-1998 */

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "SenFormController.h"

static SenFormController *instance=nil;

@interface SenFormController (Private)
@end

@implementation SenFormController
+ (SenFormController *)instance
{
    if (!instance) {
        instance=[[self alloc] init];
    }
    return instance;
}

- (id)init
{
    if (self=[super init]) {
        values=[[NSMutableDictionary alloc] init];
        controls=[[NSMutableDictionary alloc] init];
    }
    return self;
}

- (void)dealloc
{
    [values release];
    [controls release];

    [super dealloc];
}

- (void)awakeFromNib
{
    [self refreshControls];
}

- (id)objectValue
{
    return values;
}

- (void)setDictionaryValue:(NSDictionary *)aDictionary;
{
    [values setDictionary:aDictionary];
    [self refreshControls];
}

- (void)addValuesFromDictionary:(NSDictionary *)aDictionary
{
    [values addEntriesFromDictionary:aDictionary];
    [self refreshControls];
}

- (void)setObjectValue:(id)newValue forKey:(NSString *)aKey
{
    [values setObject:newValue forKey:aKey];
    [self refreshControls];
}

- (NSDictionary *)dictionaryValue
{
    return values;
}

- (id)objectValueForKey:(NSString *)aKey
{
    return [values objectForKey:aKey];
}

- (void)setControl:(id)aControl forKey:(NSString *)key
{
    [controls setObject:aControl forKey:key];
}

- (id)controlForKey:(NSString *)key
{
    return [controls objectForKey:key];
}

- (void)refreshControls:(id)sender
{
    [self refreshControls];
}

- (void)refreshControls
{
    id enumerator;
    NSString *key;
    NSControl *control;
    id value;
    
    enumerator=[controls keyEnumerator];
    while (key=[enumerator nextObject]) {
        control=(NSControl *)[controls objectForKey:key];
        if (value=[values objectForKey:key]) {
            [control setObjectValue:value];
        }
    }
}

- (void)takeObjectValuesFromAllControls:(id)sender
{
    [self takeObjectValuesFromAllControls];
}

- (void)takeObjectValuesFromAllControls
{
    id enumerator;
    NSString *key;
    NSCell *cell;

    enumerator=[controls keyEnumerator];
    while (key=[enumerator nextObject]) {
        cell=[(NSControl *)[controls objectForKey:key] selectedCell];
        if ([cell hasValidObjectValue]) {
            [values setObject:[cell objectValue] forKey:key];
        }
    }
}

- (void)takeObjectValueFromSender:(id)sender
{
    if ([sender isKindOfClass:[NSCell class]]) {
        NSControl *control=(NSControl*)[sender controlView];
        id keysEnumerator;
        NSString *key;
        
        keysEnumerator=[[controls allKeysForObject:control] objectEnumerator];
        while (key=[keysEnumerator nextObject]) {
            if ([sender hasValidObjectValue]) {
                [values setObject:[sender objectValue] forKey:key];
            }
        }
    }
}

- (id)initWithCoder:(NSCoder *)decoder
{
    int		version;
   
    self = [self init];
    
    version = [decoder versionForClassName:@"SenFormController"];
    /*
    switch (version) {
    case 0:
        break;
    default:
        break;
    } */
    return self;
}

- (void)encodeWithCoder:(NSCoder *)coder
{
    //[super encodeWithCoder:coder];

    // Version == 1
}

- (NSString *)description
{
    return [[super description] stringByAppendingString:[NSString stringWithFormat:@"{ controls = %@, \nvalues = %@ }",controls,values]];
}
@end
