/* SenPanelFactory.m created by ja on Wed 25-Feb-1998 */

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "SenPanelFactory.h"
#import "_NibOwner.h"

static SenPanelFactory *sharedPanelFactory=nil;

#ifndef WIN32
static NSString *PlatformNibKey=@"OPENSTEP nib";
#else
static NSString *PlatformNibKey=@"NT nib";
#endif

@interface SenPanelFactory (Private)
- (id)initWithDefaultRessourceFile;
- (void)registerPanelWithHandle:(id)aController withName:(NSString *)panelName;
@end

@implementation SenPanelFactory
+ (SenPanelFactory *)sharedPanelFactory
{
    if (!sharedPanelFactory) {
        sharedPanelFactory=[[self alloc] initWithDefaultRessourceFile];
    }
    return sharedPanelFactory;
}

- (id)init
{
    if (self=[super init]) {
        panels=[[NSMutableDictionary alloc] init];
    }
    return self;
}

- (id)initWithDefaultRessourceFile
{
    if (self=[self init]) {
        properties=[[NSDictionary alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"Panels" ofType:@"plist"]];
        if (!properties) {
            NSLog(@"Warning: cannot open panels property list");
        }
    }
    return self;
}

- (void)registerPanelWithHandle:(id)aController withName:(NSString *)panelName
{
    [panels setObject:aController forKey:panelName];
}

- (id)handleOfPanelWithName:(NSString *)panelName
{
    id panelOwner;
    _NibOwner *owner;
    if (panelOwner=[panels objectForKey:panelName]) {
        return panelOwner;
    } else {
        NSDictionary *panelProperties;
        NSString *nibName;
        
        if (panelProperties=[properties objectForKey:panelName]) {
            nibName=[panelProperties objectForKey:PlatformNibKey];
            if (!nibName) {
                nibName=[panelProperties objectForKey:@"nib"];
            }
            if (nibName) {
                owner=[[[_NibOwner alloc] init] autorelease];
                if ([NSBundle loadNibNamed:nibName owner:owner]) {
                    [self registerPanelWithHandle:[owner handle] withName:panelName];
                    if ([[owner handle] respondsToSelector:@selector(awakeWithProperties:)]) {
                        [[owner handle] awakeWithProperties:panelProperties];
                    }
                    return [owner handle];
                } else {
                    NSLog(@"Warning: failed to read nib for panel with name:%@",panelName);
                } 
            }
            NSLog(@"Warning: don't know how to instanciate panel with name:%@",panelName);
        } else {
            NSLog(@"Warning: cannot find panels with name:%@",panelName);
        }
    }
    return nil;
}

@end
