/*$Id: WorkAreaViewer.m,v 1.32 2002/07/04 12:21:56 stephane Exp $*/

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.


#import "WorkAreaViewer.h"
#import "BrowserController.h"
#import "WorkAreaListViewer.h"
#import "ResultsRepository.h"
#import "DirectoryContentsFilterProvider.h"
#ifdef MACOSX
#import <CvsUpdateRequest.h>
#import <CvsCommitRequest.h>
#import <CvsTagRequest.h>
#import <CvsStatusRequest.h>
#import <CvsLogRequest.h>
#import <CvsAddRequest.h>
#import <CvsDiffRequest.h>
#else
#import <Requests/CvsUpdateRequest.h>
#import <Requests/CvsCommitRequest.h>
#import <Requests/CvsTagRequest.h>
#import <Requests/CvsStatusRequest.h>
#import <Requests/CvsLogRequest.h>
#import <Requests/CvsAddRequest.h>
#import <Requests/CvsDiffRequest.h>
#endif
#import "CVLScheduler.h"
#import "RetrievePanelController.h"
#ifdef MACOSX
#import <SenPanelFactory.h>
#import <SenFormPanelController.h>
#else
#import <SenPanels/SenPanelFactory.h>
#import <SenPanels/SenFormPanelController.h>
#endif
#import <SenOyster/NSString.SenOysterRegex.h>
#import "CVLFile.h"
#ifdef MACOSX
#import <CVLOpendiffRequest.h>
#import <CvsRemoveRequest.h>
#import <CvsReleaseRequest.h>
#import <SelectorRequest.h>
#else
#import <Requests/CVLOpendiffRequest.h>
#import <Requests/CvsRemoveRequest.h>
#import <Requests/CvsReleaseRequest.h>
#import <Requests/SelectorRequest.h>
#endif
//#import "CVLRenameController.h"

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import <SenFoundation/NSObject_SenTree.h>

static NSArray *cellsOrder;
static NSArray *filterConfigurationPresets;
static NSPoint currentTopLeft= {0, 0};

//-------------------------------------------------------------------------------------

@interface WorkAreaViewer (Private)
//- (id) window;
- (void)reflectFilterConfiguration;
- (void) retrieveVersionOfFiles:(NSArray *)files inPath:(NSString *)path;
- (NSSize)suggestNewSizeForSize:(NSSize)frameSize;
- (void) viewerDoubleSelect: (NSNotification *)notification;
- (void) schedulerDidChange: (NSNotification *)notification;
- (void)updateProcessesIndicator;
- (void)_callSelectFiles:(NSSet *)someFiles;
@end

//-------------------------------------------------------------------------------------

@implementation WorkAreaViewer

+ (void)initialize
{
    cellsOrder=[NSArray arrayWithObjects:@"Locally Modified",@"Needs Update",@"Needs Merge",@"Conflicts",@"Up-To-Date",@"Not in CVS",@"Ignored",@"Being Computed",@"Unknown",nil];
    [cellsOrder retain];

    filterConfigurationPresets=[NSArray arrayWithObjects:[NSArray arrayWithObjects:@"Locally Modified",@"Needs Update",@"Needs Merge" ,@"Conflicts",@"Up-To-Date",@"Not in CVS",@"Ignored",@"Being Computed",@"Unknown",nil],[NSArray arrayWithObjects:@"Locally Modified" ,@"Needs Update",@"Needs Merge",@"Conflicts",@"Not in CVS",@"Being Computed",nil],nil];
    [filterConfigurationPresets retain];
}

+ (WorkAreaViewer*) viewerForPath: (NSString*) aPath
{
    return [self viewerForFile:[CVLFile treeAtPath:aPath]];
}

+ (WorkAreaViewer*) viewerForFile: (CVLFile *) aFile
{
    WorkAreaViewer* newViewer= [[self alloc] initForFile: aFile];

    return [newViewer autorelease];
}

- init
{
    return [self initForFile:[CVLFile treeAtPath:NSHomeDirectory()]];
} // init


- initForFile: (CVLFile *) aFile;
{
    self= [super init];
    rootFile= [aFile retain];
    windowIsMiniaturized= NO;
    currentViewer= nil;
    browserViewer= nil;
    listViewer= nil;
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(viewerDoubleSelect:)
                                                 name:@"ViewerDoubleSelect"
                                               object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(schedulerDidChange:)
                                                 name:@"SchedulerDidChange"
                                               object:[CVLScheduler sharedScheduler]];

    return self;
}


- (void) dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    [currentViewer release];
    [browserViewer release];
    [listViewer release];
    [rootFile release];
    [filterProvider release];
    [window setDelegate:nil];
//#warning COMMENTED ANTI-BUG CODE!!!
    [filterPopup setTarget:nil];
    // To avoid unexplicable bug we need to remove ourself as target of the popup...
	// It seems that popup remains in the responder chains, and it tries to send messages to its target when menus need validation!
    // Just releasing the window is not enough. Popup is still retained by ???
    [window autorelease]; // We cannot release the window, because other bugs happen!!! Autorelease seems to work...
    [super dealloc];
} // dealloc


- (void) filterPreselectionPopped:(id)sender
{
    if ([sender selectedItem]) {
        [filterProvider setFilterWithStringArray:[filterConfigurationPresets objectAtIndex:[[sender selectedItem] tag]]];
        [self reflectFilterConfiguration];
    }
}

- (void) setViewer:aViewer
{
    id	previousKeyView = [viewHolder retain];
    id	nextKeyView = [[viewHolder nextKeyView] retain];
    id newView= [aViewer view];
    NSRect holderRect= [viewHolder frame];	
  //	NXRect	viewRect= [newView frame];
    
  //	NX_X (&viewRect) = (NX_WIDTH (&holderRect) - NX_WIDTH (&viewRect)) / 2.0;
  //	NX_Y (&viewRect) = (NX_HEIGHT (&multiRect) - NX_HEIGHT (&viewRect)) / 1.5;
    [newView setFrame:holderRect];
    [newView setFrameOrigin:(NSPoint) {0.0, 0.0}];
    if (currentViewer) {
        previousKeyView = [[currentViewer previousKeyView] retain];
        nextKeyView = [[currentViewer nextKeyView] retain];
        [[currentViewer view] removeFromSuperview];
        [currentViewer release];
    }
    [viewHolder setAutoresizesSubviews: YES];
    [newView setNextKeyView:nextKeyView];
    [previousKeyView setNextKeyView:newView];
    [viewHolder addSubview: newView];
    [previousKeyView release];
    [nextKeyView release];
    currentViewer = [aViewer retain];
    [currentViewer setDelegate:self];
    [currentViewer setFilterProvider:filterProvider];
  //	[viewHolder update];
}

- (void)awakeFromNib
{
    // set default viewer;
    [self setBrowserView: self];

    [window makeFirstResponder: [currentViewer view]];
    [window setTitleWithRepresentedFilename:[rootFile path]];
    [window setMiniwindowImage:[NSImage imageNamed:@"appicon"]];
    [self updateProcessesIndicator];
} // awakeFromNib

- (void)showWindowWithDictionary:(NSDictionary *)dict
{
#if 0
    BOOL miniaturized= [[dict objectForKey: @"isMiniaturized"] isEqualToString:@"1"];
#else
    BOOL miniaturized= [[dict objectForKey: @"isMiniaturized"] intValue] == 1;
#endif
    NSString *frameString=[dict objectForKey: @"frameString"];

    if (!window)
    {
        [NSBundle loadNibNamed:[WorkAreaViewer description] owner:self];
        if ((currentTopLeft.x == 0) && (currentTopLeft.y == 0))
        {
            currentTopLeft.x= NSMinX([window frame]);
            currentTopLeft.y= NSMaxY([window frame]);
        }
    }
    if (frameString)
    {
        [window setFrameFromString:frameString];
    }
    else
    {
        currentTopLeft= [window cascadeTopLeftFromPoint: currentTopLeft];
        [window setFrameTopLeftPoint: currentTopLeft];
        [self viewerFrameSizeChanged:nil];
    }
    [window makeKeyAndOrderFront:self];
    if (miniaturized)
    {
        [window miniaturize: self];
    }
    [filterProvider setFilterWithStringArray:[dict objectForKey:@"show status"]];
    [self reflectFilterConfiguration];
}

- (NSDictionary *)stateDictionary
{
    NSMutableDictionary* browserDict= [NSMutableDictionary dictionaryWithCapacity: 3];
    [browserDict setObject: [rootFile path] forKey: @"path"];
    // Stephane: temporary check to avoid crash on quit in certain circumstances (bound to CVLSelectingFilesRequest)
    if(window)
        [browserDict setObject: [window stringWithSavedFrame] forKey: @"frameString"];
    [browserDict setObject: [NSNumber numberWithInt: windowIsMiniaturized] forKey: @"isMiniaturized"];
    [browserDict setObject: [filterProvider stringArrayFilterDescription] forKey:@"show status"];
    return browserDict;
}

- (void) show: sender
{
    if ([window isVisible]) {
        [window makeKeyAndOrderFront:self];
    } else {
        NSDictionary* defaultFilterDict= [NSDictionary dictionaryWithObject: [filterConfigurationPresets objectAtIndex: 1]
                                                                     forKey: @"show status"];
        [self showWindowWithDictionary: defaultFilterDict];
      //    [self retain];
#ifdef MACOSX
        [[NSDocumentController sharedDocumentController] noteNewRecentDocumentURL:[NSURL fileURLWithPath:[rootFile path]]];
#endif
    }
}

- (void) setBrowserView: sender
{
    if (!browserViewer)
    {
        browserViewer= [[BrowserController browserForPath:[rootFile path]] retain];
    }
    [self setViewer: browserViewer];
}


- (void) setListView: sender
{
    if (!listViewer)
    {
        listViewer= [[WorkAreaListViewer listViewerForPath:[rootFile path]] retain];
    }
    [self setViewer: listViewer];
}

- (id) viewer
{
    return currentViewer;
}

- (NSString*) rootPath
{
    return [rootFile path];
}

- (CVLFile *)rootFile
{
    return rootFile;
}

-(NSArray *)relativeSelectedPaths
{
    return [currentViewer relativeSelectedPaths];
}

-(NSArray *)selectedPaths
{
    return [currentViewer selectedPaths];
}

- (void)_callSelectFiles:(NSSet *)someFiles
{
    [currentViewer selectFiles:someFiles];
    [someFiles release];
}

- (void)selectFiles:(NSSet *)someFiles
{
    id fileEnumerator;
    CVLFile *aFile;

    fileEnumerator=[someFiles objectEnumerator];
    while (aFile=[fileEnumerator nextObject]) {
        [filterProvider configureToShowFile:aFile];
    }
    [self reflectFilterConfiguration];
    [someFiles retain];
#if 0
    [[NSRunLoop currentRunLoop] performSelector:@selector(_callSelectFiles:) target:self argument:someFiles order:100000 modes:[NSArray arrayWithObject:[[NSRunLoop currentRunLoop] currentMode]]];
#else
    // MOXS: sometimes, there is no currentRunLoop mode!!!
    // Open workarea A from Recent, then workarea B (located inside workarea A?)
    if([[NSRunLoop currentRunLoop] currentMode])
        [[NSRunLoop currentRunLoop] performSelector:@selector(_callSelectFiles:) target:self argument:someFiles order:100000 modes:[NSArray arrayWithObject:[[NSRunLoop currentRunLoop] currentMode]]];
    else{
//        NSLog(@"$$$$$$$$ NO RUN LOOP MODE $$$$$$$$$$");
        [[NSRunLoop currentRunLoop] performSelector:@selector(_callSelectFiles:) target:self argument:someFiles order:100000 modes:[NSArray arrayWithObject:NSDefaultRunLoopMode]];
    }
#endif
}

- (void) retrieveWorkAreaVersion:sender
{
    [self retrieveVersionOfFiles:nil inPath:[rootFile path]];
}

- (void) retrieveSelectedFilesVersion:sender
{
    NSArray *selectedFiles;

    if (selectedFiles=[currentViewer relativeSelectedPaths]) {
        [self retrieveVersionOfFiles:selectedFiles inPath:[rootFile path]];
    }
}

- (void) updateSelectedFiles:sender
{
    NSArray *selectedFiles=[currentViewer relativeSelectedPaths];
    if (selectedFiles) {
        [[CvsUpdateRequest cvsUpdateRequestForFiles:selectedFiles inPath:[rootFile path]] schedule];
    }
}

- (void) updateWorkAreaSheetDidDismiss:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    if(returnCode == NSAlertDefaultReturn)
        [[CvsUpdateRequest cvsUpdateRequestForFiles:nil inPath:[rootFile path]] schedule];
}

- (void) updateWorkArea:sender
{
#ifdef MACOSX
    NSBeginAlertSheet(@"CVS update", @"OK", @"Cancel", nil, window, self, NULL, @selector(updateWorkAreaSheetDidDismiss:returnCode:contextInfo:), NULL, @"Do you really want to update the whole Work Area ?");
#else
    if (NSRunAlertPanel(@"CVS update", @"Do you really want to update the whole Work Area ?", @"OK", @"Cancel", nil)==NSAlertDefaultReturn)
    {
        [[CvsUpdateRequest cvsUpdateRequestForFiles:nil inPath:[rootFile path]] schedule];
    }
#endif
}

- (IBAction) closeWorkArea:(id)sender
{
    [window performClose:sender];
}

- (void) commitSelectedFiles:sender
{
    NSArray *selectedFiles=[currentViewer relativeSelectedPaths];

    if (selectedFiles) {
        if([[NSUserDefaults standardUserDefaults] boolForKey:@"UseCvsTemplates"])
            [[CvsCommitRequest cvsCommitRequestForFiles:selectedFiles inPath:[rootFile path] message:nil] schedule];
        else{
            SenFormPanelController *commitPanel= [[SenPanelFactory sharedPanelFactory] handleOfPanelWithName:@"CvsCommitPanel"];
            [commitPanel setObjectValue:@"Commit Selected Files" forKey:@"PanelTitle"];

            if ([commitPanel showAndRunModal]==NSOKButton) {
                NSDictionary* tagInfos= [commitPanel dictionaryValue];

                if(![tagInfos objectForKey:@"message"] || [(NSString *)[tagInfos objectForKey:@"message"] length] == 0){
#ifdef MACOSX
                    NSBeginAlertSheet(@"Commit", nil, nil, nil, window, nil, NULL, NULL, NULL, @"You must give a commit message.");
#else
                    (void)NSRunAlertPanel(@"Commit", @"You must give a commit message.", nil, nil, nil);
#endif
                    return;
                }
                [[CvsCommitRequest cvsCommitRequestForFiles: selectedFiles inPath:[rootFile path] message: [tagInfos objectForKey:@"message"]] schedule];
            }
        }
    }
}

- (void) commitWorkArea:sender
{
    if([[NSUserDefaults standardUserDefaults] boolForKey:@"UseCvsTemplates"])
        [[CvsCommitRequest cvsCommitRequestForFiles:nil inPath:[rootFile path] message:nil] schedule];
    else{
        SenFormPanelController *commitPanel= [[SenPanelFactory sharedPanelFactory] handleOfPanelWithName:@"CvsCommitPanel"];
        [commitPanel setObjectValue:@"Commit All Files In Work Area" forKey:@"PanelTitle"];

        if ([commitPanel showAndRunModal]==NSOKButton) {
            NSDictionary* tagInfos= [commitPanel dictionaryValue];

            if(![tagInfos objectForKey:@"message"] || [(NSString *)[tagInfos objectForKey:@"message"] length] == 0){
#ifdef MACOSX
                NSBeginAlertSheet(@"Commit", nil, nil, nil, window, nil, NULL, NULL, NULL, @"You must give a commit message.");
#else
                (void)NSRunAlertPanel(@"Commit", @"You must give a commit message.", nil, nil, nil);
#endif
                return;
            }
            [[CvsCommitRequest cvsCommitRequestForFiles:nil inPath:[rootFile path] message: [tagInfos objectForKey:@"message"]] schedule];
        }
    }
}


- (void) tagSelectedFiles:sender
{
    NSArray *selectedFiles=[currentViewer relativeSelectedPaths];

    if (selectedFiles)
    {
        SenFormPanelController *tagPanel= [[SenPanelFactory sharedPanelFactory] handleOfPanelWithName:@"CvsTagPanel"];

        [tagPanel setObjectValue:@"Tag Selected Files" forKey:@"PanelTitle"];
        if ([tagPanel showAndRunModal]==NSOKButton) {
            NSDictionary* tagInfos= [tagPanel dictionaryValue];
            [[CvsTagRequest cvsTagRequestForFiles: selectedFiles
                                           inPath: [rootFile path]
                                              tag: [tagInfos objectForKey:@"name"]
                                         isBranch: [[tagInfos objectForKey:@"branch"] boolValue]
                                     moveIfExists: [[tagInfos objectForKey:@"moveIfExists"] boolValue]
                                  tagIfUnmodified: [[tagInfos objectForKey:@"tagIfUnmodified"] boolValue]] schedule];
        }
    }
}


- (void) tagWorkArea:sender
{
    SenFormPanelController *tagPanel= [[SenPanelFactory sharedPanelFactory] handleOfPanelWithName:@"CvsTagPanel"];

    [tagPanel setObjectValue:@"Tag All Files in Work Area" forKey:@"PanelTitle"];
    if ([tagPanel showAndRunModal]==NSOKButton) {
        NSDictionary* tagInfos= [tagPanel dictionaryValue];
        [[CvsTagRequest cvsTagRequestForFiles: nil
                                       inPath: [rootFile path]
                                          tag: [tagInfos objectForKey:@"name"]
                                     isBranch: [[tagInfos objectForKey:@"branch"] boolValue]
                                 moveIfExists: [[tagInfos objectForKey:@"moveIfExists"] boolValue]
                              tagIfUnmodified: [[tagInfos objectForKey:@"tagIfUnmodified"] boolValue]] schedule];
    }
}

- (void) doCancelChangesInFiles:(NSArray *)selectedFiles
{
    NSEnumerator *pathes = [selectedFiles objectEnumerator];
    NSString *path = nil;

    while (path = [pathes nextObject]) {
        [[NSFileManager defaultManager] removeFileAtPath:[[rootFile path] stringByAppendingPathComponent: path] handler:nil];
    }
    [[CvsUpdateRequest cvsUpdateRequestForFiles:selectedFiles inPath:[rootFile path]] schedule];
}

- (void) cancelChangesSheetDidDismiss:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    NSArray *selectedFiles = (NSArray *)contextInfo;

    if(returnCode == NSAlertDefaultReturn)
        [self doCancelChangesInFiles:selectedFiles];
    [selectedFiles release];
}

- (void) cancelChangesInSelectedFiles:sender
{
    NSArray *selectedFiles=[currentViewer relativeSelectedPaths];

    if (selectedFiles) {
#ifdef MACOSX
        NSBeginAlertSheet(@"Delete and Update", @"Delete and Update", @"Cancel", nil, window, self, NULL, @selector(cancelChangesSheetDidDismiss:returnCode:contextInfo:), [selectedFiles retain], @"All your modifications will be lost.");
#else
        if (NSRunAlertPanel(@"Delete and Update", @"All your modifications will be lost.", @"Delete and Update", @"Cancel", nil)==NSAlertDefaultReturn) {
            [self doCancelChangesInFiles:selectedFiles];
        }
#endif
    }
}


- (void) diffSelectedFiles:sender
{
    NSEnumerator	*selectedFileEnum = [[currentViewer relativeSelectedPaths] objectEnumerator];
    NSString		*aFile;

    while(aFile = [selectedFileEnum nextObject]){
        NSString			*currentFullPath = [[rootFile path] stringByAppendingPathComponent:aFile];
        CVLOpendiffRequest	*aRequest = [CVLOpendiffRequest opendiffRequestForFile:currentFullPath];

        if(aRequest)
            [aRequest schedule];
        else{
#ifdef DEBUG
            NSLog(@"opendiff not possible for %@; using diff", aFile);
#endif
            [[CvsDiffRequest cvsDiffRequestAtPath:[rootFile path] files:[NSArray arrayWithObject:aFile] context:3 outputFormat:CVLNormalOutputFormat] schedule];
        }
    }
}


- (void) addSelectedFiles:sender
{
    NSArray *selectedFiles=[currentViewer relativeSelectedPaths];

    if (selectedFiles) {
        [[CvsAddRequest cvsAddRequestAtPath:[rootFile path] files:selectedFiles] schedule];
    }
}

- (void) addSelectedFilesAsBinary:sender
{
    NSArray *selectedFiles=[currentViewer relativeSelectedPaths];

    if (selectedFiles) {
        [[CvsAddRequest cvsAddRequestAtPath:[rootFile path] files:selectedFiles forcesBinary:YES] schedule];
    }
}


- (void) removeFilesSheetDidDismiss:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    NSArray *selectedFiles = (NSArray *)contextInfo;

    if(returnCode == NSAlertDefaultReturn)
        [[CvsRemoveRequest removeRequestAtPath: [rootFile path] files: selectedFiles] schedule];
    [selectedFiles release];
}

- (void) removeSelectedFiles:sender
{
    NSArray *selectedFiles=[currentViewer relativeSelectedPaths];

    if (selectedFiles)
    {
#ifdef MACOSX
        NSBeginAlertSheet(@"CVS Remove", @"OK", @"Cancel", nil, window, self, NULL, @selector(removeFilesSheetDidDismiss:returnCode:contextInfo:), [selectedFiles retain], @"Do you really want to remove the selected file(s) from the Work Area ?");
#else
        if (NSRunAlertPanel(@"CVS Remove", @"Do you really want to remove the selected file(s) from the Work Area ?", @"OK", @"Cancel", nil)==NSAlertDefaultReturn)
        {
            [[CvsRemoveRequest removeRequestAtPath: [rootFile path] files: selectedFiles] schedule];
        }
#endif
    }
}


- (void) refreshSelectedFiles:sender
{
    ResultsRepository *results = [ResultsRepository sharedResultsRepository];

    NSArray *selectedFiles=[currentViewer selectedPaths];
    if (selectedFiles && ([selectedFiles count] > 0)) {
        NSEnumerator *fileEnumerator = [selectedFiles objectEnumerator];
        id filePath;

        [results startUpdate];
        while (filePath = [fileEnumerator nextObject]) {
            CVLFile* file= [CVLFile treeAtPath:filePath];

            if ([file isLeaf])
            {
                [file invalidateAll];
            }
            else
            {
                [file traversePostorder:@selector(invalidateAll)];
            }
        }
        [results endUpdate];
    }
}


/*
- (IBAction) renameSelectedFile:(id)sender
{
    [[CVLRenameController sharedInstance] renameFileNamed:[[currentViewer relativeSelectedPaths] lastObject] fromWorkArea:[rootFile path]];
}
*/

- (void) doReleaseWorkAreaAndDelete:(BOOL)flag
{
    CvsReleaseRequest	*aRequest = [CvsReleaseRequest cvsReleaseRequestWithPath:[rootFile path] deleteWorkArea:flag handler:self];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(releaseRequestCompleted:) name:@"RequestCompleted" object:aRequest];
    [aRequest schedule];
}

- (void) releaseWorkAreaSheetDidDismiss:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    BOOL	flag;
    
    switch(returnCode){
        case NSAlertDefaultReturn:
            flag = YES;
            break;
        case NSAlertAlternateReturn:
            flag = NO;
            break;
        default:
            return;
    }
    [self doReleaseWorkAreaAndDelete:flag];
}

- (IBAction) releaseWorkArea:(id)sender
{
#ifdef MACOSX
    NSBeginAlertSheet(@"Release Work Area", @"Release & Delete", @"Release", @"Cancel", window, self, NULL, @selector(releaseWorkAreaSheetDidDismiss:returnCode:contextInfo:), NULL, @"Do you really want to release the work area?");
#else
    BOOL	flag;

    switch(NSRunAlertPanel(@"Release Work Area", @"Do you really want to release the work area?", @"Release & Delete", @"Release", @"Cancel")){
        case NSAlertDefaultReturn:
            flag = YES;
            break;
        case NSAlertAlternateReturn:
            flag = NO;
            break;
        default:
            return;
    }
    [self doReleaseWorkAreaAndDelete:flag];
#endif
}

- (BOOL) request:(CvsReleaseRequest *)aRequest releaseWorkAreaContainingModifiedFilesNumber:(unsigned)aNumber
{
    if(NSRunAlertPanel(@"Release Work Area", @"You have %d altered %@ in work area %@. Are you sure you want to release%@ it?", @"Cancel", ([aRequest deleteWorkArea] ? @"Release & Delete":@"Release"), nil, aNumber, (aNumber == 1 ? @"file":@"files"), [aRequest workAreaPath], ([aRequest deleteWorkArea] ? @" and delete":@"")) != NSAlertAlternateReturn){
        [[NSNotificationCenter defaultCenter] removeObserver:self name:@"RequestCompleted" object:aRequest];
        return NO;
    }
    return YES;
}

- (void) releaseRequestCompleted:(NSNotification *)aNotification
{
    NSParameterAssert(aNotification != nil);
    [[NSNotificationCenter defaultCenter] removeObserver:self name:[aNotification name] object:[aNotification object]];
    if([[aNotification object] succeeded]){
        // WARNING: request will succeed even if we tell it not to release! That's why we remove ourself as observer of
        // notification in request:releaseWorkAreaContainingModifiedFilesNumber: if user does not want to continue
        if([[aNotification object] deleteWorkArea] || ![[NSUserDefaults standardUserDefaults] boolForKey:@"DoNotCloseReleasedWorkArea"])
            [window performClose:nil];
    }
    else{
        // We could warn user...
    }
}

//-------------------------------------------------------------------------------------


// @implementation WorkAreaViewer(WindowDelegate)

- (id) validRequestorForSendType:(NSString *)sendType returnType:(NSString *)returnType
{
    if (([sendType isEqualToString: NSStringPboardType]) || ([sendType isEqualToString: NSFilenamesPboardType]))
    {
        // Even if there is no selection, we can provide a path
        return self;
    }
    else
    {
        return nil;
    }
}


- updateViewer: sender
{	// called through the responder chain
    ResultsRepository *resultsRepository=[ResultsRepository sharedResultsRepository];

    [resultsRepository startUpdate];
    [(CVLFile *)[CVLFile treeAtPath:[rootFile path]] traversePostorder:@selector(invalidateAll)];
    [resultsRepository endUpdate];

    return self;
} // updateViewer:


- (void) doOpenFilesInWS:(NSArray *)files
{ // called through the responder chain
    NSString *filePath;
    id enumerator=[files objectEnumerator];

    while (filePath=[enumerator nextObject])
    {
        // If open failed, we could try to open it with a default application
        // On Windows, files whose suffix is not mapped by any app are not opened
        // Let's use TextEdit/Open File service!
        if(![[NSWorkspace sharedWorkspace] openFile:filePath]){
            // Currently there seems to be a bug in TextEdit on WO 4.01 NT...
            // It is not able to open file if we give its name through NSFilenamesPboardType
            // but it works if we give it through NSStringPboardType
            NSPasteboard	*aPasteboard = [NSPasteboard pasteboardWithUniqueName];

            [aPasteboard declareTypes:[NSArray arrayWithObjects:NSFilenamesPboardType, NSStringPboardType, nil] owner:nil];
            [aPasteboard addTypes:[NSArray arrayWithObjects:NSFilenamesPboardType, NSStringPboardType, nil] owner:nil];
            [aPasteboard setPropertyList:[NSArray arrayWithObject:filePath] forType:NSFilenamesPboardType];
            [aPasteboard setString:filePath forType:NSStringPboardType];
            
            (void)NSPerformService(@"TextEdit/Open File", aPasteboard);
        }
    }
}

- openFilesInWS: sender
{
    [self doOpenFilesInWS:[currentViewer selectedPaths]];
    return self;
} // openFilesInWS:

- (void) viewerDoubleSelect: (NSNotification *)notification
{
    if ([notification object] == currentViewer)
    {
        [self openFilesInWS: self];
    }
}


- selectInWS: sender
{ // called through the responder chain
    NSString* path= [[currentViewer selectedPaths] lastObject];
  //BOOL result= NO;
    
  // NSLog(@"select result %d", result);
    [[NSWorkspace sharedWorkspace] selectFile:path inFileViewerRootedAtPath: @"/"]; //NSHomeDirectory()
    return self;
} // selectInWS:


- (void)windowDidBecomeKey:sender
{
    [[NSNotificationCenter defaultCenter] postNotificationName:@"ViewerDidBecomeKey" object: self];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"PathSelected" object: [self selectedPaths]];
} // windowDidBecomeKey:


- (void)windowWillClose:sender
{
    [window setDelegate:nil];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"ViewerWillClose" object: self];
    //        [self release];
} // windowWillClose:

- (id)autorelease
{
    return [super autorelease];
}

- (void)release
{
    [super release];
}

- (void)windowDidMiniaturize:sender
{
    windowIsMiniaturized= YES;
} // windowDidMiniaturize:


- (void)windowDidDeminiaturize:sender
{
    windowIsMiniaturized= NO;
} // windowDidDeminiaturize:


- (NSSize)windowWillResize:sender toSize:(NSSize)frameSize
{
    return [self suggestNewSizeForSize:frameSize];
}

- (NSSize)suggestNewSizeForSize:(NSSize)frameSize
{
    if ([currentViewer respondsToSelector: @selector(viewerWillResizetoSize:)]) {
        NSSize viewerSize;
        NSSize newViewerSize;
        NSSize contentSize;
        NSRect newWindowFrame;
        NSSize newContentSize;
        NSRect contentFrame;

        contentFrame=[NSWindow contentRectForFrameRect:[window frame] styleMask:[window styleMask]];
        contentSize=contentFrame.size;

        newWindowFrame=[window frame];
        newWindowFrame.size=frameSize;
        newContentSize=[NSWindow contentRectForFrameRect:newWindowFrame styleMask:[window styleMask]].size;

        viewerSize=[[currentViewer view] frame].size;
        newViewerSize.width=newContentSize.width-(contentSize.width-viewerSize.width);
        newViewerSize.height=newContentSize.height-(contentSize.height-viewerSize.height);

        newViewerSize=[currentViewer viewerWillResizetoSize:newViewerSize];
        newContentSize.width=newViewerSize.width+(contentSize.width-viewerSize.width);
        newContentSize.height=newViewerSize.height+(contentSize.height-viewerSize.height);

        contentFrame.size=newContentSize;
        frameSize=[NSWindow frameRectForContentRect:contentFrame styleMask:[window styleMask]].size;
    }
    return frameSize;
} // windowWillResize:toSize:

- (void)viewerFrameSizeChanged:(id)sender
{
    // Invoked when user changes column size/count in preferences
    // and from -showWindowWithDictionary:
    NSRect newFrame, oldFrame;

    oldFrame = newFrame =[window frame];
    newFrame.size=[self suggestNewSizeForSize:newFrame.size];
    if(NSEqualRects(oldFrame, newFrame)){
        // Special case: if frame didn't change, then browser
        // doesn't change size => doesn't update number of columns!
        // In order to force this, we change size by 1, then restore it,
        // this way notifications are sent correctly.
        oldFrame.size.width += 1;
        [window setFrame:oldFrame display:NO];
    }
    [window setFrame:newFrame display:YES];
}

- (void) windowDidResignKey: (NSNotification*) aNotification
{
    if ([aNotification object] == window)
    {
        currentTopLeft.x= NSMinX([window frame]);
        currentTopLeft.y= NSMaxY([window frame]);
    }
}

- (void) schedulerDidChange: (NSNotification *)notification
{
    [self updateProcessesIndicator];
}

- (void)updateProcessesIndicator
{
    int requestCount;

    requestCount=[[CVLScheduler sharedScheduler] requestCount];
    if (requestCount==0) {
        [processesButton setTitle:@""];
    } else {
        int requestCountForWorkArea = [[CVLScheduler sharedScheduler] requestCountForPath:[self rootPath]];
        if (requestCountForWorkArea < 2) {
            [processesButton setTitle:[NSString stringWithFormat:@"%d / %d background process...",requestCountForWorkArea, requestCount]];
        }
        else {
            [processesButton setTitle:[NSString stringWithFormat:@"%d / %d background processes...",requestCountForWorkArea, requestCount]];
        }
        
    }
}

//-------------------------------------------------------------------------------------

// @implementation WorkAreaViewer (MenuValidation)

- (BOOL)validateMenuItem:(id <NSMenuItem>)menuItem
{
    SEL action= [menuItem action];
    NSString* actionString= NSStringFromSelector(action);
    NSRange filesRange= [actionString rangeOfString: @"Files"];
    NSRange workareaRange= [actionString rangeOfString: @"WorkArea"];
    int selectionCount= [[currentViewer selectedPaths] count];
    int currentTag= [menuItem tag];
    
  // NSLog(@"validate action %@", actionString);
    if (currentTag)
    {
        return (selectionCount == currentTag);
    }
    else if (filesRange.length)
    {
        return (selectionCount > 0);
    }
    else if (workareaRange.length)
    {
        return (BOOL) (currentViewer != nil);
    }
    return YES;  // validate other actions by default
}

//-------------------------------------------------------------------------------------

//@implementation WorkAreaViewer (NSServicesRequests)

- (BOOL)writeSelectionToPasteboard:(NSPasteboard *)pboard
                             types:(NSArray *)types
{
    if (([types containsObject: NSStringPboardType]) || (([types containsObject: NSFilenamesPboardType])))
    {
        [pboard declareTypes: [NSArray arrayWithObjects: NSFilenamesPboardType, NSStringPboardType, nil] owner:self];
        return YES;
    }
    return NO;
}


- (void)pasteboard:(NSPasteboard *)pboard  provideDataForType:(NSString *)type
{
    NSArray	*data = [currentViewer selectedPaths];

    if(!data || ![data count])
        data = [NSArray arrayWithObject:[self rootPath]]; // Work area directory path
    
    if ([type isEqualToString: NSFilenamesPboardType])
    {
        [pboard setPropertyList: data forType: NSFilenamesPboardType];
    }
    else if ([type isEqualToString: NSStringPboardType])
    {
        NSString* allSelection= [data componentsJoinedByString: RequestNewLineString];

        [pboard setString: allSelection forType: NSStringPboardType];
    }
    return;
}


//-------------------------------------------------------------------------------------

// @implementation WorkAreaViewer (Private)

- (void)reflectFilterConfiguration
{
    id cellsOrderEnumerator=[cellsOrder objectEnumerator];
    id cellsEnumerator=[[filterButtonsMatrix cells] objectEnumerator];
    NSArray *configuration=[filterProvider stringArrayFilterDescription];
    id cell;

    while (cell=[cellsEnumerator nextObject]) {
        [cell setIntValue:[configuration containsObject:[cellsOrderEnumerator nextObject]]];
    }
}

- (void) doRetrieveFile:(NSString *)filePath inPath:(NSString *)path withRevision:(NSString *)revision date:(NSString *)dateString outputFile:(NSString *)outputPath
// File is retrieved in $TEMP and is opened automatically
{
    NSString			*fullPath = [NSTemporaryDirectory() stringByAppendingPathComponent:outputPath];
    CvsUpdateRequest	*updateRequest = nil;
    SelectorRequest		*openRequest;

    updateRequest = [CvsUpdateRequest cvsUpdateRequestForFile:filePath
                                                       inPath:path
                                                     revision:revision
                                                         date:dateString
                                      removesStickyAttributes:NO
                                                       toFile:fullPath];
    openRequest = [SelectorRequest requestWithTarget:self selector:@selector(doOpenFilesInWS:) argument:[NSArray arrayWithObject:[updateRequest destinationFilePath]]];
    [openRequest addPrecedingRequest: updateRequest];
    [openRequest schedule];
}

- (void) retrieveVersionOfFiles:(NSArray *)files inPath:(NSString *)path
{
    RetrievePanelController *controller;
    NSString *dateString;
    NSString *revision;
    int result;
    BOOL	removeStickyAttributes;

    controller=[RetrievePanelController new];
    if (result=[controller runModalForFiles: files inPath:path]) {
        revision=[controller revision];
        dateString=[controller dateString];
        removeStickyAttributes = [controller removesStickyAttributes];
        if (![revision length]) revision=nil;
        if (![dateString length]) dateString=nil;

        if (result==CVL_RETRIEVE_REPLACE) {
            [[CvsUpdateRequest cvsUpdateRequestForFiles:files inPath:path revision:revision date:dateString removesStickyAttributes:removeStickyAttributes] schedule];
        }
        else if (result==CVL_RETRIEVE_OPEN)
        {
            NSString* fullPath= [NSTemporaryDirectory() stringByAppendingPathComponent: [controller buildDestinationPath]];
            CvsUpdateRequest* updateRequest= nil;
            SelectorRequest	*openRequest;

            updateRequest= [CvsUpdateRequest cvsUpdateRequestForFile: [files objectAtIndex: 0]
                                                              inPath: path
                                                            revision: revision
                                                                date: dateString
                                                   removesStickyAttributes: removeStickyAttributes
                                                              toFile: fullPath];
            openRequest = [SelectorRequest requestWithTarget:self selector:@selector(doOpenFilesInWS:) argument:[NSArray arrayWithObject:[updateRequest destinationFilePath]]];
            [openRequest addPrecedingRequest: updateRequest];
            [openRequest schedule];
        }
        else if (result == CVL_RETRIEVE_SAVE_AS)
        {
            [[CvsUpdateRequest cvsUpdateRequestForFile: [files objectAtIndex: 0]
                                                inPath: path
                                              revision: revision
                                                  date: dateString
                                    removesStickyAttributes: removeStickyAttributes
                                                toFile: [controller destinationPath]] schedule];
        }
    }
}

@end

//-------------------------------------------------------------------------------------
