/* SenOpenPanelController.m created by ja on Mon 02-Mar-1998 */

// Copyright (c) 1997-2001, Sen:te Ltd.  All rights reserved.
//
// Use of this source code is governed by the license in OpenSourceLicense.html
// found in this distribution and at http://www.sente.ch/software/ ,  where the
// original version of this source code can also be found.
// This notice may not be removed from this file.

#import "SenOpenPanelController.h"

@interface SenOpenPanelController (private)
- (void)initValues;
@end

@implementation SenOpenPanelController
+ (void)initialize
{
    if (self == [SenOpenPanelController class]) {
        [self setVersion:1];
    }
}

- (void)setAction:(SEL)aSelector
{
    action=aSelector;
}

- (id)init
{
    if (self=[super init]) {
        [self initValues];
    }
    return self;
}

- (void)initValues
{
    title=@"";
    prompt=@"";
    requiredFileType=@"";
    treatsFilePackagesAsDirectories=NO;
    canChooseFiles=YES;
    canChooseDirectories=NO;
    allowsMultipleSelection=NO;
}

- (SEL)action
{
    return action;
}

- (void)setTarget:(id)anObject
{
    [anObject retain];
    [target release];
    target=anObject;
}

- (id)target
{
    return target;
}

- (void)setUseSaveVsOpenPanel:(BOOL)flag
{
    saveVsOpenPanel=flag;
}

- (BOOL)useSaveVsOpenPanel
{
    return saveVsOpenPanel;
}

- (NSOpenPanel *)initializedOpenPanel
{
    NSOpenPanel *panel;

    if (saveVsOpenPanel) {
        panel=(NSOpenPanel *)[NSSavePanel savePanel];
    } else {
        panel=[NSOpenPanel openPanel];
    }

    if (delegate) {
        [panel setDelegate:delegate];
    }
    if (accessoryView) {
        [panel setAccessoryView:accessoryView];
    }
    if (title && ![title isEqual:@""]) {
        [panel setTitle:title];
    }
    if (prompt && ![prompt isEqual:@""]) {
        [panel setPrompt:prompt];
    }
    if (requiredFileType && ![requiredFileType isEqual:@""]) {
        [panel setRequiredFileType:requiredFileType];
    }
    [panel setTreatsFilePackagesAsDirectories:treatsFilePackagesAsDirectories];

    if (!saveVsOpenPanel) {
        [panel setCanChooseFiles:canChooseFiles];
        [panel setCanChooseDirectories:canChooseDirectories];
        [panel setAllowsMultipleSelection:allowsMultipleSelection];
    }
    return panel;
}

- (BOOL)openPanel:(id)sender
{
    int result;
    NSOpenPanel *panel;

    panel=[self initializedOpenPanel];

    if (valueField) {
        [self takeObjectValueFrom:valueField];
    }
    if (fileNames && [fileNames count]) {
        result=[panel runModalForDirectory:directory file:[[fileNames objectAtIndex:0] lastPathComponent]];
    } else if (directory) {
        result=[panel runModalForDirectory:directory file:@""];
    } else {
        result=[panel runModal];
    }

    if (result==NSOKButton) {
        [directory release];
        directory=[[panel directory] retain];
        [fileNames release];
        fileNames=[[NSArray arrayWithObject:[panel filename]] retain];
        if (action && target) {
            [target performSelector:action withObject:self];
        }
        return YES;
    }
    return NO;
}

- (id)objectValue
{
    return fileNames;
}

- (NSString *)stringValue
{
    return [fileNames objectAtIndex:0];
}

- (void)setStringValue:(NSString *)value
{
    [directory release];
    directory=[[value stringByDeletingLastPathComponent] retain];
    [fileNames release];
    fileNames=[[NSArray arrayWithObject:value] retain];
}

// Actions
- (void)takeStringValueFrom:(id)sender
{
    [self setStringValue:[sender objectValue]];
}

- (void)takeStringArrayValueFrom:(id)sender
{
    id value;

    value=[sender objectValue];

    [directory release];
    directory=[[[value objectAtIndex:0] stringByDeletingLastPathComponent] retain];
    [fileNames release];
    fileNames=[value copy];
}

- (void)takeObjectValueFrom:(id)sender
{
    id value;

    value=[sender objectValue];
    if ([value isKindOfClass:[NSString class]]) {
        [self takeStringValueFrom:sender];
    } else if ([value isKindOfClass:[NSArray class]]) {
        [self takeStringArrayValueFrom:sender];
    }
}


// NSSavePanel options
- (void)setPrompt:(NSString *)value
{
    [value retain];
    [prompt release];
    prompt=value;
}

- (NSString *)prompt
{
    return prompt;
}

- (void)setTitle:(NSString *)value
{
    [value retain];
    [title release];
    title=value;
}

- (NSString *)title
{
    return title;
}

- (void)setRequiredFileType:(NSString *)value
{
    [value retain];
    [requiredFileType release];
    requiredFileType=value;
}

- (NSString *)requiredFileType
{
    return requiredFileType;
}

- (void)setTreatsFilePackagesAsDirectories:(BOOL)flag
{
    treatsFilePackagesAsDirectories=flag;
}

- (BOOL)treatsFilePackagesAsDirectories
{
    return treatsFilePackagesAsDirectories;
}

// NSOpenPanel options
- (void)setAllowsMultipleSelection:(BOOL)flag
{
    allowsMultipleSelection=flag;
}

- (BOOL)allowsMultipleSelection
{
    return allowsMultipleSelection;
}

- (void)setCanChooseDirectories:(BOOL)flag
{
    canChooseDirectories=flag;
}

- (BOOL)canChooseDirectories
{
    return canChooseDirectories;
}

- (void)setCanChooseFiles:(BOOL)flag
{
    canChooseFiles=flag;
}

- (BOOL)canChooseFiles
{
    return canChooseFiles;
}

// NSCoding
- (id)initWithCoder:(NSCoder *)decoder
{
    int		version;

    self = [self init];

    version = [decoder versionForClassName:@"SenOpenPanelController"];

    switch (version) {
    case 1:
        title=[[decoder decodeObject] retain];
        prompt=[[decoder decodeObject] retain];
        requiredFileType=[[decoder decodeObject] retain];
        [decoder decodeValueOfObjCType:@encode(BOOL) at:&treatsFilePackagesAsDirectories];
        [decoder decodeValueOfObjCType:@encode(BOOL) at:&canChooseFiles];
        [decoder decodeValueOfObjCType:@encode(BOOL) at:&canChooseDirectories];
        [decoder decodeValueOfObjCType:@encode(BOOL) at:&allowsMultipleSelection];
    case 0:
        [decoder decodeValueOfObjCType:@encode(BOOL) at:&saveVsOpenPanel];
    default:
        break;
    }
    
    return self;
}

- (void)encodeWithCoder:(NSCoder *)coder
{
    //[super encodeWithCoder:coder];
    // Version == 1
    [coder encodeObject:title];
    [coder encodeObject:prompt];
    [coder encodeObject:requiredFileType];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&treatsFilePackagesAsDirectories];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&canChooseFiles];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&canChooseDirectories];
    [coder encodeValueOfObjCType:@encode(BOOL) at:&allowsMultipleSelection];

    // Version == 0
    [coder encodeValueOfObjCType:@encode(BOOL) at:&saveVsOpenPanel];

}

@end
